/* file main.c */
/*
 *      dio87c (Diogene87) - the communication utility of dio87d
 *      Diogene87
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <libintl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include <config.h>

#if DG87_USE_LONG_OPTIONS 
#include <getopt.h> /* include <config.h> must be before this line */
#endif /*DG87_USE_LONG_OPTIONS*/

#include <libdg87/global.h>
#include <libdg87/connection.h>
#include <libdg87/conf.h>


extern char *optarg;
extern int optind;
extern int opterr;

char *hostname = NULL;
char *user = NULL;
char *daemon_name = NULL;
char *password = NULL;
char *ip_port_str = NULL;
BOOL verbose = FALSE;

/*
 * dg87_conf_private_field_array
 */
dg87_conf_field private_field_array[] = {
  {"ADDRESS", NULL, NULL},
  {"IPPORT", NULL, NULL},
  {"USER", NULL, NULL},
  {"PASSWORD", NULL, NULL},
  {NULL, NULL, NULL}
};

/*
 * dg87_conf_public_field_array
 */
dg87_conf_field public_field_array[] = {
  {"ADDRESS", NULL, NULL},
  {"IPPORT", NULL, NULL},
  {NULL, NULL, NULL}
};

dg87_conf *private_conf = NULL;
dg87_conf *public_conf = NULL;


/******************************************************************************/
/*
 * get Value
 *
 * - from command line 
 * - from environement variables
 * - from the private configuaration file
 * - from the public configuaration file
 */
char *
get_value (char *name, char *default_value, char *value)
{
  if (name == NULL || default_value == NULL)
    return NULL;

  /* return if value is already set */
  if (value != NULL)
    return value;

  /* free memory of previous value */
  free (value);

  /* return environment variable if set */
  if ((value = getenv (name)) != NULL)
    return value;

  /* return value of private configuration file */
  if (private_conf != NULL)
    if ((value = dg87_conf_get_field_value (private_conf, name)) != NULL)
      return value;

  /* return value of public configuration file */
  if (public_conf != NULL)
    if ((value = dg87_conf_get_field_value (public_conf, name)) != NULL)
      return value;

  /* return default value */
  return default_value;
}

/******************************************************************************/
void
remove_character (char *string, char character)
{
  int i, j = 0;
  for (i = 0; string[i] != '\0'; i++)
    {
      if (string[i] == character)
	continue;
      string[j++] = string[i];
    }
  string[j] = '\0';
}

/******************************************************************************/
BOOL
read_config_files (char *config_file)
{
  char *ip_port_field = NULL;
  char *address_field = NULL;
  char *user_field = NULL;
  char *password_field = NULL;
  char *home_dir;
  struct stat fstat;
  BOOL specific = TRUE;

  if (config_file == NULL)
    {
      config_file = DIO87C_PRIVATE_CONFIGFILE;
      specific = FALSE;
    }

  /* get home directory from environment variable */
  home_dir = getenv ("HOME");

  /* if config file path is relative to home directory */
  if (config_file != NULL && config_file[0] == '~')
    {
      if (home_dir == NULL)
	{
	  fprintf (stderr, "Can't open configuration file '%s', "
		   "because HOME environment variable "
		   "is not set !\n", config_file);
	  return FALSE;
	}
      /* change current directory to home directory */
      if (chdir (home_dir) == -1)
	{
	  fprintf (stderr,
		   "Can't access to home directory '%s' \n", home_dir);
	  return FALSE;
	}
      /* make config path relative to current directory (home) */
      config_file++;		/* skip '~' character */
      if (config_file[0] == '/')
	config_file++;		/*skip '/' character */
    }

  /* check if private configuration file exists */
  if (stat (config_file, &fstat) != 0)
    {
      /* don't print error message if '-C' was not used */
      if (specific == TRUE)
	{
	  fprintf (stderr,
		   "Configuration file '%s' doesn't exist !\n", config_file);
	  return FALSE;
	}
    }
  else
    {
      /* create configure file object */
      if ((private_conf = dg87_conf_new (config_file,
					 private_field_array)) == NULL)
	return FALSE;
      /* read configuration file */
      if (dg87_conf_read (private_conf) == FALSE)
	{
	  fprintf (stderr, "Error while reading private "
		   "configuration file '%s' !\n", config_file);
	  dg87_conf_delete (private_conf);
	  return FALSE;
	}
    }

  /* check if public configuration file exists */
  if (stat (DIO87C_PUBLIC_CONFIGFILE, &fstat) != 0)
    {
      /* don't print error message if '-C' was used */
      if (verbose == TRUE)
	{
	  fprintf (stderr,
		   "Can't access to public configuration file "
		   "'%s' ! Use default values !\n", DIO87C_PUBLIC_CONFIGFILE);
	}
    }
  else
    {
      /* create configure file object */
      if ((public_conf = dg87_conf_new (DIO87C_PUBLIC_CONFIGFILE,
					public_field_array)) == NULL)
	{
	  dg87_conf_delete (private_conf);
	  return FALSE;
	}
      /* read configuration file */
      if (dg87_conf_read (public_conf) == FALSE)
	{
	  fprintf (stderr,
		   "Error while reading public configuration "
		   "file '%s' !\n", DIO87C_PUBLIC_CONFIGFILE);
	  if (private_conf != NULL)
	    dg87_conf_delete (private_conf);
	  dg87_conf_delete (public_conf);
	  return FALSE;
	}
    }

  /* get address and IP port field names */
  address_field = "DIO87D_ADDRESS";
  ip_port_field = "DIO87D_IPPORT";
  user_field = "DIO87D_USER";
  password_field = "DIO87D_PASSWORD";

  /* read values from config file */
  hostname = get_value (address_field, "localhost", hostname);
  ip_port_str = get_value (ip_port_field, DIO87D_IPPORT, ip_port_str);
  user = get_value (user_field, DG87_USER_ADMIN_NAME, user);
  password = get_value (password_field, "", password);

  if (verbose == TRUE)
    {
      if (private_conf == NULL)
	fprintf (stderr, "private configuration file was not used.\n");
      if (public_conf == NULL)
	fprintf (stderr, "public configuration file was not used.\n");
      fprintf (stderr, "host name : \t%s\n", hostname);
      fprintf (stderr, "IP port : \t%s\n", ip_port_str);
      fprintf (stderr, "user name : %s\n", user);
      fprintf (stderr, "password : %s\n", password);
    }
  if (private_conf != NULL)
    dg87_conf_delete (private_conf);
  if (public_conf != NULL)
    dg87_conf_delete (public_conf);
  return TRUE;
}

/******************************************************************************/
static void
usage (void)
{
	char** p;
#define gettext_noop(String) (String)
#ifdef DG87_USE_LONG_OPTIONS
	static char* msg[] = {
gettext_noop ("Usage : dio87c [OPTION] [--] command ...\nOptions :\n"),
gettext_noop (" -C file_name, --configuration file_name\n"),
gettext_noop ("\tuse the specified configuration file.\n"),
gettext_noop (" -H hostname, --hostname hostname\n"),
gettext_noop ("\tspecify the name of the host where dio87d is running.\n"),
gettext_noop (" -P ip_port, --port ip_port\n"),
gettext_noop ("\tspecify the IP port of the dio87d daemon.\n"),
gettext_noop (" -u user_name, --user user_name\n"),
gettext_noop ("\tspecify the user account to use (by default 'admin').\n"),
gettext_noop (" -p password, --password password\n"),
gettext_noop ("\tspecify the password of the user account to use.\n"),
gettext_noop (" -s, --show\n"),
gettext_noop ("\tshow the communication progress by printing dots.\n"),
gettext_noop (" -V, --verbose\n"),
gettext_noop ("\tverbose. print more informations.\n"),
gettext_noop ("\nReport bugs to admin@diogene87.org\n"),
NULL
	};
#else /*DG87_USE_LONG_OPTIONS*/
	static char* msg[] = {
gettext_noop ("Usage : dio87c [OPTION] [--] command ...\n"),
gettext_noop ("Options :\n"),
gettext_noop (" -C file_name\n"),
gettext_noop ("\tuse the specified configuration file.\n"),
gettext_noop (" -H hostname\n"),
gettext_noop ("\tspecify the name of the host where dio87d is running.\n"),
gettext_noop (" -P ip_port\n"),
gettext_noop ("\tspecify the IP port of the dio87d daemon.\n"),
gettext_noop (" -u user_name\n"),
gettext_noop ("\tspecify the user account to use (by default 'admin').\n"),
gettext_noop (" -p password\n"),
gettext_noop ("\tspecify the password of the user account to use.\n"),
gettext_noop (" -s\n"),
gettext_noop ("\tshow the communication progress by printing dots.\n"),
gettext_noop (" -V\n"),
gettext_noop ("\tverbose. print more informations.\n"),
gettext_noop ("\nReport bugs to admin@diogene87.org\n"),
NULL
};
#endif /*DG87_USE_LONG_OPTIONS*/
	p = msg;
	while (*p != NULL){
		fprintf(stderr, "%s", gettext (*p));
		p++;
	}
}

/******************************************************************************/
static void 
version(void)
{
  fprintf (stderr, gettext( "%s (%s) %s\n"\
						"Copyright (C) 2003,2004,2005 Dominique Parisot.\n"\
						"Diogene87 comes with NO WARRANTY,\n"\
						"to the extent permitted by law.\n"\
						"You may redistribute copies of Diogene87\n"\
						"under the terms of the GNU General Public License.\n"\
						"For more information about these matters,\n"\
						"see the file named COPYING.\n"),
					DIO87C_NAME, PACKAGE, VERSION);

}
/******************************************************************************/
int
main (int argc, char *argv[])
{
  static char stropts[] = "C:H:P:u:p:hvsw:V";
#if DG87_USE_LONG_OPTIONS
	static struct option lopts[] = {
					/* name, has_arg, flag, val */
					{ "configuration", 1, 0, 'C' },
					{ "hostname", 1, 0, 'H' },  
					{ "port", 1, 0, 'P' },  
					{ "user", 1, 0, 'u' },  
					{ "password", 1, 0, 'p' },  
					{ "help", 0, 0, 'h' },
					{ "version", 0, 0, 'v' },
					{ "show", 0, 0, 's' },
					{ "wait", 1, 0, 'w' },
					{ "verbose", 0, 0, 'V' },
					{ 0, 0, 0, 0}
	};
	int cmdx;
#endif /*DG87_USE_LONG_OPTIONS*/
  char *string;
  char *banner;
  char *config_file = NULL;
  int optch;
  int length, i;
  BOOL show_progress = FALSE;
  struct timeval tv;
  dg87_connection *connection;
  int exit_code = -1;
  int tempo = 0;
  int ip_port = -1;

	/* gettext */
	if (setlocale (LC_ALL, "") == NULL)
					fprintf(stderr, "Warning : Can't set locale !\n");
	if (bindtextdomain (PACKAGE, LOCALEDIR) == NULL)
					fprintf(stderr, "Warning : Can't bind text domain !\n");
	if (textdomain(PACKAGE) == NULL)
					fprintf(stderr, "Warning : Can't set text domain !\n");

  /* interpret options */
#if DG87_USE_LONG_OPTIONS
	while ((optch = getopt_long(argc, argv, stropts, lopts, &cmdx)) != EOF)
#else /*DG87_USE_LONG_OPTIONS*/
  while ((optch = getopt (argc, argv, stropts)) != -1)
#endif /*not DG87_USE_LONG_OPTIONS*/
		{
      switch (optch)
					{
					case 'C':		/* configuration file */
						config_file = strdup (optarg);
						break;
					case 'H':		/* host */
						hostname = strdup (optarg);
						break;
					case 'P':		/* port */
						ip_port_str = strdup (optarg);
						break;
					case 'u':		/* user name */
						user = strdup (optarg);
						break;
					case 'p':		/* user password */
						password = strdup (optarg);
						break;
					case 'h':		/* help */
						usage();
						exit (0);
						break;
					case 's':		/* show progress */
						show_progress = TRUE;
						break;
					case 'v':		/* version */
						version ();
						exit (0);
						break;
					case 'V':		/* verbose mode */
						verbose = TRUE;
						break;
					case 'w':		/* tempo */
						tempo = atoi (optarg);
						break;
					default:
						fprintf (stderr, gettext ("Use -h for help.\n"));
						exit (0);
						break;
					}
    }

	if (argc <= 1) /* first argument is the program name */
	{
		fprintf (stderr, gettext ("Incorrect call !\n"));
		usage();
		exit (0);
	}
	
  /* print informations with verbose mode */
  if (verbose == TRUE)
    {
      fprintf (stderr, gettext ("Private configuration file : %s \n"),
	       DIO87C_PRIVATE_CONFIGFILE);
      fprintf (stderr, gettext ("Public configuration file : %s \n"),
	       DIO87C_PUBLIC_CONFIGFILE);
    }
  /* read configuration file */
  if (read_config_files (config_file) == FALSE)
    exit (1);
  ip_port = atoi (ip_port_str);

  /* create connection object */
  connection = dg87_connection_out_new (hostname, ip_port);
  if (connection == NULL)
    {
      fprintf (stderr, gettext ("Can't create the object '%s' !\n"),
											"connection");
      exit (1);
    }

  /* read banner */
  while (connection->state != dg87_connection_state_message_available &&
	 connection->state != dg87_connection_state_error)
    {
      if (dg87_connection_update (connection))
	if (show_progress)
	  {
	    fprintf (stderr, ".");
	    fflush (stderr);
	  }
      /* wait 0.001 second */
      tv.tv_sec = 0;
      tv.tv_usec = 1000;
      select (0, NULL, NULL, NULL, &tv);
    }
  if (show_progress == TRUE)
    fprintf (stderr, "\n");
  if (connection->state == dg87_connection_state_error)
    {
      dg87_connection_delete (connection);
      fprintf (stderr,
	       gettext ("Can't communicate with %s on '%s' IP port %i !\n"),
	       (daemon_name == NULL) ? "diogene87" : daemon_name,
	       hostname, ip_port);
      exit (1);
    }
  banner = dg87_connection_read (connection);

  /* interpret banner */
  /*diogene87-dio87d-0.9.2cvs-(P01) */
  /* verify package name */
  if (strncmp (banner, PACKAGE, strlen (PACKAGE)) != 0)
    {
      dg87_connection_delete (connection);
      fprintf (stderr, gettext ("Wrong banner '%s' !\n"), banner);
      exit (1);
    }

  if (daemon_name != NULL && strncmp (banner + strlen (PACKAGE) + 1,
				      daemon_name, strlen (daemon_name)) != 0)
    {
      dg87_connection_delete (connection);
      fprintf (stderr, gettext ("Wrong daemon name in banner '%s' !\n"), banner);
      exit (1);
    }

  dg87_free (banner, DG87_CHAR);


  /* send user and password */
  if (user != NULL && password != NULL)
    {

      dg87_connection_printf (connection, "user \"%s\" \"%s\"",
			      user, password);
      if (dg87_connection_send (connection) == FALSE)
	{
	  dg87_connection_delete (connection);
	  fprintf (stderr, gettext ("Can't send message !\n"));
	  exit (1);
	}
      /* send password and wait response */
      while (connection->state ==
	     dg87_connection_state_sending_message ||
	     connection->state == dg87_connection_state_reading_message)
	{
	  if (dg87_connection_update (connection))
	    if (show_progress)
	      {
		fprintf (stderr, ".");
		fflush (stderr);
	      }
	  /* wait 0.01 second */
	  tv.tv_sec = 0;
	  tv.tv_usec = 10000;
	  select (0, NULL, NULL, NULL, &tv);
	}
      if (show_progress == TRUE)
	fprintf (stderr, "\n");
      if (connection->state == dg87_connection_state_error)
	{
	  dg87_connection_delete (connection);
	  fprintf (stderr, gettext ("Communication error !\n"));
	  exit (1);
	}

      /* read response */
      string = dg87_connection_read (connection);
      if (string == NULL)
	{
	  fprintf (stderr, gettext ("Response unavailable !\n"));
	  dg87_connection_delete (connection);
	  exit (1);
	}

      /* get exit code */
      length = strlen (string);
      if (length == 2 && strcmp ("OK", string) == 0)
	{
	  *string = '\0';
	  exit_code = 0;
	}
      else if (length > 3 && strcmp ("OK", string + length - 2) == 0)
	{
	  *(string + length - 3) = '\0';
	  exit_code = 0;
	}
      else
	{
	  /* find the string ERROR: */
	  while (length > 0 && string[length] != ':')
	    length--;
	  if (length > 6 && strncmp ("ERROR:", string + length - 5, 6) == 0)
	    {
	      exit_code = atoi (string + length + 1);
	      *(string + length - 5) = '\0';
	      /* ensure that an error code is return */
	      if (exit_code == 0)
		exit_code = 255;
	    }
	  else
	    exit_code = -1;
	}

      /* print the string */
      for (i = 0; i < strlen (string); i++)
	if (string[i] != '\r')
	  fprintf ((exit_code == 0) ? stdout : stderr, "%c", string[i]);
      if (exit_code != 0)
	return exit_code;
    }

  /* tempo */
  if (tempo > 0)
    sleep (tempo);

  /* send command (concat all arguments) */
  for (i = optind; i < argc; ++i)
    {
      if (strlen (argv[i]) == 0)
	/* send an empty string */
	dg87_connection_printf (connection, "\"\" ");
      else if (strchr (argv[i], ' ') == NULL
	       && strchr (argv[i], '\t') == NULL
	       && strchr (argv[i], '\"') == NULL)
	dg87_connection_printf (connection, "%s ", argv[i]);
      else
	{
	  /* TODO if string contains quote, add a backslash 
	   * before */
	  dg87_connection_printf (connection, "\"%s\" ", argv[i]);
	}
    }
  /*printf("buffer:%s\n", connection->buffer); */

  if (dg87_connection_send (connection) == FALSE)
    {
      dg87_connection_delete (connection);
      fprintf (stderr, gettext ("Can't send message !\n"));
      exit (1);
    }

  /* send request and wait response */
  while (connection->state == dg87_connection_state_sending_message ||
	 connection->state == dg87_connection_state_reading_message)
    {
      if (dg87_connection_update (connection))
	if (show_progress)
	  {
	    fprintf (stderr, ".");
	    fflush (stderr);
	  }
      /* wait 0.01 second */
      tv.tv_sec = 0;
      tv.tv_usec = 10000;
      select (0, NULL, NULL, NULL, &tv);
    }
  if (show_progress == TRUE)
    fprintf (stderr, "\n");
  if (connection->state == dg87_connection_state_error)
    {
      dg87_connection_delete (connection);
      fprintf (stderr, gettext ("Communication error !\n"));
      exit (1);
    }

  /* read response */
  string = dg87_connection_read (connection);
  if (string == NULL)
    {
      fprintf (stderr, gettext ("Response unavailable !\n"));
      dg87_connection_delete (connection);
      exit (1);
    }


  /* get exit code */
  length = strlen (string);
  if (length == 2 && strcmp ("OK", string) == 0)
    {
      *string = '\0';
      exit_code = 0;
    }
  else if (length > 3 && strcmp ("OK", string + length - 2) == 0)
    {
      *(string + length - 2) = '\0';
      exit_code = 0;
    }
  else
    {
      /* find the string ERROR: */
      while (length > 0 && string[length] != ':')
	length--;
      if (length > 6 && strncmp ("ERROR:", string + length - 5, 6) == 0)
	{
	  exit_code = atoi (string + length + 1);
	  *(string + length - 5) = '\0';
	  /* ensure that an error code is return */
	  if (exit_code == 0)
	    exit_code = 255;
	}
      else
	exit_code = -1;
    }

  remove_character (string, '\r');
  printf ("%s", string);

  /* free memory */
  free (string);
  dg87_connection_delete (connection);

  return exit_code;
}

/* End of file */
