/* file cmdengine.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "cmdengine.h"
#include "util.h"
#include "x_day.h"

/******************************************************************************/
/* ei
 * Engine Information
 * Usage : ei
 */
int
cmd_ei (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_engine *engine;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  engine = context->engine;

  /* control the argument number */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ei [options]");
      return 1;
    }

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Enabled", sep,
                          dg87_bool_to_ascii (engine->enabled), end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Day Retention", sep,
                          context->day_purge_retention, end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "IP port", sep, 
													context->server->ip_port, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Pid file", sep,
                          (context->pid_file ==
                           NULL) ? "None" : context->pid_file, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", beg,
                          "Configuration file", sep,
                          context->conf->config_file, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", beg, 
													"Data dir", sep,
                          context->data_directory, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", beg, 
													"Data file",
                          sep, context->storage->data_file, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", beg, 
													"Log level", sep,  
													dg87_daemon_get_log_priority(), end);
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* el
 * Engine Log 
 * Usage : el
 */
int
cmd_el (dg87_connection * connection)
{
  dg87_engine *engine;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  engine = context->engine;

  /* control the argument number */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "el [options]");
      return 1;
    }

  dg87_connection_printf (connection, "%s\n", engine->log->buffer);
  return 0;
}

/******************************************************************************/
int
cmd_ese (dg87_connection * connection)
{
  dg87_engine *engine;
  dg87_context *context;
  dg87_command *command;
  BOOL value;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ese BooleanValue");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  if ((value = dg87_ascii_to_bool (command->argv[0])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[0]);
      return 1;
    }

  engine = context->engine;
	dg87_engine_set_enabled(engine, value);
	DG87_SAVE_NEEDED;
  return 0;
}

/******************************************************************************/
/* etm
 * eXploitation Tasks Monitor 
 * return statistics on tasks 
 * Usage : ets [options] [DayName]
 */
int
cmd_etm (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  char state;
  dg87_context *context;
  dg87_command *command;
  dg87_element *task_element;
  dg87_element *day_element;
  dg87_exp_day *seleted_day = NULL;
  dg87_exp_day *day;
  dg87_exp_task *task;

  int n_total = 0;
  int n_waiting = 0;
  int n_running = 0;
  int n_holded = 0;
  int n_terminated = 0;
  int n_error = 0;
  int n_cancel = 0;

	BOOL print_target = FALSE;
	BOOL print_queue = FALSE;
	BOOL print_description = FALSE;

  BOOL show_holded = FALSE;
  BOOL show_running = FALSE;
  BOOL show_Error = FALSE;
  BOOL show_Cancel = FALSE;
  BOOL show_terminated = FALSE;
  BOOL show_waiting = FALSE;

  /* by default, shows job counters and not job list */
  BOOL show_counters = TRUE;
  BOOL show_list = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  if (command->argc > 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "etm [options] [DayName]");
      return 1;
    }

  /* if DayName argument was supplied */
  if (command->argc == 1)
    {
      /* find the day */
      seleted_day = dg87_util_find_exp_day (connection, command->argv[0]);
      if (seleted_day == NULL)
        return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'h':              /* holded jobs */
          show_holded = TRUE;
          break;
        case 'r':              /* running jobs */
          show_running = TRUE;
          break;
        case 'e':              /* jobs on error */
          show_Error = TRUE;
          break;
        case 'c':              /* canceled jobs */
          show_Cancel = TRUE;
          break;
        case 't':              /* terminated jobs */
          show_terminated = TRUE;
          break;
        case 'w':              /* waiting jobs */
          show_waiting = TRUE;
          break;
				case 'T': /* print target name */
					print_target = TRUE;
					break;
				case 'Q': /* print queue and target names */
					print_queue = TRUE;
					break;
				case 'D': /* print task description */
					print_description = TRUE;
					break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* if filter option was given then shows task list */
  if (show_holded == TRUE || show_running == TRUE ||
      show_Error == TRUE || show_Cancel == TRUE ||
      show_terminated == TRUE || show_waiting == TRUE)
    {
      show_counters = FALSE;
      show_list = TRUE;
    }

  /* for each day */
  day_element = context->exp_day_vector->first;
  while (day_element != NULL)
    {
      day = day_element->object;
      day_element = day_element->next;

      /* skip unwanted days if day filter was given */
      if (seleted_day != NULL && seleted_day != day)
        continue;

      /* for each task of the day */
      task_element = day->task_vector->first;
      while (task_element != NULL)
        {
          task = task_element->object;
          task_element = task_element->next;

          n_total++;
          state = '_';
          switch (task->state)
            {

              /* waiting */
            case dg87_exp_task_state_uninitialized:
            case dg87_exp_task_state_wait_for_dep:
            case dg87_exp_task_state_wait_lock:
            case dg87_exp_task_state_wait_for_condition:
            case dg87_exp_task_state_wait_tempo:
              n_waiting++;
              state = 'W';
              if (show_waiting == FALSE)
                continue;
              break;

              /* terminated */
            case dg87_exp_task_state_terminated:
              n_terminated++;
              state = 'T';
              if (show_terminated == FALSE)
                continue;
              break;

              /* holded */
            case dg87_exp_task_state_aborted_stopped:
            case dg87_exp_task_state_wait_enabled:
              n_holded++;
              state = 'H';
              if (show_holded == FALSE)
                continue;
              break;

              /* running */
            case dg87_exp_task_state_running:
              n_running++;
              state = 'R';
              if (show_running == FALSE)
                continue;
              break;

              /* error */
            case dg87_exp_task_state_deleted:
            case dg87_exp_task_state_aborted:
              n_error++;
              state = 'E';
              if (show_Error == FALSE)
                continue;
              break;

              /* cancel */
            case dg87_exp_task_state_ignore:
            case dg87_exp_task_state_ignored:
            case dg87_exp_task_state_canceled:
              n_cancel++;
              state = 'C';
              if (show_Cancel == FALSE)
                continue;
              break;
            }

          /* prints tasks if necessary */
          if (show_list == FALSE)
            continue;
          dg87_connection_printf (connection,
                                  "%s%c%s%s%s%s%s%s",
                                  beg, state,
                                  sep, task->day->name,
                                  sep, task->name,
                                  sep,
                                  dg87_exp_task_state_to_ascii (task->state));
					if (print_target == TRUE || print_queue == TRUE)
              	dg87_connection_printf (connection,
															"%s%s", sep, task->queue->target->name);
					if (print_queue == TRUE)
              	dg87_connection_printf (connection,
															"%s%s", sep, task->queue->name);
					if (print_description == TRUE)
              	dg87_connection_printf (connection,
															"%s%s", sep, task->description);
					dg87_connection_printf (connection,
															"%s\n", end);

        }/* next task */
    }/* next day */

  /* print counters if necessary */
  if (show_counters == TRUE)
    {
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Total", sep, 
															n_total, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Waiting", sep, 
															n_waiting, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Holded", sep, 
															n_holded, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Running", sep, 
															n_running, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Error", sep, 
															n_error, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Cancel", sep, 
															n_cancel, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Terminated", sep, 
															n_terminated, end);
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* ejm
 * eXploitation Jobs Monitor 
 * return statistics on jobs
 * Usage : ejm [options] [DayName]
 */
int
cmd_ejm (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  char state;
  dg87_context *context;
  dg87_command *command;
  dg87_element *job_element;
  dg87_element *task_element;
  dg87_element *day_element;
  dg87_exp_day *seleted_day = NULL;
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *job;

  int n_total = 0;
  int n_waiting = 0;
  int n_running = 0;
  int n_holded = 0;
  int n_terminated = 0;
  int n_error = 0;
  int n_cancel = 0;
	BOOL print_target = FALSE;
	BOOL print_queue = FALSE;
	BOOL print_description = FALSE;

  BOOL show_holded = FALSE;
  BOOL show_running = FALSE;
  BOOL show_Error = FALSE;
  BOOL show_Cancel = FALSE;
  BOOL show_terminated = FALSE;
  BOOL show_waiting = FALSE;

  /* by default, shows job counters and not job list */
  BOOL show_counters = TRUE;
  BOOL show_list = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  if (command->argc > 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ejm [options] [DayName]");
      return 1;
    }

  /* if DayName argument was supplied */
  if (command->argc == 1)
    {
      /* find the day */
      seleted_day = dg87_util_find_exp_day (connection, command->argv[0]);
      if (seleted_day == NULL)
        return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'h':              /* holded jobs */
          show_holded = TRUE;
          break;
        case 'r':              /* running jobs */
          show_running = TRUE;
          break;
        case 'e':              /* jobs on error */
          show_Error = TRUE;
          break;
        case 'c':              /* canceled jobs */
          show_Cancel = TRUE;
          break;
        case 't':              /* terminated jobs */
          show_terminated = TRUE;
          break;
        case 'w':              /* waiting jobs */
          show_waiting = TRUE;
          break;
				case 'T': /* print target name */
					print_target = TRUE;
					break;
				case 'Q': /* print queue and target names */
					print_queue = TRUE;
					break;
				case 'D': /* print job description */ 
					print_description = TRUE;
					break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* if filter option was given then shows job list */
  if (show_holded == TRUE || show_running == TRUE ||
      show_Error == TRUE || show_Cancel == TRUE ||
      show_terminated == TRUE || show_waiting == TRUE)
    {
      show_counters = FALSE;
      show_list = TRUE;
    }

  /* for each day */
  day_element = context->exp_day_vector->first;
  while (day_element != NULL)
    {
      day = day_element->object;
      day_element = day_element->next;

      /* skip unwanted days if day filter was given */
      if (seleted_day != NULL && seleted_day != day)
        continue;

      /* for each task of the day */
      task_element = day->task_vector->first;
      while (task_element != NULL)
        {
          task = task_element->object;
          task_element = task_element->next;

          /* for each job of the task */
          job_element = task->job_vector->first;
          while (job_element != NULL)
            {
              job = job_element->object;
              job_element = job_element->next;

              n_total++;
              state = '_';
              switch (job->state)
                {

                  /* waiting */
                case dg87_exp_job_state_uninitialized:
                case dg87_exp_job_state_wait_for_dep:
                  n_waiting++;
                  state = 'W';
                  if (show_waiting == FALSE)
                    continue;
                  break;

                  /* terminated */
                case dg87_exp_job_state_terminated:
                case dg87_exp_job_state_exit:
                  n_terminated++;
                  state = 'T';
                  if (show_terminated == FALSE)
                    continue;
                  break;

                  /* holded */
                case dg87_exp_job_state_aborted_stopped:
                case dg87_exp_job_state_deleted_stopped:
                case dg87_exp_job_state_wait_enabled:
                case dg87_exp_job_state_ready_to_run:
                case dg87_exp_job_state_ready_to_submit:
                case dg87_exp_job_state_submit:
                  n_holded++;
                  state = 'H';
                  if (show_holded == FALSE)
                    continue;
                  break;

                  /* running */
                case dg87_exp_job_state_running:
                case dg87_exp_job_state_wait_start_tempo:
                case dg87_exp_job_state_wait_end_tempo:
                  n_running++;
                  state = 'R';
                  if (show_running == FALSE)
                    continue;
                  break;

                  /* error */
                case dg87_exp_job_state_deleted:
                case dg87_exp_job_state_aborted:
                  n_error++;
                  state = 'E';
                  if (show_Error == FALSE)
                    continue;
                  break;

                  /* cancel */
                case dg87_exp_job_state_ignore:
                case dg87_exp_job_state_ignored:
                case dg87_exp_job_state_canceled:
                  n_cancel++;
                  state = 'C';
                  if (show_Cancel == FALSE)
                    continue;
                  break;
                }

              /* prints job if necessary */
              if (show_list == FALSE)
                continue;
              dg87_connection_printf (connection,
															"%s%c%s%s%s%s%s%s%s%s",
															beg, state,
															sep, job->task->day->name,
															sep, job->task->name,
															sep, job->name,
															sep,
															dg87_exp_job_state_to_ascii (job->state));
							if (print_target == TRUE || print_queue == TRUE)
              	dg87_connection_printf (connection,
															"%s%s", sep, job->task->queue->target->name);
							if (print_queue == TRUE)
              	dg87_connection_printf (connection,
															"%s%s", sep, job->task->queue->name);
							if (print_description == TRUE)
              	dg87_connection_printf (connection,
															"%s%s", sep, job->description);
              dg87_connection_printf (connection,
															"%s\n", end);
            }/* next job */
        }/* next task */
    }/* next day */

  /* print counters if necessary */
  if (show_counters == TRUE)
    {
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Total", sep, n_total, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Waiting", sep, n_waiting, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Holded", sep, n_holded, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Running", sep, n_running, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Error", sep, n_error, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Cancel", sep, n_cancel, end);
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Terminated", sep, 
															n_terminated, end);
    }

  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
int
cmd_save (dg87_connection * connection)
{
  dg87_engine *engine;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  engine = context->engine;

  /* control the argument number */
  if (command->argc != 0 || command->optc != 0)
    {
      dg87_connection_printf (connection,
                       gettext ("Incorrect call.\nUsage : %s\n"), "save");
      return 1;
    }
  if (dg87_context_save (context) == FALSE)
    {
      dg87_connection_printf (connection, gettext ("Save Error !\n"));
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* End of file */
