/* file cmdqueue.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "cmdqueue.h"
#include "util.h"
#include "x_task.h"
#include "x_day.h"

/******************************************************************************/
/* cmd_qk
 * Queue Kill
 * Usage : qk TargetName QueueName
 */
int
cmd_qk (dg87_connection * connection)
{
  dg87_target *target;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qk TargetName QueueName\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* delete the queue */
  dg87_err_clear_error ();
  if (dg87_target_queue_delete_by_name (target, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't delete the queue '%s' "\
															"in the target '%s' ! %s\n"),
											command->argv[1], target->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************
 * cmd_qa
 * Queue Add
 * Usage : qa TargetName QueueName [description]
 */
int
cmd_qa (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc < 2 || command->argc > 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qa TargetName QueueName [description]\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* add the queue to the Target */
  dg87_err_clear_error ();
  queue = dg87_target_queue_add (target, command->argv[1], FALSE);
  if (queue == NULL)
    {
      dg87_connection_printf (connection,
							gettext ("Can't add the queue '%s' on the target '%s' ! %s\n"),
							command->argv[1], target->name,
							dg87_err_get_error_str ());
      return 1;
    }
  /* change description */
  if (command->argc == 3)
    {
      dg87_err_clear_error ();
      if (dg87_queue_set_description (queue, command->argv[2]) == FALSE)
        {
          dg87_connection_printf (connection,
							gettext ("Can't add the queue '%s' on the target '%s' ! %s\n"),
							command->argv[1], target->name,
							dg87_err_get_error_str ());
          dg87_target_queue_delete (target, queue);
          return 1;
        }
    }
  return 0;
}

/******************************************************************************/
/* cmd_qc
 * Usage : qc TargetName
 * Queue count
 * return the number of queues in the Target.
 */
int
cmd_qc (dg87_connection * connection)
{
  dg87_target *target;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qc TargetName\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;
  /* print Queue count */
  dg87_connection_printf (connection, "%i\n", target->queue_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_ql
 * Usage : ql [options] TargetName
 * Queue List
 * options
 * -d : print description of queue 
 * -t : print Target name of queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first queue to display (first index is 0)
 * -Nnumber : number of queue to display (no limit if -N < 1)
 */
int
cmd_ql (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "dtB:S:E:I:N:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first queue to display */
  int number = 0; /* number of queues to display */
  int count = 0;  /* number of queue actually display */
  dg87_element *element = NULL;
  dg87_target *target;
  dg87_queue *queue;
  BOOL show_description = FALSE;
  BOOL show_target_name = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ql [options] TargetName");
      return 1;
    }

  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");
  show_description = FALSE;
  show_target_name = FALSE;

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first queue to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of queue to display */
          number = atoi (optarg);
          break;
        case 'd':              /* description */
          show_description = TRUE;
          break;
        case 't':              /* target */
          show_target_name = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first queue to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (target->queue_vector, start);
  else
    element = target->queue_vector->first;


  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of queue to display */
      count++;
      if (number > 0 && count > number)
        break;
      queue = element->object;

      /* print Target */
      dg87_connection_printf (connection, beg);
      if (show_target_name == TRUE)
        dg87_connection_printf (connection, "%s%s", queue->target->name, sep);

      /* print Queue Name */
      dg87_connection_printf (connection, queue->name);

      /* print Queue Description */
      if (show_description == TRUE)
        dg87_connection_printf (connection, "%s%s", sep, queue->description);

      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* qi
 * Usage : qi [options] TargetName QueueName
 * Queue Information 
 * get information for a queue 
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_qi (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "B:S:E:"; */
  int optch, iopt;
  char *optarg;
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qi [options] TargetName QueueName");
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
      return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection,
                                     target, command->argv[1])) == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':/* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':/* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':/* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default: /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* print target name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Target Name", sep, 
													queue->target->name, end);
  /* print queue name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Queue Name", sep, 
													queue->name, end);
  /* print queue desription */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Description", sep, 
													queue->description, end);
  /* open property */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Opened", sep,
                          dg87_bool_to_ascii (queue->open), end);
  /* max property */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Max", sep, 
													queue->max, end);
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/***************************************************************************/
/* qsd
 * Queue Set Description
 * Usage : qsd TargetName QueueName description
 */
int
cmd_qsd (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qsd TargetName QueueName description");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* find the queue */
  queue = dg87_util_find_queue (connection, target, command->argv[1]);
  if (queue == NULL)
    return 1;

  /* change the description */
  dg87_err_clear_error ();
  if (dg87_queue_set_description (queue, command->argv[2]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set description for the queue "\
                              "'%s' in the target '%s' ! %s\n"),
                              queue->name, target->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/***************************************************************************/
/* cmd_qrtc
 * Usage : qrtc TargetName QueueName
 * return the number of referential tasks in the queue.
 */
int
cmd_qrtc (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "qrtc TargetName QueueName");
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
    return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection, target,
                                     command->argv[1])) == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", queue->ref_task_list->count);
  return 0;
}

/***************************************************************************/
/* cmd_qrtl
 * Usage : qrtl [options] TargetName QueueName
 * Queue Ref Task List
 * give the list of Task of the ref which have this queue 
 * Options :
 * -d : print description of task
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int
cmd_qrtl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "dB:S:E:I:N:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first task to display */
  int number = 0;               /* number of TASKs to display */
  int count = 0;                /* number of TASK actually display */
  dg87_item *item = NULL;
  dg87_ref_task *task;
  dg87_target *target;
  dg87_queue *queue;
  BOOL show_description = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "qrtl [options] TargetName QueueName");
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
    return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection, target,
                                     command->argv[1])) == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        case 'd': /* description */
          show_description = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first task to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (queue->ref_task_list, start);
  else
    item = queue->ref_task_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      task = item->object;

      /* print ref name */
      dg87_connection_printf (connection, "%s%s", beg, task->ref->name);

      /* print Task name */
      dg87_connection_printf (connection, "%s%s", sep, task->name);

      /* print description */
      if (show_description == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep, task->description);
        }
      dg87_connection_printf (connection, "%s\n", end);

      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}
/***************************************************************************/
/* qsn
 * Queue Set Name 
 * Usage : tsn TargetName QueueName NewName 
 */
int
cmd_qsn (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qsn TargetName QueueName NewName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* find the queue */
  queue = dg87_util_find_queue (connection, target, command->argv[1]);
  if (queue == NULL)
    return 1;

  /* change the queue name */
  dg87_err_clear_error ();
  if (dg87_queue_set_name (queue, command->argv[2]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the name of the queue `%s' "\
											"of the target '%s' ! %s\n"),
											queue->name, target->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}
/***************************************************************************/
/* qso
 * Queue Set Open 
 * Usage : tsd TargetName QueueName BooleanValue 
 */
int
cmd_qso (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  BOOL value;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qso TargetName QueueName BooleanValue");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* find the queue */
  queue = dg87_util_find_queue (connection, target, command->argv[1]);
  if (queue == NULL)
    return 1;


  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }

  /* change the property */
  dg87_err_clear_error ();
  if (dg87_queue_set_open (queue, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the '%s' property of the "\
											"queue '%s' of the target '%s' ! %s\n"),
											"open", queue->name, target->name,
											dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/***************************************************************************/
/* qsm
 * Queue Set Max 
 * Usage : tsd TargetName QueueName IntegerValue 
 */
int
cmd_qsm (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  int value;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qsm TargetName QueueName IntegerValue");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* find the queue */
  queue = dg87_util_find_queue (connection, target, command->argv[1]);
  if (queue == NULL)
    return 1;

  /* change the value */
  dg87_err_clear_error ();
  value = atoi (command->argv[2]);
  if (dg87_queue_set_max (queue, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the '%s' property of the "\
											"queue '%s' of the target '%s' ! %s\n"),
											"max", queue->name, target->name,
											dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}



/***************************************************************************/
/* cmd_qxtc
 * Usage : qxtc [options] TargetName QueueName
 * return the number of exploitation tasks in the queue.
 */
int
cmd_qxtc (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qxtc TargetName QueueName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
    return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection, target,
                                     command->argv[1])) == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", queue->exp_task_list->count);
  return 0;
}

/***************************************************************************/
/* cmd_qxtl
 * Usage : qxtl [options] TargetName QueueName
 * Queue eXp Task List
 * give the list of Task of the exp which have this queue 
 * Options :
 * -d : print description of task
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int
cmd_qxtl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "dB:S:E:I:N:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first task to display */
  int number = 0;               /* number of TASKs to display */
  int count = 0;                /* number of TASK actually display */
  dg87_item *item = NULL;
  dg87_exp_task *task;
  dg87_target *target;
  dg87_queue *queue;
  BOOL show_description = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qxtl [options] TargetName QueueName");
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
    return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection, target,
                                     command->argv[1])) == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of task to display */
          number = atoi (optarg);
          break;
        case 'd':              /* description */
          show_description = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first task to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (queue->exp_task_list, start);
  else
    item = queue->exp_task_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      task = item->object;

      /* print task name and day name */
      dg87_connection_printf (connection, "%s%s%s%s",
                              beg, task->day->name, sep, task->name);

      /* print description */
      if (show_description == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep, task->description);
        }
      dg87_connection_printf (connection, "%s\n", end);

      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/***************************************************************************/
/* cmd_qxjc
 * Usage : qxjc [options] TargetName QueueName
 * return the number of exploitation jobs in the queue.
 */
int
cmd_qxjc (dg87_connection * connection)
{
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                      gettext ("Incorrect call.\nUsage : %s\n"),
											"qxjc TargetName QueueName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
    return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection, target,
                                     command->argv[1])) == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", queue->running_job_list->count);
  return 0;
}

/***************************************************************************/
/* cmd_qxjl
 * Usage : qxjl [options] TargetName QueueName
 * Queue eXp Job List
 * give the list of jobs of the exp which are in this queue 
 * Options :
 * -d : print description of task
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 * -s : state of the jobs
 */
int
cmd_qxjl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "sdB:S:E:I:N:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first job to display */
  int number = 0;               /* number of job to display */
  int count = 0;                /* number of job actually display */
  dg87_item *item = NULL;
  dg87_exp_job *job;
  dg87_target *target;
  dg87_queue *queue;
  BOOL show_description = FALSE;
  BOOL show_state = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"qxjl [options] TargetName QueueName");
      return 1;
    }

  /* find the target */
  if ((target = dg87_util_find_target (connection, command->argv[0])) == NULL)
    return 1;

  /* find the queue */
  if ((queue = dg87_util_find_queue (connection, target,
                                     command->argv[1])) == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first job to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of job to display */
          number = atoi (optarg);
          break;
        case 'd':              /* description */
          show_description = TRUE;
          break;
        case 's':              /* description */
          show_state = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the first job to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (queue->running_job_list, start);
  else
    item = queue->running_job_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      job = item->object;

      /* print job name, task name and day name */
      dg87_connection_printf (connection, "%s%s%s%s%s%s",
                              beg, job->task->day->name,
                              sep, job->task->name, sep, job->name);

      /* print state */
      if (show_state == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep,
                                  dg87_exp_job_state_to_ascii (job->state));
        }
      /* print description */
      if (show_description == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep, job->description);
        }
      dg87_connection_printf (connection, "%s\n", end);

      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/***************************************************************************/
/* End of file */
