/* file cmdtarget.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "cmdtarget.h"
#include "util.h"


/******************************************************************************/
/* tk
 * Target Kill
 * Usage : tk TargetName
 */
int
cmd_tk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tk TargetName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  if (dg87_context_target_delete_by_name (context, command->argv[0]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't delete the target '%s' ! %s!\n"),
											command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* ta
 * Target Add
 * Usage : ta TargetName [description]
 */
int
cmd_ta (dg87_connection * connection)
{
  dg87_target *target;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc < 1 || command->argc > 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ta TargetName [description]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* add the target */
  dg87_err_clear_error ();
  target = dg87_context_target_add (context, command->argv[0], FALSE);
  if (target == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't add the target '%s'! %s\n"),
											command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  /* set description */
  if (command->argc == 2)
    {
      dg87_err_clear_error ();
      if (dg87_target_set_description (target, command->argv[1]) == FALSE)
        {
          dg87_connection_printf (connection,
													gettext ("Can't add the target '%s'! %s\n"),
													command->argv[0], dg87_err_get_error_str ());
          dg87_context_target_delete (context, target);
          return 1;
        }
    }
  return 0;
}

/******************************************************************************/
/* cmd_tc
 * target count
 * Usage : tc
 * return the number of target.
 */
int
cmd_tc (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), "tc");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", context->target_vector->count);
  return 0;
}

/******************************************************************************/
/* ti
 * Target information 
 * Usage : ti [options] TargetName
 * Options 
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_ti (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator begin end */
  int optch, iopt;
  char *optarg;
  dg87_target *target;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ti [Options] TargetName");
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* Target name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Target name", sep, 
													target->name, end);
  /* target description */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Description", sep, 
													target->description, end);
  /* target open property */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Opened", sep,
                          dg87_bool_to_ascii (target->open), end);
  /* target hostname */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Host Name", sep,
                          (target->hostname ==
                           NULL) ? "" : target->hostname, end);
  /* target ip_port property */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "IP Port", sep, 
													target->ip_port, end);

  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* cmd_tl
 * target list
 * options
 * -d : print description of target
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first target to display (first index is 0)
 * -Nnumber : number of target to display (no limit if -N < 1)
 */
int
cmd_tl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator begin end */
  int optch, iopt;
  char *optarg;
  int start = 0;         /* index of the first target to display */
  int number = 0;        /* number of targets to display */
  int count = 0;         /* number of target actually display */
  dg87_element *element = NULL;
  dg87_target *target;
  BOOL show_description = FALSE;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");
  show_description = FALSE;

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first target to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of target to display */
          number = atoi (optarg);
          break;
        case 'd':              /* description */
          show_description = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first target to display */
  element = NULL;
  if (start > 0)
    element =
      dg87_vector_get_element_by_index (context->target_vector, start);
  else
    element = context->target_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      target = element->object;
      dg87_connection_printf (connection, "%s%s", beg, target->name, end);
      if (show_description == TRUE)
        dg87_connection_printf (connection, "%s%s", sep, target->description);
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/***************************************************************************/
/* tsh
 * Target Set Hostname 
 * Usage : tsh TargetName Hostname
 */
int
cmd_tsh (dg87_connection * connection)
{
  dg87_target *target;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tsh TargetName HostName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* change the hostname */
  dg87_err_clear_error ();
  if (dg87_target_set_hostname (target, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
							gettext ("Can't change the hostname of the target '%s' ! %s\n"),
							target->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/***************************************************************************/
/* tsd
 * Target Set Description
 * Usage : tsd TargetName Description
 */
int
cmd_tsd (dg87_connection * connection)
{
  dg87_target *target;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tsd TargetName description");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* change the description */
  dg87_err_clear_error ();
  if (dg87_target_set_description (target, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the description of "\
															"the target '%s' ! %s\n"),
											target->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/***************************************************************************/
/* tsn
 * Target Set Name 
 * Usage : tsn TargetName NewName 
 */
int
cmd_tsn (dg87_connection * connection)
{
  dg87_target *target;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tsn TargetName NewName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  /* change the target name */
  dg87_err_clear_error ();
  if (dg87_target_set_name (target, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
							gettext ("Can't change the name of the target '%s' ! %s\n"),
							target->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}
/***************************************************************************/
/* tso
 * Target Set Open 
 * Usage : tsd TargetName BooleanValue 
 */
int
cmd_tso (dg87_connection * connection)
{
  dg87_target *target;
  BOOL value;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tso TargetName BooleanValue");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  if ((value = dg87_ascii_to_bool (command->argv[1])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[1]);
      return 1;
    }

  /* change open property */
  dg87_err_clear_error ();
  if (dg87_target_set_open (target, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the '%s' property of "\
															"the target '%s' ! %s\n"),
											"open", target->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/***************************************************************************/
/* tsp
 * Target Set ip_port
 * Usage : tsp TargetName IntegerValue 
 */
int
cmd_tsp (dg87_connection * connection)
{
  dg87_target *target;
  int value;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"tsp TargetName IpPort");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the target */
  target = dg87_util_find_target (connection, command->argv[0]);
  if (target == NULL)
    return 1;

  value = atoi (command->argv[1]);
  if (value < 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s' for IpPort !\n"),
											command->argv[1]);
      return 1;
    }

  /* change the description */
  dg87_err_clear_error ();
  if (dg87_target_set_ip_port (target, value, FALSE) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the '%s' property of "\
															"the target '%s' ! %s\n"),
															"IP Port", target->name, 
															dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/***************************************************************************/
/* End of file */
