/* file cmdvariable.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "engine.h"
#include "cmdvariable.h"
#include "util.h"


/******************************************************************************/
/* vk
 * Variable Kill
 * Usage : vk VariableName
 */
int
cmd_vk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"vk VariableName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  dg87_err_clear_error ();
  if (dg87_engine_variable_delete_by_name (context->engine,
                                           command->argv[0]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't delete the variable '%s' ! %s!\n"),
											command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* va
 * Variable Add
 * Usage : va VariableName [value]
 */
int
cmd_va (dg87_connection * connection)
{
  dg87_variable *variable;
  dg87_context *context;
  dg87_command *command;
  char *value = "";

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc < 1 || command->argc > 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"va VariableName [value]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* initial value as optional argument */
  if (command->argc == 2)
    value = command->argv[1];

  /* add the variable */
  dg87_err_clear_error ();
  variable = dg87_engine_variable_add (context->engine,
                                       command->argv[0], value);
  if (variable == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't add the variable '%s'! %s\n"),
											command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  /* set description */
  if (command->argc == 2)
    {
      dg87_err_clear_error ();
      if (dg87_variable_set_value (variable, command->argv[1]) == FALSE)
        {
          dg87_connection_printf (connection,
                                  gettext ("Can't set value for "\
                                  "the variable '%s' ! %s\n"),
                                  command->argv[0],
                                  dg87_err_get_error_str ());
          /* TODO delete variable before return ? */
          return 1;
        }
    }
  return 0;
}

/******************************************************************************/
/* cmd_vc
 * variable count
 * Usage : vc
 * return the number of variable.
 */
int
cmd_vc (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"vc");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  dg87_connection_printf (connection,
                          "%i\n", context->engine->variable_vector->count);
  return 0;
}

/******************************************************************************/
/* vi
 * Variable information 
 * Usage : ti [options] VariableName
 * Options 
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_vi (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator begin end */
  int optch, iopt;
  char *optarg;
  dg87_variable *variable;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "vi [Options] VariableName");
      return 1;
    }
  /* find the variable */
  variable = dg87_util_find_variable (connection, command->argv[0]);
  if (variable == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* Variable name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Variable name", sep, 
													variable->name, end);
  /* variable value */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Value", sep, 
													variable->value, end);
  /* variable linked_rules value */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Linked Rules", sep,
                          variable->linked_rules, end);
  /* variable linkedConditions value */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Linked Ref Conditions", sep,
                          variable->linked_ref_conditions, end);
  /* variable linked_tasks value */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Linked Ref Tasks", sep,
                          variable->linked_ref_tasks, end);
  /* variable linkedConditions value */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Linked Exp Conditions", sep,
                          variable->linked_exp_conditions, end);
  /* variable linked_tasks value */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Linked Exp Tasks", sep,
                          variable->linked_exp_tasks, end);

  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* cmd_vl
 * variable list
 * options
 * -v : print value of variable
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first variable to display (first index is 0)
 * -Nnumber : number of variable to display (no limit if -N < 1)
 */
int
cmd_vl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator begin end */
  /*static char optstring[] = "dB:S:E:I:N:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first variable to display */
  int number = 0; /* number of variables to display */
  int count = 0;  /* number of variable actually display */
  dg87_element *element = NULL;
  dg87_variable *variable;
  BOOL show_value = FALSE;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "vl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");
  show_value = FALSE;

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first variable to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of variable to display */
          number = atoi (optarg);
          break;
        case 'v': /* value */
          show_value = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first variable to display */
  element = NULL;
  if (start > 0)
    element =
      dg87_vector_get_element_by_index (context->engine->variable_vector,
                                        start);
  else
    element = context->engine->variable_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of variable to display */
      count++;
      if (number > 0 && count > number)
        break;
      variable = element->object;
      dg87_connection_printf (connection, "%s%s", beg, variable->name, end);
      if (show_value == TRUE)
        dg87_connection_printf (connection, "%s%s", sep, variable->value);
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/***************************************************************************/
/* vsv
 * Variable Set Value
 * Usage : vsv VariableName Variable 
 */
int
cmd_vsv (dg87_connection * connection)
{
  dg87_variable *variable;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"vsv VariableName value");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the variable */
  variable = dg87_util_find_variable (connection, command->argv[0]);
  if (variable == NULL)
    return 1;

  /* change the description */
  dg87_err_clear_error ();
  if (dg87_variable_set_value (variable, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
															gettext ("Can't change the value for "\
																			"the variable '%s' ! %s\n"),
															variable->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/***************************************************************************/
/* End of file */
