/* file context.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <assert.h>
#include <config.h>
#include <dirent.h>
#include <netdb.h>
#include <libdg87/message.h>
#include "context.h"
#include "engine.h"
#include "variable.h"
#include "x_day.h"
#include "l_day.h"
#include "r_job.h"
#include "r_file.h"
#include "s_ref.h"
#include "target.h"

#define DIO87D_DATA_MIN_VERSION 1
#define DIO87D_DATA_MAX_VERSION 3
/*
 * dg87_context_field_array
 */
dg87_conf_field dg87_context_field_array[] = {
  {"IPPORT", DIO87D_IPPORT, NULL} ,
  {"IPADDR", DIO87D_IPADDR, NULL} ,
  {"HOSTNAME", NULL, NULL} , /*full qualified domain name of the server */
  {"PID_FILE", NULL, NULL} ,
  {"USER_ACCOUNT", NULL, NULL} ,
  {"DATA_DIR", DG87_DATADIR, NULL} ,
  {"SAVE_TEMPO", NULL, NULL} ,
	{"LOG_LEVEL", NULL, NULL},
  {NULL, NULL, NULL}
};

/******************************************************************************/
dg87_context *
dg87_context_new (char *conf_file)
{
  dg87_context *context;
  dg87_target *system_target;
  dg87_queue *system_queue;
  assert (conf_file != NULL);
  if (conf_file == NULL)
    return NULL;
  /* allocate memory */
  if ((context = dg87_malloc (sizeof (dg87_context), DG87_CONTEXT)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      goto alloc_error;
    }
  if ((context->conf = dg87_conf_new (conf_file,
                                      dg87_context_field_array)) == NULL)
    goto alloc_error;
  if ((context->target_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((context->ref_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((context->stat_ref_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((context->exp_day_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((context->log_day_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((context->job_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((context->file_list = dg87_list_new ()) == NULL)
    goto alloc_error;

  /* read config file */
  if (dg87_context_read_config (context) == FALSE)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	gettext ("Error while reading configuration file : '%s'"), 
											conf_file);
      goto alloc_error;
    }

  if ((context->engine = dg87_engine_new (context)) == NULL)
    goto alloc_error;

  context->storage = dg87_storage_new (DIO87D_DATA_FILE,
																		DIO87D_BACKUP_FILE, DIO87D_NAME,
																		DIO87D_DATA_MIN_VERSION,
																		DIO87D_DATA_MAX_VERSION,
																		dg87_conf_get_field_value
																		(context->conf,
																		 "USER_ACCOUNT"));
  if (context->storage == NULL) 
    goto alloc_error;

  /* create the system target and the system queue */
  system_target = dg87_context_target_add (context, "localhost", TRUE);
  if (system_target == NULL)
    goto alloc_error;
  system_queue = dg87_target_queue_get_by_name(system_target, "batch") ;
	if (system_queue == NULL)
    goto alloc_error;
	context->system_queue = system_queue;

  /* initialize structure */
  context->server = NULL;
  context->daemon = NULL;
  context->save_time = dg87_current_time;
  return context;

alloc_error:
  if (context == NULL)
    dg87_context_delete (context);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_context_read_config (dg87_context * context)
{
	dg87_target *localhost_target;
  char *value;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_conf_read (context->conf) == FALSE)
    return FALSE;

	/* read IP port and change the IP port of the localhost target */
  context->ip_port =
    atoi (dg87_conf_get_field_value (context->conf, "IPPORT"));
	localhost_target = dg87_context_target_get_by_name (context, "localhost");
	if (localhost_target != NULL)
		dg87_target_set_ip_port(localhost_target, context->ip_port, TRUE);

  /* interpret values */
  context->ip_addr = dg87_conf_get_field_value (context->conf, "IPADDR");
  context->pid_file = dg87_conf_get_field_value (context->conf, "PID_FILE");
  if (context->server != NULL)
    dg87_server_reload (context->server, context->pid_file,
                        context->ip_addr, context->ip_port);
	/* change the current directory */
  context->data_directory =
    dg87_conf_get_field_value (context->conf, "DATA_DIR");
  if (chdir (context->data_directory) == -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	gettext ("Can't open the data directory '%s' !"),
               context->data_directory);
    }

  /* read save tempo */
  value = dg87_conf_get_field_value (context->conf, "SAVE_TEMPO");
  context->save_tempo = (value == NULL) ? DG87_SAVE_TEMPO : atoi (value);
  if (context->save_tempo <= 0)
    context->save_tempo = DG87_SAVE_TEMPO;

	/* read the LOG_LEVEL priority */
  value = dg87_conf_get_field_value (context->conf, "LOG_LEVEL");
	if (dg87_daemon_set_log_priority (value) == FALSE)
				dg87_daemon_log_printf(dg87_daemon_log_error,
												gettext ("Incorrect value '%s' for LOG_LEVEL "\
																"parameter in the configuration file !"),
												(value==NULL) ? "" : value);

  /* read the hostname */
  context->hostname = dg87_conf_get_field_value (context->conf, "HOSTNAME");

  /* read day purge retention */
  value = dg87_conf_get_field_value (context->conf, "DAY_PURGE_RETENTION");
  context->day_purge_retention = (value == NULL) ?
    DG87_DAY_PURGE_RETENTION : atoi (value);
  if (context->day_purge_retention <= 2)
    context->day_purge_retention = DG87_DAY_PURGE_RETENTION;

  return TRUE;
}

/******************************************************************************/
BOOL
dg87_context_delete (dg87_context * context)
{
  dg87_target *target;
  dg87_ref_model *ref;
  dg87_sta_ref *star_ref;
  dg87_exp_day *exp_day;
  dg87_log_day *log_day;
  dg87_r_file *file;
  dg87_r_job *job;

  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  /* delete all file object */
  if (context->file_list != NULL)
    {
      while (context->file_list->count > 0)
        {
          file = dg87_list_remove_object_by_index (context->file_list, 0);
          dg87_r_file_delete (file);
        }
      dg87_list_delete (context->file_list);
    }

  /* delete all job object */
  if (context->job_list != NULL)
    {
      while (context->job_list->count > 0)
        {
          job = dg87_list_remove_object_by_index (context->job_list, 0);
          dg87_r_job_delete (job);
        }
      dg87_list_delete (context->job_list);
    }

  /* delete all days */
  if (context->exp_day_vector != NULL)
    {
      while (context->exp_day_vector->count > 0)
        {
          exp_day = dg87_vector_remove_object_by_index
            (context->exp_day_vector, 0);
          dg87_exp_day_delete (exp_day);
        }
      dg87_vector_delete (context->exp_day_vector);
    }
  /* delete all days */
  if (context->log_day_vector != NULL)
    {
      while (context->log_day_vector->count > 0)
        {
          log_day = dg87_vector_remove_object_by_index
            (context->log_day_vector, 0);
          dg87_log_day_delete (log_day, FALSE);
        }
      dg87_vector_delete (context->log_day_vector);
    }

  /* delete all refs */
  if (context->ref_vector != NULL)
    {
      while (context->ref_vector->count > 0)
        {
          ref = dg87_vector_remove_object_by_index (context->ref_vector, 0);
          dg87_ref_model_delete (ref);
        }
      dg87_vector_delete (context->ref_vector);
    }

  /* delete all refs */
  if (context->stat_ref_vector != NULL)
    {
      while (context->stat_ref_vector->count > 0)
        {
          star_ref = dg87_vector_remove_object_by_index
            (context->stat_ref_vector, 0);
          dg87_sta_ref_delete (star_ref, FALSE);
        }
      dg87_vector_delete (context->stat_ref_vector);
    }


  /* delete all targets */
  if (context->target_vector != NULL)
    {
      while (context->target_vector->count > 0)
        {
          target = dg87_vector_remove_object_by_index
            (context->target_vector, 0);
          dg87_target_delete (target, TRUE);
        }
      dg87_vector_delete (context->target_vector);
    }

  if (context->storage != NULL)
    dg87_storage_delete (context->storage);
  if (context->engine != NULL)
    dg87_engine_delete (context->engine);
  if (context->conf != NULL)
    dg87_conf_delete (context->conf);
  if (context->daemon != NULL)
    dg87_daemon_delete (context->daemon);
  dg87_free (context, DG87_CONTEXT);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_set_server (dg87_context * context, dg87_server * server)
{
  assert (context != NULL && server != NULL);
  if (context == NULL || server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context->server = server;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_make_periodic_save (dg87_context * context)
{
  assert (context != NULL);
  if (dg87_current_time > context->save_time + context->save_tempo)
    {
			if (dg87_save_needed == 0)
			{
				/* database was not changed */
      	context->save_time = dg87_current_time;
				return TRUE;
			}
      if (dg87_context_save (context) == FALSE)
				dg87_daemon_log_printf(dg87_daemon_log_error,
												gettext ("Save Error !"));
      context->save_time = dg87_current_time;
    }
  return TRUE;
}


/****************************************************************************/
dg87_target *
dg87_context_target_add (dg87_context * context, char *target_name,
                         BOOL system)
{
  dg87_target *target;
  assert (context != NULL && target_name != NULL);
  if (context == NULL || target_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  target = dg87_target_new (target_name, system, context);
  if (target == NULL)
    return NULL;

  /* add the target in the target vector */
  /* vector generate error if an other target has the same name. */
  /* Key must point on target->name */
  if (dg87_vector_add_object (context->target_vector,
                              target, target->name) == FALSE)
    {
      dg87_target_delete (target, TRUE);
      return NULL;
    }
  return target;
}

/****************************************************************************/
BOOL
dg87_context_target_rename (dg87_context * context, dg87_target* target,
                               char *new_name)
{
  char old_name[DG87_TARGET_NAME_LENGTH + 1];
  assert (context != NULL && target != NULL && new_name != NULL);
  if (context == NULL || target == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  if (strlen (new_name) > DG87_TARGET_NAME_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  /* remove object from vector */
  if (dg87_vector_remove_object (context->target_vector, target) == FALSE)
    return FALSE;
  /* copy new name */
  dg87_strnzcpy (old_name, target->name, DG87_TARGET_NAME_LENGTH);
  dg87_strnzcpy (target->name, new_name, DG87_TARGET_NAME_LENGTH);
  /* add the object in the vector with the new key */
  if (dg87_vector_add_object (context->target_vector, target, target->name) == FALSE)
    {
      dg87_strnzcpy (target->name, old_name, DG87_TARGET_NAME_LENGTH);
      dg87_vector_add_object (context->target_vector, target, target->name);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}


/****************************************************************************/
int
dg87_context_target_count (dg87_context * context)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (context->target_vector);
}

/****************************************************************************/
dg87_target *
dg87_context_target_get_by_name (dg87_context * context, char *name)
{
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  return dg87_vector_get_object_by_key (context->target_vector, name);
}

/****************************************************************************/
dg87_target *
dg87_context_target_get_by_index (dg87_context * context, int index)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (context->target_vector, index);
}

/****************************************************************************/
BOOL
dg87_context_target_delete (dg87_context * context, dg87_target * target)
{
  assert (context != NULL && target != NULL);
  if (context == NULL || target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_vector_remove_object (context->target_vector, target) == FALSE)
    return FALSE;
  if (dg87_target_delete (target, FALSE) == FALSE)
    {
      /* add target in vector again */
      dg87_vector_add_object (context->target_vector, target, target->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_target_delete_by_name (dg87_context * context, char *name)
{
  dg87_target *target;
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  target = dg87_vector_get_object_by_key (context->target_vector, name);
  if (target == NULL)
    return FALSE;
  return dg87_context_target_delete (context, target);
}

/****************************************************************************/
BOOL
dg87_context_target_delete_by_index (dg87_context * context, int index)
{
  dg87_target *target;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  target = dg87_vector_get_object_by_index (context->target_vector, index);
  if (target == NULL)
    return FALSE;
  return dg87_context_target_delete (context, target);
}

/****************************************************************************/
dg87_ref_model *
dg87_context_ref_model_add (dg87_context * context, char *referential_name)
{
  dg87_ref_model *ref;
  assert (context != NULL && referential_name != NULL);
  if (context == NULL || referential_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((ref = dg87_ref_model_new (referential_name, context)) == NULL)
    return NULL;

  /* add the ref in the referntial vector */
  /* vector generates error if an other ref has the same name. */
  /* Key must point on ref->name */
  if (dg87_vector_add_object (context->ref_vector, ref, ref->name) == FALSE)
    {
      dg87_ref_model_delete (ref);
      return NULL;
    }
  return ref;
}

/****************************************************************************/
dg87_ref_model *
dg87_context_ref_model_duplicate (dg87_context * context,
                                  dg87_ref_model * original_referential,
                                  char *new_referential_name)
{
  dg87_ref_model *ref;
  assert (context != NULL && original_referential != NULL
          && new_referential_name != NULL);
  if (context == NULL || original_referential == NULL
      || new_referential_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* duplicate the original ref */
  ref = dg87_ref_model_duplicate (original_referential, new_referential_name);
  if (ref == NULL)
    return NULL;

  /* add the ref in the referential vector */
  /* vector generates error if an other ref has the same name. */
  /* Key must point on ref->name */
  if (dg87_vector_add_object (context->ref_vector, ref, ref->name) == FALSE)
    {
      dg87_ref_model_delete (ref);
      return NULL;
    }
  return ref;
}

/****************************************************************************/
BOOL
dg87_context_ref_model_delete_by_name (dg87_context * context, char *name)
{
  dg87_ref_model *ref;
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((ref = dg87_vector_get_object_by_key (context->ref_vector,
                                            name)) == NULL)
    return FALSE;

  /* delete the referential */
  if (dg87_vector_remove_object (context->ref_vector, ref) == FALSE)
    return FALSE;
  if (dg87_ref_model_delete (ref) == FALSE)
    {
      dg87_vector_add_object (context->ref_vector, ref, ref->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
dg87_ref_model *
dg87_context_ref_model_get_by_name (dg87_context * context, char *name)
{
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  return dg87_vector_get_object_by_key (context->ref_vector, name);
}

/****************************************************************************/
BOOL
dg87_context_ref_model_rename (dg87_context * context, dg87_ref_model * ref,
                               char *new_name)
{
  char old_name[DG87_REF_NAME_LENGTH + 1];
  assert (context != NULL && ref != NULL && new_name != NULL);
  if (context == NULL || ref == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  if (strlen (new_name) > DG87_REF_NAME_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  /* remove object from vector */
  if (dg87_vector_remove_object (context->ref_vector, ref) == FALSE)
    return FALSE;
  /* copy new name */
  dg87_strnzcpy (old_name, ref->name, DG87_REF_NAME_LENGTH);
  dg87_strnzcpy (ref->name, new_name, DG87_REF_NAME_LENGTH);
  /* add the object in the vector with the new key */
  if (dg87_vector_add_object (context->ref_vector, ref, ref->name) == FALSE)
    {
      dg87_strnzcpy (ref->name, old_name, DG87_REF_NAME_LENGTH);
      dg87_vector_add_object (context->ref_vector, ref, ref->name);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_exp_day *
dg87_context_exp_day_add (dg87_context * context, char *day_name,
                          time_t start_time)
{
  dg87_exp_day *day;
  assert (context != NULL && day_name != NULL);
  if (context == NULL || day_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((day = dg87_exp_day_new (day_name, start_time, context)) == NULL)
    return NULL;

  /* add the day in the day vector */
  /* vector generate error if an other day has the same name. */
  /* Key must point on day->name */
  if (dg87_vector_add_object (context->exp_day_vector,
                              day, day->name) == FALSE)
    {
      dg87_exp_day_delete (day);
      return NULL;
    }
  return day;
}

/****************************************************************************/
int
dg87_context_exp_day_count (dg87_context * context)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (context->exp_day_vector);
}

/****************************************************************************/
dg87_exp_day *
dg87_context_exp_day_get_by_name (dg87_context * context, char *name)
{
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  return dg87_vector_get_object_by_key (context->exp_day_vector, name);
}

/****************************************************************************/
dg87_exp_day *
dg87_context_exp_day_get_by_index (dg87_context * context, int index)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (context->exp_day_vector, index);
}

/****************************************************************************/
BOOL
dg87_context_exp_day_delete_by_name (dg87_context * context, char *name)
{
  dg87_exp_day *day;
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  day = dg87_vector_remove_object_by_key (context->exp_day_vector, name);
  if (day == NULL)
    return FALSE;
  dg87_exp_day_delete (day);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_exp_day_delete_by_index (dg87_context * context, int index)
{
  dg87_exp_day *day;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  day = dg87_vector_remove_object_by_index (context->exp_day_vector, index);
  if (day == NULL)
    return FALSE;
  dg87_exp_day_delete (day);
  return TRUE;
}

/****************************************************************************/
dg87_log_day *
dg87_context_log_day_add (dg87_context * context, char *day_name)
{
  dg87_log_day *day;
  dg87_strbuf *buffer;
  assert (context != NULL && day_name != NULL);
  if (context == NULL || day_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((buffer = dg87_strbuf_new ()) == NULL)
    return NULL;
  if (dg87_strbuf_printf (buffer, DIO87D_LOGFILE_TEMPLATE, day_name) == FALSE)
    {
      dg87_strbuf_delete (buffer);
      return NULL;
    }

  /* create day */
  if ((day = dg87_log_day_new (day_name, buffer->buffer,
                               context->storage->uid,
                               context->storage->gid)) == NULL)
    {
      dg87_strbuf_delete (buffer);
      return NULL;
    }
  dg87_strbuf_delete (buffer);

  /* add the day in the log_day vector */
  /* vector generates an error if an other day has the same name. */
  /* Key must point on day->name */
  if (dg87_vector_add_object (context->log_day_vector,
                              day, day->name) == FALSE)
    {
      dg87_log_day_delete (day, FALSE);
      return NULL;
    }
  return day;
}

/****************************************************************************/
int
dg87_context_log_day_count (dg87_context * context)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (context->log_day_vector);
}

/****************************************************************************/
dg87_log_day *
dg87_context_log_day_get_by_name (dg87_context * context, char *name)
{
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  return dg87_vector_get_object_by_key (context->log_day_vector, name);
}

/****************************************************************************/
dg87_log_day *
dg87_context_log_day_get_by_index (dg87_context * context, int index)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (context->log_day_vector, index);
}

/****************************************************************************/
BOOL
dg87_context_log_day_delete_by_name (dg87_context * context, char *name,
                                     BOOL delete_file)
{
  dg87_log_day *day;
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((day = dg87_vector_remove_object_by_key (context->log_day_vector,
                                               name)) == NULL)
    return FALSE;
  if (dg87_log_day_delete (day, delete_file) == FALSE)
    {
      dg87_vector_add_object (context->log_day_vector, day, day->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_log_day_delete_by_index (dg87_context * context, int index,
                                      BOOL delete_file)
{
  dg87_log_day *day;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((day = dg87_vector_remove_object_by_index (context->log_day_vector,
                                                 index)) == NULL)
    return FALSE;
  if (dg87_log_day_delete (day, delete_file) == FALSE)
    {
      dg87_vector_add_object (context->log_day_vector, day, day->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
dg87_r_job *
dg87_r_context_job_add (dg87_context * context, char *day_name,
                        char *task_name, char *job_name, char *path,
                        char *argument, char *hostname, int ip_port)
{
  dg87_item *item;
  dg87_r_job *job;
  assert (context != NULL && task_name != NULL && job_name != NULL
          && day_name != NULL && path != NULL
          && argument != NULL && hostname != NULL);
  if (context == NULL || task_name == NULL || job_name == NULL
      || day_name == NULL || path == NULL || argument == NULL
      || hostname == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* error if a same job is already in the list */
  item = context->job_list->first;
  while (item != NULL)
    {
      job = item->object;
      item = item->next;
      if (strcmp (day_name, job->day_name) == 0 &&
          strcmp (task_name, job->task_name) == 0 &&
          strcmp (job_name, job->job_name) == 0)
        {
          dg87_err_set_error (DG87_ERR_SAME_JOB_ALREADY_EXISTS);
          return NULL;
        }

    }

  if ((job = dg87_r_job_new (day_name, task_name, job_name, path,
                             argument, hostname, ip_port)) == NULL)
    return NULL;
  if (dg87_list_add_object (context->job_list, job) == FALSE)
    {
      dg87_r_job_delete (job);
      return NULL;
    }
  return job;
}

/****************************************************************************/
BOOL
dg87_r_context_job_delete (dg87_context * context, dg87_r_job * job)
{
  assert (context != NULL && job != NULL);
  if (context == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* TODO more things to do ! if the job run, kill it */
  if (dg87_list_remove_object (context->job_list, job) == FALSE)
    return FALSE;
  dg87_r_job_delete (job);
  return TRUE;
}

/****************************************************************************/
dg87_r_file *
dg87_r_context_file_add (dg87_context * context, char *day_name,
                         char *task_name, char *path, char *hostname,
                         int ip_port)
{
  dg87_item *item;
  dg87_r_file *file;
  assert (context != NULL && day_name != NULL && task_name != NULL
          && path != NULL && hostname != NULL);
  if (context == NULL || day_name == NULL || task_name == NULL
      || path == NULL || hostname == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* error if a same file is already in the list */
  item = context->file_list->first;
  while (item != NULL)
    {
      file = item->object;
      item = item->next;
      if (strcmp (day_name, file->day_name) == 0 &&
          strcmp (task_name, file->task_name) == 0 &&
          strcmp (path, file->path) == 0)
        {
          dg87_err_set_error (DG87_ERR_SAME_FILE_ALREADY_EXISTS);
          return NULL;
        }
    }

  if ((file = dg87_r_file_new (day_name, task_name,
                               path, hostname, ip_port)) == NULL)
    return NULL;
  if (dg87_list_add_object (context->file_list, file) == FALSE)
    {
      dg87_r_file_delete (file);
      return NULL;
    }
  return file;
}

/****************************************************************************/
BOOL
dg87_r_context_file_delete (dg87_context * context, dg87_r_file * file)
{
  assert (context != NULL && file != NULL);
  if (context == NULL || file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_remove_object (context->file_list, file) == FALSE)
    return FALSE;
  dg87_r_file_delete (file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_sta_load_referentials (dg87_context * context)
{
  DIR *dir;
  struct dirent *dirent;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* unload refs which are in memory */
  /*TODO unload refs which are in memory */

  if ((dir = opendir (".")) == NULL)
    { 
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't open data directory '%s' !",
               context->data_directory);
      dg87_err_set_error (DG87_ERR_CAN_NOT_ACCESS_DIRECTORY);
      return FALSE;
    }

  /* read directory entries */
  for (;;)
    {
      dirent = readdir (dir);
      if (dirent == NULL)
        break;
      /* ignore files which dosn't start with 's_' */
      if (strncmp (dirent->d_name, DIO87D_STATFILE_TEMPLATE, 2) != 0)
        continue;

      /* try to load ref */
      if (dg87_context_sta_referential_load (context, dirent->d_name) ==
          FALSE)
        return FALSE;
    }

  closedir (dir);

  return TRUE;
}

/****************************************************************************/
dg87_sta_ref *
dg87_context_sta_referential_load (dg87_context * context, char *filename)
{
  dg87_sta_ref *ref;
  assert (context != NULL && filename != NULL);
  if (context == NULL || filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  ref = dg87_sta_ref_load (filename);
  if (ref == NULL)
    return NULL;

  /* add the ref in the referential vector */
  /* vector generate error if an other ref has the same name. */
  /* Key must point on ref->name */
  if (dg87_vector_add_object (context->stat_ref_vector,
                              ref, ref->name) == FALSE)
    {
      dg87_sta_ref_delete (ref, FALSE);
      return NULL;
    }
  return ref;

}

/****************************************************************************/
dg87_sta_ref *
dg87_context_sta_referential_add (dg87_context * context,
                                  char *referential_name)
{
  dg87_sta_ref *ref;
  dg87_strbuf *buffer;
  assert (context != NULL && referential_name != NULL);
  if (context == NULL || referential_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((buffer = dg87_strbuf_new ()) == NULL)
    return NULL;
  if (dg87_strbuf_printf (buffer, DIO87D_STATFILE_TEMPLATE,
                          referential_name) == FALSE)
    {
      dg87_strbuf_delete (buffer);
      return NULL;
    }
  /* create ref */
  if ((ref = dg87_sta_ref_new (referential_name, buffer->buffer,
                               context->storage->uid,
                               context->storage->gid)) == NULL)
    {
      dg87_strbuf_delete (buffer);
      return NULL;
    }
  dg87_strbuf_delete (buffer);

  /* add the ref in the referential vector */
  /* vector generate error if an other ref has the same name. */
  /* Key must point on ref->name */
  if (dg87_vector_add_object (context->stat_ref_vector,
                              ref, ref->name) == FALSE)
    {
      dg87_sta_ref_delete (ref, FALSE);
      return NULL;
    }
  return ref;
}

/****************************************************************************/
int
dg87_context_sta_referential_count (dg87_context * context)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (context->stat_ref_vector);
}

/****************************************************************************/
dg87_sta_ref *
dg87_context_sta_referential_get_by_name (dg87_context * context, char *name)
{
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  return dg87_vector_get_object_by_key (context->stat_ref_vector, name);
}

/****************************************************************************/
dg87_sta_ref *
dg87_context_sta_referential_get_by_index (dg87_context * context, int index)
{
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (context->stat_ref_vector, index);
}

/****************************************************************************/
BOOL
dg87_context_sta_referential_delete_by_name (dg87_context * context,
                                             char *name, BOOL delete_file)
{
  dg87_sta_ref *ref;
  assert (context != NULL && name != NULL);
  if (context == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((ref = dg87_vector_remove_object_by_key (context->stat_ref_vector,
                                               name)) == NULL)
    return FALSE;
  if (dg87_sta_ref_delete (ref, delete_file) == FALSE)
    {
      dg87_vector_add_object (context->stat_ref_vector, ref, ref->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_sta_referential_delete_by_index (dg87_context * context,
                                              int index, BOOL delete_file)
{
  dg87_sta_ref *ref;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((ref = dg87_vector_remove_object_by_index (context->stat_ref_vector,
                                                 index)) == NULL)
    return FALSE;
  if (dg87_sta_ref_delete (ref, delete_file) == FALSE)
    {
      dg87_vector_add_object (context->stat_ref_vector, ref, ref->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_load_conditions (dg87_context * context,
                              dg87_ref_task * task, dg87_storage * storage)
{
  dg87uint count;
  dg87uint minutes;
  dg87ubyte byte;
  dg87ubyte day_delta;
  dg87_ref_condition_type type;
  dg87_variable *variable;
  char *string;
  assert (context != NULL && task != NULL && storage != NULL);
  /* read conditions count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      /* read condition type */
      if (dg87_storage_read_ubyte (storage, &byte) == FALSE)
        return FALSE;
      type = byte;
      switch (type)
        {
        case condition_hmin:
          if (dg87_storage_read_uint (storage, &minutes) == FALSE)
            return FALSE;
          day_delta = 0;
          if (dg87_storage_read_ubyte (storage, &day_delta) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_hmin_add (task, minutes,
                                                day_delta) == FALSE)
            return FALSE;
          break;
        case condition_hmax:
          if (dg87_storage_read_uint (storage, &minutes) == FALSE)
            return FALSE;
          day_delta = 0;
          if (dg87_storage_read_ubyte (storage, &day_delta) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_hmax_add (task, minutes,
                                                day_delta) == FALSE)
            return FALSE;
          break;
        case condition_hlate:
          if (dg87_storage_read_uint (storage, &minutes) == FALSE)
            return FALSE;
          day_delta = 0;
          if (dg87_storage_read_ubyte (storage, &day_delta) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_hlate_add (task, minutes,
                                                 day_delta) == FALSE)
            return FALSE;
          break;
        case condition_file:
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_file_add (task, string) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              return FALSE;
            }
          dg87_free (string, DG87_CHAR);
          break;
        case condition_manual:
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_manual_add (task, string) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              return FALSE;
            }
          dg87_free (string, DG87_CHAR);
          break;
        case condition_ifequal:
          /* find variable */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          variable =
            dg87_engine_variable_get_by_name (context->engine, string);
          dg87_free (string, DG87_CHAR);
          if (variable == NULL)
            return FALSE;
          /* read value */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_ifequal_add (task,
                                                   variable, string) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              return FALSE;
            }
          dg87_free (string, DG87_CHAR);
          break;
        case condition_ifnequal:
          /* find variable */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          variable =
            dg87_engine_variable_get_by_name (context->engine, string);
          dg87_free (string, DG87_CHAR);
          if (variable == NULL)
            return FALSE;
          /* read value */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          if (dg87_ref_task_condition_ifnotequal_add (task,
                                                      variable,
                                                      string) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              return FALSE;
            }
          dg87_free (string, DG87_CHAR);
          break;
        default:
          break;
        }
    }
  return TRUE;
}

/****************************************************************************/
static dg87_ref_rule_operation *
dg87_context_load_operations (dg87_context * context,
                              dg87_ref_model * ref, dg87_storage * storage)
{
  dg87_ref_rule *target_rule;
  dg87_variable *variable;
  dg87ubyte ubyte, ubyte2;
  dg87byte byte;
  dg87int intvalue;
  dg87long longvalue;
  char *string;
  dg87_ref_rule_operator operator;
  dg87_ref_rule_type type;
  dg87_ref_rule_operation *operation;
  assert (context != NULL && ref != NULL && storage != NULL);
  /* read operation */
  if (dg87_storage_read_ubyte (storage, &ubyte) == FALSE)
    return FALSE;
  operator = ubyte;
  if (dg87_storage_read_ubyte (storage, &ubyte) == FALSE)
    return FALSE;
  type = ubyte;
  switch (type)
    {
    case dg87_ref_rule_type_dow:
      if (dg87_storage_read_byte (storage, &byte) == FALSE)
        return FALSE;
      if (storage->actual_version <= 2)
				{
					/* before version 3, the first day of the week was sunday(sunday=0),
					 * now the first day of the week is monday (monday=0) */
					if (byte == 0)
						byte = 7;
					byte--;
				}
      operation = dg87_ref_operation_dayofweek_new (operator, byte);
      break;
    case dg87_ref_rule_type_dom:
      if (dg87_storage_read_byte (storage, &byte) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_dayofmonth_new (operator, byte);
      break;
    case dg87_ref_rule_type_doy:
      if (dg87_storage_read_int (storage, &intvalue) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_dayofmonth_new (operator, intvalue);
      break;
    case dg87_ref_rule_type_wom:
      if (dg87_storage_read_byte (storage, &byte) == FALSE)
        return FALSE;
			
      operation = dg87_ref_operation_weekofmonth_new (operator, byte);
      break;
    case dg87_ref_rule_type_woy:
      if (dg87_storage_read_byte (storage, &byte) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_weekofyear_new (operator, byte);
      break;
    case dg87_ref_rule_type_moy:
      if (dg87_storage_read_byte (storage, &byte) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_monthofyear_new (operator, ubyte);
      break;
    case dg87_ref_rule_type_day:
      if (dg87_storage_read_ubyte (storage, &ubyte) == FALSE)
        return FALSE;
      if (dg87_storage_read_ubyte (storage, &ubyte2) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_day_new (operator, ubyte2, ubyte);
      break;
    case dg87_ref_rule_type_date:
      if (dg87_storage_read_long (storage, &longvalue) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_date_new (operator, longvalue);
      break;
    case dg87_ref_rule_type_always:
      operation = dg87_ref_operation_always_new ();
      break;
    case dg87_ref_rule_type_complex:
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      target_rule = dg87_ref_model_rule_get_by_name (ref, string);
      dg87_free (string, DG87_CHAR);
      if (target_rule == NULL)
        return FALSE;
      operation = dg87_ref_operation_complex_new (operator, target_rule);
      break;
    case dg87_ref_rule_type_ifequal:
      /* find variable */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      variable = dg87_engine_variable_get_by_name (context->engine, string);
      dg87_free (string, DG87_CHAR);
      if (variable == NULL)
        return FALSE;
      /* read value */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      operation = dg87_ref_operation_ifequal_new (operator, variable, string);
      dg87_free (string, DG87_CHAR);
      break;
    case dg87_ref_rule_type_ifnequal:
      /* find variable */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      variable = dg87_engine_variable_get_by_name (context->engine, string);
      dg87_free (string, DG87_CHAR);
      if (variable == NULL)
        return FALSE;
      /* read value */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      operation =
        dg87_ref_operation_ifnequal_new (operator, variable, string);
      dg87_free (string, DG87_CHAR);

      break;
    default:
      operation = NULL;
      break;
    }
  return operation;
}

/****************************************************************************/
static BOOL
dg87_context_load_rules (dg87_context * context,
                         dg87_ref_model * ref, dg87_storage * storage)
{
  dg87_ref_rule_operation *operation;
  char *string;
  dg87uint count;
  dg87_ref_rule *rule;
  dg87_element *element;
  assert (context != NULL && ref != NULL && storage != NULL);
  /* -------- load rules ---------- */
  /* read rules count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      /* load rule */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((rule = dg87_ref_model_rule_add (ref, string)) == FALSE)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
    }
  /* -------- load operations ----- */
  /* for each rule */
  element = ref->rule_vector->first;
  while (element != NULL)
    {
      rule = element->object;
      element = element->next;
      /* read operations count */
      if (dg87_storage_read_uint (storage, &count) == FALSE)
        return FALSE;
      while (count > 0)
        {
          count--;
          if ((operation = dg87_context_load_operations (context,
                                                         ref,
                                                         storage)) == NULL)
            return FALSE;
          if (dg87_ref_rule_operation_add (rule, operation) == FALSE)
            return FALSE;

        }/* next operation */
    }/* next rule */
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_load_task_rules (dg87_context * context,
                              dg87_ref_task * task, dg87_storage * storage)
{

  dg87_ref_rule_operation *operation;
  dg87uint count;
  assert (context != NULL && task != NULL && storage != NULL);
  /* read operations count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      if ((operation = dg87_context_load_operations (context,
                                                     task->ref,
                                                     storage)) == NULL)
        return FALSE;
      if (dg87_ref_task_operation_add (task, operation) == FALSE)
        return FALSE;

    }/* next operation */
  return TRUE;
}

/****************************************************************************/
static BOOL
_dg87_context_load_conditions (dg87_context * context,
                               dg87_exp_task * task, dg87_storage * storage)
{
  dg87uint count;
  dg87uint minutes;
  dg87ubyte ubyte;
  dg87_exp_condition_type type;
  dg87_variable *variable;
  dg87_exp_condition *condition;
  dg87ubyte day_delta;
  char *string;
  assert (context != NULL && task != NULL && storage != NULL);
  /* read conditions count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  condition = NULL;
  while (count > 0)
    {
      count--;
      /* read condition type */
      if (dg87_storage_read_ubyte (storage, &ubyte) == FALSE)
        return FALSE;
      type = ubyte;
      switch (type)
        {
        case condition_hmin:
          if (dg87_storage_read_uint (storage, &minutes) == FALSE)
            return FALSE;
          day_delta = 0;
          if (dg87_storage_read_ubyte (storage, &day_delta) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_hmin_add (task, minutes,
                                                        day_delta);
          break;
        case condition_hmax:
          if (dg87_storage_read_uint (storage, &minutes) == FALSE)
            return FALSE;
          day_delta = 0;
          if (dg87_storage_read_ubyte (storage, &day_delta) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_hmax_add (task, minutes,
                                                        day_delta);
          break;
        case condition_hlate:
          if (dg87_storage_read_uint (storage, &minutes) == FALSE)
            return FALSE;
          day_delta = 0;
          if (dg87_storage_read_ubyte (storage, &day_delta) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_hlate_add (task, minutes,
                                                         day_delta);
          break;
        case condition_file:
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_file_add (task, string);
          dg87_free (string, DG87_CHAR);
          if (dg87_storage_read_ubyte (storage, &ubyte) == FALSE)
            return FALSE;
          condition->data.condition_File->command_sended = ubyte;
          break;
        case condition_manual:
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_manual_add (task, string);
          dg87_free (string, DG87_CHAR);
          break;
        case condition_ifequal:
          /* find variable */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          variable =
            dg87_engine_variable_get_by_name (context->engine, string);
          dg87_free (string, DG87_CHAR);
          if (variable == NULL)
            return FALSE;
          /* read value */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_ifequal_add (task,
                                                           variable, string);
          dg87_free (string, DG87_CHAR);
          break;
        case condition_ifnequal:
          /* find variable */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          variable =
            dg87_engine_variable_get_by_name (context->engine, string);
          dg87_free (string, DG87_CHAR);
          if (variable == NULL)
            return FALSE;
          /* read value */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            return FALSE;
          condition = dg87_exp_task_condition_ifnotequal_add (task,
                                                              variable,
                                                              string);
          dg87_free (string, DG87_CHAR);
          break;
        default:
          break;
        }
      if (condition == NULL)
        return FALSE;
      if (dg87_exp_condition_load (condition, storage) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_load (dg87_context * context)
{
  dg87uint count, count2, count3, ipport;
  dg87_storage *storage;
  char *string, *string2;
  dg87_target *target;
  dg87_queue *queue;
  dg87_variable *variable;
  dg87_user *user;
  dg87_ref_model *ref;
  dg87_ref_task *task, *task2;
  dg87_ref_job *job, *job2;
  dg87_ref_jobdep *job_dep;
  dg87_ref_taskdep *taskdep;
  dg87_exp_day *exp_day;
  dg87_exp_task *exp_task, *exp_task2;
  dg87_exp_job *exp_job, *exp_job2;
  dg87_exp_jobdep *exp_job_dep;
  dg87_exp_taskdep *exp_taskdep;
  dg87_r_job *followed_job;
  dg87_r_file *followed_file;
  char *hostname;
  char *path, *argument;
  dg87int ip_port;
  char *day_name, *task_name, *job_name;
  dg87ubyte b;
  dg87ubyte system;
  time_t start_time;
  dg87long longvalue;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  storage = context->storage;
  if (dg87_storage_open (storage, dg87_storage_mode_read) == FALSE)
    return FALSE;
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load started");
  /* --------- Engine ---------------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
  	"file version : %i", context->storage->actual_version);
  if (dg87_engine_load (context->engine, storage) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }

  /* --------- Targets and queues ---------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load targets and queues");
  /* read target count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  while (count > 0)
    {
      count--;
      /* load target name */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* load the system property */
      system = FALSE;
      if (storage->actual_version >= 2)
        {
          if (dg87_storage_read_ubyte (storage, &system) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        }
			else
				{
					if (strcmp(string, "localhost") == 0)
						system = TRUE;
				}
			/* create or find the target */
      if (system == FALSE)
        {
          if ((target =
               dg87_context_target_add (context, string, FALSE)) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              dg87_storage_close (storage);
              return FALSE;
            }
        }
      else
        {
          if ((target =
               dg87_context_target_get_by_name (context, string)) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              dg87_storage_close (storage);
              return FALSE;
            }
        }
      dg87_free (string, DG87_CHAR);
      if (dg87_target_load (target, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Queues --------- */
      /* read queue count */
      if (dg87_storage_read_uint (storage, &count2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      while (count2 > 0)
        {
          count2--;
          /* load queue */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* load the system property */
          system = FALSE;
          if (storage->actual_version >= 2)
            {
              if (dg87_storage_read_ubyte (storage, &system) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }
					else
						{
							if (target->system == TRUE && strcmp(string, "batch") == 0)
								system = TRUE;
						}
					/* create or find the queue */
          if (system == FALSE)
            {
              if ((queue =
                   dg87_target_queue_add (target, string, FALSE)) == FALSE)
                {
                  dg87_free (string, DG87_CHAR);
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }
          else
            {
              if ((queue =
                   dg87_target_queue_get_by_name (target, string)) == FALSE)
                {
                  dg87_free (string, DG87_CHAR);
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }
          dg87_free (string, DG87_CHAR);
          if (dg87_queue_load (queue, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        }
    }
  /* --------- Variables ------------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load variables...");
  /* read variables count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      /* load variable */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if ((variable = dg87_engine_variable_add (context->engine,
                                                string, "")) == FALSE)
        {
          dg87_free (string, DG87_CHAR);
          dg87_storage_close (storage);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      if (dg87_variable_load (variable, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }
  /* --------- Users ----------------------- */
  /* read users count */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load users...");
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      /* load user */
      if (dg87_storage_read_ubyte (storage, &b) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_read_string (storage, &string) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (b == TRUE)
        user = dg87_server_user_get_by_name (context->server, string);
      else
        user = dg87_server_user_add (context->server, string);
      dg87_free (string, DG87_CHAR);
      if (user == NULL)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_user_load (user, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }

  /* --------- Referentials ---------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load referentials...");
  /* read referentials count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  while (count > 0)
    {
      count--;
      /* load referential */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if ((ref = dg87_context_ref_model_add (context, string)) == FALSE)
        {
          dg87_free (string, DG87_CHAR);
          dg87_storage_close (storage);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      if (dg87_ref_model_load (ref, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Rules ---------- */
      if (dg87_context_load_rules (context, ref, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Tasks ---------- */
      /* read task count */
      if (dg87_storage_read_uint (storage, &count2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      while (count2 > 0)
        {
          count2--;
          /* load task */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if ((task = dg87_ref_model_task_add (ref, string)) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              dg87_storage_close (storage);
              return FALSE;
            }
          dg87_free (string, DG87_CHAR);
          if (dg87_ref_task_load (task, storage, context) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

          /* --------- Jobs ----------- */
          /* read job count */
          if (dg87_storage_read_uint (storage, &count3) == FALSE)
            return FALSE;
          while (count3 > 0)
            {
              count3--;
              /* load job */
              if (dg87_storage_read_string (storage, &string) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if ((job = dg87_ref_task_job_add (task, string)) == FALSE)
                {
                  dg87_free (string, DG87_CHAR);
                  dg87_storage_close (storage);
                  return FALSE;
                }
              dg87_free (string, DG87_CHAR);
              if (dg87_ref_job_load (job, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next job */

          /* --------- Jobs Deps ----------- */
          /* read jobdep count */
          if (dg87_storage_read_uint (storage, &count3) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          while (count3 > 0)
            {
              count3--;
              /* find job before */
              if (dg87_storage_read_string (storage, &string) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              job = dg87_ref_task_job_get_by_name (task, string);
              dg87_free (string, DG87_CHAR);
              /* find job after */
              if (dg87_storage_read_string (storage, &string) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              job2 = dg87_ref_task_job_get_by_name (task, string);
              dg87_free (string, DG87_CHAR);
              /* load dependence */
              if ((job_dep = dg87_ref_task_jobdep_add (task,
                                                       job, job2,
                                                       dg87_ref_jobdep_always))
                  == NULL)
                {
                  dg87_free (string, DG87_CHAR);
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_ref_jobdep_load (job_dep, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next jobdep */

          /* --------- Rules ---------- */
          if (dg87_context_load_task_rules (context, task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

          /* --------- Tasks conditions----- */
          if (dg87_context_load_conditions (context, task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

        }/*next task*/

      /* --------- Task Deps ----------- */
      /* read taskdep count */
      if (dg87_storage_read_uint (storage, &count2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      while (count2 > 0)
        {
          count2--;
          /* find task before */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          task = dg87_ref_model_task_get_by_name (ref, string);
          dg87_free (string, DG87_CHAR);
          /* find task after */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          task2 = dg87_ref_model_task_get_by_name (ref, string);
          dg87_free (string, DG87_CHAR);
          /* load dependence */
          if ((taskdep = dg87_ref_model_taskdep_add (ref, task,
                                                     task2,
                                                     dg87_ref_taskdep_always))
              == NULL)
            {
              dg87_free (string, DG87_CHAR);
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_ref_taskdep_load (taskdep, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        }/* next taskdep */
    }/* next referential */

  /* --------- Days ------------------------ */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load exploitation days...");
  /* read days count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  while (count > 0)
    {
      count--;
      /* load day */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_read_long (storage, &longvalue) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      start_time = longvalue;
      if ((exp_day = dg87_context_exp_day_add (context,
                                               string, start_time)) == FALSE)
        {
          dg87_free (string, DG87_CHAR);
          dg87_storage_close (storage);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      if (dg87_exp_day_load (exp_day, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Tasks ---------- */
      /* read task count */
      if (dg87_storage_read_uint (storage, &count2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      while (count2 > 0)
        {
          count2--;
          /* load task */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if ((exp_task = dg87_exp_day_task_add (exp_day, string)) == FALSE)
            {
              dg87_free (string, DG87_CHAR);
              dg87_storage_close (storage);
              return FALSE;
            }
          dg87_free (string, DG87_CHAR);
          if (dg87_exp_task_load (exp_task, storage, context) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

          /* --------- Jobs ----------- */
          /* read job count */
          if (dg87_storage_read_uint (storage, &count3) == FALSE)
            return FALSE;
          while (count3 > 0)
            {
              count3--;
              /* load job */
              if (dg87_storage_read_string (storage, &string) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if ((exp_job =
                   dg87_exp_task_job_add (exp_task, string)) == FALSE)
                {
                  dg87_free (string, DG87_CHAR);
                  dg87_storage_close (storage);
                  return FALSE;
                }
              dg87_free (string, DG87_CHAR);
              if (dg87_exp_job_load (exp_job, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next job */

          /* --------- Jobs Deps ----------- */
          /* read jobdep count */
          if (dg87_storage_read_uint (storage, &count3) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          while (count3 > 0)
            {
              count3--;
              /* find job before */
              if (dg87_storage_read_string (storage, &string) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              exp_job = dg87_exp_task_job_get_by_name (exp_task, string);
              dg87_free (string, DG87_CHAR);
              /* find job after */
              if (dg87_storage_read_string (storage, &string) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              exp_job2 = dg87_exp_task_job_get_by_name (exp_task, string);
              dg87_free (string, DG87_CHAR);
              /* load dependence */
              if ((exp_job_dep = dg87_exp_task_jobdep_add (exp_task,
                                                           exp_job, exp_job2,
                                                           dg87_exp_jobdep_type_always))
                  == NULL)
                {
                  dg87_free (string, DG87_CHAR);
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_exp_jobdep_load (exp_job_dep, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next jobdep */

          /* --------- Tasks conditions----- */
          if (_dg87_context_load_conditions (context,
                                             exp_task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

        }/* next task */

      /* --------- Task Deps ----------- */
      /* read taskdep count */
      if (dg87_storage_read_uint (storage, &count2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      while (count2 > 0)
        {
          count2--;
          /* find task before */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          exp_task = dg87_exp_day_task_get_by_name (exp_day, string);
          dg87_free (string, DG87_CHAR);
          /* find task after */
          if (dg87_storage_read_string (storage, &string) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          exp_task2 = dg87_exp_day_task_get_by_name (exp_day, string);
          dg87_free (string, DG87_CHAR);
          /* load dependence */
          if ((exp_taskdep =
               dg87_exp_day_taskdep_add (exp_day, exp_task, exp_task2,
                                         dg87_exp_taskdep_type_always)) ==
              NULL)
            {
              dg87_free (string, DG87_CHAR);
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_exp_taskdep_load (exp_taskdep, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        } /* next taskdep */
    }/* next day */

  /* --------- Files ----------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load files to monitor...");
  /* read file count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      /* load day name */
      if (dg87_storage_read_string (storage, &day_name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* load task name */
      if (dg87_storage_read_string (storage, &task_name) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      /* load path */
      if (dg87_storage_read_string (storage, &path) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (day_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          return FALSE;
        }
      /* load hostname */
      if (dg87_storage_read_string (storage, &hostname) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (path, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          return FALSE;
        }
      /* load ip_port */
      if (dg87_storage_read_int (storage, &ip_port) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (hostname, DG87_CHAR);
          dg87_free (path, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          return FALSE;
        }
      /* create file */
      followed_file = dg87_r_context_file_add (context,
                                               day_name, task_name, path,
                                               hostname, ip_port);
      dg87_free (day_name, DG87_CHAR);
      dg87_free (task_name, DG87_CHAR);
      dg87_free (path, DG87_CHAR);
      dg87_free (hostname, DG87_CHAR);
      if (followed_file == NULL)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_r_file_load (followed_file, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }

  /* --------- Jobs ----------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load jobs to monitor...");
  /* read job count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    return FALSE;
  while (count > 0)
    {
      count--;
      /* load day name */
      if (dg87_storage_read_string (storage, &day_name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* load task name */
      if (dg87_storage_read_string (storage, &task_name) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      /* load job name */
      if (dg87_storage_read_string (storage, &job_name) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (task_name, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      /* load path */
      if (dg87_storage_read_string (storage, &path) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (job_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      /* load argument */
      if (dg87_storage_read_string (storage, &argument) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (job_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          dg87_free (path, DG87_CHAR);
          return FALSE;
        }
      /* load hostname */
      if (dg87_storage_read_string (storage, &hostname) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (argument, DG87_CHAR);
          dg87_free (path, DG87_CHAR);
          dg87_free (job_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      /* load IP port */
      if (dg87_storage_read_int (storage, &ip_port) == FALSE)
        {
          dg87_storage_close (storage);
          dg87_free (hostname, DG87_CHAR);
          dg87_free (argument, DG87_CHAR);
          dg87_free (path, DG87_CHAR);
          dg87_free (job_name, DG87_CHAR);
          dg87_free (task_name, DG87_CHAR);
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      /* create job */
      followed_job = dg87_r_context_job_add (context,
                                             day_name, task_name, job_name,
                                             path, argument, hostname,
                                             ip_port);
      dg87_free (hostname, DG87_CHAR);
      dg87_free (argument, DG87_CHAR);
      dg87_free (path, DG87_CHAR);
      dg87_free (job_name, DG87_CHAR);
      dg87_free (task_name, DG87_CHAR);
      dg87_free (day_name, DG87_CHAR);
      if (followed_job == NULL)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_r_job_load (followed_job, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }/* next job */

  /* --------- Messages -------------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load messages...");
  /* read messages count */
  if (dg87_storage_read_uint (storage, &count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  while (count > 0)
    {
      count--;
      /* load host name */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* load IP port */
      if (dg87_storage_read_uint (storage, &ipport) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* load message */
      if (dg87_storage_read_string (storage, &string2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* post message */
      if (dg87_server_post_message (context->server,
                                    string, ipport, string2) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }

  /* close file */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Load terminated");
  dg87_storage_close (storage);
	DG87_SAVE_DONE; /* save is not needed after loading */
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_save_conditions (dg87_context * context,
                              dg87_ref_task * task, dg87_storage * storage)
{
  dg87_item *item;
  dg87_ref_condition *condition;
  assert (context != NULL && task != NULL && storage != NULL);
  /* save conditions count */
  if (dg87_storage_write_uint (storage, task->condition_list->count) == FALSE)
    return FALSE;
  /* for each condition of the task */
  item = task->condition_list->first;
  while (item != NULL)
    {
      condition = item->object;
      item = item->next;
      /* save condition */
      if (dg87_storage_write_ubyte (storage, condition->type) == FALSE)
        return FALSE;
      switch (condition->type)
        {
        case condition_hmin:
          if (dg87_storage_write_uint (storage,
                                       condition->data.condition_hmin->
                                       minutes) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_hmin->
                                       day_delta) == FALSE)
            return FALSE;
          break;
        case condition_hmax:
          if (dg87_storage_write_uint (storage,
                                       condition->data.condition_hmax->
                                       minutes) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_hmax->
                                       day_delta) == FALSE)
            return FALSE;
          break;
        case condition_hlate:
          if (dg87_storage_write_uint (storage,
                                       condition->data.condition_hlate->
                                       minutes) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_hlate->
                                       day_delta) == FALSE)
            return FALSE;
          break;
        case condition_file:
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_File->
                                         path) == FALSE)
            return FALSE;
          break;
        case condition_manual:
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_manual->
                                         message) == FALSE)
            return FALSE;
          break;
        case condition_ifequal:
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_ifequal->
                                         variable->name) == FALSE)
            return FALSE;
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_ifequal->
                                         value) == FALSE)
            return FALSE;
          break;
        case condition_ifnequal:
          if (dg87_storage_write_string (storage,
                                         condition->data.
                                         condition_ifnotequal->variable->
                                         name) == FALSE)
            return FALSE;
          if (dg87_storage_write_string (storage,
                                         condition->data.
                                         condition_ifnotequal->value) ==
              FALSE)
            return FALSE;

          break;
        default:
          break;
        }
    }/* next condition */
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_save_operations (dg87_context * context,
                              dg87_ref_rule_operation * operation,
                              dg87_storage * storage)
{
  /* save operation */
  assert (context != NULL && operation != NULL && storage != NULL);
  if (dg87_storage_write_ubyte (storage, operation->oper) == FALSE)
    return FALSE;
  if (dg87_storage_write_ubyte (storage, operation->type) == FALSE)
    return FALSE;
  /* save data */
  switch (operation->type)
    {
    case dg87_ref_rule_type_dow:
      if (dg87_storage_write_byte (storage, operation->
                                   rule.dow.day_of_week) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_dom:
      if (dg87_storage_write_byte (storage, operation->
                                   rule.dom.day_of_month) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_doy:
      if (dg87_storage_write_int (storage, operation->
                                  rule.doy.day_of_year) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_wom:
      if (dg87_storage_write_byte (storage, operation->
                                   rule.wom.week_of_month) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_woy:
      if (dg87_storage_write_byte (storage, operation->
                                   rule.woy.week_of_year) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_moy:
      if (dg87_storage_write_byte (storage, operation->
                                   rule.moy.month_of_year) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_day:
      if (dg87_storage_write_ubyte (storage, operation->rule.day.day) == FALSE)
        return FALSE;
      if (dg87_storage_write_ubyte (storage, operation->
                                   rule.day.month) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_date:
      if (dg87_storage_write_long (storage, operation->
                                   rule.date.date) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_always:
      /* nothing to save */
      break;
    case dg87_ref_rule_type_complex:
      if (dg87_storage_write_string (storage,
                                     operation->rule.rule->name) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_ifequal:
      if (dg87_storage_write_string (storage,
                                     operation->rule.ifequal.
                                     variable->name) == FALSE)
        return FALSE;
      if (dg87_storage_write_string (storage,
                                     operation->rule.ifequal.value) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_ifnequal:
      if (dg87_storage_write_string (storage,
                                     operation->rule.ifnequal.
                                     variable->name) == FALSE)
        return FALSE;
      if (dg87_storage_write_string (storage,
                                     operation->rule.ifnequal.value) == FALSE)
        return FALSE;
      break;
    default:
      break;
    }
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_save_rules (dg87_context * context,
                         dg87_ref_model * ref, dg87_storage * storage)
{
  dg87_element *element;
  dg87_ref_rule *rule;
  dg87_item *item;
  dg87_ref_rule_operation *operation;
  assert (context != NULL && ref != NULL && storage != NULL);
  /* -------- save rules ---------- */
  /* save rules count */
  if (dg87_storage_write_uint (storage, ref->rule_vector->count) == FALSE)
    return FALSE;
  /* for each rule */
  element = ref->rule_vector->first;
  while (element != NULL)
    {
      rule = element->object;
      element = element->next;
      /* save rule name */
      if (dg87_storage_write_string (storage, rule->name) == FALSE)
        return FALSE;
    }
  /* -------- save operations ----- */
  /* for each rule */
  element = ref->rule_vector->first;
  while (element != NULL)
    {
      rule = element->object;
      element = element->next;
      /* save operations count */
      if (dg87_storage_write_uint (storage,
                                   rule->operation_list->count) == FALSE)
        return FALSE;
      /* for each operation */
      item = rule->operation_list->first;
      while (item != NULL)
        {
          operation = item->object;
          item = item->next;
          /* save rule operations */
          if (dg87_context_save_operations (context,
                                            operation, storage) == FALSE)
            return FALSE;
        }
    }
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_save_task_rules (dg87_context * context,
                              dg87_ref_task * task, dg87_storage * storage)
{
  dg87_item *item;
  dg87_ref_rule_operation *operation;
  assert (context != NULL && task != NULL && storage != NULL);
  /* save operations count */
  if (dg87_storage_write_uint (storage,
                               task->rule_operation_list->count) == FALSE)
    return FALSE;
  /* for each operation */
  item = task->rule_operation_list->first;
  while (item != NULL)
    {
      operation = item->object;
      item = item->next;
      /* save rule operations */
      if (dg87_context_save_operations (context, operation, storage) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_context_save_exp_conditions (dg87_context * context,
                                  dg87_exp_task * task,
                                  dg87_storage * storage)
{
  dg87_item *item;
  dg87_exp_condition *condition;
  assert (context != NULL && task != NULL && storage != NULL);
  /* save conditions count */
  if (dg87_storage_write_uint (storage, task->condition_list->count) == FALSE)
    return FALSE;
  /* for each condition of the task */
  item = task->condition_list->first;
  while (item != NULL)
    {
      condition = item->object;
      item = item->next;
      /* save condition */
      if (dg87_storage_write_ubyte (storage, condition->type) == FALSE)
        return FALSE;
      switch (condition->type)
        {
        case condition_hmin:
          if (dg87_storage_write_uint (storage,
                                       condition->data.condition_hmin->
                                       minutes) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_hmin->
                                       day_delta) == FALSE)
            return FALSE;
          break;
        case condition_hmax:
          if (dg87_storage_write_uint (storage,
                                       condition->data.condition_hmax->
                                       minutes) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_hmin->
                                       day_delta) == FALSE)
            return FALSE;
          break;
        case condition_hlate:
          if (dg87_storage_write_uint (storage,
                                       condition->data.condition_hlate->
                                       minutes) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_hmin->
                                       day_delta) == FALSE)
            return FALSE;
          break;
        case condition_file:
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_File->
                                         path) == FALSE)
            return FALSE;
          if (dg87_storage_write_ubyte (storage,
                                       condition->data.condition_File->
                                       command_sended) == FALSE)
            return FALSE;

          break;
        case condition_manual:
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_manual->
                                         message) == FALSE)
            return FALSE;
          break;
        case condition_ifequal:
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_ifequal->
                                         variable->name) == FALSE)
            return FALSE;
          if (dg87_storage_write_string (storage,
                                         condition->data.condition_ifequal->
                                         value) == FALSE)
            return FALSE;
          break;
        case condition_ifnequal:
          if (dg87_storage_write_string (storage,
                                         condition->data.
                                         condition_ifnotequal->variable->
                                         name) == FALSE)
            return FALSE;
          if (dg87_storage_write_string (storage,
                                         condition->data.
                                         condition_ifnotequal->value) ==
              FALSE)
            return FALSE;

          break;
        default:
          break;
        }
      if (dg87_exp_condition_save (condition, storage) == FALSE)
        return FALSE;
    }/* next condition */
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_context_save (dg87_context * context)
{
  dg87_element *element;
  dg87_element *element2;
  dg87_element *element3;
  dg87_item *item;
  dg87_target *target;
  dg87_queue *queue;
  dg87_variable *variable;
  dg87_storage *storage;
  dg87_user *user;
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_ref_jobdep *job_dep;
  dg87_ref_taskdep *taskdep;
  dg87_exp_day *exp_day;
  dg87_exp_task *exp_task;
  dg87_exp_job *exp_job;
  dg87_exp_jobdep *exp_job_dep;
  dg87_exp_taskdep *exp_taskdep;
  dg87_r_job *followed_job;
  dg87_r_file *followed_file;

  dg87_message *message;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  storage = context->storage;
  if (dg87_storage_open (storage, dg87_storage_mode_write) == FALSE)
    return FALSE;
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save started");
  /* --------- Engine ---------------------- */
  if (dg87_engine_save (context->engine, storage) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* --------- Targets --------------------- */
  /* save target count */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save targets...");
  if (dg87_storage_write_uint (storage,
                               context->target_vector->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* for each target */
  element = context->target_vector->first;
  while (element != NULL)
    {
      target = element->object;
      element = element->next;
      /* save target */
      if (dg87_storage_write_string (storage, target->name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_write_ubyte (storage, target->system) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_target_save (target, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Queues --------------- */
      /* save queue count */
      if (dg87_storage_write_uint (storage,
                                   target->queue_vector->count) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* for each queue of the target */
      element2 = target->queue_vector->first;
      while (element2 != NULL)
        {
          queue = element2->object;
          element2 = element2->next;
          /* save queue */
          if (dg87_storage_write_string (storage, queue->name) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_storage_write_ubyte (storage, queue->system) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_queue_save (queue, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        }
    }
  /* --------- Variables ------------------- */
  /* save variable count */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save variables...");
  if (dg87_storage_write_uint (storage,
                               context->engine->variable_vector->count) ==
      FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* for each variable */
  element = context->engine->variable_vector->first;
  while (element != NULL)
    {
      variable = element->object;
      element = element->next;
      /* save variable */
      if (dg87_storage_write_string (storage, variable->name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_variable_save (variable, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }
  /* --------- Users ----------------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save users...");
  /* save user count */
  if (dg87_storage_write_uint (storage,
                               context->server->user_vector->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* for each user */
  element = context->server->user_vector->first;
  while (element != NULL)
    {
      user = element->object;
      element = element->next;
      /* save User */
      if (dg87_storage_write_ubyte (storage, user->system_account) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_write_string (storage, user->name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_user_save (user, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }
  /* --------- Referentials ---------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save referentials...");
  /* save referentials count */
  if (dg87_storage_write_uint (storage, context->ref_vector->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* for each referential */
  element = context->ref_vector->first;
  while (element != NULL)
    {
      ref = element->object;
      element = element->next;
      /* save referential */
      if (dg87_storage_write_string (storage, ref->name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_ref_model_save (ref, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Rules ---------- */
      if (dg87_context_save_rules (context, ref, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Tasks ---------------- */
      /* save tasks count */
      if (dg87_storage_write_uint (storage, ref->task_vector->count) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* for each task of the referential */
      element2 = ref->task_vector->first;
      while (element2 != NULL)
        {
          task = element2->object;
          element2 = element2->next;
          /* save task */
          if (dg87_storage_write_string (storage, task->name) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_ref_task_save (task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* --------- Jobs ---------------- */
          /* save jobs count */
          if (dg87_storage_write_uint (storage,
                                       task->job_vector->count) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* for each job of the task */
          element3 = task->job_vector->first;
          while (element3 != NULL)
            {
              job = element3->object;
              element3 = element3->next;
              /* save job */
              if (dg87_storage_write_string (storage, job->name) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_ref_job_save (job, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next job */

          /* --------- Jobs Dep-------------- */
          /* save job dependencies count */
          if (dg87_storage_write_uint (storage,
                                       task->job_dep_list->count) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* for each jobdep of the task */
          item = task->job_dep_list->first;
          while (item != NULL)
            {
              job_dep = item->object;
              item = item->next;
              /* save jobdep */
              if (dg87_storage_write_string (storage,
                                             job_dep->upstream_job->name) ==
                  FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_storage_write_string (storage,
                                             job_dep->downstream_job->name) ==
                  FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_ref_jobdep_save (job_dep, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next jobdep */

          /* --------- Task Rules ---------- */
          if (dg87_context_save_task_rules (context, task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

          /* --------- Tasks conditions ----- */
          if (dg87_context_save_conditions (context, task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

        }/* next task */

      /* --------- Task Dependencies -------------- */
      /* save task dependencies count */
      if (dg87_storage_write_uint (storage,
                                   ref->taskdep_list->count) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* for each taskdep of the task */
      item = ref->taskdep_list->first;
      while (item != NULL)
        {
          taskdep = item->object;
          item = item->next;
          /* save taskdep */
          if (dg87_storage_write_string (storage,
                                         taskdep->upstream_task->name) ==
              FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_storage_write_string (storage,
                                         taskdep->downstream_task->name) ==
              FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_ref_taskdep_save (taskdep, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        }/* next taskdep */


    }/* next referential */

  /* --------- Days ------------------------ */
  /* save days count */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save exploitation days...");
  if (dg87_storage_write_uint (storage,
                               context->exp_day_vector->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }

  /* for each days */
  element = context->exp_day_vector->first;
  while (element != NULL)
    {
      exp_day = element->object;
      element = element->next;
      /* save day */
      if (dg87_storage_write_string (storage, exp_day->name) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_write_long (storage, exp_day->start_time) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_exp_day_save (exp_day, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }

      /* --------- Tasks ---------------- */
      /* save tasks count */
      if (dg87_storage_write_uint (storage,
                                   exp_day->task_vector->count) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* for each task of the day */
      element2 = exp_day->task_vector->first;
      while (element2 != NULL)
        {
          exp_task = element2->object;
          element2 = element2->next;
          /* save task */
          if (dg87_storage_write_string (storage, exp_task->name) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_exp_task_save (exp_task, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* --------- Jobs ---------------- */
          /* save jobs count */
          if (dg87_storage_write_uint (storage,
                                       exp_task->job_vector->count) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* for each job of the task */
          element3 = exp_task->job_vector->first;
          while (element3 != NULL)
            {
              exp_job = element3->object;
              element3 = element3->next;
              /* save job */
              if (dg87_storage_write_string (storage, exp_job->name) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_exp_job_save (exp_job, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next job */

          /* --------- Jobs Dep-------------- */
          /* save job dependencies count */
          if (dg87_storage_write_uint (storage,
                                       exp_task->job_dep_list->count) ==
              FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          /* for each jobdep of the task */
          item = exp_task->job_dep_list->first;
          while (item != NULL)
            {
              exp_job_dep = item->object;
              item = item->next;
              /* save jobdep */
              if (dg87_storage_write_string (storage,
                                             exp_job_dep->upstream_job->
                                             name) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_storage_write_string (storage,
                                             exp_job_dep->downstream_job->
                                             name) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
              if (dg87_exp_jobdep_save (exp_job_dep, storage) == FALSE)
                {
                  dg87_storage_close (storage);
                  return FALSE;
                }
            }/* next jobdep */

          /* --------- Tasks conditions ----- */
          if (dg87_context_save_exp_conditions (context, exp_task,
                                                storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }

        }/* next task */

      /* --------- Task Dependencies -------------- */
      /* save task dependencies count */
      if (dg87_storage_write_uint (storage,
                                   exp_day->taskdep_list->count) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* for each taskdep of the task */
      item = exp_day->taskdep_list->first;
      while (item != NULL)
        {
          exp_taskdep = item->object;
          item = item->next;
          /* save taskdep */
          if (dg87_storage_write_string (storage,
                                         exp_taskdep->upstream_task->name) ==
              FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_storage_write_string (storage,
                                         exp_taskdep->downstream_task->
                                         name) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
          if (dg87_exp_taskdep_save (exp_taskdep, storage) == FALSE)
            {
              dg87_storage_close (storage);
              return FALSE;
            }
        }/* next taskdep */
    }/* next day */

  /* --------- Files ---------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save files to monitor...");
  /* save files count */
  if (dg87_storage_write_uint (storage, context->file_list->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* for each file */
  item = context->file_list->first;
  while (item != NULL)
    {
      followed_file = item->object;
      item = item->next;
      /* save day name */
      if (dg87_storage_write_string (storage, followed_file->day_name) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save task name */
      if (dg87_storage_write_string (storage, followed_file->task_name) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save path */
      if (dg87_storage_write_string (storage, followed_file->path) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save hostname */
      if (dg87_storage_write_string (storage, followed_file->hostname) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save IP port */
      if (dg87_storage_write_int (storage, followed_file->ip_port) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }

  /* --------- Jobs ---------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save jobs to monitor...");
  /* save jobs count */
  if (dg87_storage_write_uint (storage, context->job_list->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  /* for each job */
  item = context->job_list->first;
  while (item != NULL)
    {
      followed_job = item->object;
      item = item->next;
      /* save day name */
      if (dg87_storage_write_string (storage, followed_job->day_name) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save task name */
      if (dg87_storage_write_string (storage, followed_job->task_name) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save job name */
      if (dg87_storage_write_string (storage, followed_job->job_name) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save path */
      if (dg87_storage_write_string (storage, followed_job->path) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save argument */
      if (dg87_storage_write_string (storage, followed_job->argument) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save hostname */
      if (dg87_storage_write_string (storage, followed_job->hostname) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      /* save IP port */
      if (dg87_storage_write_int (storage, followed_job->ip_port) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_r_job_save (followed_job, storage) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }/* next job */

  /* --------- Messages -------------------- */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save messages...");
  /* save messages count */
  if (dg87_storage_write_uint (storage,
                               context->server->message_list->count) == FALSE)
    {
      dg87_storage_close (storage);
      return FALSE;
    }
  item = context->server->message_list->first;
  while (item != NULL)
    {
      message = item->object;
      item = item->next;
      if (dg87_storage_write_string (storage, message->hostname) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_write_uint (storage, message->ip_port) == FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
      if (dg87_storage_write_string (storage, message->message_string) ==
          FALSE)
        {
          dg87_storage_close (storage);
          return FALSE;
        }
    }

  /* close file */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
									"Save terminated");
  dg87_storage_close (storage);
	DG87_SAVE_DONE;
  return TRUE;
}

/****************************************************************************/
dg87_log_day *
dg87_context_log_load_day (dg87_context * context, char *filename)
{
  dg87_log_day *day;
  assert (context != NULL && filename != NULL);
  if (context == NULL || filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  day = dg87_log_day_load (filename);
  if (day == NULL)
    return NULL;

  /* add the day in the day vector */
  /* vector generate error if an other day has the same name. */
  /* Key must point on day->name */
  if (dg87_vector_add_object (context->log_day_vector,
                              day, day->name) == FALSE)
    {
      dg87_log_day_delete (day, FALSE);
      return NULL;
    }
  return day;

}

/****************************************************************************/
BOOL
dg87_context_log_load_days (dg87_context * context)
{
  DIR *dir;
  struct dirent *dirent;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* unload days which are in memory */
  /*TODO unload days which are in memory */

  if ((dir = opendir (".")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't open data directory '%s' !",
               context->data_directory);
      dg87_err_set_error (DG87_ERR_CAN_NOT_ACCESS_DIRECTORY);
      return FALSE;
    }

  /* read directory entries */
  for (;;)
    {
      dirent = readdir (dir);
      if (dirent == NULL)
        break;
      /* ignore files which doesn't start with 'l_' */
      if (strncmp (dirent->d_name, DIO87D_LOGFILE_TEMPLATE, 2) != 0)
        continue;

      /* try to load day */
      if (dg87_context_log_load_day (context, dirent->d_name) == FALSE)
        return FALSE;
    }

  closedir (dir);

  return TRUE;
}

/****************************************************************************/
char *
dg87_context_get_hostname (dg87_context * context, BOOL use_conf_file)
{
  int size = 127;
  char *buffer;
  char *newbuffer;
  struct hostent *hoste;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* try to get the hostname from the configuration file */
  if (use_conf_file == TRUE && context->hostname != NULL)
    {
      buffer = dg87_strdup (context->hostname, DG87_CHAR);
      if (buffer == NULL)
        {
          dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
          return NULL;
        }
      return buffer;
    }
  /* get the hostname by calling gethostname */
  buffer = dg87_malloc (size + 1, DG87_CHAR);
  if (buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  buffer[0] = '\0';
  while (gethostname (buffer, size) != 0)
    {
      if (errno == ENAMETOOLONG)
        {
          /* increase the buffer size and try again */
          size *= 2;
          newbuffer = realloc (buffer, size + 1);
          if (newbuffer == NULL)
            {
              dg87_free (buffer, DG87_CHAR);
              return NULL;
            }
          buffer = newbuffer;
        }
      else
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	"gethostname error !");
          dg87_free (buffer, DG87_CHAR);
          return NULL;
        }
      return NULL;
    }
  /* get the fully qualified name */
  hoste = gethostbyname (buffer);
  if (hoste != NULL)
    {
      dg87_free (buffer, DG87_CHAR);
      buffer = dg87_strdup (hoste->h_name, DG87_CHAR);
      if (buffer == NULL)
        {
          dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
          return NULL;
        }
      return buffer;
    }

  return buffer;
}

/****************************************************************************/

#ifdef DEBUG
int
dg87_context_check (dg87_context * context, int (*print) (char *))
{
  dg87_element *element;

  /* check engine */
  if (context->engine == NULL)
    {
      print ("Context : engine is null !\n");
      return 1;
    }
  if (dg87_engine_check (context->engine, print) != 0)
    return 1;

  /* check ref vector */
  if (context->ref_vector == NULL)
    {
      print ("Context : ref is null !\n");
      return 1;
    }
  if (dg87_vector_check (context->ref_vector, print) != 0)
    return 1;
  element = context->ref_vector->first;
  while (element != NULL)
    {
      if (dg87_ref_model_check (element->object, print) != 0)
        return 1;
      element = element->next;
    }

  /* check the target vector */
  if (context->target_vector == NULL)
    {
      print ("Context : target_vector is null !\n");
      return 1;
    }
  if (dg87_vector_check (context->target_vector, print) != 0)
    return 1;
  element = context->target_vector->first;
  while (element != NULL)
    {
      if (dg87_target_check (element->object, print) != 0)
        return 1;
      element = element->next;
    }
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
