/* file d_cmdjob.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "d_cmdjob.h"
#include "util.h"

/******************************************************************************/
/* rjk
 * Ref Job Kill
 * Usage : rjk RefName TaskName JobName
 */
int
cmd_rjk (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjk RefName TaskName JobName\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* delete the job */
  dg87_err_clear_error ();
  if (dg87_ref_task_job_delete_by_name (task, command->argv[2]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't delete the job '%s' of the task '%s' "\
															"in the referential '%s' ! %s\n"),
											command->argv[1], task, task->ref->name,
											dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************
 * cmd_rja
 * Ref Job Add
 * Usage : rja RefName TaskName Taskjob [description]
 */
int
cmd_rja (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "rja RefName TaskName JobName [description]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* add the job to the task */
  dg87_err_clear_error ();
  if ((job = dg87_ref_task_job_add (task, command->argv[2])) == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't a add job '%s' to the task '%s' "\
															" in the referential '%s' ! %s\n"),
											command->argv[2], task->name, task->ref->name,
											dg87_err_get_error_str ());
      return 1;
    }

  /* change description */
  if (command->argc == 4)
    {
      dg87_err_clear_error ();
      if (dg87_ref_job_set_description (job, command->argv[3]) == FALSE)
        {
          dg87_connection_printf (connection,
									gettext ("Can't change the description of the job '%s' of "\
													"the task '%s' in the referential '%s' ! %s\n"),
									job->name, task->name, task->ref->name,
									dg87_err_get_error_str ());
          /* delete the job before exit */
          dg87_ref_task_job_delete (task, job);
          return 1;
        }
    }
  return 0;
}

/******************************************************************************/
/* cmd_rjc
 * Usage : rjc RefName TaskName
 * Task Job Count
 * return the number of jobs in the TASK.
 */
int
cmd_rjc (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjc RefName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* return jobs count */
  dg87_connection_printf (connection, "%i\n", task->job_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_rjl
 * rjl [options] RefName TaskName
 * Ref Job List
 * Options
 * -d : print description of job 
 * -t : print task name of job
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 */
int
cmd_rjl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  /*static char optstring[] = "dtB:S:E:I:N:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;         /* index of the first task to display */
  int number = 0;        /* number of jobs to display */
  int count = 0;         /* number of job actually display */
  dg87_ref_model *ref;
  dg87_element *element = NULL;
  dg87_ref_task *task;
  dg87_ref_job *job;
  BOOL show_description = FALSE;
  BOOL show_task_name = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "rjl [options] RefName TaskName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':/* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':/* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':/* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':/* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N':/* number of task to display */
          number = atoi (optarg);
          break;
        case 'd':/* description */
          show_description = TRUE;
          break;
        case 't':/* Task */
          show_task_name = TRUE;
          break;

        default: /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first job to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (task->job_vector, start);
  else
    element = task->job_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      job = element->object;

      /* response */
      dg87_connection_printf (connection, beg);
      if (show_task_name == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", job->task->name, sep);
        }
      dg87_connection_printf (connection, job->name);
      if (show_description == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep, job->description);
        }
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rji
 * ref job information 
 * get information for a job
 * Usage : rji [options] RefName TaskName JobName
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_rji (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rji [options] RefName TaskName JobName");
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':/* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':/* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':/* end separator */
          free (end);
          end = strdup (optarg);
          break;

        default: /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* job name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Job Name", sep, 
													job->name, end);
  /* Task name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Name", sep, 
													job->task->name, end);
  /* day name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Referential Name", sep, 
													job->task->ref->name, end);
  /* job desription */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Description", sep, 
													job->description, end);
  /* Task Target */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Target", sep,
                          (job->task->queue == NULL) ? "none" :
                          job->task->queue->target->name, end);
  /* Target Queue */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Target Queue", sep,
                          (job->task->queue == NULL) ? "none" :
                          job->task->queue->name, end);
  /* job Path */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Path", sep,
                          (job->path == NULL) ? "none" : job->path, end);
  /* argument */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Argument", sep,
                          (job->argument ==
                           NULL) ? "none" : job->argument, end);
  /* start tempo */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Start Tempo", sep, 
													job->start_tempo, end);
  /* end tempo */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "End Tempo", sep, 
													job->end_tempo, end);
  /* wait_all_dependencies */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Wait All Dependencies", sep,
                          dg87_bool_to_ascii (job->wait_all_dependencies),
                          end);
  /* on_error_continue */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "On Error Continue", sep,
                          dg87_bool_to_ascii (job->on_error_continue), end);

  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rjsn
 * Ref Job Set Name 
 * change the name of a job
 * Usage : rjsn RefName TaskName JobName NewName 
 */
int
cmd_rjsn (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjsn RefName TaskName JobName NewName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  if (dg87_ref_job_set_name (job, command->argv[3]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set the name of the job '%s' "\
																			"in the task '%s' in the "\
																			"referential '%s' ! %s\n"),
                              job->name, job->task->name, job->task->ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjsd
 * Ref Job Set Description
 * change the description of a job
 * Usage : rjsd RefName TaskName JobName description
 */
int
cmd_rjsd (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjsd RefName TaskName JobName description");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  if (dg87_ref_job_set_description (job, command->argv[3]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the description of the job '%s' "\
											"in the task '%s' in the referential '%s' ! %s\n"),
											job->name, job->task->name, job->task->ref->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjsp
 * Ref Job Set Path 
 * change the path of a job.
 * Usage : rjsp RefName TaskName JobName Path
 */
int
cmd_rjsp (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjsp RefName TaskName JobName Path");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  if (dg87_ref_job_set_path (job, command->argv[3]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set the path of the job '%s' "\
																			"in the task '%s' in the referential "\
																			"'%s' ! %s\n"),
                              job->name, job->task->name, job->task->ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjsa
 * Ref Job Set Argument 
 * change the argument of a job.
 * Usage : rjsa RefName TaskName JobName argument
 */
int
cmd_rjsa (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjsa RefName TaskName JobName argument");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  if (dg87_ref_job_set_argument (job, command->argv[3]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set the argument of the job "\
																			"'%s' in the task '%s' in the "\
																			"referential '%s' ! %s\n"),
                              job->name, job->task->name, job->task->ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjst
 * Ref Job Set Tempo
 * change the start_tempo and end_tempo of a job
 * Usage : rjst RefName TaskName JobName start_tempo end_tempo
 */
int
cmd_rjst (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *job;
  int start_tempo, end_tempo;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjst RefName TaskName JobName start_tempo end_tempo");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  /* change tempo */
  start_tempo = atoi (command->argv[3]);
  end_tempo = atoi (command->argv[4]);
  dg87_err_clear_error ();
  if (dg87_ref_job_set_tempo (job, start_tempo, end_tempo) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set the tempo of the job '%s' "\
                              "in the task '%s' in the referential '%s' ! "\
															"%s\n"),
                              job->name, job->task->name, job->task->ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjswad
 * Ref Job Set _wait_all_dependencies
 * Usage : rjswad RefName TaskName JobName true/false
 */
int
cmd_rjswad (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_ref_job *job;
  BOOL value;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjswad RefName TaskName JobName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[3])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[3]);
      return 1;
    }

  /* change the property wait_all_dependencies */
  dg87_err_clear_error ();
  if (dg87_ref_job_set_wait_all_dependencies (job, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the job '%s' "\
															"of the task '%s' in the referential "\
															"'%s' ! %s\n"),
															"wait_all_dependencies",
                              job->name, task->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjsoec
 * Ref Job Set on_error_continue
 * Usage : rjsoec RefName TaskName JobName true/false
 */
int
cmd_rjsoec (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_ref_job *job;
  BOOL value;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjsoec RefName TaskName JobName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* find the job */
  job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[3])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[3]);
      return 1;
    }

  /* change the property on_error_continue */
  dg87_err_clear_error ();
  if (dg87_ref_job_set_on_error_continue (job, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the job '%s' "\
															"of the task '%s' in the referential "\
															"'%s' ! %s\n"),
											"on_error_continue",
											job->name, task->name, task->ref->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}


/******************************************************************************/
/* rjdl
 * Ref Job Dependence List
 * Usage : rjdl [options] RefName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -a job_name : only jobdep after this job
 *  -b job_name : only jobdep before this job 
 *  -t : print dependence type
 */
int
cmd_rjdl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "tB:S:E:I:N:b:a:"; */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first job to display */
  int number = 0;               /* number of jobs to display */
  int count = 0;                /* number of job actually display */
  dg87_item *item = NULL;
  dg87_ref_jobdep *job_dep;
  int filter = 0;               /* 0 = no; 1 = before; 2 = after */
  dg87_ref_job *filter_job = NULL;
  char *filter_job_name = NULL;
  dg87_ref_model *ref;
  dg87_ref_task *task = NULL;
  BOOL show_dep_type = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjdl [options] RefName TaskName");
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first job to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of job to display */
          number = atoi (optarg);
          break;
        case 'b': /* only job before */
          filter_job_name = strdup (optarg);
          filter = 1;
          break;
        case 'a': /* only job after */
          filter_job_name = strdup (optarg);
          filter = 2;
          break;
        case 't': /* type of dependence */
          show_dep_type = TRUE;
          break;
        default:  /* unknown option */
          if (filter_job_name != NULL)
            free (filter_job_name);
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the filter job */
  if (filter > 0)
    {
      filter_job = dg87_util_find_ref_job (connection, task, filter_job_name);
      if (filter_job == NULL)
        {
          free (filter_job_name);
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first JobDep to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (task->job_dep_list, start);
  else
    item = task->job_dep_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      job_dep = item->object;
      if (filter == 0 ||
          (filter == 1 && job_dep->downstream_job == filter_job) ||
          (filter == 2 && job_dep->upstream_job == filter_job))
        {
          /* print job before and after */
          dg87_connection_printf (connection, "%s%s%s%s", beg,
                                  job_dep->upstream_job->name, sep,
                                  job_dep->downstream_job->name);
          /* print job dependence type */
          if (show_dep_type == TRUE)
            {
              dg87_connection_printf (connection,
                                      "%s%s", sep,
                                      dg87_ref_jobdep_type_to_ascii (job_dep->
                                                                     type));
            }
          dg87_connection_printf (connection, "%s\n", end);
        }
      item = item->next;
    }
  /* free memory */
  if (filter_job_name != NULL)
    free (filter_job_name);
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rjdc
 * Ref Job Dependence Count
 * Usage : rjdc RefName TaskName
 */
int
cmd_rjdc (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjdc RefName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", task->job_dep_list->count);
  return 0;
}

/******************************************************************************/
/* rjda
 * Ref Job Dependence Add
 * Usage RefName TaskName JobBeforeName JobAfterName [type]
 */
int
cmd_rjda (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_job *upstream_job, *downstream_job;
  dg87_ref_jobdep_type detype;
  dg87_ref_jobdep *jobdep;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 5)
    {
      dg87_connection_printf (connection,
							gettext ("Incorrect call.\nUsage : %s\n"),
							"rjda RefName TaskName JobBeforeName JobAfterName [type]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the referential */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  dg87_err_clear_error ();
  /* find the job before */
  upstream_job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (upstream_job == NULL)
    return 1;

  /* find the job after */
  downstream_job =
    dg87_util_find_ref_job (connection, task, command->argv[3]);
  if (downstream_job == NULL)
    return 1;

  /* interpret the dependence type */
  detype = dg87_ref_jobdep_always;
  if (command->argc == 5)
    {
      detype = dg87_ref_jobdep_ascii_to_type (command->argv[4]);
      if (detype == dg87_ref_jobdep_unknown)
        {
          dg87_connection_printf (connection,
													gettext ("Incorrect dependence type '%s' !\n"),
													command->argv[4]);
          return 1;
        }
    }

  /* add a dependence between jobs */
  dg87_err_clear_error ();
  jobdep =
    dg87_ref_task_jobdep_add (task, upstream_job, downstream_job, detype);
  if (jobdep == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't add a dependence between jobs '%s' "\
                              "and '%s' in the task '%s' in the "\
															"referential '%s' ! %s\n"),
											upstream_job->name, downstream_job->name,
											task->name, task->ref->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rjdk
 * Ref Job Dependencies Kill
 * Usage : rjdk RefName TaskName JobBeforeName JobAfterName
 */
int
cmd_rjdk (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_job *upstream_job, *downstream_job;
  dg87_ref_jobdep *job_dep;
  dg87_item *item;
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_jobdep_type dep_type;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rjdk RefName TaskName JobBeforeName JobAfterName Detype");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find dependence type */
  if ((dep_type = dg87_ref_jobdep_ascii_to_type (command->argv[4])) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect dependence type '%s'\n"),
                              command->argv[4]);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;


  /* find the job before */
  upstream_job = dg87_util_find_ref_job (connection, task, command->argv[2]);
  if (upstream_job == NULL)
    return 1;

  /* find the job after */
  downstream_job =
    dg87_util_find_ref_job (connection, task, command->argv[3]);
  if (downstream_job == NULL)
    return 1;

  /* TODO move this code in a new function dg87_ref_task_jobdep_delete */
  /* find a dependence between jobs and delete it */
  item = task->job_dep_list->first;
  while (item != NULL)
    {
      job_dep = item->object;
      if (job_dep->upstream_job == upstream_job &&
          job_dep->downstream_job == downstream_job
          && job_dep->type == dep_type)
        {
          dg87_err_clear_error ();
          if (dg87_list_remove_item (task->job_dep_list, item) == NULL)
            {
              dg87_connection_printf (connection,
															gettext ("Can't delete the dependence between "\
															"jobs '%s' and '%s' in the task '%s' in the "\
															"referential '%s' ! %s\n"),
															upstream_job->name, downstream_job->name, 
															task->name, task->ref->name,
															dg87_err_get_error_str ());
              return 1;
            }
          /* success */
          dg87_ref_jobdep_delete (job_dep);
          return 0;
        }
      item = item->next;
    }
  /* we don't find the dependence */
  dg87_connection_printf (connection,
                          gettext ("Can't find a dependence between jobs "\
                          "'%s' and '%s' in the task '%s' in the referential "\
													"'%s' with type '%s' !\n"),
                          upstream_job->name, downstream_job->name,
                          task->name, task->ref->name,
                          dg87_ref_jobdep_type_to_ascii (dep_type));
  return 1;
}

/******************************************************************************/
/* End of file */
