/* file d_cmdref.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "d_cmdref.h"
#include "util.h"
#include "context.h"
#include "engine.h"

/******************************************************************************/
/* rk
 * Ref Kill
 * Usage : rk RefName
 */
int
cmd_rk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"rk RefName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* delete the ref */
  dg87_err_clear_error ();
  if (dg87_context_ref_model_delete_by_name (context, command->argv[0]) ==
      FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't delete the referential '%s' ! %s\n"),
											command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rd
 * Ref Duplicate
 * Usage : rd OriginalRefName DuplicateRefName
 */
int
cmd_rd (dg87_connection * connection)
{
  dg87_ref_model *ref, *original_referential;
  dg87_context *context;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rd OriginalRefName DuplicatedRefName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the original ref */
  original_referential =
    dg87_util_find_ref_model (connection, command->argv[0]);
  if (original_referential == NULL)
    return 1;

  /* add the ref */
  dg87_err_clear_error ();
  ref = dg87_context_ref_model_duplicate (context,
																	original_referential, command->argv[1]);
  if (ref == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't duplicate the referential '%s' ! %s\n"),
											command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* ra
 *  Ref Add
 * Usage : ra RefName
 */
int
cmd_ra (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_context *context;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  command = connection->command;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"ra RefName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* add the ref */
  dg87_err_clear_error ();
  ref = dg87_context_ref_model_add (context, command->argv[0]);
  if (ref == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add a ref '%s' ! %s\n"),
                              command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_rc
 * Ref Count
 * return the number of Refs.
 * Usage : rc 
 */
int
cmd_rc (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"rc");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", context->ref_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_rl
 *  Ref List
 * Usage : rl [options] 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int
cmd_rl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first task to display */
  int number = 0; /* number of referentials to display */
  int count = 0;  /* number of referentials actually display */
  dg87_element *element = NULL;
  dg87_ref_task *ref;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              "Incorrect call.\nUsage : %s\n",
                              "rl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first referential to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of referential to display */
          number = atoi (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the first referential to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (context->ref_vector, start);
  else
    element = context->ref_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      ref = element->object;
      /* print referential name */
      dg87_connection_printf (connection, "%s%s", beg, ref->name);
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* ri
 * Ref Information
 * get information for a ref
 * Usage : ri RefName
 */
int
cmd_ri (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_ref_model *ref;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ri [options] RefName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* referential name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Referential Name", sep, 
													ref->name, end);

  /* auto_planned property */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Auto Planned", sep,
                          dg87_bool_to_ascii (ref->auto_planned), end);
  /* auto_purged property */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Auto Purged", sep,
                          dg87_bool_to_ascii (ref->auto_purged), end);
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rp
 * Referential plan
 * This command creates new days from referentials.
 * rp [options] [ReferentialName [date [DayName]]
 * options :
 *  -d : disable day after creation (it must be enabled with xdse)
 */
int
cmd_rp (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_command *command;
  dg87_context *context;
  char *start_time_str;
  char *day_name;
  time_t start_time = 0;
  struct tm dc;
  int optch, iopt;
  char *optarg;
  BOOL enable_day = TRUE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control the argument number */
  if (command->argc > 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rp [options] [ReferentialName [date [DayName]]");
      return 1;
    }
  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'd': /* disable the day after creation */
          enable_day = FALSE;
          break;
        default:  /* unknown option */
          return 1;
        }
    }
  context = connection->server->data;

  if (command->argc == 0)
    {
      /* force auto plan */
      if (dg87_engine_auto_plan (context->engine, TRUE) == FALSE)
        {
          dg87_connection_printf (connection, "%s\n",
                                  context->engine->log->buffer);
          return 1;
        }
      dg87_connection_printf (connection, "%s\n",
                              context->engine->log->buffer);
      return RETVAL_OK;
    }

  /* find the referential */
  if ((ref = dg87_util_find_ref_model (connection, command->argv[0])) == NULL)
    return 1;

  /* find the start time */
  if (command->argc > 1)
    {
      /* get start time from parameter */
      if ((start_time = dg87_ascii_to_date (command->argv[1])) == -1)
        {
          dg87_connection_printf (connection,
													gettext ("Can't plan the referential '%s', because "\
													"the date '%s' is not valid !\n"),
													command->argv[0], command->argv[1]);
          return 1;
        }
    }
  else
    {
      /* start time is the next day at midnight */
      start_time = time (0) + 24 * 3600;
      dc = *localtime (&start_time);
      dc.tm_hour = 0;
      dc.tm_min = 0;
      dc.tm_sec = 0;
      start_time = mktime (&dc);
    }
  if ((start_time_str = dg87_date_to_ascii (start_time)) == NULL)
    return 1;

  /* find the day name */
  if (command->argc == 3)
    {
      if ((day_name = dg87_strdup (command->argv[2], DG87_CHAR)) == NULL)
        {
          dg87_free (start_time_str, DG87_CHAR);
          return 1;
        }
    }
  else
    {
      /* create the day name xxx_yyyymmdd */
      dc = *localtime (&start_time);
      if ((day_name = dg87_malloc (DG87_REF_NAME_LENGTH + 10,
                                   DG87_CHAR)) == NULL)
        {
          dg87_free (start_time_str, DG87_CHAR);
          return 1;
        }
      sprintf (day_name, "%s_%04d%02d%02d", ref->name,
               dc.tm_year + 1900, dc.tm_mon + 1, dc.tm_mday);
    }

  /* plan the ref */
  if (dg87_engine_plan (context->engine, ref, day_name,
                        start_time, enable_day) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't plan the referential '%s' ! %s\n"),
											ref->name, dg87_err_get_error_str ());
      dg87_free (start_time_str, DG87_CHAR);
      dg87_free (day_name, DG87_CHAR);
      return 1;
    }
  dg87_free (start_time_str, DG87_CHAR);
  dg87_free (day_name, DG87_CHAR);
	/* log-buffer must be the second paramater of dg87_connection_printf
	 * because, it contains '\n' which must be replaced by CR+LF */
  dg87_connection_printf (connection, context->engine->log->buffer);
  dg87_connection_printf (connection, "\n");
  return RETVAL_OK;
}

/******************************************************************************/
/* rs
 * Ref Simulate 
 * Simulate a day planification and print the names of the task which will be 
 * planed. The DayName parameter format is YYYY-MM-DD
 * Usage : rs [options] RefName DayName
 */
int
cmd_rs (dg87_connection * connection)
{
  char *sep, *beg, *end;/* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0; /* index of the first task to display */
  int number = 0;/* number of tasks to display */
  int count = -1;
  int index = 0;

  dg87_ref_model *ref;
  dg87_element *task_element;
  dg87_ref_task *task;
  dg87_command *command;
  time_t day;
  BOOL result;
  BOOL show_description = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control the argument number */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rs [options] RefName Date");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* interpret date argument */
  day = dg87_ref_rule_ascii_to_date (command->argv[1]);
  if (day == -1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s' for date argument.\n"),
											command->argv[1]);
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          count = -1;
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        case 'd': /* show description */
          show_description = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* for each task */
  task_element = ref->task_vector->first;
  while (task_element != NULL)
    {
      task = task_element->object;
      task_element = task_element->next;
      result = FALSE;
      if (dg87_ref_task_calculate (task, day, &result) == FALSE)
        {
          dg87_connection_printf (connection,
													gettext ("Error while rule calculation ! %s\n"),
													dg87_err_get_error_str ());
          /* free memory */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }

      /* skip tasks which are not planed */
      if (result == FALSE)
        continue;

      /* skip until first task to display */
      if (count < 0)
        {
          index++;
          if (index <= start)
            continue;
          count = 0;
        }
      /* stop if number of task to display is reached */
      if (count >= 0)
        {
          count++;
          if (number > 0 && count > number)
            break;
        }
      dg87_connection_printf (connection, "%s%s", beg, task->name);
      if (show_description == TRUE)
        dg87_connection_printf (connection, "%s%s", sep, task->description);
      dg87_connection_printf (connection, "%s\n", end);
    }

  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;

}

/******************************************************************************/
/* rsa
 * Ref Set Auto 
 * Usage : rsa RefName AutoPlanned AutoPurge 
 */
int
cmd_rsa (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;
  BOOL auto_planned, auto_purged;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rsa RefName AutoPlanned AutoPurge");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the referential */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* interpret auto_planned parameter */
  if ((auto_planned = dg87_ascii_to_bool (command->argv[1])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[1]);
      return 1;
    }
  /* interpret auto_purged parameter */
  if ((auto_purged = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }
  /* change the auto properties */
  dg87_err_clear_error ();
  if (dg87_ref_model_set_auto (ref, auto_planned, auto_purged) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't change the auto properties of "\
                              	"the referential '%s' ! %s\n"),
                              ref->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rsn
 * Ref Set Name 
 * Usage : rsn RefName NewName 
 */
int
cmd_rsn (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rsn RefName NewName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* change the name */
  dg87_err_clear_error ();
  if (dg87_ref_model_set_name (ref, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the name of "\
															"the referential '%s' ! %s\n"),
											ref->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* End of file */
