/* file d_cmdtask.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "context.h"
#include "d_cmdtask.h"
#include "util.h"
#include "variable.h"

/******************************************************************************/
/* rtk
 * Ref Task Kill
 * Usage : rtk RefName TaskName
 */
int
cmd_rtk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_task *task;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtk RefName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* delete the task */
  dg87_err_clear_error ();
  context = connection->server->data;
  if (dg87_ref_model_task_delete_by_name (ref, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the task '%s' in the "\
																			"referential '%s' ! %s\n"),
                              command->argv[0], ref->name,
															dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rta
 * Ref Task Add
 * Usage : rta RefName TaskName [description]
 */
int
cmd_rta (dg87_connection * connection)
{
  dg87_context *context;
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 2 || command->argc > 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rta RefName TaskName [description]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* add the task */
  dg87_err_clear_error ();
  context = connection->server->data;
  task = dg87_ref_model_task_add (ref, command->argv[1]);
  if (task == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add a task '%s' in the "\
																			"referential '%s' ! %s\n"),
                              command->argv[1], ref->name,
															dg87_err_get_error_str ());
      return 1;
    }
	/* link the task with the system queue (localhost/batch) */
	if (dg87_ref_task_set_queue (task, context->system_queue) == FALSE)
		{
      dg87_connection_printf (connection,
											 gettext ("Can't link the task '%s' of the referential"\
															 "'%s' with the system queue ! %s\n"),
											 task->name, task->ref->name, dg87_err_get_error_str ());
      dg87_ref_model_task_delete (ref, task);
      return 1;
     }

  /* change the description */
  if (command->argc == 3)
    {
      dg87_err_clear_error ();
      if (dg87_ref_task_set_description (task, command->argv[2]) == FALSE)
        {
          dg87_connection_printf (connection,
                                  gettext ("Can't add a task '%s' in the "\
																					"referential '%s' ! %s\n"),
                                  task->name, task->ref->name,
																	dg87_err_get_error_str ());
          dg87_ref_model_task_delete (ref, task);
          return 1;
        }
    }
  return 0;
}

/******************************************************************************/
/* cmd_rtd
 * Ref Task Duplicate 
 * Copy a task 
 * Usage : rtd RefName TaskName [DestRef] NewTaskName
 */
int
cmd_rtd (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *original_ref, *new_referential;
  dg87_ref_task *original_task, *new_task;
  char *new_task_name;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtd RefName TaskName [DestRef] NewTaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the original ref */
  original_ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (original_ref == NULL)
    return 1;
  /* find the original task */
  original_task = dg87_util_find_ref_model_task (connection,
                                                 original_ref,
                                                 command->argv[1]);
  if (original_task == NULL)
    return 1;

  /* find the new referential */
  if (command->argc == 4)
    {
      /* find destination ref */
      new_referential =
        dg87_util_find_ref_model (connection, command->argv[2]);
      if (new_referential == NULL)
        return 1;
      new_task_name = command->argv[3];
    }
  else
    {
      new_referential = original_ref;
      new_task_name = command->argv[2];
    }

  /* verify that new task doesn't already exits */
  if (new_referential == original_ref
      && strcmp (new_task_name, original_task->name) == 0)
    {
      dg87_connection_printf (connection,
											gettext ("Can't duplicate the task '%s' of the "\
															"referential '%s', because the names of "\
															"the original task and the new task "\
															"are the same !\n"),
											new_task_name, original_task->ref->name);
      return 1;
    }
  if (dg87_ref_model_task_get_by_name (new_referential, new_task_name) !=
      NULL)
    {
      dg87_connection_printf (connection,
											gettext ("A task '%s' already exists in the "\
															"referential '%s' !\n"),
											new_task_name, new_referential->name);
      return 1;
    }

  /* duplicate task */
  dg87_err_clear_error ();
  if ((new_task =
       dg87_ref_model_task_duplicate (new_referential, original_task,
                                      new_task_name)) == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't duplicate the task '%s' "\
																			"of the referential '%s' ! %s\n"),
                              original_task, original_task->ref->name,
															dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* cmd_rtc
 * Ref Task Count
 * return the number of Tasks in a referentiel.
 * Usage : rtc RefName
 */
int
cmd_rtc (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"rtc RefName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  dg87_connection_printf (connection, "%i\n", ref->task_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_rtl
 * Ref Task List
 * Usage : rtl [options] RefName
 * options
 * -d : print description of task
 * -t : show target and queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int
cmd_rtl (dg87_connection * connection)
{
  dg87_context *context;
  char *sep, *beg, *end;        /* separator, begin, end */
  /* "dtB:S:E:I:N:" */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first task to display */
  int number = 0;               /* number of TASKs to display */
  int count = 0;                /* number of TASK actually display */
  dg87_element *element = NULL;
  dg87_ref_task *task;
  BOOL show_description = FALSE;
  BOOL show_target = FALSE;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtl [options] RefName");
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");
  show_description = FALSE;

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        case 'd': /* description */
          show_description = TRUE;
          break;
        case 't': /* show target */
          show_target = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first task to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (ref->task_vector, start);
  else
    element = ref->task_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      task = element->object;
      dg87_connection_printf (connection, "%s%s", beg, task->name);
      if (show_target == TRUE)
        dg87_connection_printf (connection,
                                "%s%s%s%s", sep,
                                (task->queue == NULL) ? "none" :
                                task->queue->target->name,
                                sep,
                                (task->queue == NULL) ? "none" :
                                task->queue->name);
      if (show_description == TRUE)
        dg87_connection_printf (connection, "%s%s", sep, task->description);
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* cmd_rts
 * Ref Task Search 
 * Usage : rts [options] referential
 * options
 * -d : print description of task
 * -t : show target and queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int
cmd_rts (dg87_connection * connection)
{
  return 0;
}

/******************************************************************************/
/* rti
 * Ref Task Information 
 * get information for a Task 
 * Usage : rti RefName TaskName
 */
int
cmd_rti (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rti RefName TaskName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* print informations */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Name", sep, 
													task->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Referential Name", sep, 
													task->ref->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Description", sep, 
													task->description, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Enabled", sep,
                          dg87_bool_to_ascii (task->enabled), end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Wait All Dependencies", sep,
                          dg87_bool_to_ascii (task->wait_all_dependencies),
                          end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "On Error Continue", sep,
                          dg87_bool_to_ascii (task->on_error_continue), end);

  /* cyclical properties */
  if (task->iteration == 0)
    {
      dg87_connection_printf (connection, "%s%s%s%s%s\n",
                              beg, "Iterations", sep, 
															"unlimited", end);
    }
  else
    {
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Iterations", sep, 
															task->iteration, end);
    }
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Interval", sep, 
													task->interval, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "From", sep,
                          (task->from_begin == TRUE) ? "begin" : "end", end);

  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Lock Variable", sep,
                          (task->lock_variable == NULL) ? "none"
                          : task->lock_variable->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Lock Action", sep,
                          (task->lock_variable == NULL) ? "none"
                          : (task->lock_action == TRUE) ? "lock" : "unlock",
                          end);

  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Target", sep,
                          (task->queue ==
                           NULL) ? "none" : task->queue->target->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Queue", sep,
                          (task->queue ==
                           NULL) ? "none" : task->queue->name, end);
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rtst
 * Ref Task Set Task 
 * Usage : rtst RefName TaskName TargetName [QueueName]
 */
int
cmd_rtst (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_target *target;
  dg87_queue *queue;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtst RefName TaskName TargetName [QueueName]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the referential and the task */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
	/* find the target and the queue */
	if (command->argc == 3)
		{
  		/* the queue_name is not specified */
  		if (strcmp (command->argv[2], "none") == 0)
    		{
					target = NULL;
					queue = NULL;
				}
			else
    		{
      		target = dg87_util_find_target (connection, command->argv[2]);
      		if (target == NULL)
        		return 1;
					queue = dg87_util_find_queue (connection, target, "batch");
					if (queue == NULL)
						return 1;
    		}
		}
	else
		{
			/* the queue_name argument is specified */
			target = dg87_util_find_target (connection, command->argv[2]);
			if (target == NULL)
				return 1;
			queue = dg87_util_find_queue (connection, target, command->argv[3]);
			if (queue == NULL)
				return 1;
		}

  /* queue can be null if the target name is "none" */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_queue (task, queue) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't link the task '%s' of the referential"\
															"'%s' with the target '%s' (with queue '%s') !"\
															"%s\n"),
											task->name, task->ref->name,
											(target == NULL) ? "none" : target->name,
											(queue == NULL) ? "none" : queue->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}


/******************************************************************************/
/* rtsl
 * Ref Task Set Lock 
 * Usage : rtsl RefName TaskName VariableName [lock/unlock]
 */
int
cmd_rtsl (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_variable *variable;
  BOOL lock_action;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsl RefName TaskName VariableName/none [lock/unlock]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret variable argument */
  if (strcmp (command->argv[2], "none") == 0)
    {
      if (command->argc == 4)
        {
          dg87_connection_printf (connection,
									gettext ("The 'Action' argument is not attempted !\n"));
          return 1;
        }
      variable = NULL;
      lock_action = FALSE;
    }
  else
    {
      if (command->argc == 3)
        {
          dg87_connection_printf (connection,
													gettext ("The 'Action' argument is required !\n"));
          return 1;
        }
      if ((variable = dg87_util_find_variable (connection,
                                               command->argv[2])) == NULL)
        return 1;
      /* interpret action argument */
      if (strcmp (command->argv[3], "lock") == 0)
        {
          lock_action = TRUE;
        }
      else if (strcmp (command->argv[3], "unlock") == 0)
        {
          lock_action = FALSE;
        }
      else
        {
          dg87_connection_printf (connection,
									gettext ("Incorrect value for action argument '%s'!\n"),
									command->argv[3]);
          return 1;
        }
    }

  /* change the variable lock */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_lock (task, variable, lock_action) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the variable "\
															"lock of the task '%s' ! %s\n"),
											task->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtsn
 * Ref Task Set Name 
 * Usage : rtsn RefName TaskName NewName 
 */
int
cmd_rtsn (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsn RefName TaskName NewName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* change the name */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_name (task, command->argv[2]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the name of the task '%s' in the "\
															"referential '%s' ! %s\n"),
											task->name, task->ref->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtsc
 * referential task set cyclical
 * change the cyclical properties of the task 
 * Usage : xtsc RefName TaskName iteration [interval [begin/end]]
 */
int
cmd_rtsc (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;
  dg87uint iteration;
  dg87uint interval;
  BOOL from_begin;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsc RefName TaskName iteration "\
											"[interval [begin/end]]\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  if ((ref = dg87_util_find_ref_model (connection, command->argv[0])) == NULL)
      return 1;
  if ((task = dg87_util_find_ref_model_task (connection, ref,
                                             command->argv[1])) == NULL)
      return 1;

  /* by default, task isn't cyclical */
  iteration = 0;
  interval = 0;
  from_begin = TRUE;

  if (strcmp (command->argv[2], "unlimited") != 0)
    {
      /* interpret iteration argument */
      if ((iteration = atoi (command->argv[2])) <= 0)
        {
          dg87_connection_printf (connection,
													gettext ("Incorrect '%s' value for the iteration " \
																	"argument for the task '%s' of the "\
																	"referential '%s'!\n"),
													command->argv[2], task->name, task->ref->name);
          return 1;
        }
    }
  /* interpret interval argument */
  if (command->argc > 3)
    {
      if ((interval = atoi (command->argv[3])) <= 0)
        {
          dg87_connection_printf (connection,
                                  gettext ("Incorrect '%s' value for the "\
																					"interval argument for the task "\
																					"'%s' of the referential '%s' !\n"),
                                  command->argv[3], 
																	task->name, task->ref->name);
          return 1;
        }
    }
  if (command->argc == 5)
    {
      if (strcmp (command->argv[4], "begin") == 0)
        from_begin = TRUE;
      else if (strcmp (command->argv[4], "end") == 0)
        from_begin = FALSE;
      else
        {
          dg87_connection_printf (connection,
													gettext ("Incorrect value '%s' for the fifth "\
																	"argument for task '%s' of the referential "\
																	"'%s' ! Use 'begin' or 'end' values.\n"),
													command->argv[4], task->name, task->ref->name);
          return 1;
        }
    }

  if (dg87_ref_task_set_cyclical (task, iteration, interval, from_begin) ==
      FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the cyclical properties for the "\
															"task '%s' of the referential '%s' ! %s\n"),
											task->name, task->ref->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* rtsd
 * Ref Task Set Description
 * Usage : rtsd RefName TaskName description
 */
int
cmd_rtsd (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsd RefName TaskName description");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* change the description */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_description (task, command->argv[2]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the description of the task '%s' "\
															"of the referential '%s' ! %s\n"),
											task->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtse
 * Ref Task Set Enable 
 * Usage : rtse RefName TaskName true/false
 */
int
cmd_rtse (dg87_connection * connection)
{
  dg87_ref_task *task;
  BOOL value;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtse RefName TaskName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }

  /* change the property Enabled */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_enabled (task, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the task '%s' "\
															"of the referential '%s' ! %s\n"),
											"enabled", task->name, task->ref->name, 
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtswad
 * Ref Task Set Wait All Dependencies
 * Usage : rtswad RefName TaskName true/false
 */
int
cmd_rtswad (dg87_connection * connection)
{
  dg87_ref_task *task;
  BOOL value;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtswad RefName TaskName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }

  /* change the property wait_all_dependencies */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_wait_all_dependencies (task, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the task '%s' "\
															"of the referential '%s' ! %s\n"),
															"wait_all_dependencies", task->name, 
															task->ref->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtsoec
 * Ref Task Set On Error Continue 
 * Usage : rtsoec RefName TaskName true/false
 */
int
cmd_rtsoec (dg87_connection * connection)
{
  dg87_ref_task *task;
  BOOL value;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsoec RefName TaskName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }

  /* change the property on_error_continue */
  dg87_err_clear_error ();
  if (dg87_ref_task_set_on_error_continue (task, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the task '%s' "\
															"of the referential '%s' ! %s\n"),
															"on_error_continue", task->name, 
															task->ref->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}


/******************************************************************************/
/* rtdl
 * Ref Task Dependence List
 * Usage : rtdl [options] RefName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of taskdep to display (no limit if -N < 1)
 *  -a TaskName : only taskdep after this TASK
 *  -b TaskName : only taskdep before this TASK
 *  -t : print dependence type
 */
int
cmd_rtdl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  /*static char optstring[] = "tB:S:E:I:N:b:a:r"; */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first task to display */
  int number = 0;               /* number of TASKs to display */
  int count = 0;                /* number of TASK actually display */
  dg87_item *item = NULL;
  dg87_ref_taskdep *taskdep;
  dg87_ref_task *filter_task = NULL;
  int filter = 0;               /* 0 = no; 1 = before; 2 = after */
  BOOL show_dep_type = FALSE;
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtdl [options] RefName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;


  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        case 'b': /* only task before */
          filter_task = dg87_ref_model_task_get_by_name (ref, optarg);
          filter = 1;
          break;
        case 'a': /* only task after */
          filter_task = dg87_ref_model_task_get_by_name (ref, optarg);
          filter = 2;
          break;
        case 't': /* type of dependence */
          show_dep_type = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* error if filter and task is null */
  if (filter > 0 && filter_task == NULL)
    {
      dg87_connection_printf (connection, 
											gettext ("Can't find the filter task !\n"));
      free (sep);
      free (beg);
      free (end);
      return 1;
    }

  /* find the first Taskdep to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (ref->taskdep_list, start);
  else
    item = ref->taskdep_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      taskdep = item->object;
      if (filter == 0 ||
          (filter == 1 && taskdep->downstream_task == filter_task) ||
          (filter == 2 && taskdep->upstream_task == filter_task))
        {
          /* print name of TaskBefore and TaskAfter */
          dg87_connection_printf (connection, "%s%s%s%s", beg,
                                  taskdep->upstream_task->name, sep,
                                  taskdep->downstream_task->name);
          if (show_dep_type == TRUE)
            {
              dg87_connection_printf (connection,
                                      "%s%s", sep,
                                      dg87_ref_taskdep_type_to_ascii
                                      (taskdep->type));
            }
          dg87_connection_printf (connection, "%s\n", end);
        }
      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rtdc
 * Ref Task Dependence Count
 * Usage : rtdc RefName
 */
int
cmd_rtdc (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtdc RefName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  dg87_connection_printf (connection, "%i\n", ref->taskdep_list->count);
  return 0;
}

/******************************************************************************/
/* rtda
 * Ref Task Dependence Add
 * Usage : rtdc RefName TaskBeforeName TaskAfterName [type]
 */
int
cmd_rtda (dg87_connection * connection)
{
  dg87_ref_task *upstream_task, *downstream_task;
  dg87_ref_taskdep *taskdep;
  dg87_ref_taskdep_type detype;
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtda RefName TaskBeforeName TaskAfterName [type]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the referential */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task before */
  upstream_task =
    dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (upstream_task == NULL)
    return 1;

  /* find the task after */
  downstream_task =
    dg87_util_find_ref_model_task (connection, ref, command->argv[2]);
  if (downstream_task == NULL)
    return 1;

  /* interpret the dependence type */
  detype = dg87_ref_taskdep_always;
  if (command->argc == 4)
    {
      detype = dg87_ref_taskdep_ascii_to_type (command->argv[3]);
      if (detype == dg87_ref_taskdep_unknown)
        {
          dg87_connection_printf (connection,
													gettext ("Incorrect dependence type '%s' !\n"),
													command->argv[3]);
          return 1;
        }
    }

  /* add a dependence between tasks */
  dg87_err_clear_error ();
  context = connection->server->data;
  taskdep =
    dg87_ref_model_taskdep_add (ref, upstream_task, downstream_task, detype);
  if (taskdep == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add a dependence between "\
                              	"tasks '%s' and '%s' in the referential '%s'"\
																"! %s\n"),
                              upstream_task->name, downstream_task->name,
															ref->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* rtdk
 * Ref Task Dependence Kill
 * Usage : rtdk RefName TaskBeforeName TaskAfterName Type
 */
int
cmd_rtdk (dg87_connection * connection)
{
  dg87_ref_task *upstream_task, *downstream_task;
  dg87_ref_taskdep *taskdep;
  dg87_item *item;
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_taskdep_type dep_type;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtdk RefName TaskBeforeName TaskAfterName Detype");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find dependence type */
  if ((dep_type = dg87_ref_taskdep_ascii_to_type (command->argv[3])) == -1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect dependence type '%s'\n"),
                              command->argv[3]);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task before */
  upstream_task =
    dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (upstream_task == NULL)
    return 1;

  /* find the task after */
  downstream_task =
    dg87_util_find_ref_model_task (connection, ref, command->argv[2]);
  if (downstream_task == NULL)
    return 1;

  /* TODO move this code in a new function dg87_ref_model_taskdep_delete */
  context = connection->server->data;
  item = ref->taskdep_list->first;
  while (item != NULL)
    {

      taskdep = item->object;
      if (taskdep->upstream_task == upstream_task &&
          taskdep->downstream_task == downstream_task
          && taskdep->type == dep_type)
        {
          dg87_err_clear_error ();
          if (dg87_list_remove_item (ref->taskdep_list, item) == NULL)
            {
              dg87_connection_printf (connection,
                                      gettext ("Can't delete the dependence "\
                                      "between task '%s' and '%s' in the "\
																			"referential '%s' ! %s\n"),
                                      upstream_task->name,
                                      downstream_task->name,
                                      dg87_err_get_error_str ());
              return 1;
            }
          /* success */
          dg87_ref_taskdep_delete (taskdep);
          return 0;
        }
      item = item->next;
    }
  /* we don't find the dependence */
  dg87_connection_printf (connection,
                          gettext ("Can't find a dependence between tasks "\
																	"'%s' and '%s' with type '%s' in the "\
																	"referential '%s' !\n"),
                          upstream_task->name, downstream_task->name,
                          dg87_ref_taskdep_type_to_ascii (dep_type), ref->name);
  return 0;
}

/******************************************************************************/
/* rtcl
 * Ref Task Condition List
 * Usage : rtcl [options] RefName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first condition to display (first index is 0)
 *  -Nnumber : number of condition to display (no limit if -N < 1)
 *  -a TaskName : only condition after this TASK
 *  -b TaskName : only condition before this TASK
 */
int
cmd_rtcl (dg87_connection * connection)
{
  char *str_hour;
  char *sep, *beg, *end;/* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first condition to display */
  int number = 0; /* number of conditions to display */
  int count = 0;  /* number of conditions actually display */
  dg87_item *item = NULL;
  dg87_ref_condition *condition;
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;
  dg87ubyte delta;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "rtcl [options] RefName TaskName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first condition to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of condition to display */
          number = atoi (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first condition to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (task->condition_list, start);
  else
    item = task->condition_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      condition = item->object;
      switch (condition->type)
        {
        case condition_hmin:
          delta = condition->data.condition_hmin->day_delta;
          str_hour =
            dg87_ref_condition_minute_to_hour (condition->data.
                                               condition_hmin->minutes);
          dg87_connection_printf (connection, 
													"%s%s%s%s%s+%i%s\n", beg, 
													"hmin", sep, str_hour, sep, delta, end);
          dg87_free (str_hour, DG87_CHAR);
          break;
        case condition_hmax:
          delta = condition->data.condition_hmax->day_delta;
          str_hour =
            dg87_ref_condition_minute_to_hour (condition->data.
                                               condition_hmax->minutes);
          dg87_connection_printf (connection, 
													"%s%s%s%s%s+%i%s\n", beg, 
													"hmax", sep, str_hour, sep, delta, end);
          dg87_free (str_hour, DG87_CHAR);
          break;
        case condition_hlate:
          delta = condition->data.condition_hlate->day_delta;
          str_hour =
            dg87_ref_condition_minute_to_hour (condition->data.
                                               condition_hlate->minutes);
          dg87_connection_printf (connection, 
													"%s%s%s%s%s+%i%s\n", beg, 
													"hlate", sep, str_hour, sep, delta, end);
          dg87_free (str_hour, DG87_CHAR);
          break;
        case condition_file:
          dg87_connection_printf (connection, 
													"%s%s%s%s%s\n", beg, "file", sep, 
													condition->data.condition_File->path, end);
          break;
        case condition_manual:
          dg87_connection_printf (connection, 
													"%s%s%s%s%s\n", beg, "manual", sep, 
													condition->data.condition_manual->message, end);
          break;
        case condition_ifequal:
          dg87_connection_printf (connection, 
													"%s%s%s%s%s%s%s\n", beg, 
													dg87_ref_condition_type_to_ascii(condition->type), 
													sep, 
													condition->data.condition_ifequal-> variable->name, 
													sep, condition->data.condition_ifequal->value, end);
          break;
        case condition_ifnequal:
          dg87_connection_printf (connection, 
													"%s%s%s%s%s%s%s\n", beg, 
													dg87_ref_condition_type_to_ascii(condition->type), 
													sep, 
													condition->data.condition_ifequal->variable->name, 
													sep, condition->data.condition_ifequal->value, end);
          break;

        default:
          dg87_connection_printf (connection,
                                  gettext ("Unknown type of condition !\n"));
          break;
        }
      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rtcc
 * Ref Task Condition Count
 * Usage : rtcc RefName TaskName
 */
int
cmd_rtcc (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtcc RefName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", task->condition_list->count);
  return 0;
}

/******************************************************************************/
/* rtca
 * Ref Task Condition Add
 * Usage : rtca RefName TaskName Type Argument [Argument2]
 * with type = hmin, hmax, hlate, file, manual, if_equal or if_not_equal 
 *
 * - with type=hmin, hmax, hlate, argument is an hour, argument2 is a deltaDay
 * - with type=file, argument is a file string 
 * - with type=manual, argument is a message 
 * - with type=if_equal, if_not_equal, argument are a variable name and a value
 */
int
cmd_rtca (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_ref_condition *condition = NULL;
  dg87_ref_condition_type type;
  int minutes;
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_variable *variable;
  dg87ubyte day_delta;
  dg87ubyte nbarg = 0;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtca RefName TaskName type argument [argument2]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret condition type argument */
  if ((type = dg87_ref_condition_ascii_to_type (command->argv[2])) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect condition type '%s' ! \n"),
                              command->argv[2]);
      return 1;
    }

  /* control number of arguments */
  switch (type)
    {
    case condition_hmin:
    case condition_hmax:
    case condition_hlate:
    case condition_ifequal:
    case condition_ifnequal:
      nbarg = 5;
      break;
    case condition_file:
    case condition_manual:
      nbarg = 4;
      break;
    case condition_error:
      nbarg = 0;                /* just to avoid warnings */
      break;
    }
  if (command->argc != nbarg)
    {
      dg87_connection_printf (connection,
											gettext ("With a '%s' condition, %i parameters "\
															"are required !\n"),
											dg87_ref_condition_type_to_ascii (type), nbarg);
      return 1;
    }

  /* add the condition */
  dg87_err_clear_error ();
  switch (type)
    {
    case condition_hmin:
      minutes = dg87_ref_condition_hour_to_minute (command->argv[3]);
      day_delta = atoi (command->argv[4]);
      condition = dg87_ref_task_condition_hmin_add (task, minutes, day_delta);
      break;
    case condition_hmax:
      minutes = dg87_ref_condition_hour_to_minute (command->argv[3]);
      day_delta = atoi (command->argv[4]);
      condition = dg87_ref_task_condition_hmax_add (task, minutes, day_delta);
      break;
    case condition_hlate:
      minutes = dg87_ref_condition_hour_to_minute (command->argv[3]);
      day_delta = atoi (command->argv[4]);
      condition =
        dg87_ref_task_condition_hlate_add (task, minutes, day_delta);
      break;
    case condition_file:
      condition = dg87_ref_task_condition_file_add (task, command->argv[3]);
      break;
    case condition_manual:
      condition = dg87_ref_task_condition_manual_add (task, command->argv[3]);
      break;
    case condition_ifequal:
      variable = dg87_util_find_variable (connection, command->argv[3]);
      if (variable == NULL)
        return 1;
      condition = dg87_ref_task_condition_ifequal_add (task,
                                                       variable,
                                                       command->argv[4]);
      break;
    case condition_ifnequal:
      variable = dg87_util_find_variable (connection, command->argv[3]);
      if (variable == NULL)
        return 1;
      condition = dg87_ref_task_condition_ifnotequal_add (task,
                                                          variable,
                                                          command->argv[4]);
      break;

    default:
      dg87_connection_printf (connection,
                              gettext ("Incorrect type of condition '%s' !\n"),
                              command->argv[2]);
      return 1;
    }

  if (condition == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add the condition ! %s\n"),
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtck
 * Ref Task Condition Kill 
 * rtck RefName TaskName type Data
 * with type = hmin, hmax, hlate, file or manuel
 * argument is an hour, a file string or a message for manuel condition
 */
int
cmd_rtck (dg87_connection * connection)
{
  dg87_ref_task *task;
  dg87_ref_condition *condition;
  int type;
  dg87_command *command;
  dg87_ref_model *ref;
  char *argument;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtck RefName TaskName typeOfCondition data1 [data2]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;


  /* find the type of condition */
  type = dg87_ref_condition_ascii_to_type (command->argv[2]);
  if (type == condition_error)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect condition type '%s' !\n"),
                              command->argv[1]);
      return 1;
    }

  /* find the condition */
  argument = (command->argc == 5) ? command->argv[4] : NULL;
  if ((condition = dg87_ref_task_condition_find (task,
                                                 type, command->argv[3],
                                                 argument)) == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find this condition in the task "\
																			"'%s' of the referential '%s'\n"), 
															task->name, ref->name);
      return 1;
    }
  /* delete the condition */
  dg87_err_clear_error ();
  if (dg87_ref_task_condition_delete (task, condition) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete this condition !\n"),
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rtsr
 * Ref Task Simulate Range
 * Usage : rts RefName TaskName date1 [date2]
 */
int
cmd_rtsr (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_task *task;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsr RefName TaskName date1 [date2]");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* error message if task is disabled */
  if (task->enabled == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("The task '%s' of the referential '%s' "\
															"is disabled !\n"),
											task->name, ref->name);
      return 1;
    }
  return cmd_operation_rsr (connection, task->rule_operation_list);
}

/******************************************************************************/
/* rtsy
 * Ref Task Simulate Year
 * Usage : rts RefName TaskName Year
 */
int
cmd_rtsy (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_task *task;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtsy RefName TaskName Year");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* error message if task is disabled */
  if (task->enabled == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("The task '%s' of the referential '%s' "\
															"is disabled !\n"),
											task->name, ref->name);
      return 1;
    }
  return cmd_operation_rsy (connection, task->rule_operation_list);
}

/******************************************************************************/
/* rtok
 * Referential Task Operation Kill
 * Usage : rtok RefName TaskName OperationIndex 
 */
int
cmd_rtok (dg87_connection * connection)
{
  int index;
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtok RefName TaskName OperationIndex");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;
  /* find the operation */
  dg87_err_clear_error ();
  index = atoi (command->argv[2]) - 1;
  if (dg87_ref_task_operation_delete (task, index) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the operation of the "\
																			"task '%s' of the referential '%s' ! "\
																			"%s\n"),
                              task->name, ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************
 * cmd_rtoa
 * Ref Task Operation Add
 * Usage : rtoa RefName TaskName operator type [data]
 */
int
cmd_rtoa (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_ref_rule_operation *operation;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 6)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtoa RefName TaskName operator type [data1] [data2]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* create operation */
  operation = cmd_operation_ra (connection, ref);
  if (operation == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add an operation to the "\
																			"task '%s' of the "\
																			"referential '%s' ! %s\n"),
                              task->name, ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }

  /* add operation to the list */
  if (dg87_ref_task_operation_add (task, operation) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add an operation to the "\
																			"task '%s' of the "\
																			"referential '%s' ! %s\n"),
                              task->name, ref->name,
                              dg87_err_get_error_str ());
      dg87_ref_operation_delete (operation);
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* cmd_rtoc
 * Usage : rtoc RefName TaskName
 * Task Operation Count
 * return the number of operations in the task.
 */
int
cmd_rtoc (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtoc RefName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* return operations count */
  dg87_connection_printf (connection, "%i\n",
                          task->rule_operation_list->count);
  return 0;
}

/******************************************************************************/
/* cmd_rtol
 * rtol [options] RefName TaskName
 * Ref Task Operation List
 * Options
 * -i : print index of operation
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first operation to display (first index is 0)
 * -Nnumber : number of operation to display (no limit if -N < 1)
 */
int
cmd_rtol (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtol [options] RefName TaskName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  return cmd_operation_ol (connection, task->rule_operation_list);
}

/******************************************************************************/
/* cmd_rtom
 * Usage : rtom RefName TaskName OperationIndex delta
 * Referential Task Operation Move 
 * Move an operation in the list
 */
int
cmd_rtom (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_task *task;
  dg87_command *command;
  int index, delta;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rtom RefName TaskName OperationIndex Delta");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_ref_model_task (connection, ref, command->argv[1]);
  if (task == NULL)
    return 1;

  /* get index from parameter */
  index = atoi (command->argv[2]) - 1;

	/* interpret the delta parameter */
	if (strcmp(command->argv[3], "up") == 0)
					delta = -1;
	else if (strcmp(command->argv[3], "down") == 0)
					delta = 1;
	else
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s' for the delta parameter "\
															"of the command %s, "\
															"please use 'up' or 'down'.\n"),
											command->argv[3], command->function_name);
      return 1;
    }

  if (dg87_ref_task_operation_move (task, index, delta) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't move an operation in the "
																			"task '%s' of the " \
																			"referential '%s' ! %s\n"),
                              task->name, ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* End of file */
