/* file d_condition.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_CONDITION_H__
#define __DG87_REF_CONDITION_H__

#include "global.h"
#include <libdg87/storage.h>

/* condition types */
typedef enum
{
  condition_error = 0,
  condition_hmin = 1,           /* hour min (can't start before) */
  condition_hmax = 2,           /* hour max (can't start after) */
  condition_hlate = 3,          /* To Late : don't execute the task */
  condition_file = 4,           /* test if file is present */
  condition_manual = 5,         /* must be validate manually */
  condition_ifequal = 6,        /* test a variable is equal to a value */
  condition_ifnequal = 7        /* test a variable is not equal to a value */
}
dg87_ref_condition_type;

/* struct for hMin, hMax and hLate Conditions */
struct dg87_ref_condition_h
{
  int minutes;                  /* number of minutes from 00H00 */
  dg87ubyte day_delta;          /* 0=this day, 1=the next day, 2=two days later... */
};
/* struct for File Condition */
struct dg87_ref_condition_file
{
  char *path;
};
/* struct for Manual Condition */
struct dg87_ref_condition_manual
{
  char *message;                /* mnemonic to identify the condition */
};
/* struct for if_equal and if_not_equal Condition */
struct dg87_ref_condition_eq
{
  dg87_variable *variable;
  char *value;
};

/* Condition Object */
struct _dg87_ref_condition
{
  dg87_ref_condition_type type;
  union
  {
    struct dg87_ref_condition_h *condition_hmin;
    struct dg87_ref_condition_h *condition_hmax;
    struct dg87_ref_condition_h *condition_hlate;
    struct dg87_ref_condition_file *condition_File;
    struct dg87_ref_condition_manual *condition_manual;
    struct dg87_ref_condition_eq *condition_ifequal;
    struct dg87_ref_condition_eq *condition_ifnotequal;
  }
  data;
};

/*
 * dg87_ref_condition_duplicate
 * create a new condition object by copying another
 */
dg87_ref_condition *dg87_ref_condition_duplicate (dg87_ref_condition *
                                                  original_condition);

/*
 * dg87_ref_condition_hmax_new
 * create a condition hMax and return a pointer on this object.
 * The number of minutes from 00H00 must be pass in parameter.
 * The second parameter indicates which day is used :
 * 0=this day, 1=the next day, 2=two days later...
 */
dg87_ref_condition *dg87_ref_condition_hmax_new (int minutes,
                                                 dg87ubyte day_delta);

/*
 * dg87_ref_condition_hmin_new
 * create a condition hMin and return a pointer on this object.
 * The number of minutes from 00H00 must be pass in parameter.
 * The second parameter indicates which day is used :
 * 0=this day, 1=the next day, 2=two days later...
 */
dg87_ref_condition *dg87_ref_condition_hmin_new (int minutes,
                                                 dg87ubyte day_delta);

/*
 * dg87_ref_conditionLate_new
 * create a condition Late and return a pointer on this object.
 * The number of minutes from 00H00 must be pass in parameter.
 * The second parameter indicates which day is used :
 * 0=this day, 1=the next day, 2=two days later...
 */
dg87_ref_condition *dg87_ref_condition_hlate_new (int minutes,
                                                  dg87ubyte day_delta);

/*
 * dg87_ref_condition_file_new
 * create a condition File and return a pointer on this object.
 * The Path of the file must be pass in parameter
 */
dg87_ref_condition *dg87_ref_condition_file_new (char *file_path);

/*
 * dg87_ref_condition_manual_new
 * create a condition Manual and return a pointer on this object.
 * A mnemonic must be pass in parameter to help the user to
 * identify when he must validate this condition. 
 */
dg87_ref_condition *dg87_ref_condition_manual_new (char *message);

/*
 * dg87_ref_condition_ifequal_new
 * create a if_equal condition and return a pointer on this object.
 * The name of the variable and the value which the variable should
 * have must be passed as parameters.
 */
dg87_ref_condition *dg87_ref_condition_ifequal_new (dg87_variable * variable,
                                                    char *value);

/*
 * dg87_ref_condition_ifnotequal_new
 * create a if_not_equal condition and return a pointer on this object.
 * The name of the variable and the value which the variable should
 * have must be passed as parameters.
 */
dg87_ref_condition *dg87_ref_condition_ifnotequal_new (dg87_variable *
                                                       variable, char *value);


/*
 * dg87_ref_condition_delete
 * free memory of the condition object in parameter.
 */
BOOL dg87_ref_condition_delete (dg87_ref_condition * condition);

/*
 * dg87_ref_condition_load
 */
BOOL dg87_ref_condition_load (dg87_ref_condition * condition,
                              dg87_storage * storage);

/*
 * dg87_ref_condition_save
 */
BOOL dg87_ref_condition_save (dg87_ref_condition * condition,
                              dg87_storage * storage);

/*
 * dg87_ref_condition_asciiToCondition 
 * convert a string into a condition type.
 * return -1 if error.
 */
dg87_ref_condition_type dg87_ref_condition_ascii_to_type (char *condition);

/*
 * dg87_ref_condition_condition_to_ascii
 * convert a condition type into a string
 */
char *dg87_ref_condition_type_to_ascii (dg87_ref_condition_type);

/*
 * dg87_ref_condition_hour_to_minute
 * convert a string HH:MM in a number of minutes from midnight
 */
int dg87_ref_condition_hour_to_minute (char *hour);

/*
 * dg87_ref_condition_minute_to_hour
 * convert a number of minutes from midnigh in the format HH:MM
 * return NULL on error
 * the string return must be free with dg87_free
 */
char *dg87_ref_condition_minute_to_hour (int minutes);

#ifdef DEBUG
/*
 * dg87_ref_condition_ckeck
 */
int dg87_ref_condition_check (dg87_ref_condition * condition,
                              int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_CONDITION_H__*/
/* End of file */
