/* file d_job.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "d_job.h"

/****************************************************************************/
dg87_ref_job *
dg87_ref_job_new (dg87_ref_task * task, char *name)
{
  dg87_ref_job *job;
  assert (task != NULL && name != NULL);
  if (task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_JOB_NAME_LENGTH) == FALSE)
    return NULL;
  /* alloc memory */
  if ((job = dg87_malloc (sizeof (dg87_ref_job), DG87_JOB)) == NULL)
    goto alloc_error;
  /* initialize structure */
  dg87_strnzcpy (job->name, name, DG87_JOB_NAME_LENGTH);
  job->task = task;
  job->description[0] = 0;
  job->path = NULL;
  job->argument = NULL;
  job->start_tempo = 0;
  job->end_tempo = 0;
  job->wait_all_dependencies = TRUE;
  job->on_error_continue = TRUE;
	DG87_SAVE_NEEDED;
  return job;
alloc_error:
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  if (job != NULL)
    dg87_ref_job_delete (job);
  return NULL;
}

/****************************************************************************/
dg87_ref_job *
dg87_ref_job_duplicate (dg87_ref_task * new_task, dg87_ref_job * original_job)
{
  dg87_ref_job *new_job;
  assert (new_task != NULL && original_job != NULL);
  if (new_task == NULL || original_job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* create a job with the same name */
  new_job = dg87_ref_job_new (new_task, original_job->name);
  if (new_job == NULL)
    return NULL;
  /* copy job properties */
  dg87_strnzcpy (new_job->description, original_job->description,
                 DG87_JOB_DESC_LENGTH);
  new_job->start_tempo = original_job->start_tempo;
  new_job->end_tempo = original_job->end_tempo;
  new_job->wait_all_dependencies = original_job->wait_all_dependencies;
  new_job->on_error_continue = original_job->on_error_continue;
  /* copy path */
  if (original_job->path != NULL)
    {
      new_job->path = dg87_strdup (original_job->path, DG87_CHAR);
      if (new_job->path == NULL)
        {
          dg87_ref_job_delete (new_job);
          return NULL;
        }
    }
  /* copy argument */
  if (original_job->argument != NULL)
    {
      new_job->argument = dg87_strdup (original_job->argument, DG87_CHAR);
      if (new_job->argument == NULL)
        {
          dg87_ref_job_delete (new_job);
          return NULL;
        }
    }
	DG87_SAVE_NEEDED;
  return new_job;
}

/****************************************************************************/
BOOL
dg87_ref_job_delete (dg87_ref_job * job)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (job->path != NULL)
    dg87_free (job->path, DG87_CHAR);
  if (job->argument != NULL)
    dg87_free (job->argument, DG87_CHAR);
  dg87_free (job, DG87_JOB);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_load (dg87_ref_job * job, dg87_storage * storage)
{
  char *string;
  dg87ubyte b, b2;
  assert (job != NULL && storage != NULL);
  if (job == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read job description */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_ref_job_set_description (job, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);
  /* read job path */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_ref_job_set_path (job, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);
  /* read job arugment */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_ref_job_set_argument (job, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);

  /* read start tempo and end tempo */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_storage_read_ubyte (storage, &b2) == FALSE)
    return FALSE;
  if (dg87_ref_job_set_tempo (job, b, b2) == FALSE)
    return FALSE;

  /* read wait_all_dependencies */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_job_set_wait_all_dependencies (job, b) == FALSE)
    return FALSE;

  /* read on_error_continue */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_job_set_on_error_continue (job, b) == FALSE)
    return FALSE;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_save (dg87_ref_job * job, dg87_storage * storage)
{
  assert (job != NULL && storage != NULL);
  if (job == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write job description */
  if (dg87_storage_write_string (storage, job->description) == FALSE)
    return FALSE;
  /* write job path */
  if (dg87_storage_write_string (storage,
                                 (job->path == NULL) ? "" : job->path) ==
      FALSE)
    return FALSE;
  /* write job argument */
  if (dg87_storage_write_string (storage,
                                 (job->argument ==
                                  NULL) ? "" : job->argument) == FALSE)
    return FALSE;
  /* write start tempo */
  if (dg87_storage_write_ubyte (storage, job->start_tempo) == FALSE)
    return FALSE;
  /* write end tempo */
  if (dg87_storage_write_ubyte (storage, job->end_tempo) == FALSE)
    return FALSE;
  /* write wait_all_dependencies */
  if (dg87_storage_write_ubyte (storage, job->wait_all_dependencies) == FALSE)
    return FALSE;
  /* write on_error_continue */
  if (dg87_storage_write_ubyte (storage, job->on_error_continue) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_name (dg87_ref_job * job, char *new_name)
{
  assert (job != NULL && new_name != NULL);
  if (job == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_name (new_name, DG87_JOB_NAME_LENGTH) == FALSE)
    return FALSE;
  if (dg87_ref_task_job_rename (job->task, job, new_name) == FALSE)
    return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_description (dg87_ref_job * job, char *description)
{
  assert (job != NULL && description != NULL);
  if (job == NULL || description == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_description (description, DG87_JOB_DESC_LENGTH) == FALSE)
    return FALSE;
  dg87_strnzcpy (job->description, description, DG87_JOB_DESC_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_tempo (dg87_ref_job * job, int start_tempo, int end_tempo)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (start_tempo < 0 || start_tempo > JOB_MAX_TEMPO
      || end_tempo < 0 || end_tempo > JOB_MAX_TEMPO)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  job->start_tempo = start_tempo;
  job->end_tempo = end_tempo;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_path (dg87_ref_job * job, char *path)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  if (job->path != NULL)
    dg87_free (job->path, DG87_CHAR);
  job->path = NULL;
  if (path == NULL || strlen (path) == 0)
    return TRUE;

  job->path = dg87_malloc (strlen (path) + 1, DG87_CHAR);
  if (job->path == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  strcpy (job->path, path);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_argument (dg87_ref_job * job, char *argument)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  if (job->argument != NULL)
    dg87_free (job->argument, DG87_CHAR);
  job->argument = NULL;
  if (argument == NULL || strlen (argument) == 0)
    return TRUE;

  job->argument = dg87_malloc (strlen (argument) + 1, DG87_CHAR);
  if (job->argument == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  strcpy (job->argument, argument);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_wait_all_dependencies (dg87_ref_job * job, BOOL value)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job->wait_all_dependencies = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_set_on_error_continue (dg87_ref_job * job, BOOL value)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job->on_error_continue = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_find_upstream_jobs (dg87_ref_job * job, dg87_list * job_list)
{
  dg87_ref_jobdep *job_dep;
  dg87_item *jobdep_item;
  assert (job != NULL && job_list != NULL);
  if (job == NULL || job_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (job_list, job) == FALSE)
    return FALSE;
  jobdep_item = job->task->job_dep_list->first;
  while (jobdep_item != NULL)
    {
      job_dep = jobdep_item->object;
      jobdep_item = jobdep_item->next;
      if (job_dep->downstream_job != job)
        continue;
      if (dg87_ref_job_find_upstream_jobs (job_dep->upstream_job,
                                           job_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_job_find_downstream_jobs (dg87_ref_job * job, dg87_list * job_list)
{
  dg87_ref_jobdep *job_dep;
  dg87_item *jobdep_item;
  assert (job != NULL && job_list != NULL);
  if (job == NULL || job_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (job_list, job) == FALSE)
    return FALSE;
  jobdep_item = job->task->job_dep_list->first;
  while (jobdep_item != NULL)
    {
      job_dep = jobdep_item->object;
      jobdep_item = jobdep_item->next;
      if (job_dep->upstream_job != job)
        continue;
      if (dg87_ref_job_find_downstream_jobs (job_dep->downstream_job,
                                             job_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_ref_job_check (dg87_ref_job * job, int (*print) (char *))
{
  if (strlen (job->name) == 0)
    {
      print ("Job : name length is zero !\n");
      return 1;
    }
  if (job->task == NULL)
    {
      print ("Job : Task pointer is null !\n");
      return 1;
    }
  if (job->start_tempo > JOB_MAX_TEMPO)
    {
      print ("Job : incorrect value for start_tempo !\n");
      return 1;
    }
  if (job->end_tempo > JOB_MAX_TEMPO)
    {
      print ("Job : incorrect value for end_tempo\n");
      return 1;
    }
  /* path and argument could be null -> no test */

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
