/* file d_jobdep.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "d_job.h"
#include "d_jobdep.h"
/*****************************************************************************/
dg87_ref_jobdep *
dg87_ref_jobdep_new (dg87_ref_job * upstream_job,
                     dg87_ref_job * downstream_job, dg87_ref_jobdep_type type)
{
  dg87_ref_jobdep *job_dep;
  dg87_list *upstream_job_list;
  dg87_list *downstream_job_list;
  assert (upstream_job != NULL && downstream_job != NULL);
  if (upstream_job == NULL || downstream_job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* jobs must be in the same task to have a dependence */
  if (upstream_job->task != downstream_job->task)
    {
      dg87_err_set_error (DG87_ERR_JOBS_NOT_IN_SAME_TASK);
      return NULL;
    }

  /* find all upstream jobs */
  upstream_job_list = dg87_list_new ();
  if (upstream_job_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  dg87_ref_job_find_upstream_jobs (upstream_job, upstream_job_list);
  /* find all downstream tasks */
  downstream_job_list = dg87_list_new ();
  if (downstream_job_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  dg87_ref_job_find_downstream_jobs (downstream_job, downstream_job_list);
  /* detect circular dependencies */
  if (dg87_list_intersection (downstream_job_list, upstream_job_list) == TRUE)
    {
      dg87_list_delete (downstream_job_list);
      dg87_list_delete (upstream_job_list);
      dg87_err_set_error (DG87_ERR_CIRCULAR_DEPENDENCE);
      return NULL;
    }
  dg87_list_delete (downstream_job_list);
  dg87_list_delete (upstream_job_list);
  /* allocate memory */
  if ((job_dep = dg87_malloc (sizeof (dg87_ref_jobdep), DG87_JOBDEP)) == NULL)
    goto alloc_error;
  /* initialize structure */
  job_dep->upstream_job = upstream_job;
  job_dep->downstream_job = downstream_job;
  job_dep->type = type;
	DG87_SAVE_NEEDED;
  return job_dep;

alloc_error:
  if (job_dep != NULL)
    dg87_ref_jobdep_delete (job_dep);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/*****************************************************************************/
BOOL
dg87_ref_jobdep_delete (dg87_ref_jobdep * job_dep)
{
  assert (job_dep != NULL);
  if (job_dep == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_free (job_dep, DG87_JOBDEP);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_jobdep_load (dg87_ref_jobdep * job_dep, dg87_storage * storage)
{
  dg87ubyte b;
  assert (job_dep != NULL && storage != NULL);
  if (job_dep == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read dependence type */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_jobdep_set_type (job_dep, b) == FALSE)
    return FALSE;

  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_jobdep_save (dg87_ref_jobdep * job_dep, dg87_storage * storage)
{
  assert (job_dep != NULL && storage != NULL);
  if (job_dep == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write dependence type */
  if (dg87_storage_write_ubyte (storage, job_dep->type) == FALSE)
    return FALSE;
  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_jobdep_set_type (dg87_ref_jobdep * job_dep,
                          dg87_ref_jobdep_type type)
{
  assert (job_dep != NULL);
  if (job_dep == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job_dep->type = type;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/*****************************************************************************/
dg87_ref_jobdep_type
dg87_ref_jobdep_ascii_to_type (char *str_type)
{
  assert (str_type != NULL);
  if (str_type == NULL)
    return dg87_ref_jobdep_unknown;
  if (strcasecmp (str_type, "always") == 0)
    return dg87_ref_jobdep_always;
  else if (strcasecmp (str_type, "onsuccess") == 0)
    return dg87_ref_jobdep_on_success;
  else if (strcasecmp (str_type, "onerror") == 0)
    return dg87_ref_jobdep_on_error;
  return dg87_ref_jobdep_unknown;
}

/*****************************************************************************/
char *
dg87_ref_jobdep_type_to_ascii (dg87_ref_jobdep_type dep_type)
{
  switch (dep_type)
    {
    case dg87_ref_jobdep_always:
      return "always";
    case dg87_ref_jobdep_on_success:
      return "onsuccess";
    case dg87_ref_jobdep_on_error:
      return "onerror";
    default:
      return "unknown";         /* to remove warning */
    }
  return NULL;
}

/*****************************************************************************/
#ifdef DEBUG
int
dg87_ref_jobdep_check (dg87_ref_jobdep * jobdep, int (*print) (char *))
{
  if (jobdep->upstream_job == NULL)
    {
      print ("JobDep : JobBefore is NULL !\n");
      return 1;
    }
  if (jobdep->downstream_job == NULL)
    {
      print ("JobDep : JobAfter is NULL !\n");
      return 1;
    }
  return 0;
}
#endif /*DEBUG*/
/*****************************************************************************/
/* End of file */
