/* file d_ref.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <libdg87/global.h>
#include <libdg87/element.h>
#include <libdg87/connection.h>
#include <libdg87/dgerr.h>
#include "d_ref.h"
#include "global.h"
#include "context.h"

/****************************************************************************/
dg87_ref_model *
dg87_ref_model_new (char *name, dg87_context * context)
{
  dg87_ref_model *ref;
  assert (name != NULL && context != NULL);
  if (name == NULL || context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_REF_NAME_LENGTH) == FALSE)
    return NULL;

  /* allocate memory for ref */
  if ((ref = dg87_malloc (sizeof (dg87_ref_model), DG87_REF)) == NULL)
    goto alloc_error;
  if ((ref->task_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((ref->taskdep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((ref->rule_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  /* initialize the structure of the object */
  dg87_strnzcpy (ref->name, name, DG87_REF_NAME_LENGTH);
  ref->name[DG87_REF_NAME_LENGTH] = '\0';
  ref->auto_planned = FALSE;
  ref->auto_purged = FALSE;
  ref->context = context;
	DG87_SAVE_NEEDED;
  return ref;

alloc_error:
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  if (ref != NULL)
    dg87_ref_model_delete (ref);
  return NULL;

}

/****************************************************************************/
dg87_ref_model *
dg87_ref_model_duplicate (dg87_ref_model * original_referential,
                          char *new_referential_name)
{
  dg87_ref_model *new_referential;
  dg87_element *element, *element2;
  dg87_item *item;
  dg87_ref_task *original_task, *new_task;
  dg87_ref_task *upstream_task, *downstream_task;
  dg87_ref_rule *original_rule, *new_rule;
  dg87_ref_rule_operation *original_operation, *new_operations;
  dg87_ref_taskdep *new_taskdep, *original_taskdep;
  assert (original_referential != NULL && new_referential_name != NULL);
  if (original_referential == NULL || new_referential_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((new_referential =
       dg87_ref_model_new (new_referential_name,
                           original_referential->context)) == NULL)
    return NULL;
  /* copy all rules */
  /* first step is to copy only rules (without operations) */
  element = original_referential->rule_vector->first;
  while (element != NULL)
    {
      original_rule = element->object;
      element = element->next;
      /* duplicate rule */
      new_rule = dg87_ref_rule_duplicate (original_rule);
      if (new_rule == NULL)
        {
          dg87_ref_model_delete (new_referential);
          return NULL;
        }
      /* add it to rule vector of the new ref */
      if (dg87_vector_add_object (new_referential->rule_vector,
                                  new_rule, new_rule->name) == FALSE)
        {
          dg87_ref_rule_delete (new_rule);
          dg87_ref_model_delete (new_referential);
          return NULL;
        }
    }
  /* second step is to copy each operation of each rule */
  element = original_referential->rule_vector->first;
  element2 = new_referential->rule_vector->first;
  while (element != NULL && element2 != NULL)
    {
      original_rule = element->object;
      element = element->next;
      new_rule = element2->object;
      element2 = element2->next;
      /* for each operation of the rule */
      item = original_rule->operation_list->first;
      while (item != NULL)
        {
          original_operation = item->object;
          item = item->next;
          /* duplicate operation */
          new_operations = dg87_ref_operation_duplicate (original_operation,
                                                         new_referential->
                                                         rule_vector);
          if (new_operations == NULL)
            {
              dg87_ref_model_delete (new_referential);
              return NULL;
            }
          /* add it to the operation list of the rule */
          if (dg87_ref_rule_operation_add (new_rule, new_operations) == FALSE)
            {
              dg87_ref_model_delete (new_referential);
              return NULL;
            }
        }
    }

  /* copy all tasks */
  element = original_referential->task_vector->first;
  while (element != NULL)
    {
      original_task = element->object;
      element = element->next;
      /* duplicate task */
      new_task = dg87_ref_task_duplicate (original_task,
                                          new_referential->rule_vector, NULL);
      if (new_task == NULL)
        {
          dg87_ref_model_delete (new_referential);
          return NULL;
        }
      /* add it to task vector of the new ref */
      if (dg87_vector_add_object (new_referential->task_vector,
                                  new_task, new_task->name) == FALSE)
        {
          dg87_ref_task_delete (new_task);
          dg87_ref_model_delete (new_referential);
          return NULL;
        }
      new_task->ref = new_referential;

    }
  /* copy task_dep_list */
  item = original_referential->taskdep_list->first;
  while (item != NULL)
    {
      original_taskdep = item->object;
      item = item->next;
      upstream_task =
        dg87_vector_get_object_by_key (new_referential->task_vector,
                                       original_taskdep->upstream_task->name);
      downstream_task =
        dg87_vector_get_object_by_key (new_referential->task_vector,
                                       original_taskdep->downstream_task->
                                       name);
      if (upstream_task == NULL || downstream_task == NULL)
        {
          dg87_ref_model_delete (new_referential);
          return NULL;
        }
      if ((new_taskdep =
           dg87_ref_model_taskdep_add (new_referential, upstream_task,
                                       downstream_task,
                                       original_taskdep->type)) == NULL)
        {
          dg87_ref_model_delete (new_referential);
          return NULL;
        }
    }
	DG87_SAVE_NEEDED;
  return new_referential;
}

/****************************************************************************/
BOOL
dg87_ref_model_delete (dg87_ref_model * ref)
{
  dg87_ref_task *task;
  dg87_ref_taskdep *taskdep;
  dg87_ref_rule *rule;
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete all taskdep objects */
  if (ref->taskdep_list != NULL)
    {
      while (ref->taskdep_list->count > 0)
        {
          taskdep = dg87_list_remove_object_by_index (ref->taskdep_list, 0);
          dg87_ref_taskdep_delete (taskdep);
        }
      dg87_list_delete (ref->taskdep_list);
    }

  /* delete all task object */
  if (ref->task_vector != NULL)
    {
      while (ref->task_vector->count > 0)
        {
          task = dg87_vector_remove_object_by_index (ref->task_vector, 0);
          dg87_list_clear (task->upstream_taskdep_list);
          dg87_list_clear (task->downstream_taskdep_list);
          dg87_ref_task_delete (task);
        }
      dg87_vector_delete (ref->task_vector);
    }

  /* delete all rule object */
  if (ref->rule_vector != NULL)
    {
      while (ref->rule_vector->count > 0)
        {
          rule = dg87_vector_remove_object_by_index (ref->rule_vector, 0);
          rule->linked_rules = 0;       /* force rule deletion */
          dg87_ref_rule_delete (rule);
        }
      dg87_vector_delete (ref->rule_vector);
    }

  dg87_free (ref, DG87_REF);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_model_set_auto (dg87_ref_model * ref, BOOL auto_planned,
                         BOOL auto_purged)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  ref->auto_planned = auto_planned;
  ref->auto_purged = auto_purged;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_model_set_name (dg87_ref_model * ref, char *new_name)
{
  assert (ref != NULL && new_name != NULL);
  if (ref == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_name (new_name, DG87_REF_NAME_LENGTH) == FALSE)
    return FALSE;
  if (dg87_context_ref_model_rename (ref->context, ref, new_name) == FALSE)
    return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_model_load (dg87_ref_model * ref, dg87_storage * storage)
{
  dg87ubyte auto_planned, auto_purged;
  if (dg87_storage_read_ubyte (storage, &auto_planned) == FALSE)
    return FALSE;
  if (dg87_storage_read_ubyte (storage, &auto_purged) == FALSE)
    return FALSE;
  if (dg87_ref_model_set_auto (ref, auto_planned, auto_purged) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_model_save (dg87_ref_model * ref, dg87_storage * storage)
{
  /* write auto_planned property */
  if (dg87_storage_write_ubyte (storage, ref->auto_planned) == FALSE)
    return FALSE;
  /* write auto_purged property */
  if (dg87_storage_write_ubyte (storage, ref->auto_purged) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
dg87_ref_task *
dg87_ref_model_task_add (dg87_ref_model * ref, char *task_name)
{
  dg87_ref_task *task = NULL;
  assert (ref != NULL && task_name != NULL);
  if (ref == NULL || task_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  task = dg87_ref_task_new (task_name);
  if (task == NULL)
    return NULL;

  /* add the task in the task list */
  /* vector generate error if an other task has not the same name */
  /* key must point on task->name */
  if (dg87_vector_add_object (ref->task_vector, task, task->name) == FALSE)
    {
      dg87_ref_task_delete (task);
      return NULL;
    }
  task->ref = ref;

	DG87_SAVE_NEEDED;
  return task;
}

/****************************************************************************/
dg87_ref_task *
dg87_ref_model_task_duplicate (dg87_ref_model * ref,
                               dg87_ref_task * original_task,
                               char *new_task_name)
{
  dg87_ref_task *new_task = NULL;
  assert (ref != NULL && original_task != NULL && new_task_name != NULL);
  if (ref == NULL || original_task == NULL || new_task_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  new_task = dg87_ref_task_duplicate (original_task, ref->rule_vector,
                                      new_task_name);
  if (new_task == NULL)
    return NULL;

  /* add the task in the task list */
  /* vector generate error if an other task has not the same name */
  /* key must point on task->name */
  if (dg87_vector_add_object (ref->task_vector,
                              new_task, new_task->name) == FALSE)
    {
      dg87_ref_task_delete (new_task);
      return NULL;
    }
  new_task->ref = ref;

	DG87_SAVE_NEEDED;
  return new_task;
}

/****************************************************************************/
BOOL
dg87_ref_model_task_rename (dg87_ref_model * ref, dg87_ref_task * task,
                            char *new_name)
{
  char old_name[DG87_TASK_NAME_LENGTH + 1];
  assert (ref != NULL && task != NULL && new_name != NULL);
  if (ref == NULL || task == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  if (strlen (new_name) >= DG87_TASK_NAME_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  /* remove object from vector */
  if (dg87_vector_remove_object (ref->task_vector, task) == FALSE)
    return FALSE;
  /* copy new name */
  dg87_strnzcpy (old_name, task->name, DG87_TASK_NAME_LENGTH);
  dg87_strnzcpy (task->name, new_name, DG87_TASK_NAME_LENGTH);
  /* add the object in the vector with the new key */
  if (dg87_vector_add_object (ref->task_vector, task, task->name) == FALSE)
    {
      dg87_strnzcpy (task->name, old_name, DG87_TASK_NAME_LENGTH);
      dg87_vector_add_object (ref->task_vector, task, task->name);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
int
dg87_ref_model_task_count (dg87_ref_model * ref)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (ref->task_vector);
}

/****************************************************************************/
dg87_ref_task *
dg87_ref_model_task_get_by_name (dg87_ref_model * ref, char *name)
{
  dg87_element *element;
  assert (ref != NULL && name != NULL);
  if (ref == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }

  /* browse the list to find an item with the same name */
  element = ref->task_vector->first;
  while (element != NULL)
    {
      if (strcmp (element->key, name) == 0)
        return element->object;
      element = element->next;
    }
  dg87_err_set_error (DG87_ERR_CAN_NOT_FIND_OBJECT);
  return NULL;
}

/****************************************************************************/
dg87_ref_task *
dg87_ref_model_task_get_by_index (dg87_ref_model * ref, int index)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (ref->task_vector, index);
}

/****************************************************************************/
BOOL
dg87_ref_model_task_delete (dg87_ref_model * ref, dg87_ref_task * task)
{
  assert (ref != NULL && task != NULL);
  if (ref == NULL || task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_vector_remove_object (ref->task_vector, task) == FALSE)
    return FALSE;
  if (dg87_ref_task_delete (task) == FALSE)
    {
      /* add task in vector again */
      dg87_vector_add_object (ref->task_vector, task, task->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_model_task_delete_by_name (dg87_ref_model * ref, char *name)
{
  dg87_ref_task *task;
  assert (ref != NULL && name != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task = dg87_vector_get_object_by_key (ref->task_vector, name);
  if (task == NULL)
    return FALSE;
  return dg87_ref_model_task_delete (ref, task);
}

/****************************************************************************/
BOOL
dg87_ref_model_task_delete_by_index (dg87_ref_model * ref, int index)
{
  dg87_ref_task *task;
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task = dg87_vector_get_object_by_index (ref->task_vector, index);
  if (task == NULL)
    return FALSE;
  return dg87_ref_model_task_delete (ref, task);
}

/****************************************************************************/
dg87_ref_taskdep *
dg87_ref_model_taskdep_add (dg87_ref_model * ref,
                            dg87_ref_task * upstream_task,
                            dg87_ref_task * downstream_task,
                            dg87_ref_taskdep_type type)
{
  dg87_ref_taskdep *taskdep;
  assert (ref != NULL && upstream_task != NULL && downstream_task != NULL);
  if (ref == NULL || upstream_task == NULL || downstream_task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* create and add the taskdep */
  if ((taskdep =
       dg87_ref_taskdep_new (upstream_task, downstream_task, type)) == NULL)
    return NULL;

  dg87_list_add_object (ref->taskdep_list, taskdep);
  dg87_list_add_object (upstream_task->downstream_taskdep_list, taskdep);
  dg87_list_add_object (downstream_task->upstream_taskdep_list, taskdep);

  return taskdep;
}

/****************************************************************************/
int
dg87_ref_model_taskdep_count (dg87_ref_model * ref)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (ref->taskdep_list);
}

/****************************************************************************/
dg87_ref_taskdep *
dg87_ref_model_taskdep_get (dg87_ref_model * ref, int index)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (ref->taskdep_list, index);
}

/****************************************************************************/
BOOL
dg87_ref_model_taskdep_delete (dg87_ref_model * ref, int index)
{
  dg87_ref_taskdep *taskdep;
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  taskdep = dg87_list_remove_object_by_index (ref->taskdep_list, index);
  if (taskdep == NULL)
    return FALSE;
  dg87_ref_taskdep_delete (taskdep);
  return TRUE;
}

/****************************************************************************/
dg87_ref_rule *
dg87_ref_model_rule_add (dg87_ref_model * ref, char *rule_name)
{
  dg87_ref_rule *rule = NULL;
  assert (ref != NULL && rule_name != NULL);
  if (ref == NULL || rule_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((rule = dg87_ref_rule_new (rule_name, ref)) == NULL)
    return NULL;

  /* add the rule in the rule vector */
  /* vector generate error if an other rule has not the same name */
  /* key must point on rule->name */
  if (dg87_vector_add_object (ref->rule_vector, rule, rule->name) == FALSE)
    {
      dg87_ref_rule_delete (rule);
      return NULL;
    }
  return rule;
}

/****************************************************************************/
BOOL
dg87_ref_model_rule_rename (dg87_ref_model * ref, dg87_ref_rule * rule,
                            char *new_name)
{
  char old_name[DG87_RULE_NAME_LENGTH + 1];
  assert (ref != NULL && rule != NULL && new_name != NULL);
  if (ref == NULL || rule == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  if (strlen (new_name) >= DG87_RULE_NAME_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  /* remove object from vector */
  if (dg87_vector_remove_object (ref->rule_vector, rule) == FALSE)
    return FALSE;
  /* copy new name */
  dg87_strnzcpy (old_name, rule->name, DG87_RULE_NAME_LENGTH);
  dg87_strnzcpy (rule->name, new_name, DG87_RULE_NAME_LENGTH);
  /* add the object in the vector with the new key */
  if (dg87_vector_add_object (ref->rule_vector, rule, rule->name) == FALSE)
    {
      dg87_strnzcpy (rule->name, old_name, DG87_RULE_NAME_LENGTH);
      dg87_vector_add_object (ref->rule_vector, rule, rule->name);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
int
dg87_ref_model_rule_count (dg87_ref_model * ref)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (ref->rule_vector);
}

/****************************************************************************/
dg87_ref_rule *
dg87_ref_model_rule_get_by_name (dg87_ref_model * ref, char *name)
{
  dg87_element *element;
  assert (ref != NULL && name != NULL);
  if (ref == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }

  /* browse the list to find an item with the same name */
  element = ref->rule_vector->first;
  while (element != NULL)
    {
      if (strcmp (element->key, name) == 0)
        return element->object;
      element = element->next;
    }
  dg87_err_set_error (DG87_ERR_CAN_NOT_FIND_OBJECT);
  return NULL;
}

/****************************************************************************/
dg87_ref_rule *
dg87_ref_model_rule_get_by_index (dg87_ref_model * ref, int index)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (ref->rule_vector, index);
}

/****************************************************************************/
BOOL
dg87_ref_model_rule_delete_by_name (dg87_ref_model * ref, char *name)
{
  dg87_ref_rule *rule;
  assert (ref != NULL && name != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  rule = dg87_vector_remove_object_by_key (ref->rule_vector, name);
  if (rule == NULL)
    return FALSE;
  if (dg87_ref_rule_delete (rule) == FALSE)
    {
      /* add the rule in the vector again */
      dg87_vector_add_object (ref->rule_vector, rule, rule->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_model_rule_delete_by_index (dg87_ref_model * ref, int index)
{
  dg87_ref_rule *rule;
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  rule = dg87_vector_remove_object_by_index (ref->rule_vector, index);
  if (rule == NULL)
    return FALSE;
  dg87_ref_rule_delete (rule);
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_ref_model_check (dg87_ref_model * ref, int (*print) (char *))
{
  dg87_element *element;
  dg87_item *item;
  dg87_ref_taskdep *taskdep;

  /* check task_vector */
  if (ref->task_vector == NULL)
    {
      print ("Ref : task_vector is null !\n");
      return 1;
    }
  if (dg87_vector_check (ref->task_vector, print) != 0)
    return 1;
  element = ref->task_vector->first;
  while (element != NULL)
    {
      if (dg87_ref_task_check (element->object, print) != 0)
        return 1;
      element = element->next;
    }

  /* check taskdep list */
  if (ref->taskdep_list == NULL)
    {
      print ("Ref : taskdep list is null !\n");
      return 1;
    }
  if (dg87_list_check (ref->taskdep_list, print) != 0)
    return 1;
  item = ref->taskdep_list->first;
  while (item != NULL)
    {
      taskdep = item->object;
      if (dg87_ref_taskdep_check (taskdep, print) != 0)
        return 1;
      item = item->next;
    }
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
