/* file d_ref.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_MODEL_H__
#define __DG87_REF_MODEL_H__

#include <libdg87/vector.h>
#include "d_task.h"
#include "d_taskdep.h"
#include "d_rule.h"


struct _dg87_ref_model
{
  char name[DG87_REF_NAME_LENGTH + 1];
  dg87_context *context;
  BOOL auto_planned;
  BOOL auto_purged;
  dg87_vector *task_vector;
  dg87_list *taskdep_list;
  dg87_vector *rule_vector;
};

/*
 * dg87_ref_model_new
 * create a new ref and return a pointer on this.
 */
dg87_ref_model *dg87_ref_model_new (char *name, dg87_context * context);

/*
 * dg87_ref_model_delete
 */
BOOL dg87_ref_model_delete (dg87_ref_model * ref);

/*
 * dg87_ref_model_load
 */
BOOL dg87_ref_model_load (dg87_ref_model * ref, dg87_storage * storage);

/*
 * dg87_ref_model_save
 */
BOOL dg87_ref_model_save (dg87_ref_model * ref, dg87_storage * storage);

/*
 * dg87_ref_model_duplicate
 */
dg87_ref_model *dg87_ref_model_duplicate (dg87_ref_model *
                                          original_referential,
                                          char *new_referential_name);

/*
 * dg87_ref_model_set_name
 * Change the name of a referential.
 * An error occurres if another referential has the same name in the context.
 */
BOOL dg87_ref_model_set_name (dg87_ref_model * ref, char *name);

/*
 * dg87_ref_model_set_auto
 * Change the auto_planned and the auto_purge properties of a referential.
 */
BOOL dg87_ref_model_set_auto (dg87_ref_model * ref, BOOL auto_planned,
                              BOOL auto_purged);

/*************************************************************************/
/*  functions for tasks                                                  */
/*************************************************************************/

/*
 * dg87_ref_model_task_add
 * create an TASK object, add this object in the TaskList
 * of the ref and return a pointer on this object.
 * Return NULL if the TASK object can't be create.
 */
dg87_ref_task *dg87_ref_model_task_add (dg87_ref_model * ref,
                                        char *task_name);

/*
 * dg87_ref_model_task_duplicate
 */
dg87_ref_task *dg87_ref_model_task_duplicate (dg87_ref_model * ref,
                                              dg87_ref_task * original_task,
                                              char *new_task_name);

/*
 * dg87_ref_model_task_rename
 * You don't have to use this function. Use dg87_ref_task_set_name instead.
 * This function is called by dg87_ref_task_set_name to change the name of a task.
 * It returns FALSE if another task as the same name.
 */
BOOL dg87_ref_model_task_rename (dg87_ref_model * ref, dg87_ref_task * task,
                                 char *new_name);

/*
 * dg87_ref_model_task_count
 * return the number of TASK.
 * return -1 if ref is NULL.
 */
int dg87_ref_model_task_count (dg87_ref_model * ref);

/*
 * dg87_ref_model_task_get_by_name
 * return a pointer on the TASK which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_ref_task *dg87_ref_model_task_get_by_name (dg87_ref_model * ref,
                                                char *name);

/*
 * dg87_ref_task_get_by_index
 * return a pointer on the TASK at the given position.
 */
dg87_ref_task *dg87_ref_model_task_get_by_index (dg87_ref_model * ref,
                                                 int index);

/*
 * dg87_ref_model_task_delete_by_name
 * delete the task which has the name given in parameter.
 */
BOOL dg87_ref_model_task_delete_by_name (dg87_ref_model * ref, char *name);

/*
 * dg87_ref_model_task_delete_by_index
 * delete the task at the given index.
 */
BOOL dg87_ref_model_task_delete_by_index (dg87_ref_model * ref, int index);

/*
 * dg87_ref_model_task_delete
 */
BOOL dg87_ref_model_task_delete (dg87_ref_model * ref, dg87_ref_task * task);

/*************************************************************************/
/*  functions for taskdep                                                  */
/*************************************************************************/

/*
 * dg87_ref_model_taskdep_add
 * add a dependence between two tasks of a referential . The name of the tasks 
 * must be pass as parameters.
 */
dg87_ref_taskdep *dg87_ref_model_taskdep_add (dg87_ref_model * ref,
                                              dg87_ref_task * upstream_task,
                                              dg87_ref_task * downstream_task,
                                              dg87_ref_taskdep_type type);

/*
 * dg87_ref_model_taskdep_count
 * return the number of taskdep the ref.
 * return -1 if ref is NULL.
 */
int dg87_ref_model_taskdep_count (dg87_ref_model * ref);

/*
 * dg87_ref_model_taskdep_get
 * return a pointer on the taskdep at the given index.
 */
dg87_ref_taskdep *dg87_ref_model_taskdep_get (dg87_ref_model * ref,
                                              int index);

/*
 * dg87_ref_model_taskdep_delete
 * delete the taskdep at the given index.
 */
BOOL dg87_ref_model_taskdep_delete (dg87_ref_model * ref, int index);

/*************************************************************************/
/*  functions for rules                                                  */
/*************************************************************************/

/*
 * dg87_ref_model_rule_add
 * create an rule object, add this object in the rulevector
 * of the ref and return a pointer on this object.
 * Return NULL if the rule object can't be create.
 */
dg87_ref_rule *dg87_ref_model_rule_add (dg87_ref_model * ref,
                                        char *rule_name);

/*
 * dg87_ref_model_rule_rename
 * You don't have to use this function. Use dg87_ref_rule_set_name instead.
 * This function is called by dg87_ref_rule_set_name to change the name of a rule.
 * It returns FALSE if another rule as the same name.
 */
BOOL dg87_ref_model_rule_rename (dg87_ref_model * ref, dg87_ref_rule * rule,
                                 char *new_name);

/*
 * dg87_ref_model_rule_count
 * return the number of rule.
 * return -1 if ref is NULL.
 */
int dg87_ref_model_RuleCount (dg87_ref_model * ref);

/*
 * dg87_ref_model_rule_get_by_name
 * return a pointer on the rule which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_ref_rule *dg87_ref_model_rule_get_by_name (dg87_ref_model * ref,
                                                char *name);

/*
 * dg87_ref_rule_get_by_index
 * return a pointer on the rule at the given position.
 */
dg87_ref_rule *dg87_ref_model_rule_get_by_index (dg87_ref_model * ref,
                                                 int index);

/*
 * dg87_ref_model_rule_delete_by_name
 * delete the rule which has the name given in parameter.
 */
BOOL dg87_ref_model_rule_delete_by_name (dg87_ref_model * ref, char *name);

/*
 * dg87_ref_model_rule_delete_by_index
 * delete the rule at the given index.
 */
BOOL dg87_ref_model_rule_delete_by_index (dg87_ref_model * ref, int index);

#ifdef DEBUG
/*
 * dg87_ref_model_ckeck
 */
int dg87_ref_model_check (dg87_ref_model * ref, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_MODEL_H__*/
/* End of file */
