/* file d_rule.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "global.h"
#include "variable.h"
#include <libdg87/dgerr.h>
#include "d_rule.h"
#include "d_ref.h"
#include <libdg87/element.h>
#include <libdg87/connection.h>
#include "global.h"

/****************************************************************************/
dg87_ref_rule *
dg87_ref_rule_new (char *name, dg87_ref_model * ref)
{
  dg87_ref_rule *rule;
  assert (name != NULL && ref != NULL);
  if (name == NULL || ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_RULE_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((rule = dg87_malloc (sizeof (dg87_ref_rule), DG87_RULE)) == NULL)
    goto alloc_error;
  if ((rule->operation_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  /* initialize structure */
  dg87_strnzcpy (rule->name, name, DG87_RULE_NAME_LENGTH);
  rule->linked_rules = 0;
  rule->linked_tasks = 0;
  rule->ref = ref;
	DG87_SAVE_NEEDED;
  return rule;

alloc_error:
  if (rule != NULL)
    dg87_ref_rule_delete (rule);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_rule *
dg87_ref_rule_duplicate (dg87_ref_rule * original_rule)
{
  dg87_ref_rule *new_rule;
  assert (original_rule != NULL);
  if (original_rule == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* create a rule with the same name */
  new_rule = dg87_ref_rule_new (original_rule->name, original_rule->ref);
  if (new_rule == NULL)
    return NULL;
	DG87_SAVE_NEEDED;
  return new_rule;
}

/****************************************************************************/
BOOL
dg87_ref_rule_delete (dg87_ref_rule * rule)
{
  assert (rule != NULL);
  if (rule == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* verify that rule isn't used by another rule or by a task */
  if (rule->linked_rules > 0)
    {
      dg87_err_set_error (DG87_ERR_RULE_USED_BY_ANOTER_RULE);
      return FALSE;
    }
  if (rule->linked_tasks > 0)
    {
      dg87_err_set_error (DG87_ERR_RULE_USED_BY_TASK);
      return FALSE;
    }
  /* delete operation */
  if (rule->operation_list != NULL)
    {
      while (rule->operation_list->count > 0)
        {
          if (dg87_ref_rule_operation_delete (rule, 0) == FALSE)
            return FALSE;
        }
      dg87_list_delete (rule->operation_list);
    }

  dg87_free (rule, DG87_RULE);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_rule_set_name (dg87_ref_rule * rule, char *new_name)
{
  assert (rule != NULL && new_name != NULL);
  if (rule == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_name (new_name, DG87_RULE_NAME_LENGTH) == FALSE)
    return FALSE;
  if (dg87_ref_model_rule_rename (rule->ref, rule, new_name) == FALSE)
    return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_rule_load (dg87_ref_rule * rule, dg87_storage * storage)
{
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_rule_save (dg87_ref_rule * rule, dg87_storage * storage)
{
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_dom_calculate
 * This function calculates day_of_month operation.
 */
BOOL
dg87_ref_operation_dom_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  time_t limit;
  struct tm dc;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  if (operation->rule.dom.day_of_month > 0)
    {
      /* find first day of the month */
      dc.tm_mday = 1;
      limit = mktime (&dc);
      /* calculate limit date */
      limit += (operation->rule.dom.day_of_month - 1) * (24 * 3600);
    }
  else
    {
      /* find first day of next month */
      if (dc.tm_mon == 11)
        {
          /* december : next month is january of next year */
          dc.tm_year += 1;
          dc.tm_mon = 0;
          dc.tm_mday = 1;
        }
      else
        {
          dc.tm_mon += 1;
          dc.tm_mday = 1;
        }
      limit = mktime (&dc);
      /* calculate limit date */
      limit += operation->rule.dom.day_of_month * (24 * 3600);

    }
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (day >= limit && day < limit + (24 * 3600))
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (day >= limit)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (day <= limit)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (day >= limit && day < limit + (24 * 3600))
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (day >= limit)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (day <= limit)
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_dow_calculate
 * This function calculates day_of_week operation.
 */
BOOL
dg87_ref_operation_dow_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  struct tm dc;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (dc.tm_wday == operation->rule.dow.day_of_week)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (dc.tm_wday >= operation->rule.dow.day_of_week)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (dc.tm_wday <= operation->rule.dow.day_of_week)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (dc.tm_wday == operation->rule.dow.day_of_week)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (dc.tm_wday >= operation->rule.dow.day_of_week)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (dc.tm_wday <= operation->rule.dow.day_of_week)
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_doy_calculate
 * This function calculates day_of_year operation.
 */
BOOL
dg87_ref_operation_doy_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  time_t limit;
  struct tm dc;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  if (operation->rule.doy.day_of_year > 0)
    {
      /* find first day of year */
      dc.tm_mon = 0;
      dc.tm_mday = 1;
      limit = mktime (&dc);
      /* calculate limit date */
      limit += (operation->rule.doy.day_of_year - 1) * (24 * 3600);
    }
  else
    {
      /* find first day of next year */
      dc.tm_year += 1;
      dc.tm_mon = 0;
      dc.tm_mday = 1;
      limit = mktime (&dc);
      /* calculate limit date */
      limit += operation->rule.doy.day_of_year * (24 * 3600);

    }
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (day >= limit && day < limit + (24 * 3600))
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (day >= limit)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (day <= limit)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (day >= limit && day < limit + (24 * 3600))
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (day >= limit)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (day <= limit)
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_day_calculate
 * This function calculates day operation (dd/mm).
 */
BOOL
dg87_ref_operation_day_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  struct tm dc;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (dc.tm_mday == operation->rule.day.day &&
          dc.tm_mon == operation->rule.day.month)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (dc.tm_mon > operation->rule.day.month ||
          (dc.tm_mon == operation->rule.day.month &&
           dc.tm_mday >= operation->rule.day.day))
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (dc.tm_mon < operation->rule.day.month ||
          (dc.tm_mon == operation->rule.day.month &&
           dc.tm_mday <= operation->rule.day.day))
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (dc.tm_mday == operation->rule.day.day &&
          dc.tm_mon == operation->rule.day.month)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (dc.tm_mon > operation->rule.day.month ||
          (dc.tm_mon == operation->rule.day.month &&
           dc.tm_mday >= operation->rule.day.day))
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (dc.tm_mon < operation->rule.day.month ||
          (dc.tm_mon == operation->rule.day.month &&
           dc.tm_mday <= operation->rule.day.day))
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_date_calculate
 * This function calculates date operation (yyyy-mm-dd).
 */
BOOL
dg87_ref_operation_date_calculate (dg87_ref_rule_operation * operation,
                                   time_t day, BOOL * result)
{
  struct tm dc;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (day == operation->rule.date.date)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (day >= operation->rule.date.date)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (day <= operation->rule.date.date)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (day == operation->rule.date.date)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (day >= operation->rule.date.date)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (day <= operation->rule.date.date)
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_always_calculate
 * This function calculates always operation
 */
BOOL
dg87_ref_operation_always_calculate (dg87_ref_rule_operation * operation,
                                     time_t day, BOOL * result)
{
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (operation->oper != dg87_ref_rule_operator_add)
    {
      dg87_err_set_error (DG87_ERR_RULE_WRONG_OPERATOR_ALWAYS);
      return FALSE;
    }
  *result = TRUE;
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_complex_calculate
 * This function calculates complex operation
 */
BOOL
dg87_ref_operation_complex_calculate (dg87_ref_rule_operation * operation,
                                      time_t day, BOOL * result)
{
  BOOL res;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_ref_rule_calculate (operation->rule.rule, day, &res) == FALSE)
    return FALSE;

  switch (operation->oper)
    {

    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (res == TRUE)
        *result = TRUE;
      break;

    case dg87_ref_rule_operator_remove:
      if (res == TRUE)
        *result = FALSE;
      break;

    case dg87_ref_rule_operator_add_until:
    case dg87_ref_rule_operator_add_from:
    case dg87_ref_rule_operator_remove_from:
    case dg87_ref_rule_operator_remove_until:
      dg87_err_set_error (DG87_ERR_RULE_WRONG_OPERATOR_COMPLEX);
      return FALSE;
      break;

    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_ifequal_calculate
 * This function calculates ifequal operation
 */
BOOL
dg87_ref_operation_ifequal_calculate (dg87_ref_rule_operation * operation,
                                      time_t day, BOOL * result)
{
  dg87_variable *variable;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  variable = operation->rule.ifequal.variable;
  if (strcmp (operation->rule.ifequal.variable->value,
              operation->rule.ifequal.value) == 0)
    {
      if (operation->oper == dg87_ref_rule_operator_add)
        *result = TRUE;
      else if (operation->oper == dg87_ref_rule_operator_remove)
        *result = FALSE;

    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_ifnequal_calculate
 * This function calculates ifnequal operation
 */
BOOL
dg87_ref_operation_ifnequal_calculate (dg87_ref_rule_operation * operation,
                                       time_t day, BOOL * result)
{
  dg87_variable *variable;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  variable = operation->rule.ifequal.variable;
  if (strcmp (operation->rule.ifequal.variable->value,
              operation->rule.ifequal.value) != 0)
    {
      if (operation->oper == dg87_ref_rule_operator_add)
        *result = TRUE;
      else if (operation->oper == dg87_ref_rule_operator_remove)
        *result = FALSE;

    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_moy_calculate
 * This function calculates month_of_year operation
 */
BOOL
dg87_ref_operation_moy_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  struct tm dc;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (dc.tm_mon == operation->rule.moy.month_of_year)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (dc.tm_mon >= operation->rule.moy.month_of_year)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (dc.tm_mon <= operation->rule.moy.month_of_year)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (dc.tm_mon == operation->rule.moy.month_of_year)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (dc.tm_mon >= operation->rule.moy.month_of_year)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (dc.tm_mon <= operation->rule.moy.month_of_year)
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_wom_calculate
 * This function calculates week_of_month operation
 */
BOOL
dg87_ref_operation_wom_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  struct tm dc;
  time_t first_day;
  time_t fdow;                  /* first day of week */
  time_t ldow;                  /* last day of week */
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dc = *localtime (&day);
  if (operation->rule.wom.week_of_month > 0)
    {
      /* find first day of month */
      dc.tm_mday = 1;
      first_day = mktime (&dc);
      /* come back to begin of week */
      dc = *localtime (&first_day);
      first_day -= dc.tm_wday * 24 * 3600;
      /* calculate first and last day of week */
      fdow = first_day + 7 * 24 * 3600
        * (operation->rule.wom.week_of_month - 1);
      ldow = fdow + 7 * 24 * 3600;
    }
  else
    {
      /* find last day of month */
      dc.tm_mon++;
      dc.tm_mday = 1;
      first_day = mktime (&dc) - 24 * 3600;
      /* go to first day of next week */
      dc = *localtime (&first_day);
      first_day += (7 - dc.tm_wday) * 24 * 3600;
      /* calculate first and last day of week */
      fdow = first_day + 7 * 24 * 3600 * operation->rule.wom.week_of_month;
      ldow = fdow + 7 * 24 * 3600;
    }
  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (day >= fdow && day < ldow)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (day >= fdow)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (day < ldow)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (day >= fdow && day < ldow)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (day >= fdow)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (day < ldow)
        *result = FALSE;
      break;
    }
  return TRUE;
}

/****************************************************************************/
/* dg87_ref_operation_woy_calculate
 * This function calculates week_of_year operation
 */
BOOL
dg87_ref_operation_woy_calculate (dg87_ref_rule_operation * operation,
                                  time_t day, BOOL * result)
{
  struct tm dc;
  char buffer[50];
  int current_week_number;
  int selected_week_number;
	int last_week;
	int delta;
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	/* get the week number of the current week */

	/* about the `strftime' function :
	 * if the first week of a year doesn't start on the 1st january, then
	 * %V is the number of the week of the last year (53) (range 0-53),
	 * %W is 0 (range 0 to 53).
	 */
  dc = *localtime (&day);
  delta = operation->rule.woy.week_of_year; 

  if (delta > 0)
		{
			/* get the real number of the selected week (range 01 to 53) */
			strftime(buffer, 10, "%V", &dc); /* %V range 01-53 */
			current_week_number = atoi(buffer);
      selected_week_number = delta;
		}
  else
		{
			/* get the number of the week (first week of the year is 0 or 1) */
			strftime(buffer, 10, "%W", &dc); /* %W :range 0-53 */
			current_week_number = atoi(buffer);

			if (delta == -1 && current_week_number == 0)
				{
					/* `delta=-1' means that we want to select the last week of the
					 * year, and current_week_number=0 means that the current day 
					 * is in the week of the previous year 
					 * -> we have to select them...
					 */
					current_week_number = 53;
					selected_week_number = 53;
				}
			else
				{
					/* get the number of the last week of this year */
					dc.tm_mday = 31; dc.tm_mon = 11; /* the 31st of december */
					mktime(&dc); /* recompute the date */
					strftime(buffer, 10, "%W", &dc); /* %W -> 0-53 */
					last_week = atoi(buffer);
					/* delta is a negativ value */
      		selected_week_number = last_week + 1 + delta;
				}
		}

  switch (operation->oper)
    {
    case dg87_ref_rule_operator_shift_after:
    case dg87_ref_rule_operator_shift_before:
    case dg87_ref_rule_operator_add:
      if (current_week_number == selected_week_number)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_from:
      if (current_week_number >= selected_week_number)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_add_until:
      if (current_week_number <= selected_week_number)
        *result = TRUE;
      break;
    case dg87_ref_rule_operator_remove:
      if (current_week_number == selected_week_number)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_from:
      if (current_week_number >= selected_week_number)
        *result = FALSE;
      break;
    case dg87_ref_rule_operator_remove_until:
      if (current_week_number <= selected_week_number)
        *result = FALSE;
      break;
    }
  return TRUE;
}
/****************************************************************************/
/* 
 * dg87_ref_operation_calculate
 * This function set to true the value of the parameter 'result' if the day 
 * given in parameter is selected by the operation.
 * If the day isn't selected, the value of *result is not changed.
 * Operation witch operator is shiftbefore of shiftafter are not calculated
 * by this function.
 * The function returns FALSE if an error occurres. 
 */
static BOOL
dg87_ref_operation_calculate (dg87_ref_rule_operation * operation,
                              time_t day, BOOL * result)
{
  assert (operation != NULL && result != NULL);
  if (operation == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  switch (operation->type)
    {
    case dg87_ref_rule_type_dow:       /* day of week */
      if (dg87_ref_operation_dow_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_dom:       /* day of month */
      if (dg87_ref_operation_dom_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_doy:       /* day of year */
      if (dg87_ref_operation_doy_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_wom:       /* week of month */
      if (dg87_ref_operation_wom_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;

    case dg87_ref_rule_type_woy:       /* week of year */
      if (dg87_ref_operation_woy_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_moy:       /* month of year */
      if (dg87_ref_operation_moy_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_day:       /* date dd/mm */
      if (dg87_ref_operation_day_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_date:      /* date dd/mm/yyyy */
      if (dg87_ref_operation_date_calculate (operation, day, result) == FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_always:    /* all days */
      if (dg87_ref_operation_always_calculate (operation, day, result) ==
          FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_complex:   /* composed rule */
      if (dg87_ref_operation_complex_calculate (operation, day, result) ==
          FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_ifequal:   /* if variable equal value */
      if (dg87_ref_operation_ifequal_calculate (operation, day, result) ==
          FALSE)
        return FALSE;
      break;
    case dg87_ref_rule_type_ifnequal:  /* if variable not equal value */
      if (dg87_ref_operation_ifnequal_calculate (operation, day, result) ==
          FALSE)
        return FALSE;
      break;

    }

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_operationlist_calculate (dg87_list * operation_list,
                                  time_t day, BOOL * result, int n_operations)
{
  int i_operation;
  int delta, limit;
  time_t day_delta;
  dg87_item *item;
  dg87_ref_rule_operation *operation;
  BOOL res;
  assert (operation_list != NULL && result != NULL);
  if (operation_list == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (n_operations < -1)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  if (n_operations == -1)
    n_operations = operation_list->count;
  *result = FALSE;

  /* for each operation of the rule */
  i_operation = 0;
  item = operation_list->first;
  while (item != NULL && i_operation < n_operations)
    {
      operation = item->object;
      item = item->next;
      i_operation++;

      /* optimization : nothing to do if
       * - operator is add and day is already selected
       * - operator is remove and day is already unselected */
      /* TODO move optimization in dg87_ref_operation_calculate ? */
      /* TODO optimization with shift_after and shif_before */
      if (*result == TRUE && (operation->oper == dg87_ref_rule_operator_add ||
                              operation->oper ==
                              dg87_ref_rule_operator_add_until
                              || operation->oper ==
                              dg87_ref_rule_operator_add_from))
        continue;
      if (*result == FALSE
          && (operation->oper == dg87_ref_rule_operator_remove
              || operation->oper == dg87_ref_rule_operator_remove_until
              || operation->oper == dg87_ref_rule_operator_remove_from))
        continue;

      /* for operator different from shiftafter or shiftbefore */
      if (operation->oper != dg87_ref_rule_operator_shift_before &&
          operation->oper != dg87_ref_rule_operator_shift_after)
        {
          if (dg87_ref_operation_calculate (operation, day, result) == FALSE)
            {
              return FALSE;
            }
          continue;
        }

      /* 
       * here, we are sure that operator is shift_after or shift_before.
       *
       * - if the day is selected by the shift operation, then
       *   the day is unselected. (shift)
       *
       * - if day is just outside of a range of days which are 
       *   selected by the shift operation, then day is selected
       *   if one (or more) days of the range should be selected
       *   by previous operations in the list.
       *
       * - else no change
       */

      /* 
       * si day dans la plage shift 
       *      le deselectionner 
       *      operation suivante
       * fin si
       */
      res = FALSE;
      /* TODO changer l'operateur pour dg87_d_operator_add ? */
      if (dg87_ref_operation_calculate (operation, day, &res) == FALSE)
        {
          /* TODO erreur calcul : sortir */
          return FALSE;
        }
      /* if day is selected by the operation, */
      if (res == TRUE)
        {
          *result = FALSE;
          /* next operation */
          continue;
        }

      if (operation->oper == dg87_ref_rule_operator_shift_before)
        delta = 24 * 3600;      /* shift before <<< */
      else
        delta = -24 * 3600;     /* shift after   >>> */

      /* si jour_teste+delta n'est pas dans plage shift alors
       *      operation suivante
       * fin si
       */

      /* the day is just at the bound of range of days which are 
       * selected by the shift operation. 
       * If one or more days in this range are 
       * selected by previous operations of the list, then
       * the day must be selected.
       * (selected days by previous operations are shifted at the
       * bound of the range)
       */
      limit = 0;
      day_delta = day;
      while (limit < 365)
        {
          /* next day in the range of selected days */
          day_delta += delta;
          limit++;

          /* break if outside of range of selected days */
          /* TODO change operator for operator_add ? */
          res = FALSE;
          if (dg87_ref_operation_calculate (operation,
                                            day_delta, &res) == FALSE)
            {
              return FALSE;
            }
          if (res == FALSE)
            {
              break;
            }

          res = FALSE;

          if (dg87_ref_operationlist_calculate (operation_list,
                                                day_delta, &res,
                                                n_operations - 1) == FALSE)
            {
              return FALSE;
            }

          if (res == TRUE)
            {
              *result = TRUE;
              continue;
            }
        }

    }                           /* next operation */
  return TRUE;

}

/****************************************************************************/
BOOL
dg87_ref_rule_calculate (dg87_ref_rule * rule, time_t day, BOOL * result)
{
  assert (rule != NULL && result != NULL);
  if (rule == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  return dg87_ref_operationlist_calculate (rule->operation_list,
                                           day, result, -1);
}

/****************************************************************************/
/*
 * dg87_ref_rule_use_rule
 * This function is recursive. The function return TRUE if the searched_rule
 * is used by the rule.
 */
BOOL
dg87_ref_rule_use_rule (dg87_ref_rule * rule, dg87_ref_rule * searched_rule)
{
  dg87_item *item;
  dg87_ref_rule_operation *operation;
  assert (rule != NULL && searched_rule != NULL);
  if (rule == NULL || searched_rule == NULL)
    return FALSE;
  if (rule == searched_rule)
    return TRUE;
  /* for each operation in the rule */
  item = rule->operation_list->first;
  while (item != NULL)
    {
      operation = item->object;
      /* if operation uses another rule */
      if (operation->type == dg87_ref_rule_type_complex)
        {
          /* determine if this rule uses the search rule */
          if (dg87_ref_rule_use_rule (operation->rule.rule,
                                      searched_rule) == TRUE)
            return TRUE;
        }
      item = item->next;
    }
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_ref_rule_operation_add (dg87_ref_rule * rule,
                             dg87_ref_rule_operation * operation)
{
  assert (rule != NULL && operation != NULL);
  if (rule == NULL || operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (operation->type == dg87_ref_rule_type_complex)
    {
      /* avoid circular reference */
      if (dg87_ref_rule_use_rule (operation->rule.rule, rule) == TRUE)
        {
          dg87_err_set_error (DG87_ERR_CIRCULAR_RULE_REFERENCE);
          return FALSE;
        }
    }
  /* add the operation to the list */
  if (dg87_list_add_object (rule->operation_list, operation) == FALSE)
    {
      return FALSE;
    }
  /* increment linked_rules counter for complex rule */
  if (operation->type == dg87_ref_rule_type_complex)
    operation->rule.rule->linked_rules++;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_rule_operation_delete (dg87_ref_rule * rule, int index)
{
  dg87_ref_rule_operation *operation;
  assert (rule != NULL);
  if (rule == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* remove the operation from the list */
  operation = dg87_list_remove_object_by_index (rule->operation_list, index);
  if (operation == NULL)
    return FALSE;
  /* if complex rule, decrement linked_rules counter */
  if (operation->type == dg87_ref_rule_type_complex)
    operation->rule.rule->linked_rules--;
  /* delete operation object */
  if (dg87_ref_operation_delete (operation) == FALSE)
    {
      dg87_list_add_object (rule->operation_list, operation);
      if (operation->type == dg87_ref_rule_type_complex)
        operation->rule.rule->linked_rules++;
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_rule_operation_move (dg87_ref_rule * rule, int index, int delta)
{
  assert (rule != NULL);
  if (rule == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return dg87_list_move_item (rule->operation_list, index, delta);

}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_dayofweek_new (dg87_ref_rule_operator oper, int day)
{
  dg87_ref_rule_operation *operation;
  if (day < 0 || day > 6)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_dow;
  operation->rule.dow.day_of_week = day;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_dayofmonth_new (dg87_ref_rule_operator oper, int day)
{
  dg87_ref_rule_operation *operation;
  if (day < -31 || day > 31 || day == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_dom;
  operation->rule.dom.day_of_month = day;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_dayofyear_new (dg87_ref_rule_operator oper, int day)
{
  dg87_ref_rule_operation *operation;
  if (day < -366 || day == 0 || day > 366)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_doy;
  operation->rule.doy.day_of_year = day;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_weekofmonth_new (dg87_ref_rule_operator oper, int week)
{
  dg87_ref_rule_operation *operation;
  if (week < -5 || week == 0 || week > 5)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_wom;
  operation->rule.wom.week_of_month = week;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_weekofyear_new (dg87_ref_rule_operator oper, int week)
{
  dg87_ref_rule_operation *operation;
  if (week < -53 || week == 0 || week > 53)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_woy;
  operation->rule.woy.week_of_year = week;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_monthofyear_new (dg87_ref_rule_operator oper, int month)
{
  dg87_ref_rule_operation *operation;
  if (month < 0 || month > 11)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_moy;
  operation->rule.moy.month_of_year = month;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_day_new (dg87_ref_rule_operator oper, int month, int day)
{
  dg87_ref_rule_operation *operation;
  if (day < 1 || day > 31 || month < 0 || month > 11)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_day;
  operation->rule.day.day = day;
  operation->rule.day.month = month;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_date_new (dg87_ref_rule_operator oper, time_t date)
{
  dg87_ref_rule_operation *operation;
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_date;
  operation->rule.date.date = date;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_always_new (void)
{
  dg87_ref_rule_operation *operation;
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = dg87_ref_rule_operator_add;
  operation->type = dg87_ref_rule_type_always;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_complex_new (dg87_ref_rule_operator oper,
                                dg87_ref_rule * rule)
{
  dg87_ref_rule_operation *operation;
  if (rule == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_complex;
  operation->rule.rule = rule;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_ifequal_new (dg87_ref_rule_operator oper,
                                dg87_variable * variable, char *value)
{
  dg87_ref_rule_operation *operation;
  char *variable_value;
  assert (variable != NULL && value != NULL);
  if (variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  variable_value = dg87_strdup (value, DG87_CHAR);
  if (variable_value == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      dg87_free (operation, DG87_OPERATION);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_ifequal;
  operation->rule.ifequal.variable = variable;
  operation->rule.ifequal.value = variable_value;
  /* increment variable use counter */
  variable->linked_rules++;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_ifnequal_new (dg87_ref_rule_operator oper,
                                 dg87_variable * variable, char *value)
{
  dg87_ref_rule_operation *operation;
  char *variable_value;
  assert (variable != NULL && value != NULL);
  if (variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  operation = dg87_malloc (sizeof (dg87_ref_rule_operation), DG87_OPERATION);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  variable_value = dg87_strdup (value, DG87_CHAR);
  if (variable_value == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      dg87_free (operation, DG87_OPERATION);
      return NULL;
    }
  operation->oper = oper;
  operation->type = dg87_ref_rule_type_ifnequal;
  operation->rule.ifequal.variable = variable;
  operation->rule.ifequal.value = variable_value;
  /* increment variable use counter */
  variable->linked_rules++;
	DG87_SAVE_NEEDED;
  return operation;
}

/****************************************************************************/
BOOL
dg87_ref_operation_delete (dg87_ref_rule_operation * operation)
{
  dg87_variable *variable;
  assert (operation != NULL);
  if (operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* variable decrement counter */
  if (operation->type == dg87_ref_rule_type_ifequal)
    {
      variable = operation->rule.ifequal.variable;
      variable->linked_rules--;
      dg87_free (operation->rule.ifequal.value, DG87_CHAR);
    }
  if (operation->type == dg87_ref_rule_type_ifnequal)
    {
      variable = operation->rule.ifnequal.variable;
      variable->linked_rules--;
      dg87_free (operation->rule.ifnequal.value, DG87_CHAR);
    }
  dg87_free (operation, DG87_OPERATION);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_ref_rule_operation *
dg87_ref_operation_duplicate (dg87_ref_rule_operation * original_operation,
                              dg87_vector * new_rule_vector)
{
  dg87_ref_rule_operation *new_operations;
  dg87_ref_rule *rule;
  new_operations = NULL;
  assert (original_operation != NULL && new_rule_vector != NULL);
  switch (original_operation->type)
    {
    case dg87_ref_rule_type_dow:
      new_operations =
        dg87_ref_operation_dayofweek_new (original_operation->oper,
                                          original_operation->rule.dow.
                                          day_of_week);
      break;
    case dg87_ref_rule_type_dom:
      new_operations =
        dg87_ref_operation_dayofmonth_new (original_operation->oper,
                                           original_operation->rule.dom.
                                           day_of_month);
      break;
    case dg87_ref_rule_type_doy:
      new_operations =
        dg87_ref_operation_dayofyear_new (original_operation->oper,
                                          original_operation->rule.doy.
                                          day_of_year);
      break;
    case dg87_ref_rule_type_wom:
      new_operations =
        dg87_ref_operation_weekofmonth_new (original_operation->oper,
                                            original_operation->rule.wom.
                                            week_of_month);
      break;
    case dg87_ref_rule_type_woy:
      new_operations =
        dg87_ref_operation_weekofyear_new (original_operation->oper,
                                           original_operation->rule.woy.
                                           week_of_year);
      break;
    case dg87_ref_rule_type_moy:
      new_operations =
        dg87_ref_operation_monthofyear_new (original_operation->oper,
                                            original_operation->rule.moy.
                                            month_of_year);
      break;
    case dg87_ref_rule_type_day:
      new_operations = dg87_ref_operation_day_new (original_operation->oper,
                                                   original_operation->rule.
                                                   day.month,
                                                   original_operation->rule.
                                                   day.day);
      break;
    case dg87_ref_rule_type_date:
      new_operations = dg87_ref_operation_date_new (original_operation->oper,
                                                    original_operation->rule.
                                                    date.date);
      break;
    case dg87_ref_rule_type_always:
      new_operations = dg87_ref_operation_always_new ();
      break;
    case dg87_ref_rule_type_complex:
      /* find the rule in the vector of new rules */
      rule = dg87_vector_get_object_by_key (new_rule_vector,
                                            original_operation->rule.rule->
                                            name);
      if (rule == NULL)
        return NULL;
      new_operations =
        dg87_ref_operation_complex_new (original_operation->oper, rule);
      break;
    case dg87_ref_rule_type_ifequal:
      new_operations =
        dg87_ref_operation_ifequal_new (original_operation->oper,
                                        original_operation->rule.ifequal.
                                        variable,
                                        original_operation->rule.ifequal.
                                        value);
      break;
    case dg87_ref_rule_type_ifnequal:
      new_operations =
        dg87_ref_operation_ifnequal_new (original_operation->oper,
                                         original_operation->rule.ifnequal.
                                         variable,
                                         original_operation->rule.ifnequal.
                                         value);
      break;

    }
	DG87_SAVE_NEEDED;
  return new_operations;
}

/****************************************************************************/
dg87_ref_rule_type
dg87_ref_rule_ascii_to_type (char *rule_type)
{
  assert (rule_type != NULL);
  if (rule_type == NULL)
    return -1;

  if (strcasecmp (rule_type, "day_of_week") == 0)
    return dg87_ref_rule_type_dow;
  if (strcasecmp (rule_type, "day_of_month") == 0)
    return dg87_ref_rule_type_dom;
  if (strcasecmp (rule_type, "day_of_year") == 0)
    return dg87_ref_rule_type_doy;
  if (strcasecmp (rule_type, "week_of_month") == 0)
    return dg87_ref_rule_type_wom;
  if (strcasecmp (rule_type, "week_of_year") == 0)
    return dg87_ref_rule_type_woy;
  if (strcasecmp (rule_type, "month_of_year") == 0)
    return dg87_ref_rule_type_moy;
  if (strcasecmp (rule_type, "day") == 0)
    return dg87_ref_rule_type_day;
  if (strcasecmp (rule_type, "date") == 0)
    return dg87_ref_rule_type_date;
  if (strcasecmp (rule_type, "always") == 0)
    return dg87_ref_rule_type_always;
  if (strcasecmp (rule_type, "rule") == 0)
    return dg87_ref_rule_type_complex;
  if (strcasecmp (rule_type, "if_equal") == 0)
    return dg87_ref_rule_type_ifequal;
  if (strcasecmp (rule_type, "if_not_equal") == 0)
    return dg87_ref_rule_type_ifnequal;

  return -1;
}

/****************************************************************************/
char *
dg87_ref_rule_type_to_ascii (dg87_ref_rule_type rule_type)
{
  switch (rule_type)
    {
    case dg87_ref_rule_type_dow:
      return "day_of_week";
    case dg87_ref_rule_type_dom:
      return "day_of_month";
    case dg87_ref_rule_type_doy:
      return "day_of_year";
    case dg87_ref_rule_type_wom:
      return "week_of_month";
    case dg87_ref_rule_type_woy:
      return "week_of_year";
    case dg87_ref_rule_type_moy:
      return "month_of_year";
    case dg87_ref_rule_type_day:
      return "day";
    case dg87_ref_rule_type_date:
      return "date";
    case dg87_ref_rule_type_always:
      return "always";
    case dg87_ref_rule_type_complex:
      return "rule";
    case dg87_ref_rule_type_ifequal:
      return "if_equal";
    case dg87_ref_rule_type_ifnequal:
      return "if_not_equal";
    }
  return "error";
}

/****************************************************************************/
dg87_ref_rule_operator
dg87_ref_rule_ascii_to_operator (char *operator)
{
  assert (operator != NULL);
  if (operator == NULL)
    return -1;

  if (strcasecmp (operator, "add") == 0)
    return dg87_ref_rule_operator_add;
  if (strcasecmp (operator, "add_until") == 0)
    return dg87_ref_rule_operator_add_until;
  if (strcasecmp (operator, "add_from") == 0)
    return dg87_ref_rule_operator_add_from;
  if (strcasecmp (operator, "remove") == 0)
    return dg87_ref_rule_operator_remove;
  if (strcasecmp (operator, "remove_until") == 0)
    return dg87_ref_rule_operator_remove_until;
  if (strcasecmp (operator, "remove_from") == 0)
    return dg87_ref_rule_operator_remove_from;
  if (strcasecmp (operator, "shift_after") == 0)
    return dg87_ref_rule_operator_shift_after;
  if (strcasecmp (operator, "shift_before") == 0)
    return dg87_ref_rule_operator_shift_before;
  return -1;
}

/****************************************************************************/
char *
dg87_ref_rule_operator_to_ascii (dg87_ref_rule_operator oper)
{
  switch (oper)
    {
    case dg87_ref_rule_operator_add:
      return "add";
    case dg87_ref_rule_operator_add_until:
      return "add_until";
    case dg87_ref_rule_operator_add_from:
      return "add_from";
    case dg87_ref_rule_operator_remove:
      return "remove";
    case dg87_ref_rule_operator_remove_until:
      return "remove_until";
    case dg87_ref_rule_operator_remove_from:
      return "remove_from";
    case dg87_ref_rule_operator_shift_after:
      return "shift_after";
    case dg87_ref_rule_operator_shift_before:
      return "shift_before";
    }
  return "error";
}

/******************************************************************************/
BOOL
dg87_ref_rule_ascii_to_day (char *string, int *month, int *day)
{
  char buffer[6];
  assert (string != NULL && month != NULL);
  if (string == NULL || strlen (string) != 5)
    return FALSE;
  if (string[2] != '-')
    return FALSE;
  dg87_strnzcpy (buffer, string, 5);
  buffer[2] = '\0';
  *month = atoi (string) - 1;
  *day = atoi (string + 3);
  if (*month < 0 || *month > 11 || *day < 1 || *day > 31)
    return FALSE;
  return TRUE;
}

/******************************************************************************/
char *
dg87_ref_rule_day_to_ascii (int month, int day)
{
  char *string = dg87_malloc (sizeof (char) * 6, DG87_CHAR);
  if (string == NULL)
    return NULL;
  sprintf (string, "%02i-%02i", month + 1, day);
  return string;

  return FALSE;
}

/******************************************************************************/
time_t
dg87_ref_rule_ascii_to_date (char *string)
{
  time_t td;
  struct tm dc;
  int year, month, day;
  char buffer[11];
  assert (string != NULL);
  if (string == NULL || strlen (string) != 10)
    return -1;
  if (string[4] != '-' || string[7] != '-')
    return -1;
  dg87_strnzcpy (buffer, string, 10);
  buffer[4] = '\0';
  buffer[7] = '\0';

  year = atoi (buffer);
  month = atoi (buffer + 5);
  day = atoi (buffer + 8);

  if (year < 1900 || month < 1 || month > 12 || day < 1 || day > 31)
    {
      return -1;
    }
  dc.tm_year = year - 1900;
  dc.tm_mon = month - 1;
  dc.tm_mday = day;
  dc.tm_hour = 0;
  dc.tm_min = 0;
  dc.tm_sec = 0;
  dc.tm_isdst = 0;              /* don't change tm_isdst */
  td = mktime (&dc);
  return td;
}

/******************************************************************************/
char *
dg87_ref_rule_date_to_ascii (time_t time)
{
  struct tm dc;
  char format[] = "%Y-%m-%d";
  char *string = dg87_malloc (sizeof (char) * 11, DG87_CHAR);
  if (string == NULL)
    return NULL;
  dc = *localtime (&time);
  strftime (string, 11, format, &dc);
  return string;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_ref_rule_check (dg87_ref_rule * rule, int (*print) (char *))
{
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
