/* file d_rule.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_RULE_H__
#define __DG87_REF_RULE_H__

#include <libdg87/vector.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include <time.h>


typedef enum
{
  dg87_ref_rule_type_dow,       /* day of week */
  dg87_ref_rule_type_dom,       /* day of month */
  dg87_ref_rule_type_doy,       /* day of year */
  dg87_ref_rule_type_wom,       /* week of month */
  dg87_ref_rule_type_woy,       /* week of year */
  dg87_ref_rule_type_moy,       /* month of year */
  dg87_ref_rule_type_day,       /* date mm-dd */
  dg87_ref_rule_type_date,      /* date dd/mm/yyyy */
  dg87_ref_rule_type_always,    /* all days */
  dg87_ref_rule_type_complex,   /* composed rule */
  dg87_ref_rule_type_ifequal,   /* if variable equal value */
  dg87_ref_rule_type_ifnequal   /* if variable equal value */
}
dg87_ref_rule_type;

/* rule to select a day of week (1-7) */
typedef struct
{
  int day_of_week;
}
dg87_ref_rule_dow;

/* rule to select a day of month (1 to 31 and -31 to -1) */
typedef struct
{
  int day_of_month;
}
dg87_ref_rule_dom;

/* rule to select a day of year (1-365) */
typedef struct
{
  int day_of_year;
}
dg87_ref_rule_doy;

/* rule to select a week of month (1-4) */
typedef struct
{
  int week_of_month;
}
dg87_ref_rule_wom;

/* rule to select a week of year (1-52)*/
typedef struct
{
  int week_of_year;
}
dg87_ref_rule_woy;

/* rule to select a month of year (1-12) */
typedef struct
{
  int month_of_year;
}
dg87_ref_rule_moy;

/* rule to select a day of the year (dd/mm) */
typedef struct
{
  int day;
  int month;
}
dg87_ref_rule_day;

/* rule to select a date */
typedef struct
{
  time_t date;
}
dg87_ref_rule_date;


/* rule always */
typedef struct
{
  /* empty */
  int dummy;
}
dg87_ref_rule_always;

/* rule ifequal */
typedef struct
{
  dg87_variable *variable;
  char *value;
}
dg87_ref_rule_ifequal;

/* rule ifnequal (if not equal) */
typedef struct
{
  dg87_variable *variable;
  char *value;
}
dg87_ref_rule_ifnequal;


/* generic rule */
typedef union
{
  dg87_ref_rule_dow dow;        /* day of week */
  dg87_ref_rule_dom dom;        /* day of month */
  dg87_ref_rule_doy doy;        /* day of year */
  dg87_ref_rule_wom wom;        /* week of month */
  dg87_ref_rule_woy woy;        /* week of year */
  dg87_ref_rule_moy moy;        /* month of year */
  dg87_ref_rule_day day;        /* day of year dd/mm */
  dg87_ref_rule_date date;      /* date dd/mm/yyyy */
  dg87_ref_rule_always always;  /* all days */
  dg87_ref_rule *rule;          /* composed rule */
  dg87_ref_rule_ifequal ifequal;        /* if variable equal value */
  dg87_ref_rule_ifnequal ifnequal;      /* if variable equal value */
}
dg87_ref_generic_rule;

/* enum operator */
typedef enum
{
  dg87_ref_rule_operator_add,
  dg87_ref_rule_operator_add_until,
  dg87_ref_rule_operator_add_from,
  dg87_ref_rule_operator_remove,
  dg87_ref_rule_operator_remove_until,
  dg87_ref_rule_operator_remove_from,
  dg87_ref_rule_operator_shift_after,
  dg87_ref_rule_operator_shift_before
}
dg87_ref_rule_operator;

/* struct operation */
typedef struct
{
  dg87_ref_rule_operator oper;
  dg87_ref_rule_type type;
  dg87_ref_generic_rule rule;
}
dg87_ref_rule_operation;


/* struct rule */
struct _dg87_ref_rule
{
  char name[DG87_RULE_NAME_LENGTH + 1];
  dg87_ref_model *ref;
  dg87_list *operation_list;
  dg87uint linked_rules;
  dg87uint linked_tasks;
};

/*
 * dg87_ref_rule_new
 * create a new rule and return a pointer on this.
 */
dg87_ref_rule *dg87_ref_rule_new (char *name, dg87_ref_model * ref);

/*
 * dg87_ref_rule_delete
 */
BOOL dg87_ref_rule_delete (dg87_ref_rule * rule);

/*
 * dg87_ref_rule_set_name
 * Change the name of a rule.
 * An error occurres if another rule has the same name in the referential.
 */
BOOL dg87_ref_rule_set_name (dg87_ref_rule * rule, char *name);

/*
 * dg87_ref_rule_load
 */
BOOL dg87_ref_rule_load (dg87_ref_rule * rule, dg87_storage * storage);

/*
 * dg87_ref_rule_save
 */
BOOL dg87_ref_rule_save (dg87_ref_rule * rule, dg87_storage * storage);

/*
 * dg87_ref_rule_calculate
 */
BOOL dg87_ref_rule_calculate (dg87_ref_rule * rule, time_t day,
                              BOOL * result);

/*
 * dg87_ref_rule_duplicate
 */
dg87_ref_rule *dg87_ref_rule_duplicate (dg87_ref_rule * pOrignialRule);

/*
 * dg87_ref_rule_opearation_add
 */
BOOL dg87_ref_rule_operation_add (dg87_ref_rule * rule,
                                  dg87_ref_rule_operation * operation);

/*
 * dg87_ref_rule_operation_delete
 */
BOOL dg87_ref_rule_operation_delete (dg87_ref_rule * rule, int index);

/*
 * dg87_ref_rule_operation_move
 */
BOOL dg87_ref_rule_operation_move (dg87_ref_rule * rule, int index,
                                   int delta);

/******************************************************************************/
/*   functions for operations                                                 */
/******************************************************************************/

/*
 * dg87_ref_operation_dayofweek_new (1 to 7)
 */
dg87_ref_rule_operation
  *dg87_ref_operation_dayofweek_new (dg87_ref_rule_operator oper, int day);

/*
 * dg87_ref_operation_dayofmonth_new (1 to 31 and -31 to -1)
 */
dg87_ref_rule_operation
  *dg87_ref_operation_dayofmonth_new (dg87_ref_rule_operator oper, int day);

/*
 * dg87_ref_operation_dayofyear_new (1 to 365)
 */
dg87_ref_rule_operation
  *dg87_ref_operation_dayofyear_new (dg87_ref_rule_operator oper, int day);

/*
 * dg87_ref_operation_weekofmonth_new (1 to 4)
 */
dg87_ref_rule_operation
  *dg87_ref_operation_weekofmonth_new (dg87_ref_rule_operator oper, int week);

/*
 * dg87_ref_operation_weekofyear_new (1 to 52)
 */
dg87_ref_rule_operation
  *dg87_ref_operation_weekofyear_new (dg87_ref_rule_operator oper, int week);

/*
 * dg87_ref_operation_monthofyear_new (1 to 12)
 */
dg87_ref_rule_operation
  *dg87_ref_operation_monthofyear_new (dg87_ref_rule_operator oper,
                                       int month);

/*
 * dg87_ref_operation_day_new (dd/mm)
 */
dg87_ref_rule_operation *dg87_ref_operation_day_new (dg87_ref_rule_operator
                                                     oper, int month,
                                                     int day);

/*
 * dg87_ref_operation_date_new 
 */
dg87_ref_rule_operation *dg87_ref_operation_date_new (dg87_ref_rule_operator
                                                      oper, time_t date);

/*
 * dg87_ref_operation_always_new (add all days)
 */
dg87_ref_rule_operation *dg87_ref_operation_always_new (void);

/*
 * dg87_ref_operation_complex_new 
 */
dg87_ref_rule_operation
  *dg87_ref_operation_complex_new (dg87_ref_rule_operator oper,
                                   dg87_ref_rule * rule);

/*
 * dg87_ref_operation_ifequal_new
 */
dg87_ref_rule_operation
  *dg87_ref_operation_ifequal_new (dg87_ref_rule_operator oper,
                                   dg87_variable * variable_name,
                                   char *value);

/*
 * dg87_ref_operation_ifnequal_new
 */
dg87_ref_rule_operation
  *dg87_ref_operation_ifnequal_new (dg87_ref_rule_operator oper,
                                    dg87_variable * variable_name,
                                    char *value);


/*
 * dg87_ref_operation_delete
 */
BOOL dg87_ref_operation_delete (dg87_ref_rule_operation * operation);

/*
 * dg87_ref_operation_duplicate
 */
dg87_ref_rule_operation *dg87_ref_operation_duplicate (dg87_ref_rule_operation
                                                       * operation,
                                                       dg87_vector *
                                                       new_rule_vector);

/******************************************************************************/
/*   utilities                                                                */
/******************************************************************************/

/*
 * dg87_ref_rule_ascii_to_type
 */
dg87_ref_rule_type dg87_ref_rule_ascii_to_type (char *rule_type);

/*
 * dg87_ref_rule_type_to_ascii
 */
char *dg87_ref_rule_type_to_ascii (dg87_ref_rule_type rule_type);

/*
 * dg87_ref_rule_ascii_to_operator
 */
dg87_ref_rule_operator dg87_ref_rule_ascii_to_operator (char *oper);

/*
 * dg87_ref_rule_operator_to_ascii
 */
char *dg87_ref_rule_operator_to_ascii (dg87_ref_rule_operator oper);


/*
 * dg87_ref_rule_operator_to_ascii
 */
BOOL dg87_ref_operationlist_calculate (dg87_list * operation_list,
                                       time_t day, BOOL * result,
                                       int n_operations);
/******************************************************************************/
/*   utilities                                                                */
/******************************************************************************/

/*
 * dg87_ref_rule_ascii_to_day
 * This function extracts the month and the day from a string 
 * with a format hh:mm.
 * returns FALSE on error.
 */
BOOL dg87_ref_rule_ascii_to_day (char *string, int *month, int *day);

/* dg87_ref_rule_day_to_ascii
 * return a string witch represent the time given in parameter. 
 * The format of the string is MM-DD where MM is month, DD is month day.
 * The function return -1 on error.
 * The string must be free with a call to dg87_free
 */
char *dg87_ref_rule_day_to_ascii (int month, int day);


/* dg87_ref_rule_ascii_to_date
 * convert a string to a time_t.
 * The format of the string is YYYY-MM-DD where YYYY is year,
 * MM is month, DD is month day.
 * The function return -1 on error.
 */
time_t dg87_ref_rule_ascii_to_date (char *string);

/* dg87_ref_rule_date_to_ascii
 * return a string witch represent the time given in parameter. 
 * The format of the string is YYYY-MM-DD where YYYY is year,
 * MM is month, DD is month day.
 * The function return -1 on error.
 * The string must be free with a call to dg87_free
 */
char *dg87_ref_rule_date_to_ascii (time_t time);

#ifdef DEBUG
/*
 * dg87_ref_rule_ckeck
 */
int dg87_ref_rule_check (dg87_ref_rule * rule, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_RULE_H__*/
/* End of file */
