/* file d_task.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <libdg87/list.h>
#include <libdg87/vector.h>
#include <libdg87/global.h>

#include "context.h"
#include "target.h"
#include "d_job.h"
#include "d_jobdep.h"
#include "d_task.h"
#include "d_ref.h"
#include "target.h"
#include "d_condition.h"
#include "variable.h"
#include "engine.h"


/****************************************************************************/
dg87_ref_task *
dg87_ref_task_new (char *name)
{
  dg87_ref_task *task;
  assert (name != NULL);
  if (name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_TASK_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((task = dg87_malloc (sizeof (dg87_ref_task), DG87_TASK)) == NULL)
    goto alloc_error;
  if ((task->job_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((task->job_dep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->upstream_taskdep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->downstream_taskdep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->condition_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->rule_operation_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  dg87_strnzcpy (task->name, name, DG87_TASK_NAME_LENGTH);
  /* initialize structure */
  task->description[0] = 0;
  task->enabled = TRUE;
  task->queue = NULL;
  task->ref = NULL;
  task->wait_all_dependencies = TRUE;
  task->on_error_continue = TRUE;
  task->iteration = 1;          /* task is not cyclical */
  task->interval = 0;           /* not used with no-cyclical task */
  task->from_begin = TRUE;      /* not used with no-cyclical task */
  task->lock_variable = NULL;
  task->lock_action = FALSE;
	DG87_SAVE_NEEDED;
  return task;

alloc_error:
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  if (task != NULL)
    dg87_ref_task_delete (task);
  return NULL;
}

/****************************************************************************/
dg87_ref_task *
dg87_ref_task_duplicate (dg87_ref_task * original_task,
                         dg87_vector * new_rule_vector, char *new_name)
{
  dg87_ref_task *new_task;
  dg87_element *element;
  dg87_item *item;
  dg87_ref_job *original_job, *new_job;
  dg87_ref_job *upstream_job, *downstream_job;
  dg87_ref_jobdep *original_job_dep, *new_jobdep;
  dg87_ref_condition *original_condition, *new_condition;
  dg87_ref_rule_operation *original_operation, *new_operations;
  /* new_name parameter can be NULL */
  assert (original_task != NULL && new_rule_vector != NULL);
  if (original_task == NULL || new_rule_vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* create a task with the same name or with a new name */
  if (new_name == NULL)
    new_name = original_task->name;
  new_task = dg87_ref_task_new (new_name);
  if (new_task == NULL)
    return NULL;
  /* copy task properties */
  dg87_strnzcpy (new_task->description, original_task->description,
                 DG87_TASK_DESC_LENGTH);
  new_task->enabled = original_task->enabled;
  new_task->wait_all_dependencies = original_task->wait_all_dependencies;
  new_task->on_error_continue = original_task->on_error_continue;
  new_task->ref = NULL;
  /* set task queue */
  if (dg87_ref_task_set_queue (new_task, original_task->queue) == FALSE)
    {
      dg87_ref_task_delete (new_task);
      return NULL;
    }
  /* copy lock-variable properties */
  new_task->lock_variable = original_task->lock_variable;
  if (new_task->lock_variable != NULL)
    new_task->lock_variable->linked_ref_tasks++;
  new_task->lock_action = original_task->lock_action;
  /* copy cyclical properties */
  new_task->iteration = original_task->iteration;
  new_task->interval = original_task->interval;
  new_task->from_begin = original_task->from_begin;

  /* copy job_vector */
  element = original_task->job_vector->first;
  while (element != NULL)
    {
      original_job = element->object;
      element = element->next;
      /* duplicate job */
      new_job = dg87_ref_job_duplicate (new_task, original_job);
      if (new_job == NULL)
        {
          dg87_ref_task_delete (new_task);
          return NULL;
        }
      /* add it to job vector of the new task */
      if (dg87_vector_add_object (new_task->job_vector,
                                  new_job, new_job->name) == FALSE)
        {
          dg87_ref_job_delete (new_job);
          dg87_ref_task_delete (new_task);
          return NULL;
        }
    }
  /* copy job_dep_list */
  item = original_task->job_dep_list->first;
  while (item != NULL)
    {
      original_job_dep = item->object;
      item = item->next;
      upstream_job = dg87_vector_get_object_by_key (new_task->job_vector,
                                                    original_job_dep->
                                                    upstream_job->name);
      downstream_job =
        dg87_vector_get_object_by_key (new_task->job_vector,
                                       original_job_dep->downstream_job->
                                       name);
      if (upstream_job == NULL || downstream_job == NULL)
        {
          dg87_ref_task_delete (new_task);
          return NULL;
        }
      if ((new_jobdep = dg87_ref_task_jobdep_add (new_task, upstream_job,
                                                  downstream_job,
                                                  original_job_dep->type)) ==
          NULL)
        {
          dg87_ref_task_delete (new_task);
          return NULL;
        }
    }

  /* copy condition_list */
  item = original_task->condition_list->first;
  while (item != NULL)
    {
      original_condition = item->object;
      item = item->next;
      /* duplicate condition */
      new_condition = dg87_ref_condition_duplicate (original_condition);
      if (new_condition == NULL)
        {
          dg87_ref_task_delete (new_task);
          return NULL;
        }
      /* add it to condition list of the new task */
      if (dg87_list_add_object (new_task->condition_list,
                                new_condition) == FALSE)
        {
          dg87_ref_condition_delete (new_condition);
          dg87_ref_task_delete (new_task);
          return NULL;
        }
    }
  /* copy rule_operation_list */
  item = original_task->rule_operation_list->first;
  while (item != NULL)
    {
      original_operation = item->object;
      item = item->next;
      /* duplicate operation */
      new_operations = dg87_ref_operation_duplicate
        (original_operation, new_rule_vector);
      if (new_operations == NULL)
        {
          dg87_ref_task_delete (new_task);
          return NULL;
        }
      /* add it to ruleoperation list of the new task */
      if (dg87_ref_task_operation_add (new_task, new_operations) == FALSE)
        {
          dg87_ref_operation_delete (new_operations);
          dg87_ref_task_delete (new_task);
          return NULL;
        }
    }
	DG87_SAVE_NEEDED;
  return new_task;
}

/****************************************************************************/
BOOL
dg87_ref_task_delete (dg87_ref_task * task)
{
  dg87_ref_job *job;
  dg87_ref_jobdep *job_dep;
  dg87_ref_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* can't delete task if task has dependencies whith other task */
  if (task->upstream_taskdep_list != NULL &&
      task->upstream_taskdep_list->count > 0)
    {
      dg87_err_set_error (DG87_ERR_TASK_HAS_DEPENDENCES);
      return FALSE;
    }
  if (task->downstream_taskdep_list != NULL
      && task->downstream_taskdep_list->count > 0)
    {
      dg87_err_set_error (DG87_ERR_TASK_HAS_DEPENDENCES);
      return FALSE;
    }

  if (task->upstream_taskdep_list != NULL)
    dg87_list_delete (task->upstream_taskdep_list);
  if (task->downstream_taskdep_list != NULL)
    dg87_list_delete (task->downstream_taskdep_list);

  /* remove Task from Queue */
  if (task->queue != NULL)
    dg87_list_remove_object (task->queue->ref_task_list, task);

  /* decrement lock variable counter */
  if (task->lock_variable != NULL)
    task->lock_variable->linked_ref_tasks--;

  /* delete all jobdep objects */
  if (task->job_dep_list != NULL)
    {
      while (task->job_dep_list->count > 0)
        {
          job_dep = dg87_list_remove_object_by_index (task->job_dep_list, 0);
          dg87_ref_jobdep_delete (job_dep);
        }
      dg87_list_delete (task->job_dep_list);
    }

  /* delete all Job objects */
  if (task->job_vector != NULL)
    {
      while (task->job_vector->count > 0)
        {
          job = dg87_vector_remove_object_by_index (task->job_vector, 0);
          dg87_ref_job_delete (job);
        }
      dg87_vector_delete (task->job_vector);
    }

  /* delete all Condition object */
  if (task->condition_list != NULL)
    {
      while (task->condition_list->count > 0)
        {
          condition = dg87_list_remove_object_by_index
            (task->condition_list, 0);
          dg87_ref_condition_delete (condition);
        }
      dg87_list_delete (task->condition_list);
    }

  /* delete all rule_operation_list objects */
  if (task->rule_operation_list != NULL)
    {
      while (task->rule_operation_list->count > 0)
        {
          if (dg87_ref_task_operation_delete (task, 0) == FALSE)
            return FALSE;
        }
      dg87_list_delete (task->rule_operation_list);
    }

  dg87_free (task, DG87_TASK);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_load (dg87_ref_task * task,
                    dg87_storage * storage, dg87_context * context)
{
  dg87uint ui1, ui2;
  dg87ubyte b;
  char *string;
  dg87_target *target;
  dg87_queue *queue;
  dg87_variable *variable;
  assert (task != NULL && storage != NULL && context != NULL);
  if (task == NULL || storage == NULL || context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read task description */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_ref_task_set_description (task, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);

  /* read wait_all_dependencies */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_task_set_wait_all_dependencies (task, b) == FALSE)
    return FALSE;

  /* read on_error_continue */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_task_set_on_error_continue (task, b) == FALSE)
    return FALSE;

  /* read enabled property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_task_set_enabled (task, b) == FALSE)
    return FALSE;
  /* read cyclical properties */
  if (dg87_storage_read_uint (storage, &ui1) == FALSE)
    return FALSE;
  if (dg87_storage_read_uint (storage, &ui2) == FALSE)
    return FALSE;
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_task_set_cyclical (task, ui1, ui2, b) == FALSE)
    return FALSE;
  /* read lock properties */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (b == TRUE)
    {
      /* find variable */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((variable =
           dg87_engine_variable_get_by_name (context->engine,
                                             string)) == NULL)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      /* read lock_action */
      if (dg87_storage_read_ubyte (storage, &b) == FALSE)
        return FALSE;
      if (dg87_ref_task_set_lock (task, variable, b) == FALSE)
        return FALSE;
    }

  /* read target and queue */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (b == TRUE)
    {                           /* queue is not NULL */
      /* find target */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((target =
           dg87_context_target_get_by_name (context, string)) == NULL)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      /* find queue */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((queue = dg87_target_queue_get_by_name (target, string)) == NULL)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      /* set queue */
      if (dg87_ref_task_set_queue (task, queue) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_save (dg87_ref_task * task, dg87_storage * storage)
{
  assert (task != NULL && storage != NULL);
  if (task == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write task description */
  if (dg87_storage_write_string (storage, task->description) == FALSE)
    return FALSE;
  /* write wait_all_dependencies */
  if (dg87_storage_write_ubyte (storage, task->wait_all_dependencies) == FALSE)
    return FALSE;
  /* write on_error_continue */
  if (dg87_storage_write_ubyte (storage, task->on_error_continue) == FALSE)
    return FALSE;
  /* write enabled property */
  if (dg87_storage_write_ubyte (storage, task->enabled) == FALSE)
    return FALSE;
  /* write cyclical properties */
  if (dg87_storage_write_uint (storage, task->iteration) == FALSE)
    return FALSE;
  if (dg87_storage_write_uint (storage, task->interval) == FALSE)
    return FALSE;
  if (dg87_storage_write_ubyte (storage, task->from_begin) == FALSE)
    return FALSE;
  /* save lock properties */
  if (dg87_storage_write_ubyte (storage, (task->lock_variable == NULL) ?
                               FALSE : TRUE) == FALSE)
    return FALSE;
  if (task->lock_variable != NULL)
    {
      if (dg87_storage_write_string (storage,
                                     task->lock_variable->name) == FALSE)
        return FALSE;
      if (dg87_storage_write_ubyte (storage, task->lock_action) == FALSE)
        return FALSE;

    }

  /* save target and queue */
  if (dg87_storage_write_ubyte (storage,
                               (task->queue == NULL) ? FALSE : TRUE) == FALSE)
    return FALSE;
  if (task->queue != NULL)
    {
      if (dg87_storage_write_string (storage,
                                     task->queue->target->name) == FALSE)
        return FALSE;
      if (dg87_storage_write_string (storage, task->queue->name) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_operation_move (dg87_ref_task * task, int index, int delta)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return dg87_list_move_item (task->rule_operation_list, index, delta);
}

/****************************************************************************/
BOOL
dg87_ref_task_calculate (dg87_ref_task * task, time_t day, BOOL * result)
{
  assert (task != NULL && result != NULL);
  if (task == NULL || result == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (task->enabled == FALSE)
    {
      *result = FALSE;
      return TRUE;
    }
  return dg87_ref_operationlist_calculate (task->rule_operation_list, day,
                                           result, -1);
}

/****************************************************************************/
BOOL
dg87_ref_task_set_name (dg87_ref_task * task, char *new_name)
{
  assert (task != NULL && new_name != NULL);
  if (task == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_name (new_name, DG87_TASK_NAME_LENGTH) == FALSE)
    return FALSE;
  if (dg87_ref_model_task_rename (task->ref, task, new_name) == FALSE)
    return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_description (dg87_ref_task * task, char *description)
{
  assert (task != NULL && description != NULL);
  if (task == NULL || description == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_description (description, DG87_TASK_DESC_LENGTH) == FALSE)
    return FALSE;
  dg87_strnzcpy (task->description, description, DG87_TASK_DESC_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_cyclical (dg87_ref_task * task, dg87uint iteration,
                            dg87uint interval, BOOL from_begin)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->iteration = iteration;
  if (task->iteration == 1)
    {
      /* no cyclical task */
      interval = 0;
      from_begin = TRUE;
    }
  task->interval = interval;
  task->from_begin = from_begin;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_lock (dg87_ref_task * task, dg87_variable * variable,
                        BOOL lock_action)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* decrement variable counter */
  if (task->lock_variable != NULL)
    task->lock_variable->linked_ref_tasks--;
  task->lock_variable = variable;
  task->lock_action = lock_action;
  /* increment variable counter */
  if (task->lock_variable != NULL)
    task->lock_variable->linked_ref_tasks++;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_queue (dg87_ref_task * task, dg87_queue * queue)
{
  dg87_list *list;

  assert (task != NULL);
  if (task == NULL)
    {                           /* no test on queue, it can be null ! */
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  /* nothing to do, already done */
  if (task->queue == queue)
    return TRUE;
  /* remove task of previous target */
  if (task->queue != NULL)
    {
      list = task->queue->ref_task_list;
      dg87_list_remove_object (list, task);
    }
  /* add this task to ref_task_vector of queue */
  task->queue = queue;
  /* add task of actual Queue */
  if (task->queue != NULL)
    {
      list = task->queue->ref_task_list;
      dg87_list_add_object (list, task);
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_enabled (dg87_ref_task * task, BOOL enabled)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->enabled = enabled;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_wait_all_dependencies (dg87_ref_task * task, BOOL value)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->wait_all_dependencies = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_set_on_error_continue (dg87_ref_task * task, BOOL value)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->on_error_continue = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_ref_job *
dg87_ref_task_job_add (dg87_ref_task * task, char *job_name)
{
  dg87_ref_job *job;
  assert (task != NULL && job_name != NULL);
  if (task == NULL || job_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job = dg87_ref_job_new (task, job_name);
  if (job == NULL)
    return NULL;

  /* vector generate error if an other job has the same name */
  /* key must point on job->name */
  if (dg87_vector_add_object (task->job_vector, job, job->name) == FALSE)
    {
      dg87_ref_job_delete (job);
      return NULL;
    }
	DG87_SAVE_NEEDED;
  return job;
}

/****************************************************************************/
BOOL
dg87_ref_task_job_rename (dg87_ref_task * task, dg87_ref_job * job,
                          char *new_name)
{
  char old_name[DG87_JOB_NAME_LENGTH + 1];
  assert (task != NULL && job != NULL && new_name != NULL);
  if (task == NULL || job == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  if (strlen (new_name) > DG87_JOB_NAME_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  /* remove object from vector */
  if (dg87_vector_remove_object (task->job_vector, job) == FALSE)
    return FALSE;
  /* copy new name */
  dg87_strnzcpy (old_name, job->name, DG87_JOB_NAME_LENGTH);
  dg87_strnzcpy (job->name, new_name, DG87_JOB_NAME_LENGTH);
  /* add the object in the vector with the new key */
  if (dg87_vector_add_object (task->job_vector, job, job->name) == FALSE)
    {
      dg87_strnzcpy (job->name, old_name, DG87_JOB_NAME_LENGTH);
      dg87_vector_add_object (task->job_vector, job, job->name);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
int
dg87_ref_task_job_count (dg87_ref_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (task->job_vector);
}

/****************************************************************************/
dg87_ref_job *
dg87_ref_task_job_get_by_name (dg87_ref_task * task, char *name)
{
  assert (task != NULL && name != NULL);
  if (task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_key (task->job_vector, name);
}

/****************************************************************************/
dg87_ref_job *
dg87_ref_task_job_get_by_index (dg87_ref_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (task->job_vector, index);
}

/****************************************************************************/
BOOL
dg87_ref_task_job_delete (dg87_ref_task * task, dg87_ref_job * job)
{
  dg87_ref_jobdep *job_dep;
  dg87_item *item;
  assert (task != NULL && job != NULL);
  if (task == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* remove all dependencies link to this job */
  item = task->job_dep_list->first;
  while (item != NULL)
    {
      job_dep = item->object;
      if (job_dep->upstream_job == job || job_dep->downstream_job == job)
        {
          dg87_item *temp = item;
          item = item->next;
          dg87_list_remove_item (task->job_dep_list, temp);
          dg87_ref_jobdep_delete (job_dep);
        }
      else
        {
          item = item->next;
        }
    }
  if (dg87_vector_remove_object (task->job_vector, job) == FALSE)
    return FALSE;
  if (dg87_ref_job_delete (job) == FALSE)
    {
      /* add job in vector again */
      /* TODO les dpendances supprimes au dbut sont perdues */
      dg87_vector_add_object (task->job_vector, job, job->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_job_delete_by_name (dg87_ref_task * task, char *name)
{
  dg87_ref_job *job;
  assert (task != NULL && name != NULL);
  if (task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job = dg87_vector_get_object_by_key (task->job_vector, name);
  if (job == NULL)
    return FALSE;
  return dg87_ref_task_job_delete (task, job);
}

/****************************************************************************/
BOOL
dg87_ref_task_job_delete_by_index (dg87_ref_task * task, int index)
{
  dg87_ref_job *job;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job = dg87_vector_get_object_by_index (task->job_vector, index);
  if (job == NULL)
    return FALSE;
  return dg87_ref_task_job_delete (task, job);
}

/****************************************************************************/
dg87_ref_jobdep *
dg87_ref_task_jobdep_add (dg87_ref_task * task,
                          dg87_ref_job * upstream_job,
                          dg87_ref_job * downstream_job,
                          dg87_ref_jobdep_type detype)
{
  dg87_ref_jobdep *job_dep;

  assert (task != NULL && upstream_job != NULL && downstream_job != NULL);
  if (task == NULL || upstream_job == NULL || downstream_job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (upstream_job->task != task || downstream_job->task != task)
    {
      dg87_err_set_error (DG87_ERR_JOBS_NOT_IN_SAME_TASK);
      return NULL;
    }

  /* create and add the job dep */
  if ((job_dep =
       dg87_ref_jobdep_new (upstream_job, downstream_job, detype)) == NULL)
    return NULL;
  dg87_list_add_object (task->job_dep_list, job_dep);

  return job_dep;
}

/****************************************************************************/
int
dg87_ref_task_jobdep_count (dg87_ref_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->job_dep_list);
}

/****************************************************************************/
dg87_ref_jobdep *
dg87_ref_task_jobdep_get (dg87_ref_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->job_dep_list, index);
}

/****************************************************************************/
BOOL
dg87_ref_task_jobdep_delete_by_index (dg87_ref_task * task, int index)
{
  dg87_ref_jobdep *job_dep;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job_dep = dg87_list_remove_object_by_index (task->job_dep_list, index);
  if (job_dep == NULL)
    return FALSE;
  dg87_ref_jobdep_delete (job_dep);
  return TRUE;
}

/****************************************************************************/
int
dg87_ref_task_upstream_taskdep_count (dg87_ref_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->upstream_taskdep_list);
}

/****************************************************************************/
dg87_ref_taskdep *
dg87_ref_task_upstream_taskdep_get (dg87_ref_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->upstream_taskdep_list, index);
}

/****************************************************************************/
int
dg87_ref_task_downstream_taskdep_count (dg87_ref_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->downstream_taskdep_list);
}

/****************************************************************************/
dg87_ref_taskdep *
dg87_ref_task_downstream_taskdep_get (dg87_ref_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->downstream_taskdep_list, index);
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_hmin_add (dg87_ref_task * task, int minutes,
                                  dg87ubyte day_delta)
{
  dg87_ref_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_hmin_new (minutes, day_delta);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_hmax_add (dg87_ref_task * task, int minutes,
                                  dg87ubyte day_delta)
{
  dg87_ref_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_hmax_new (minutes, day_delta);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_hlate_add (dg87_ref_task * task, int minutes,
                                   dg87ubyte day_delta)
{
  dg87_ref_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_hlate_new (minutes, day_delta);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_file_add (dg87_ref_task * task, char *file_path)
{
  dg87_ref_condition *condition;
  assert (task != NULL && file_path != NULL);
  if (task == NULL || file_path == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_file_new (file_path);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_manual_add (dg87_ref_task * task, char *message)
{
  dg87_ref_condition *condition;
  assert (task != NULL && message != NULL);
  if (task == NULL || message == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_manual_new (message);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_ifequal_add (dg87_ref_task * task,
                                     dg87_variable * variable, char *value)
{
  dg87_ref_condition *condition;
  assert (task != NULL && variable != NULL && value != NULL);
  if (task == NULL || variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_ifequal_new (variable, value);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_ifnotequal_add (dg87_ref_task * task,
                                        dg87_variable * variable, char *value)
{
  dg87_ref_condition *condition;
  assert (task != NULL && variable != NULL && value != NULL);
  if (task == NULL || variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_ref_condition_ifnotequal_new (variable, value);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
int
dg87_ref_task_condition_count (dg87_ref_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->condition_list);
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_get (dg87_ref_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->condition_list, index);
}

/****************************************************************************/
BOOL
dg87_ref_task_condition_delete (dg87_ref_task * task,
                                dg87_ref_condition * condition)
{
  assert (task != NULL && condition != NULL);
  if (task == NULL || condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_remove_object (task->condition_list, condition) == FALSE)
    return FALSE;
  if (dg87_ref_condition_delete (condition) == FALSE)
    {
      dg87_list_add_object (task->condition_list, condition);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_task_condition_find (dg87_ref_task * task,
                              dg87_ref_condition_type type, char *data1,
                              char *data2)
{
  dg87_item *item;
  int minutes = 0;
  dg87_ref_condition *condition;
  assert (task != NULL && data1 != NULL);       /* data2 can be null */
  if (task == NULL || data1 == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* control second argument */
  switch (type)
    {
    case condition_hmin:
    case condition_hmax:
    case condition_hlate:
    case condition_ifequal:
    case condition_ifnequal:
      if (data2 == NULL)
        return NULL;
      break;
    case condition_file:
    case condition_manual:
      if (data2 != NULL)
        return NULL;
      break;
    case condition_error:
      return NULL;
    }
  /* for each condition of the task */
  item = task->condition_list->first;
  while (item != NULL)
    {
      condition = item->object;
      item = item->next;

      if (condition->type != type)
        continue;

      switch (condition->type)
        {

        case condition_hmin:
          minutes = dg87_ref_condition_hour_to_minute (data1);
          if (condition->data.condition_hmin->minutes != minutes)
            continue;
          if (condition->data.condition_hmin->day_delta != atoi (data2))
            continue;
          return condition;

        case condition_hmax:
          minutes = dg87_ref_condition_hour_to_minute (data1);
          if (condition->data.condition_hmax->minutes != minutes)
            continue;
          if (condition->data.condition_hmax->day_delta != atoi (data2))
            continue;
          return condition;

        case condition_hlate:
          minutes = dg87_ref_condition_hour_to_minute (data1);
          if (condition->data.condition_hlate->minutes != minutes)
            continue;
          if (condition->data.condition_hlate->day_delta != atoi (data2))
            continue;
          return condition;

        case condition_file:
          if (strcmp (condition->data.condition_File->path, data1) != 0)
            continue;
          return condition;

        case condition_manual:
          if (strcmp (condition->data.condition_manual->message, data1) != 0)
            continue;
          return condition;

        case condition_ifequal:
          if (strcmp
              (condition->data.condition_ifequal->variable->name, data1) != 0)
            continue;
          if (strcmp (condition->data.condition_ifequal->value, data2) != 0)
            continue;
          return condition;

        case condition_ifnequal:
          if (strcmp
              (condition->data.condition_ifnotequal->variable->name,
               data1) != 0)
            continue;
          if (strcmp (condition->data.condition_ifnotequal->value, data2)
              != 0)
            continue;
          return condition;

        case condition_error:  /* to remove compilators warning */
          return NULL;

        }                       /* end switch */
      item = item->next;
    }                           /* next condition */
  return NULL;
}

/****************************************************************************/
BOOL
dg87_ref_task_operation_add (dg87_ref_task * task,
                             dg87_ref_rule_operation * operation)
{
  assert (task != NULL && operation != NULL);
  if (task == NULL || operation == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* add the operation to the list */
  if (dg87_list_add_object (task->rule_operation_list, operation) == FALSE)
    {
      return FALSE;
    }
  /* increment linked rules counter for complex rule */
  if (operation->type == dg87_ref_rule_type_complex)
    operation->rule.rule->linked_tasks++;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_operation_delete (dg87_ref_task * task, int index)
{
  dg87_ref_rule_operation *operation;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* remove the operation from the list */
  operation = dg87_list_remove_object_by_index (task->rule_operation_list,
                                                index);
  if (operation == NULL)
    return FALSE;
  /* if complex rule, decrement linked_ref_tasks counter */
  if (operation->type == dg87_ref_rule_type_complex)
    operation->rule.rule->linked_tasks--;
  /* delete operation object */
  if (dg87_ref_operation_delete (operation) == FALSE)
    {
      dg87_list_add_object (task->rule_operation_list, operation);
      if (operation->type == dg87_ref_rule_type_complex)
        operation->rule.rule->linked_tasks++;
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_task_find_upstream_tasks (dg87_ref_task * task,
                                   dg87_list * task_list)
{
  dg87_ref_taskdep *taskdep;
  dg87_item *taskdep_item;
  assert (task != NULL && task_list != NULL);
  if (task == NULL || task_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (task_list, task) == FALSE)
    return FALSE;
  taskdep_item = task->upstream_taskdep_list->first;
  while (taskdep_item != NULL)
    {
      taskdep = taskdep_item->object;
      taskdep_item = taskdep_item->next;
      if (dg87_ref_task_find_upstream_tasks (taskdep->upstream_task,
                                             task_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_task_find_downstream_tasks (dg87_ref_task * task,
                                     dg87_list * task_list)
{
  dg87_ref_taskdep *taskdep;
  dg87_item *taskdep_item;
  assert (task != NULL && task_list != NULL);
  if (task == NULL || task_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (task_list, task) == FALSE)
    return FALSE;
  taskdep_item = task->downstream_taskdep_list->first;
  while (taskdep_item != NULL)
    {
      taskdep = taskdep_item->object;
      taskdep_item = taskdep_item->next;
      if (dg87_ref_task_find_downstream_tasks (taskdep->downstream_task,
                                               task_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_ref_task_check (dg87_ref_task * task, int (*print) (char *))
{
  dg87_element *element;
  dg87_item *item;
  dg87_ref_job *job;

  /* check name */
  if (strlen (task->name) == 0)
    {
      print ("Task : name length is zero !\n");
      return 1;
    }
  /* check enabled */
  if (task->enabled != TRUE && task->enabled != FALSE)
    {
      print ("Task : incorrect value for enabled property !\n");
      return 1;
    }
  /* check job_vector */
  if (task->job_vector == NULL)
    {
      print ("Task : JobVector is null !\n");
      return 1;
    }
  if (dg87_vector_check (task->job_vector, print) != 0)
    return 1;
  element = task->job_vector->first;
  while (element != NULL)
    {
      job = element->object;
      if (dg87_ref_job_check (job, print) != 0)
        return 1;
      if (job->task != task)
        {
          print ("Task : Job Task pointer incorrect !\n");
          return 1;
        }
      element = element->next;
    }

  /* check Jobdep list */
  if (task->job_dep_list == NULL)
    {
      print ("Task : Jobdep list is null !\n");
      return 1;
    }
  if (dg87_list_check (task->job_dep_list, print) != 0)
    return 1;
  item = task->job_dep_list->first;
  while (item != NULL)
    {
      if (dg87_ref_jobdep_check (item->object, print) != 0)
        return 1;
      item = item->next;
    }

  /* check Taskdep_before_list */
  if (task->upstream_taskdep_list == NULL)
    {
      print ("Task : Taskdep_before_list is null !\n");
      return 1;
    }
  if (dg87_list_check (task->upstream_taskdep_list, print) != 0)
    return 1;

  /* check Taskdep_after_list */
  if (task->downstream_taskdep_list == NULL)
    {
      print ("Task : Taskdep_after_list is null !\n");
      return 1;
    }
  if (dg87_list_check (task->downstream_taskdep_list, print) != 0)
    return 1;

  /* check Queue */
  if (task->queue != NULL)
    {
      if (dg87_list_find_item (task->queue->ref_task_list, task) == NULL)
        {
          print ("Task : task not in Queue.RefTaskList !\n");
          return 1;
        }
    }
  /* check condition_list */
  if (task->condition_list == NULL)
    {
      print ("Task : ConditionList is null !\n");
      return 1;
    }
  if (dg87_list_check (task->condition_list, print) != 0)
    return 1;
  item = task->condition_list->first;
  while (item != NULL)
    {
      if (dg87_ref_condition_check (item->object, print) != 0)
        return 1;
      item = item->next;
    }

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
