/* file d_task.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_TASK_H__
#define __DG87_REF_TASK_H__

#include "global.h"
#include "d_job.h"
#include "d_jobdep.h"
#include "d_taskdep.h"
#include "d_condition.h"
#include "d_rule.h"
#include "queue.h"
#include <libdg87/vector.h>
#include <libdg87/list.h>


struct _dg87_ref_task
{
  char name[DG87_TASK_NAME_LENGTH + 1];
  char description[DG87_TASK_DESC_LENGTH + 1];
  BOOL enabled;
  dg87_vector *job_vector;      /* jobs of the task */
  dg87_list *job_dep_list;      /* dependencies between jobs of the task */
  dg87_list *upstream_taskdep_list;
  dg87_list *downstream_taskdep_list;
  dg87_list *condition_list;
  dg87_list *rule_operation_list;
  dg87_queue *queue;            /* queue where to execute the task */
  dg87_ref_model *ref;
  BOOL wait_all_dependencies;
  BOOL on_error_continue;
  /* cyclical properties */
  dg87uint iteration;           /* number of iteration for cyclical tasks */
  dg87uint interval;            /* temporization between two executions */
  BOOL from_begin;              /* temporization starts from begin/end of task */
  /* lock properties */
  dg87_variable *lock_variable; /* variable to [un]lock */
  BOOL lock_action;             /* TRUE : lock, FALSE unlock */
};

/*
 * dg87_ref_task_new 
 * create a new task and initialize the name property
 */
dg87_ref_task *dg87_ref_task_new (char *name);

/*
 * dg87_ref_task_duplicate
 * create a new task by copying an other.
 * The 'new_name' parameter can be NULL.
 */
dg87_ref_task *dg87_ref_task_duplicate (dg87_ref_task * original_task,
                                        dg87_vector * new_rule_vector,
                                        char *new_name);

/*
 * dg87_ref_task_delete
 */
BOOL dg87_ref_task_delete (dg87_ref_task * task);

/*
 * dg87_ref_task_load
 */
BOOL dg87_ref_task_load (dg87_ref_task * task, dg87_storage * storage,
                         dg87_context * context);

/*
 * dg87_ref_task_save
 */
BOOL dg87_ref_task_save (dg87_ref_task * task, dg87_storage * storage);

/*
 * dg87_ref_task_operation_move
 */
BOOL dg87_ref_task_operation_move (dg87_ref_task * task, int index,
                                   int delta);

/*
 * dg87_ref_task_calculate
 */
BOOL dg87_ref_task_calculate (dg87_ref_task * task, time_t day,
                              BOOL * result);

/*
 * dg87_ref_task_set_name
 * Change the name of a task.
 * An error occurres if another task has the same name in the referential.
 */
BOOL dg87_ref_task_set_name (dg87_ref_task * task, char *name);

/*
 * dg87_ref_task_set_description
 * set the description of the task.
 */
BOOL dg87_ref_task_set_description (dg87_ref_task * task, char *description);

/*
 * dg87_ref_task_set_cyclical
 * set cyclical properties of a task.
 * Arguments are :
 * - the number of iterations (unlimited if zero, no cyclical if one)
 * - the number of seconds for temporization between two executions
 * - a boolean value which indicates if the temporization start from begin
 * or from end of the task
 */
BOOL dg87_ref_task_set_cyclical (dg87_ref_task * task, dg87uint iteration,
                                 dg87uint interval, BOOL from_begin);

/*
 * dg87_ref_task_set_lock
 * set lock properties of the task.
 * If variable is NULL, task doesn't use a variable lock.
 * If lock_action is TRUE, variable must lock variable before starting.
 * if lock_action is FALSE, variable must unlock variable just after ending.
 *
 */
BOOL dg87_ref_task_set_lock (dg87_ref_task * task, dg87_variable * variable,
                             BOOL lock_action);

/*
 * dg87_ref_task_set_queue
 * set the queue where the job of the task will be execute.
 */
BOOL dg87_ref_task_set_queue (dg87_ref_task * task, dg87_queue * queue);

/*
 * dg87_ref_task_set_enabled
 */
BOOL dg87_ref_task_set_enabled (dg87_ref_task * task, BOOL enabled);

/*
 * dg87_ref_task_set_wait_all_dependencies
 */
BOOL dg87_ref_task_set_wait_all_dependencies (dg87_ref_task * task,
                                              BOOL value);

/*
 * dg87_ref_task_set_on_error_continue
 */
BOOL dg87_ref_task_set_on_error_continue (dg87_ref_task * task, BOOL value);

/*************************************************************************/
/*  functions for job                                                    */
/*************************************************************************/

/*
 * dg87_ref_task_job_add
 * create a job object, add this object in the JobVector
 * of the task and return a pointer on this object.
 * Return NULL if the Job object can't be create.
 */
dg87_ref_job *dg87_ref_task_job_add (dg87_ref_task * task, char *job_name);

/*
 * dg87_ref_task_job_rename
 * You don't have to use this function. Use dg87_ref_job_set_name instead.
 * This function is called by dg87_ref_job_set_name to change the name of a job.
 * It returns FALSE if another job as the same name.
 */
BOOL dg87_ref_task_job_rename (dg87_ref_task * task, dg87_ref_job * job,
                               char *new_name);


/*
 * dg87_ref_task_job_count
 * return the number of job in the task.
 * return -1 if task is NULL.
 */
int dg87_ref_task_job_count (dg87_ref_task * task);


/*
 * dg87_ref_task_job_get_by_name
 * return a pointer on the job which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_ref_job *dg87_ref_task_job_get_by_name (dg87_ref_task * task,
                                             char *name);

/*
 * dg87_ref_task_job_get_by_index
 * return a pointer on the job at the given index.
 */
dg87_ref_job *dg87_ref_task_job_get_by_index (dg87_ref_task * task,
                                              int index);


/*
 * dg87_ref_task_job_delete_by_name
 * delete the job which has the name given in parameter.
 */
BOOL dg87_ref_task_job_delete_by_name (dg87_ref_task * task, char *name);

/*
 * dg87_ref_task_job_delete_by_index
 * delete the job at the given index.
 */
BOOL dg87_ref_task_job_delete_by_index (dg87_ref_task * task, int index);

/*
 * dg87_ref_task_job_delete
 */
BOOL dg87_ref_task_job_delete (dg87_ref_task * task, dg87_ref_job * job);

/*************************************************************************/
/*  functions for job dependencies                                        */
/*************************************************************************/

/*
 * dg87_ref_task_job_depAdd
 * add a dependence between two jobs of the task. The name of the jobs must
 * be pass in parameters.
 */
dg87_ref_jobdep *dg87_ref_task_jobdep_add (dg87_ref_task * task,
                                           dg87_ref_job * upstream_job,
                                           dg87_ref_job * downstream_job,
                                           dg87_ref_jobdep_type dep_type);

/*
 * dg87_ref_task_jobdep_count
 * return the number of jobdep the task.
 * return -1 if task is NULL.
 */
int dg87_ref_task_jobdep_count (dg87_ref_task * task);

/*
 * dg87_ref_task_jobdep_get
 * return a pointer on the jobdep at the given index.
 */
dg87_ref_jobdep *dg87_ref_task_jobdep_get (dg87_ref_task * task, int index);

/*
 * dg87_ref_task_jobdep_delete_by_index
 * delete the jobdep at the given index.
 */
BOOL dg87_ref_task_jobdep_delete_by_index (dg87_ref_task * task, int index);


/*************************************************************************/
/*  functions for taskdep dependencies                                      */
/*************************************************************************/

/*
 * dg87_ref_task_upstream_taskdep_count
 * return the number of taskdep "before" the task.
 * return -1 if task is NULL.
 */
int dg87_ref_task_upstream_taskdep_count (dg87_ref_task * task);

/*
 * dg87_ref_task_upstream_taskdep_get
 * return a pointer on the taskdep "before" at the given index.
 */
dg87_ref_taskdep *dg87_ref_task_upstream_taskdep_get (dg87_ref_task * task,
                                                      int index);

/*
 * dg87_ref_task_downstream_taskdep_count
 * return the number of taskdep "after" the task.
 * return -1 if task is NULL.
 */
int dg87_ref_task_downstream_taskdep_count (dg87_ref_task * task);

/*
 * dg87_ref_task_downstream_taskdep_get
 * return a pointer on the taskdep "after" at the given index.
 */
dg87_ref_taskdep *dg87_ref_task_downstream_taskdep_get (dg87_ref_task * task,
                                                        int index);

/*************************************************************************/
/*  functions for condition list                                         */
/*************************************************************************/

/*
 * dg87_ref_task_condition_hmin_add
 */
dg87_ref_condition *dg87_ref_task_condition_hmin_add (dg87_ref_task * task,
                                                      int minutes,
                                                      dg87ubyte day_delta);

/*
 * dg87_ref_task_condition_hmax_add
 */
dg87_ref_condition *dg87_ref_task_condition_hmax_add (dg87_ref_task * task,
                                                      int minutes,
                                                      dg87ubyte day_delta);

/*
 * dg87_ref_task_condition_hlate_add
 */
dg87_ref_condition *dg87_ref_task_condition_hlate_add (dg87_ref_task * task,
                                                       int minutes,
                                                       dg87ubyte day_delta);

/*
 * dg87_ref_task_condition_file_add
 */
dg87_ref_condition *dg87_ref_task_condition_file_add (dg87_ref_task * task,
                                                      char *file_path);

/*
 * dg87_ref_task_condition_manual_add
 */
dg87_ref_condition *dg87_ref_task_condition_manual_add (dg87_ref_task * task,
                                                        char *message);

/*
 * dg87_ref_task_condition_ifequal_add
 */
dg87_ref_condition *dg87_ref_task_condition_ifequal_add (dg87_ref_task * task,
                                                         dg87_variable *
                                                         variable,
                                                         char *value);
/*
 * dg87_ref_task_condition_ifnotequal_add
 */
dg87_ref_condition *dg87_ref_task_condition_ifnotequal_add (dg87_ref_task *
                                                            task,
                                                            dg87_variable *
                                                            variable,
                                                            char *value);

/*
 * dg87_ref_task_condition_count
 * return the number of condition in the task.
 * return -1 if task is NULL.
 */
int dg87_ref_task_condition_count (dg87_ref_task * task);

/*
 * dg87_ref_task_condition_get
 * return a pointer on the condition at the given index.
 */
dg87_ref_condition *dg87_ref_task_condition_get (dg87_ref_task * task,
                                                 int index);


/*
 * dg87_ref_task_condition_delete
 * delete the condition object at the given index.
 */
BOOL dg87_ref_task_condition_delete (dg87_ref_task * task,
                                     dg87_ref_condition * condition);

/*
 * dg87_ref_task_condition_find
 */
dg87_ref_condition *dg87_ref_task_condition_find (dg87_ref_task * task,
                                                  dg87_ref_condition_type
                                                  type, char *data1,
                                                  char *data2);

/*************************************************************************/
/*  functions for ruleOperation list                                     */
/*************************************************************************/

/*
 * dg87_ref_task_operation_add
 */
BOOL dg87_ref_task_operation_add (dg87_ref_task * task,
                                  dg87_ref_rule_operation * operation);

/*
 * dg87_ref_task_operation_delete
 */
BOOL dg87_ref_task_operation_delete (dg87_ref_task * task, int index);

/*
 * dg87_ref_task_find_upstream_tasks
 * This function adds to the list given in parameter all upstream tasks
 * of a task.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_ref_task_find_upstream_tasks (dg87_ref_task * task,
                                        dg87_list * task_list);

/*
 * dg87_ref_task_find_downstream_tasks
 * This function adds to the list given in parameter all downstream tasks
 * of a task.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_ref_task_find_downstream_tasks (dg87_ref_task * task,
                                          dg87_list * task_list);

#ifdef DEBUG
/*
 * dg87_ref_task_ckeck
 */
int dg87_ref_task_check (dg87_ref_task * task, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_TASK_H__*/
/* End of file */
