/* file d_taskdep.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "d_task.h"
#include "d_taskdep.h"

/*****************************************************************************/
dg87_ref_taskdep *
dg87_ref_taskdep_new (dg87_ref_task * upstream_task,
                      dg87_ref_task * downstream_task,
                      dg87_ref_taskdep_type type)
{
  dg87_ref_taskdep *taskdep;
  dg87_list *upstream_task_list;
  dg87_list *downstream_task_list;
  assert (upstream_task != NULL && downstream_task != NULL);
  if (upstream_task == NULL || downstream_task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* find all upstream tasks */
  upstream_task_list = dg87_list_new ();
  if (upstream_task_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  dg87_ref_task_find_upstream_tasks (upstream_task, upstream_task_list);
  /* find all downstream tasks */
  downstream_task_list = dg87_list_new ();
  if (downstream_task_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  dg87_ref_task_find_downstream_tasks (downstream_task, downstream_task_list);
  /* detect circular dependencies */
  if (dg87_list_intersection (downstream_task_list, upstream_task_list) ==
      TRUE)
    {
      dg87_list_delete (downstream_task_list);
      dg87_list_delete (upstream_task_list);
      dg87_err_set_error (DG87_ERR_CIRCULAR_DEPENDENCE);
      return NULL;
    }
  dg87_list_delete (downstream_task_list);
  dg87_list_delete (upstream_task_list);

  /* allocate memory */
  if ((taskdep =
       dg87_malloc (sizeof (dg87_ref_taskdep), DG87_TASKDEP)) == NULL)
    goto alloc_error;
  /* initialize structure */
  taskdep->upstream_task = upstream_task;
  taskdep->downstream_task = downstream_task;
  taskdep->type = type;
	DG87_SAVE_NEEDED;
  return taskdep;

alloc_error:
  if (taskdep != NULL)
    dg87_ref_taskdep_delete (taskdep);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/*****************************************************************************/
BOOL
dg87_ref_taskdep_delete (dg87_ref_taskdep * taskdep)
{
  assert (taskdep != NULL);
  if (taskdep == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* remove taskdep from vectors */
  if (taskdep->upstream_task != NULL)
    dg87_list_remove_object (taskdep->upstream_task->downstream_taskdep_list,
                             taskdep);
  if (taskdep->downstream_task != NULL)
    dg87_list_remove_object (taskdep->downstream_task->upstream_taskdep_list,
                             taskdep);

  dg87_free (taskdep, DG87_TASKDEP);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_taskdep_load (dg87_ref_taskdep * taskdep, dg87_storage * storage)
{
  dg87ubyte b;
  assert (taskdep != NULL && storage != NULL);
  if (taskdep == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read dependence type */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_ref_taskdep_set_type (taskdep, b) == FALSE)
    return FALSE;

  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_taskdep_save (dg87_ref_taskdep * taskdep, dg87_storage * storage)
{
  assert (taskdep != NULL && storage != NULL);
  if (taskdep == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write dependence type */
  if (dg87_storage_write_ubyte (storage, taskdep->type) == FALSE)
    return FALSE;
  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_ref_taskdep_set_type (dg87_ref_taskdep * taskdep,
                           dg87_ref_taskdep_type type)
{
  assert (taskdep != NULL);
  if (taskdep == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  taskdep->type = type;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/*****************************************************************************/
dg87_ref_taskdep_type
dg87_ref_taskdep_ascii_to_type (char *str_type)
{
  assert (str_type != NULL);
  if (str_type == NULL)
    return dg87_ref_taskdep_unknown;
  if (strcasecmp (str_type, "always") == 0)
    return dg87_ref_taskdep_always;
  else if (strcasecmp (str_type, "onsuccess") == 0)
    return dg87_ref_taskdep_on_success;
  else if (strcasecmp (str_type, "onerror") == 0)
    return dg87_ref_taskdep_on_error;
  return dg87_ref_taskdep_unknown;
}

/*****************************************************************************/
char *
dg87_ref_taskdep_type_to_ascii (dg87_ref_taskdep_type dep_type)
{
  switch (dep_type)
    {
    case dg87_ref_taskdep_always:
      return "always";
    case dg87_ref_taskdep_on_success:
      return "onsuccess";
    case dg87_ref_taskdep_on_error:
      return "onerror";
    default:
      return "unknown";         /* to remove warning */
    }
  return NULL;
}

/*****************************************************************************/
#ifdef DEBUG
int
dg87_ref_taskdep_check (dg87_ref_taskdep * taskdep, int (*print) (char *))
{
  /* check upstream_task */
  if (taskdep->upstream_task == NULL)
    {
      print ("taskdep : UpStream task is NULL !\n");
      return 1;
    }
  if (dg87_list_find_item (taskdep->upstream_task->downstream_taskdep_list,
                           taskdep) == NULL)
    {
      print ("Taskdep : not found in TaskBefore->TaskdepAfter !\n");
      return 1;
    }
  /* check TaskAfter */
  if (taskdep->downstream_task == NULL)
    {
      print ("taskdep : TaskAfter is NULL !\n");
      return 1;
    }
  if (dg87_list_find_item (taskdep->downstream_task->upstream_taskdep_list,
                           taskdep) == NULL)
    {
      print ("Taskdep : not found in TaskAfter->TaskdepBefore !\n");
      return 1;
    }
  return 0;
}
#endif /*DEBUG*/
/*****************************************************************************/
/* End of file */
