/* file engine.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <sys/time.h>
#include <assert.h>
#include "engine.h"
#include "variable.h"
#include <config.h>
#include "x_day.h"
#include "r_job.h"
#include "r_file.h"
#include "r_caller.h"
#include "l_day.h"
#include "x_caller.h"

/* local function prototypes */
static BOOL dg87_engine_create_day (dg87_engine * engine,
                                    dg87_ref_model * ref, BOOL enabled);
/****************************************************************************/
static BOOL
dg87_engine_clear_log (dg87_engine * engine)
{
  assert (engine != NULL);
  if (engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_strbuf_cpy (engine->log, "") == FALSE)
    return FALSE;
  return TRUE;
}


/****************************************************************************/
dg87_engine *
dg87_engine_new (dg87_context * context)
{
  struct tm dc;
  dg87_engine *engine;
  assert (context != NULL);
  if (context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((engine = dg87_malloc (sizeof (dg87_engine), DG87_ENGINE)) == NULL)
    goto alloc_error;
  if ((engine->variable_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  /* initialize structure */
  engine->context = context;
  engine->enabled = TRUE;
  if ((engine->log = dg87_strbuf_new ()) == NULL)
    goto alloc_error;
  if ((engine->new_day_name = dg87_strdup ("", DG87_CHAR)) == NULL)
    goto alloc_error;
  engine->new_day_start_time = 0;
  /* next auto plan : next day at midnight */
  engine->next_auto_plan_time = dg87_current_time + 24 * 3600;
  dc = *localtime (&engine->next_auto_plan_time);
  dc.tm_hour = 0;
  dc.tm_min = 0;
  dc.tm_sec = 0;
  engine->next_auto_plan_time = mktime (&dc);
  engine->next_auto_purge_time = mktime (&dc);
  engine->check_time = dg87_current_time;
	DG87_SAVE_NEEDED;
  return engine;

alloc_error:
  if (engine != NULL)
    dg87_engine_delete (engine);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_engine_delete (dg87_engine * engine)
{
  dg87_variable *variable;
  assert (engine != NULL);
  if (engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (engine->new_day_name != NULL)
    dg87_free (engine->new_day_name, DG87_CHAR);
  if (engine->log != NULL)
    dg87_strbuf_delete (engine->log);
  if (engine->variable_vector != NULL)
    {
      while (engine->variable_vector->count > 0)
        {
          variable =
            dg87_vector_remove_object_by_index (engine->variable_vector, 0);
          /* force variable deletion */
          variable->linked_rules = 0;
          variable->linked_ref_conditions = 0;
          variable->linked_exp_conditions = 0;
          variable->linked_ref_tasks = 0;
          variable->linked_exp_tasks = 0;
          dg87_variable_delete (variable);
        }
      dg87_vector_delete (engine->variable_vector);
    }
  dg87_free (engine, DG87_ENGINE);
  return TRUE;
}

/******************************************************************************/
BOOL dg87_engine_set_enabled (dg87_engine * engine, BOOL value)
{
  if (engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	if (engine->enabled == value)
					return TRUE;
	engine->enabled = value;
	DG87_SAVE_NEEDED;
	return TRUE;
}
/******************************************************************************/
/* dg87_engine_replace_variables
 * this functions parses InBuffer to replace variables $<VarName> by
 * its value.
 * If a variable is not declared, the $<VarName> is copied.
 */
char *
dg87_engine_replace_variables (dg87_engine * engine, char *in_buffer)
{
  char *sub_beg, *sub_end;
  char *var_beg, *var_end;
  char *buffer;
  char *var_name;
  dg87_variable *variable;
  dg87_element *element;
  dg87_strbuf *out_buf;
  int var_size;
  assert (in_buffer != NULL && engine != NULL);
  if (in_buffer == NULL || engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* optimization if no variables */
  if (strstr (in_buffer, "$<") == NULL)
    return dg87_strdup (in_buffer, DG87_CHAR);

  /* initialize out buffer */
  out_buf = dg87_strbuf_new ();
  if (out_buf == NULL)
    return NULL;

  /* substring points on begin of string */
  sub_beg = in_buffer;
  sub_end = sub_beg;

  while (*sub_beg != '\0')
    {
      variable = NULL;
      var_beg = NULL;
      var_end = NULL;

      /* find begin of variable */
      if ((var_beg = strstr (sub_end, "$<")) == NULL)
        {
          /* skip to end of string */
          sub_end = strchr (sub_end, '\0');
        }
      /* find end of variable */
      else if ((var_end = strchr (var_beg, '>')) == NULL)
        {
          /* skip to end of string */
          sub_end = strchr (sub_end, '\0');
        }
      else
        {
          var_size = var_end - var_beg - 2;
          /* extract variable name */
          var_name = dg87_malloc (var_size + 1, DG87_CHAR);
          if (var_name == NULL)
            {
              dg87_strbuf_delete (out_buf);
              return NULL;
            }
          dg87_strnzcpy (var_name, var_beg + 2, var_end - var_beg - 2);
          var_name[var_size] = '\0';

          /* find variable */
          element = engine->variable_vector->first;
          while (element != NULL)
            {
              variable = element->object;
              if (strcmp (variable->name, var_name) == 0)
                break;
              variable = NULL;
              element = element->next;
            }
          dg87_free (var_name, DG87_CHAR);

          /* unknown variable */
          if (variable == NULL)
            {
              /* search next variable */
              sub_end = var_end;
              continue;
            }
          /* end of substring is start of variable */
          sub_end = var_beg;
        }

      /* copy substring */
      if (dg87_strbuf_ncat (out_buf, sub_beg, sub_end - sub_beg) == FALSE)
        {
          dg87_strbuf_delete (out_buf);
          return NULL;
        }
      sub_beg = sub_end;

      /* continue if no variable value to copy */
      if (variable == NULL)
        continue;

      /* copy variable value */
      if (dg87_strbuf_cat (out_buf, variable->value) == FALSE)
        {
          dg87_strbuf_delete (out_buf);
          return NULL;
        }

      /* next sub string starts after end of variable */
      sub_beg = var_end + 1;
      sub_end = sub_beg;

    }
  buffer = dg87_strdup (out_buf->buffer, DG87_CHAR);
  dg87_strbuf_delete (out_buf);
  return buffer;
}


/****************************************************************************/
#ifdef DEBUG
int
dg87_engine_check (dg87_engine * engine, int (*print) (char *))
{
  if (engine->new_day_name == NULL)
    {
      print ("Engine : newDayName is null !\n");
      return 1;
    }
  if (engine->context == NULL)
    {
      print ("Engine : context is null !\n");
      return 1;
    }
  if (engine->log == NULL)
    {
      print ("Engine : Log is null !\n");
      return 1;
    }
  /* TODO check variables */
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/*
 * dg87_engine_translate_dependence
 * This function creates a dependence between DownStreamTask and UpStreamTask
 * if upstreamTask is planned.
 * Otherwise, the function tries to create dependencies between the 
 * downStreamTask and all upstream tasks of the UpStreamTask.
 * The function returns FALSE if an error occures.
 */
static BOOL
dg87_engine_translate_dependence (dg87_engine * engine,
                                  dg87_vector * task_to_plan_vector,
                                  dg87_list * taskdep_to_plan_list,
                                  dg87_ref_task * downstream_task,
                                  dg87_ref_task * upstream_task,
                                  dg87_ref_taskdep_type type)
{
  dg87_item *item;
  dg87_ref_taskdep *taskdep;
  assert (engine != NULL && task_to_plan_vector != NULL &&
          taskdep_to_plan_list != NULL &&
          downstream_task != NULL && upstream_task != NULL);
  if (engine == NULL || task_to_plan_vector == NULL ||
      taskdep_to_plan_list == NULL ||
      downstream_task == NULL || upstream_task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_strbuf_printf (engine->log,
                      gettext ("Translate dependence : %s -> %s \n"),
                      upstream_task->name, downstream_task->name);

  /* if UpStreamTask isn't planned */
  if (dg87_vector_get_element_by_key (task_to_plan_vector,
                                      upstream_task->name) == NULL)
    {

      /* the UpStreamTask isn't planned */
      /* for each upstream task */
      item = upstream_task->upstream_taskdep_list->first;
      while (item != NULL)
        {
          taskdep = item->object;
          item = item->next;
          /* skip dependence which type is on_error */
          if (taskdep->type == dg87_ref_taskdep_on_error)
            continue;
          /* translate dependence on the upstream task */
          if (dg87_engine_translate_dependence (engine,
                                                task_to_plan_vector,
                                                taskdep_to_plan_list,
                                                downstream_task,
                                                taskdep->upstream_task,
                                                type) == FALSE)
            return FALSE;
        }
    }
  else
    {
      /* create a virtual task dependence */
      if ((taskdep = dg87_ref_taskdep_new (upstream_task, downstream_task,
                                           type)) == NULL)
        return FALSE;

      /* Add the new taskdep to the list */
      if (dg87_list_add_object (taskdep_to_plan_list, taskdep) == FALSE)
        {
          dg87_ref_taskdep_delete (taskdep);
          return FALSE;
        }
    }
  return TRUE;
}

/****************************************************************************/
static BOOL
dg87_engine_create_day (dg87_engine * engine,
                        dg87_ref_model * ref_to_plan, BOOL enabled)
{
  dg87_list *taskdep_to_plan_list = NULL;
  dg87_vector *task_to_plan_vector = NULL;
  dg87_exp_day *exp_day = NULL;
  dg87_exp_task *exp_task;
  dg87_exp_job *exp_job;
  dg87_exp_job *exp_upstream_job, *exp_downstream_job;
  dg87_exp_task *exp_upstream_task, *exp_downstream_task;
  dg87_ref_task *ref_task;
  dg87_ref_job *ref_job;
  dg87_ref_condition *ref_condition;
  dg87_ref_jobdep *ref_job_dep;
  dg87_ref_taskdep *ref_task_dep;
  dg87_element *task_element, *job_element;
  dg87_item *condition_item, *jobdep_item, *taskdep_item;
  dg87_log_day *log_day;
  dg87_log_node task_node;
  dg87_log_node job_node;
  char *buffer = NULL;
  time_t day;
  struct tm dc;
  BOOL upstream_task_selected;
  BOOL downstream_task_selected;
  BOOL result;
  int n_jobs = 0, n_tasks = 0;
	char* step_msg = NULL;
  assert (engine != NULL);
  assert (ref_to_plan != NULL);

  if ((task_to_plan_vector = dg87_vector_new ()) == NULL)
    goto error;
  if ((taskdep_to_plan_list = dg87_list_new ()) == NULL)
    goto error;

  dg87_strbuf_printf (engine->log, 
									gettext ("New day : %s\n"), engine->new_day_name);
  buffer = dg87_date_to_ascii (engine->new_day_start_time);
  dg87_strbuf_printf (engine->log, 
									gettext ("Start time : %s\n"), buffer);
  dg87_free (buffer, DG87_CHAR);
  buffer = NULL;

  /* get the date of the day */
  day = engine->new_day_start_time;
  dc = *localtime (&day);
  dc.tm_hour = 0;
  dc.tm_min = 0;
  dc.tm_sec = 0;
  day = mktime (&dc);

  /* for each task of the referential */
  task_element = ref_to_plan->task_vector->first;
  while (task_element != NULL)
    {
      ref_task = task_element->object;
      task_element = task_element->next;

      /* check if task must be planed */
      result = FALSE;
      if (dg87_ref_task_calculate (ref_task, day, &result) == FALSE)
        goto error;
      /* skip tasks which are not selected */
      if (result == FALSE)
        continue;
      /* Add task to plan to the list */
      if (dg87_vector_add_object (task_to_plan_vector,
                                  ref_task, ref_task->name) == FALSE){
				step_msg = gettext ("while adding a task to the list");
        goto error;
			}
    }

  /* remove tasks that can't be planed due to 'on_error' dependence */
  task_element = task_to_plan_vector->first;
  while (task_element != NULL)
    {
      ref_task = task_element->object;
      task_element = task_element->next;

      /*no problem with tasks which don't have upstream dependencies */
      if (ref_task->upstream_taskdep_list->count == 0)
        continue;

      /* for each upstream dependence of the task */
      result = TRUE;
      taskdep_item = ref_task->upstream_taskdep_list->first;
      while (taskdep_item != NULL)
        {
          ref_task_dep = taskdep_item->object;
          taskdep_item = taskdep_item->next;
          /* a task which has one (or more) 'on_success' or
           * 'always' dependence musn't be deleted.*/
          if (ref_task_dep->type == dg87_ref_taskdep_on_success ||
              ref_task_dep->type == dg87_ref_taskdep_always)
            {
              result = FALSE;
              break;
            }
          /* a task which has one (or more) 'onerror' dependence
           * which upstream task is selected musn't be deleted */
          if (ref_task_dep->type == dg87_ref_taskdep_on_error &&
              dg87_vector_get_element_by_key (task_to_plan_vector,
                                              ref_task_dep->upstream_task->
                                              name) != NULL)
            {
              result = FALSE;
              break;
            }
        }
      if (result == FALSE)
        continue;

      /* task must be removed */
      dg87_strbuf_printf (engine->log,
                          gettext ("WARNING : the task '%s' can't be planed,"\
																	"due to 'onerror' dependence problem.\n"),
                          ref_task->name);
      if (dg87_vector_remove_object (task_to_plan_vector, ref_task) == FALSE){
				step_msg = "while removing a task from the list";
        goto error;
			}
    }

  /* for each taskdep of the referential */
  taskdep_item = ref_to_plan->taskdep_list->first;
  while (taskdep_item != NULL)
    {
      ref_task_dep = taskdep_item->object;
      taskdep_item = taskdep_item->next;
      /* test if upstream and downstream task are selected */
      upstream_task_selected =
        (dg87_vector_get_element_by_key (task_to_plan_vector,
                                         ref_task_dep->upstream_task->name) ==
         NULL) ? FALSE : TRUE;
      downstream_task_selected =
        (dg87_vector_get_element_by_key
         (task_to_plan_vector,
          ref_task_dep->downstream_task->name) == NULL) ? FALSE : TRUE;

      /* skip dependence if downstream task isn't selected */
      if (downstream_task_selected == FALSE)
        continue;

      /* on_error dependence can't be translated */
      if (upstream_task_selected == FALSE &&
          ref_task_dep->type == dg87_ref_taskdep_on_error)
        {
          dg87_strbuf_printf (engine->log,
                              gettext ("WARNING : There is a '%s' dependence "\
																			"between task '%s' and '%s', and the "\
																			"upstream task is not selected !\n"),
                              dg87_ref_taskdep_type_to_ascii (ref_task_dep->
                                                              type),
                              ref_task_dep->upstream_task->name,
                              ref_task_dep->downstream_task->name);
          continue;
        }

      /* create dependence with upstream tasks */
      if (dg87_engine_translate_dependence (engine,
                                            task_to_plan_vector,
                                            taskdep_to_plan_list,
                                            ref_task_dep->downstream_task,
                                            ref_task_dep->upstream_task,
                                            ref_task_dep->type) == FALSE)
			{
				step_msg = gettext ("while translating a dependence");
        goto error;
			}
    }

  assert (engine != NULL);

  /* if the log day doesn't exist yet */
  if ((log_day = dg87_context_log_day_get_by_name (engine->context,
                                                   engine->new_day_name)) ==
      NULL)
    {
      /* create the log day */
      if ((log_day = dg87_context_log_day_add (engine->context,
                                               engine->new_day_name)) == NULL)
        {
					step_msg = gettext ("while creating a log day");
          goto error;
        }
    }
  /* for each task to plan */
  task_element = task_to_plan_vector->first;
  while (task_element != NULL)
    {
      ref_task = task_element->object;
      task_element = task_element->next;
      /* if the task log node doesn't exist yet */
      if (dg87_log_day_task_get_by_name (log_day, ref_task->name,
                                         &task_node) == FALSE)
        {
          /* create the log node for the task */
          if (dg87_log_day_task_add (log_day, ref_task->name) == FALSE)
            {
							step_msg = gettext ("while creating a log entry for a task");
              goto error;
            }
          if (dg87_log_day_task_get_by_name (log_day, ref_task->name,
                                             &task_node) == FALSE){
						step_msg = gettext ("while searching a log entry for a task");
            goto error;
					}
        }
      /* for each job of the task */
      job_element = ref_task->job_vector->first;
      while (job_element != NULL)
        {
          ref_job = job_element->object;
          job_element = job_element->next;
          /* if the job log node doesn't exist */
          if (dg87_log_day_job_get_by_name (log_day, &task_node,
                                            ref_job->name,
                                            &job_node) == FALSE)
            {
              /* create a log entry for the job */
              if (dg87_log_day_job_add (log_day, &task_node,
                                        ref_job->name) == FALSE)
                {
									step_msg = gettext ("while creating a log entry for a job");
                  goto error;
                }
            }
        }                       /* next job */
    }                           /* next task */

  /* create the exploitation day */
  if ((exp_day = dg87_context_exp_day_add (engine->context,
                                           engine->new_day_name,
                                           engine->new_day_start_time)) ==
      NULL)
    {
			step_msg = gettext ("while creating the exploitation day");
      goto error;
    }

  /* set the referential name */
  if (dg87_exp_day_set_referential_name (exp_day, ref_to_plan->name) == FALSE)
    {
			step_msg = gettext ("while setting the referential name");
      goto error;
    }
  /* set autopurge property */
  if (dg87_exp_day_set_auto_purged (exp_day, ref_to_plan->auto_purged) ==
      FALSE)
    {
			step_msg = gettext ("while setting the autopurge property");
      goto error;
    }
  /* create the stat day entry */
  dg87_caller__create_day_entry (exp_day);
  /* add log entry */
  dg87_caller__send_day_command (exp_day, "Creation");

  /* for each task to plan */
  task_element = task_to_plan_vector->first;
  while (task_element != NULL)
    {
      ref_task = task_element->object;
      task_element = task_element->next;
      n_tasks++;
      /* create the exploitation task */
      if ((exp_task =
           dg87_exp_day_task_add (exp_day, ref_task->name)) == NULL)
        {
					step_msg = gettext ("while creating a task in the exploitation day");
          goto error;
        }
      /* set task description */
      if (dg87_exp_task_set_description (exp_task,
                                         ref_task->description) == FALSE)
        {
					step_msg = gettext ("while setting the description of a task");
          goto error;
        }
      /* set waitAllDependencies property */
      if (dg87_exp_task_set_wait_all_dependencies (exp_task,
                                                   ref_task->
                                                   wait_all_dependencies) ==
          FALSE)
        {
					step_msg = gettext ("while setting the 'wait_all_dependencies' property of a task");
          goto error;
        }
      /* set on_error_continue property */
      if (dg87_exp_task_set_on_error_continue (exp_task,
                                               ref_task->on_error_continue) ==
          FALSE)
        {
					step_msg = gettext ("while setting the 'on_error_continue' property of a task");
          goto error;
        }

      /* set task queue */
      if (dg87_exp_task_set_queue (exp_task, ref_task->queue) == FALSE)
        {
					step_msg = gettext ("while linking a task with a queue");
          goto error;
        }

      /* set cyclical properties */
      if (dg87_exp_task_set_cyclical (exp_task, ref_task->iteration,
                                      ref_task->interval,
                                      ref_task->from_begin) == FALSE)
        {
					step_msg = gettext ("while setting the 'cyclical' properties of a task");
          goto error;
        }

      /* set lock property */
      if (dg87_exp_task_set_lock (exp_task, ref_task->lock_variable,
                                  ref_task->lock_action) == FALSE)
        {
					step_msg = gettext ("while setting the 'lock' properties of a task");
          goto error;
        }
      /* create the stat task entry */
      dg87_caller__create_task_entry (exp_task);


      /* for each condition of the task */
      condition_item = ref_task->condition_list->first;

      while (condition_item != NULL)
        {
          ref_condition = condition_item->object;
          condition_item = condition_item->next;
          /* create the condition */
          switch (ref_condition->type)
            {
            case condition_hmin:
              if (dg87_exp_task_condition_hmin_add (exp_task,
                                                    ref_condition->data.
                                                    condition_hmin->minutes,
                                                    ref_condition->data.
                                                    condition_hmin->day_delta)
                  == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }
              break;
            case condition_hmax:
              if (dg87_exp_task_condition_hmax_add (exp_task,
                                                    ref_condition->data.
                                                    condition_hmax->minutes,
                                                    ref_condition->data.
                                                    condition_hmax->day_delta)
                  == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }
              break;
            case condition_hlate:
              if (dg87_exp_task_condition_hlate_add (exp_task,
                                                     ref_condition->data.
                                                     condition_hlate->minutes,
                                                     ref_condition->data.
                                                     condition_hlate->
                                                     day_delta) == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }
              break;

            case condition_file:
              if (dg87_exp_task_condition_file_add (exp_task,
                                                    ref_condition->data.
                                                    condition_File->path)
                  == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }
              break;
            case condition_manual:
              if (dg87_exp_task_condition_manual_add (exp_task,
                                                      ref_condition->data.
                                                      condition_manual->
                                                      message) == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }

              break;
            case condition_ifequal:
              if (dg87_exp_task_condition_ifequal_add (exp_task,
                                                       ref_condition->data.
                                                       condition_ifequal->
                                                       variable,
                                                       ref_condition->data.
                                                       condition_ifequal->
                                                       value) == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }
              break;
            case condition_ifnequal:
              if (dg87_exp_task_condition_ifnotequal_add (exp_task,
                                                          ref_condition->data.
                                                          condition_ifnotequal->
                                                          variable,
                                                          ref_condition->data.
                                                          condition_ifnotequal->
                                                          value) == FALSE)
                {
									step_msg = gettext ("while adding a condition");
                  goto error;
                }
              break;
            default:
							step_msg = gettext ("while adding a condition");
              goto error;
            }
        }                       /* next condition */


      job_element = ref_task->job_vector->first;
      /* for each job of the task */
      while (job_element != NULL)
        {
          ref_job = job_element->object;
          job_element = job_element->next;
          n_jobs++;

          /* create the exploitation job */
          if ((exp_job = dg87_exp_task_job_add (exp_task,
                                                ref_job->name)) == NULL)
            {
							step_msg = gettext ("while adding a exploitation job");
              goto error;
            }
          /* set description */
          if (dg87_exp_job_set_description (exp_job,
                                            ref_job->description) == FALSE)
            {
							step_msg = gettext ("while setting the description of an exploitation job");
              goto error;
            }
          /* set path */
          if ((buffer = dg87_engine_replace_variables (engine,
                                                       (ref_job->path ==
                                                        NULL) ? "" : ref_job->
                                                       path)) == NULL)
						{
							step_msg = gettext ("while setting the path of an exploitation job");
            	goto error;
						}

          if (dg87_exp_job_set_path (exp_job, buffer) == FALSE)
						{
							step_msg = gettext ("while setting the path of an exploitation job");
            	goto error;
						}
          dg87_free (buffer, DG87_CHAR);
          buffer = NULL;

          /* set argument */
          if ((buffer = dg87_engine_replace_variables (engine,
                                                       (ref_job->argument ==
                                                        NULL) ? "" : ref_job->
                                                       argument)) == NULL)
						{
							step_msg = gettext ("while setting the arguments of an exploitation job");
            	goto error;
						}

          if (dg87_exp_job_set_argument (exp_job, buffer) == FALSE)
						{
							step_msg = gettext ("while setting the arguments of an exploitation job");
            	goto error;
						}
          dg87_free (buffer, DG87_CHAR);
          buffer = NULL;

          /* set temporization */
          if (dg87_exp_job_set_tempo (exp_job, ref_job->start_tempo,
                                      ref_job->end_tempo) == FALSE)
            {
							step_msg = gettext ("while setting the temporizations of an exploitation job");
              goto error;
            }
          /* set wait_all_dependencies property */
          if (dg87_exp_job_set_wait_all_dependencies (exp_job,
                                                      ref_job->
                                                      wait_all_dependencies)
              == FALSE)
            {
							step_msg = gettext ("while setting the 'wait_all_dependencies' property of an exploitation job");
              goto error;
            }
          /* set on_error_continue property */
          if (dg87_exp_job_set_on_error_continue (exp_job,
                                                  ref_job->
                                                  on_error_continue) == FALSE)
            {
							step_msg = gettext ("while setting the 'on_error_continue' property of an exploitation job");
              goto error;
            }
          dg87_caller__create_job_entry (exp_job);

        }                       /* next job */

      /* for each job dependence of the task */
      jobdep_item = ref_task->job_dep_list->first;
      while (jobdep_item != NULL)
        {
          ref_job_dep = jobdep_item->object;
          jobdep_item = jobdep_item->next;

          if ((exp_upstream_job = dg87_exp_task_job_get_by_name (exp_task,
                                                                 ref_job_dep->
                                                                 upstream_job->
                                                                 name)) ==
              NULL)
            {
							step_msg = gettext ("while searching the upstream job of a dependence");
              goto error;
            }
          if ((exp_downstream_job = dg87_exp_task_job_get_by_name (exp_task,
                                                                   ref_job_dep->
                                                                   downstream_job->
                                                                   name)) ==
              NULL)
            {
							step_msg = gettext ("while searching the downstream job of a dependence");
              goto error;
            }

          if (dg87_exp_task_jobdep_add (exp_task, exp_upstream_job,
                                        exp_downstream_job,
                                        ref_job_dep->type) == NULL)
            {
							step_msg = gettext ("while creating a job dependence");
              goto error;
            }
        }                       /* next job dependence */

    }                           /* next task */

  /* for each task dependence */
  taskdep_item = taskdep_to_plan_list->first;
  while (taskdep_item != NULL)
    {
      ref_task_dep = taskdep_item->object;
      taskdep_item = taskdep_item->next;
      if ((exp_upstream_task = dg87_exp_day_task_get_by_name (exp_day,
                                                              ref_task_dep->
                                                              upstream_task->
                                                              name)) == NULL)
        {
					step_msg = gettext ("while searching the upstream task of a dependence");
          goto error;
        }
      if ((exp_downstream_task = dg87_exp_day_task_get_by_name (exp_day,
                                                                ref_task_dep->
                                                                downstream_task->
                                                                name)) ==
          NULL)
        {
					step_msg = gettext ("while searching the downstream task of a dependence");
          goto error;
        }
      if (dg87_exp_day_taskdep_add (exp_day, exp_upstream_task,
                                    exp_downstream_task,
                                    ref_task_dep->type) == NULL)
        {
					step_msg = gettext ("while adding a task dependence");
          goto error;
        }
    }                           /* next task dependence */

  if (dg87_exp_day_set_enabled (exp_day, enabled) == FALSE)
    {
			step_msg = gettext ("while enabling the day");
      goto error;
    }

  if (task_to_plan_vector != NULL)
    dg87_vector_delete (task_to_plan_vector);
  /* free task dependence list */
  if (taskdep_to_plan_list != NULL)
    {
      while (taskdep_to_plan_list->count > 0)
        {
          ref_task_dep =
            dg87_list_remove_object_by_index (taskdep_to_plan_list, 0);
          dg87_ref_taskdep_delete (ref_task_dep);
        }
      dg87_list_delete (taskdep_to_plan_list);
    }
  dg87_strbuf_printf (engine->log, 
									gettext ("%i task(s) created\n"), n_tasks);
  dg87_strbuf_printf (engine->log, 
									gettext ("%i job(s) created\n"), n_jobs);
  dg87_strbuf_printf (engine->log, 
									gettext ("Normal termination\n"));
  return TRUE;

error:

  dg87_strbuf_printf (engine->log, 
									gettext ("ERROR %s : %s !\n"),
									(step_msg == NULL) ? "..." : step_msg, 
									dg87_err_get_error_str ()); 
  if (buffer != NULL)
    dg87_free (buffer, DG87_CHAR);
  if (task_to_plan_vector != NULL)
    dg87_vector_delete (task_to_plan_vector);
  /* free task dependence list */
  if (taskdep_to_plan_list != NULL)
    {
      while (taskdep_to_plan_list->count > 0)
        {
          ref_task_dep =
            dg87_list_remove_object_by_index (taskdep_to_plan_list, 0);
          dg87_ref_taskdep_delete (ref_task_dep);
        }
      dg87_list_delete (taskdep_to_plan_list);
    }
  /* delete the new day if it was created */
  if (exp_day != NULL)
    dg87_context_exp_day_delete_by_name (engine->context, exp_day->name);
  dg87_strbuf_printf (engine->log, 
									gettext ("Abnormal termination\n"));
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_engine_run (dg87_engine * engine, dg87_server * server)
{
  dg87_context *context;
  dg87_element *day_element;
  dg87_element *task_element;
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_element *target_element;
  dg87_element *queue_element;
  dg87_item *item;
  dg87_queue *queue;
  dg87_target *target;
  dg87_r_job *job;
  dg87_r_file *file;
  pid_t wpid;
  fd_set testreadfds;
  struct timeval tv;
  time_t current_time;
  int status;

  assert (engine != NULL && server != NULL);
  if (engine == NULL || server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* save database */
  dg87_context_make_periodic_save (engine->context);

  /* exit if engine is not enabled */
  if (engine->enabled == FALSE)
    return FALSE;

  dg87_engine_auto_plan (engine, FALSE);
  dg87_engine_auto_purge (engine, FALSE);

  context = engine->context;

  /* file monitoring */
  current_time = dg87_current_time; 
  if (current_time >= engine->check_time + DG87_CHECK_INTERVAL)
    {
      engine->check_time = current_time + DG87_CHECK_INTERVAL;
      /* for each file to monitor */
      item = engine->context->file_list->first;
      while (item != NULL)
        {
          file = item->object;
          item = item->next;
          if (dg87_r_file_check_presence (file) == TRUE)
            {
              /* if file is present, send a message */
              dg87_r_caller_transmit_file (engine->context, file);
              /* remove file object from the list */
              if (dg87_list_remove_object (engine->context->file_list,
                                           file) == TRUE)
                dg87_r_file_delete (file);
            }
        }
    }

  /* for each job */
  item = engine->context->job_list->first;
  while (item != NULL)
    {
      job = item->object;
      item = item->next;
      switch (job->state)
        {
        case dg87_r_job_state_uninitialize:
          dg87_r_job_run (job, engine->context);
          break;
        case dg87_r_job_state_old:
          /* job can be remove from the list */
          if (dg87_list_remove_object (engine->context->job_list,
                                       job) == TRUE)
            dg87_r_job_delete (job);
          break;
        default:
          /* nothing to do */
          break;
        }
      /* send a message if the state of job is changed */
      dg87_r_job_transmit_state (engine->context, job);
    }

  /* construct fds with jobs outputs */
  FD_ZERO (&testreadfds);
  tv.tv_sec = 0;
  tv.tv_usec = 1000;
  item = engine->context->job_list->first;
  while (item != NULL)
    {
      job = item->object;
      item = item->next;
      if (job->fd != -1)
        FD_SET (job->fd, &testreadfds);
    }
  if (select (FD_SETSIZE, &testreadfds, NULL, NULL, &tv) > 0)
    {
      item = engine->context->job_list->first;
      while (item != NULL)
        {
          job = item->object;
          item = item->next;
          if (FD_ISSET (job->fd, &testreadfds))
            dg87_r_job_catch_output (job, engine->context);
        }
    }

  /* test if one job is finished */
  wpid = waitpid (0, &status, WNOHANG);
  if (wpid > 0)
    {
      /* find the job with the same pid */
      job = NULL;
      item = engine->context->job_list->first;
      while (item != NULL)
        {
          job = item->object;
          if (job->pid == wpid)
            break;
          item = item->next;
        }
      if (job != NULL)
        {
          if (WIFEXITED (status) != 0)
            {
              /* normal termination */
              job->status = WEXITSTATUS (status);
              dg87_r_job_change_state (job, dg87_r_job_state_exit);
            }
          else
            {
              /* job could not be run */
              dg87_r_job_change_state (job, dg87_r_job_state_deleted);
            }
        }
    }

  /* transmit change of state of jobs */
  item = engine->context->job_list->first;
  while (item != NULL)
    {
      job = item->object;
      dg87_r_job_transmit_state (engine->context, job);
      item = item->next;
    }


  /* exploitation */
  /* for each day */
  day_element = engine->context->exp_day_vector->first;
  while (day_element != NULL)
    {
      day = day_element->object;
      day_element = day_element->next;
      if (day->enabled == FALSE)
        continue;
      /* do nothing if start_time is not reached */
      if (day->start_time > time (0))
        continue;
      /*  update each task of the day */
      task_element = day->task_vector->first;
      while (task_element != NULL)
        {
          task = task_element->object;
          dg87_exp_task_manage (task);
          task_element = task_element->next;
        }
    }
  /* for each target */
  target_element = engine->context->target_vector->first;
  while (target_element != NULL)
    {
      target = target_element->object;

      /* for each queue */
      queue_element = target->queue_vector->first;
      while (queue_element != NULL)
        {
          queue = queue_element->object;
          dg87_queue_manage (engine->context, queue);
          queue_element = queue_element->next;
        }

      target_element = target_element->next;
    }


  return TRUE;
}

/****************************************************************************/
BOOL
dg87_engine_plan (dg87_engine * engine, dg87_ref_model * ref,
                  char *new_day_name, time_t start_time, BOOL enable_day)
{
  dg87_context *context;
  assert (engine != NULL && ref != NULL && new_day_name != NULL);
  if (engine == NULL || ref == NULL || new_day_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = engine->context;
  /* copy day name */
  if (engine->new_day_name != NULL)
    dg87_free (engine->new_day_name, DG87_CHAR);
  if ((engine->new_day_name = dg87_strdup (new_day_name, DG87_CHAR)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  /* copy date */
  engine->new_day_start_time = start_time;

  /* initialize engine to plan the referential */
  dg87_engine_clear_log (engine);
  dg87_engine_create_day (engine, ref, enable_day);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_engine_auto_plan (dg87_engine * engine, BOOL force)
{
  char *day_name;
  char *str_date;
  time_t start_time, current_time;
  struct tm dc;
  dg87_element *element;
  dg87_ref_model *ref;
  assert (engine != NULL);
  current_time = dg87_current_time;
  start_time = current_time + 24 * 3600;
  dc = *localtime (&start_time);
  dc.tm_hour = 0;
  dc.tm_min = 0;
  dc.tm_sec = 0;
  start_time = mktime (&dc);

  if (force == FALSE && engine->next_auto_plan_time > current_time)
    return TRUE;

  dc = *localtime (&current_time);
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"autoplan time %s", asctime (&dc));
  dg87_engine_clear_log (engine);

  /* find all referentials to plan */
  element = engine->context->ref_vector->first;
  while (element != NULL)
    {
      ref = element->object;
      element = element->next;
      if (ref->auto_planned == FALSE)
        continue;
			dg87_daemon_log_printf(dg87_daemon_log_info,
      	"Referential [%s] is selected for auto plan",
               ref->name);
      /* start time is tomorrow at midnight */
      start_time = dg87_current_time + 24 * 3600;
      dc = *localtime (&start_time);
      dc.tm_hour = 0;
      dc.tm_min = 0;
      dc.tm_sec = 0;
      start_time = mktime (&dc);

      /* create the day name xxx_yyyymmdd */
      dc = *localtime (&start_time);
      if ((day_name = dg87_malloc (DG87_REF_NAME_LENGTH + 10,
                                   DG87_CHAR)) == NULL)
        return FALSE;
      sprintf (day_name, "%s_%04d%02d%02d", ref->name,
               dc.tm_year + 1900, dc.tm_mon + 1, dc.tm_mday);
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"day name [%s]", day_name);
      dg87_strbuf_printf (engine->log, "day name [%s]\n", day_name);

      /* create the date */
      if ((str_date = dg87_malloc (20, DG87_CHAR)) == NULL)
        {
          dg87_free (day_name, DG87_CHAR);
          return FALSE;
        }
      sprintf (str_date, "%04d-%02d-%02d %02d:%02d:%02d",
               dc.tm_year + 1900, dc.tm_mon + 1, dc.tm_mday,
               dc.tm_hour, dc.tm_min, dc.tm_sec);

			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"start time [%s]", str_date);
      dg87_strbuf_printf (engine->log, "start time [%s]\n", str_date);
      dg87_free (str_date, DG87_CHAR);

      /* plan the day */
      /* copy day name */
      if (engine->new_day_name != NULL)
        dg87_free (engine->new_day_name, DG87_CHAR);
      engine->new_day_name = day_name;

      /* copy the date */
      engine->new_day_start_time = start_time;

      /* initialize the engine to plan the referential */
      dg87_engine_create_day (engine, ref, TRUE);
    }

  dc = *localtime (&start_time);
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"next autoplan time %s", asctime (&dc));

  engine->next_auto_plan_time = start_time;
	DG87_SAVE_NEEDED;

  return TRUE;
}

/****************************************************************************/
dg87_variable *
dg87_engine_variable_add (dg87_engine * engine,
                          char *variable_name, char *value)
{
  dg87_variable *variable;
  assert (engine != NULL && variable_name != NULL && value != NULL);
  if (engine == NULL || variable_name == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  variable = dg87_variable_new (variable_name, value);
  if (variable == NULL)
    return NULL;

  /* vector generate error if an other variable has the same name */
  /* key must point on variable->name */
  if (dg87_vector_add_object (engine->variable_vector,
                              variable, variable->name) == FALSE)
    {
      dg87_variable_delete (variable);
      return NULL;
    }
  return variable;
}

/****************************************************************************/
int
dg87_engine_variable_count (dg87_engine * engine)
{
  assert (engine != NULL);
  if (engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (engine->variable_vector);
}

/****************************************************************************/
dg87_variable *
dg87_engine_variable_get_by_name (dg87_engine * engine, char *name)
{
  assert (engine != NULL && name != NULL);
  if (engine == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_key (engine->variable_vector, name);
}

/****************************************************************************/
dg87_variable *
dg87_engine_variable_get_by_index (dg87_engine * engine, int index)
{
  assert (engine != NULL);
  if (engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (engine->variable_vector, index);
}

/****************************************************************************/
BOOL
dg87_engine_variable_delete_by_name (dg87_engine * engine, char *name)
{
  dg87_variable *variable;
  assert (engine != NULL && name != NULL);
  if (engine == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  variable = dg87_vector_remove_object_by_key (engine->variable_vector, name);
  if (variable == NULL)
    return FALSE;
  if (dg87_variable_delete (variable) == FALSE)
    {
      /* add the variable in vector again */
      dg87_vector_add_object (engine->variable_vector,
                              variable, variable->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_engine_save (dg87_engine * engine, dg87_storage * storage)
{
  assert (engine != NULL && storage != NULL);
  if (engine == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* 'enabled' property */
  if (dg87_storage_write_ubyte (storage, engine->enabled) == FALSE)
    return FALSE;

  /* next_auto_plan_time property */
  if (dg87_storage_write_long (storage, engine->next_auto_plan_time) == FALSE)
    return FALSE;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_engine_load (dg87_engine * engine, dg87_storage * storage)
{
  BOOL value;
  dg87long long_value;
  assert (engine != NULL && storage != NULL);
  if (engine == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* 'enabled' property */
  if (dg87_storage_read_ubyte (storage, &value) == FALSE)
    return FALSE;
  engine->enabled = value;

  if (dg87_storage_read_long (storage, &long_value) == FALSE)
    return FALSE;
  engine->next_auto_plan_time = long_value;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_engine_variable_delete_by_index (dg87_engine * engine, int index)
{
  dg87_variable *variable;
  assert (engine != NULL);
  if (engine == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  variable = dg87_vector_remove_object_by_index
    (engine->variable_vector, index);
  if (variable == NULL)
    return FALSE;
  dg87_variable_delete (variable);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_engine_auto_purge (dg87_engine * engine, BOOL force)
{
  time_t current_time, next_time;
  struct tm dc;
  dg87_element *element;
  dg87_exp_day *day;

  assert (engine != NULL);
  current_time = dg87_current_time;
  if (force == FALSE && engine->next_auto_purge_time > current_time)
    return TRUE;

  dc = *localtime (&current_time);
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"Autopurge time %s", asctime (&dc));
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"Day purge retention %i",
           engine->context->day_purge_retention);

  /* for each day */
  element = engine->context->exp_day_vector->first;
  while (element != NULL)
    {
      day = element->object;
      element = element->next;
      if (day->auto_purged == FALSE)
        continue;
      if (day->start_time > current_time -
          24 * 3600 * engine->context->day_purge_retention)
        continue;
			dg87_daemon_log_printf(dg87_daemon_log_info,
      	"Day purge [%s]", day->name);
      dg87_caller__send_day_command (day, "Purge");
      if (dg87_context_exp_day_delete_by_name (engine->context,
                                               day->name) == FALSE)
        return FALSE;
    }

  /* calculate the time of the next purge */
  next_time = current_time + 24 * 3600;
  dc = *localtime (&next_time);
  dc.tm_hour = 0;
  dc.tm_min = 0;
  dc.tm_sec = 0;
  next_time = mktime (&dc);

  dc = *localtime (&next_time);
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"Next autopurge time %s", asctime (&dc));
  engine->next_auto_purge_time = next_time;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
/* End of file */
