/* file l_cmdday.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>
#include <time.h>

#include "util.h"
#include "context.h"
#include "l_cmdday.h"

/******************************************************************************/
/* ldk
 * Log Day Kill 
 * Usage : ldk DayName
 */
int
cmd_ldk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  command = connection->command;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"ldk DayName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* delete the day */
  dg87_err_clear_error ();
  if (dg87_context_log_day_delete_by_name (context,
                                           command->argv[0], TRUE) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the day '%s' ! %s\n"),
                              command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_ldl
 * Log Day List
 * Usage : ldl [options] 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */

int
cmd_ldl (dg87_connection * connection)
{
  char *sep, *beg, *end;  /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;   /* index of the first task to display */
  int number = 0;  /* number of day to display */
  int count = 0;   /* number of day actually display */
  dg87_element *element = NULL;
  dg87_log_day *day;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ldl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first Task to display */
  element = NULL;
  if (start > 0)
    element =
      dg87_vector_get_element_by_index (context->log_day_vector, start);
  else
    element = context->log_day_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      day = element->object;
      /* print day name */
      dg87_connection_printf (connection, "%s%s", beg, day->name);
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* cmd_ldc
 * Log Day Count
 * return the number of Days.
 * Usage : ldc 
 */
int
cmd_ldc (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"ldc");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", context->log_day_vector->count);

  return 0;
}

/******************************************************************************/
/* cmd_ldel
 * Log Day Entry List 
 * Usage : ldel [options] day_name 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_ldel (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  char *str_date;
  dg87_log_day *day;
  dg87_command *command;
  dg87_log_entry entry;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ldel [options] DayName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* read first entry */
  if (dg87_log_day_entry_read_first (day, &entry) == FALSE)
    {
      /* quit if no entries or if reading error */
      free (sep);
      free (beg);
      free (end);
      return 0;
    }

  /* for each entry */
  while (1)
    {
      /* print entry informations */
      str_date = dg87_date_to_ascii (entry.date);
      dg87_connection_printf (connection, "%s%s%s%s",
                              beg, str_date,
                              sep,
                              dg87_log_day_entry_type_to_ascii (entry.type));
      dg87_free (str_date, DG87_CHAR);
      dg87_connection_printf (connection, "%s%s", sep, entry.data);
      dg87_connection_printf (connection, "%s\n", end);
      /* read next entry */
      if (dg87_log_day_entry_read_next (day, &entry) == FALSE)
        {
          /* quit if no entries or if reading error */
          free (sep);
          free (beg);
          free (end);
          return 0;
        }
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* cmd_ldec
 * Log Day Entry Count 
 * return the number of entries of a day.
 * Usage : ldea day_name 
 */
int
cmd_ldec (dg87_connection * connection)
{
  dg87_log_day *day;
  dg87_command *command;
  dg87uint number;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ldec DayName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  if (dg87_log_day_entry_get_numbers (day, &number) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't get entries number for day '%s'\n"),
											day->name);
      return 1;
    }
  /* read root node */
  dg87_connection_printf (connection, "%i\n", number);
  return 0;
}

/******************************************************************************/
/* End of file */
