/* file l_cmdjob.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <assert.h>
#include <string.h>

#include "context.h"
#include "l_cmdday.h"
#include "util.h"

/******************************************************************************/
/* cmd_ljl
 * Log Job List 
 * Usage : ljl [options] day_name task_name
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_ljl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_command *command;
  dg87_log_node job_node;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ljl [options] DayName TaskName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* read first job */
  if (dg87_log_day_job_read_first (day, &task_node, &job_node) == FALSE)
    {
      /* quit if no entries or if reading error */
      free (sep);
      free (beg);
      free (end);
      return 0;
    }

  /* for each job */
  while (1)
    {
      /* print job informations */
      dg87_connection_printf (connection, "%s%s", beg, job_node.name);
      dg87_connection_printf (connection, "%s\n", end);
      /* quit if there is no next node */
      if (job_node.next_node == 0)
	      break;
      /* read next job */
      if (dg87_log_day_job_read_next (day, &job_node) == FALSE)
        {
          /* quit if no entries or if reading error */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* cmd_ljc
 * Log Job Count
 * return the number of jobs of a day.
 * Usage : ltc day_name task_name
 */
int
cmd_ljc (dg87_connection * connection)
{
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_command *command;
  dg87uint number;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ljc DayName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;

  if (dg87_log_day_job_get_count (day, &task_node, &number) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't get jobs count for "\
															" the task '%s' of the day '%s'\n"),
											task_node.name, day->name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", number);
  return 0;
}

/******************************************************************************/
/* cmd_ljea
 * Log Job Entry Add 
 * Usage : ljta day_name task_name job_name date type [data]
 */
int
cmd_ljea (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_log_node job_node;
  time_t time;
  dg87_log_day_entry_type entry_type;
  char *data;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc < 5 || command->argc > 6)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ljea day_name task_name job_name date type [data]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;
  /* find the job */
  if (dg87_util_find_log_job_node (connection, day, &task_node,
                                   command->argv[2], &job_node) == FALSE)
    return 1;

  /* interpret date */
  if ((time = dg87_ascii_to_date (command->argv[3])) == -1)
    {
      dg87_connection_printf (connection,
                              gettext ("Invalid date format '%s' !\n"),
                              command->argv[3]);
      return 1;
    }
  /* interpret type */
  if ((entry_type =
       dg87_log_day_ascii_to_entry_type (command->argv[4])) == -1)
    {
      dg87_connection_printf (connection,
                              gettext ("Invalid entry type '%s' !\n"),
                              command->argv[4]);
      return 1;
    }
  /* interpret data (optional argument) */
  if (command->argc == 5)
    data = NULL;
  else
    data = command->argv[5];
  if (dg87_log_day_job_entry_add (day, &job_node, entry_type,
                                  time, data) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add an entry to job '%s' of "\
                              		"the task '%s' of the day '%s' : %s !\n"),
                              job_node.name, task_node.name, day->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_ljel
 * Log Job Entry List 
 * Usage : ljel [options] day_name task_name job_name
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -o : prints jobs outputs
 */
int
cmd_ljel (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  char *str_date;
  char *temp;
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_log_node job_node;
  dg87_command *command;
  dg87_log_entry entry;
  BOOL print_job_output = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ltel [options] DayName TaskName JobName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;
  /* find the job */
  if (dg87_util_find_log_job_node (connection, day, &task_node,
                                   command->argv[2], &job_node) == FALSE)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'o':
          print_job_output = TRUE;
          break;
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* read first entry */
  if (dg87_log_day_job_entry_read_first (day, &job_node, &entry) == FALSE)
    {
      /* quit if no entries or if reading error */
      free (sep);
      free (beg);
      free (end);
      return 0;
    }

  /* for each entry */
  while (1)
    {
      /* ignore jobs outputs without -o option */
      if (print_job_output == TRUE
          || entry.type != dg87_log_day_entry_type_output)
        {
          str_date = dg87_date_to_ascii (entry.date);
          temp = dg87_log_day_entry_type_to_ascii (entry.type);
          assert (temp != NULL);
          dg87_connection_printf (connection, "%s%s%s%s",
                                  beg, str_date, sep, temp);
          dg87_free (str_date, DG87_CHAR);
          dg87_connection_printf (connection, "%s%s", sep, entry.data);
          dg87_connection_printf (connection, "%s\n", end);
        }
      /* read next entry */
      if (dg87_log_day_job_entry_read_next (day, &job_node, &entry) == FALSE)
        {
          /* quit if no entries or if reading error */
          free (sep);
          free (beg);
          free (end);
          return 0;
        }
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* cmd_ljec
 * Log Job Entry Count 
 * return the number of entries of a job.
 * Usage : ltea day_name task_name job_name
 */
int
cmd_ljec (dg87_connection * connection)
{
  dg87_log_day *day;
  dg87_command *command;
  dg87uint number;
  dg87_log_node task_node;
  dg87_log_node job_node;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"ljec DayName TaskName JobName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;
  /* find the job */
  if (dg87_util_find_log_job_node (connection, day, &task_node,
                                   command->argv[2], &job_node) == FALSE)
    return 1;
  if (dg87_log_day_job_entry_count (day, &job_node, &number) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't get entries number for "\
																			" the job '%s' of the task '%s' "\
																			"of the day '%s'\n"),
                              job_node.name, task_node.name, day->name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", number);
  return 0;
}

/******************************************************************************/
/* End of file */
