/* file l_cmdtask.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <assert.h>
#include <string.h>

#include "context.h"
#include "l_cmdday.h"
#include "util.h"

/******************************************************************************/
/* cmd_ltl
 * Log Task List 
 * Usage : ltl [options] day_name 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_ltl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_log_day *day;
  dg87_command *command;
  dg87_log_node task_node;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              "Incorrect call.\nUsage : %s\n",
                              "ltl [options] DayName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* read first task */
  if (dg87_log_day_task_read_first (day, &task_node) == FALSE)
    {
      /* quit if no entries or if reading error */
      free (sep);
      free (beg);
      free (end);
      return 0;
    }

  /* for each task */
  while (1)
    {
      /* print entry informations */
      dg87_connection_printf (connection, "%s%s", beg, task_node.name);
      dg87_connection_printf (connection, "%s\n", end);
      /* quit if there is no next node */
      if (task_node.next_node == 0)
	      break;
      /* read next task */
      if (dg87_log_day_task_read_next (day, &task_node) == FALSE)
        {
          /* quit if no entries or if reading error */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* cmd_ltc
 * Log Task Count
 * return the number of tasks of a day.
 * Usage : ltc day_name
 */
int
cmd_ltc (dg87_connection * connection)
{
  dg87_log_day *day;
  dg87_command *command;
  dg87uint number;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              "Incorrect call.\nUsage : %s\n", "ltc DayName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
                              "Unknown option '%c' for command '%s'\n",
                              command->optv[0], command->function_name);
      return 1;
    }
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  if (dg87_log_day_task_get_count (day, &number) == FALSE)
    {
      dg87_connection_printf (connection,
                              "Can't get tasks number for day '%s'\n",
                              day->name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", number);
  return 0;
}

/******************************************************************************/
/* cmd_ltel
 * Log Task Entry List 
 * Usage : ltel [options] day_name task_name
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_ltel (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  char *str_date;
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_command *command;
  dg87_log_entry entry;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              "Incorrect call.\nUsage : %s\n",
                              "ltel [options] DayName TaskName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* read first entry */
  if (dg87_log_day_task_entry_read_first (day, &task_node, &entry) == FALSE)
    {
      /* quit if no entries or if reading error */
      free (sep);
      free (beg);
      free (end);
      return 0;
    }

  /* for each entry */
  while (1)
    {
      /* print entry informations */
      str_date = dg87_date_to_ascii (entry.date);
      dg87_connection_printf (connection, "%s%s%s%s",
                              beg, str_date,
                              sep,
                              dg87_log_day_entry_type_to_ascii (entry.type));
      dg87_free (str_date, DG87_CHAR);
      dg87_connection_printf (connection, "%s%s", sep, entry.data);
      dg87_connection_printf (connection, "%s\n", end);
      /* read next entry */
      if (dg87_log_day_task_entry_read_next (day, &task_node, &entry) ==
          FALSE)
        {
          /* quit if no entries or if reading error */
          free (sep);
          free (beg);
          free (end);
          return 0;
        }
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* cmd_ltec
 * Log Task Entry Count 
 * return the number of entries of a task.
 * Usage : ltea day_name TaskName
 */
int
cmd_ltec (dg87_connection * connection)
{
  dg87_log_day *day;
  dg87_command *command;
  dg87uint number;
  dg87_log_node task_node;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ltec DayName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_log_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_log_task_node (connection, day,
                                    command->argv[1], &task_node) == FALSE)
    return 1;
  if (dg87_log_day_task_entry_count (day, &task_node, &number) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't get the number of entries of "\
															"the task '%s' of the day '%s'\n"),
											task_node.name, day->name);
      return 1;
    }
  /* read root node */
  dg87_connection_printf (connection, "%i\n", number);
  return 0;
}

/******************************************************************************/
/* End of file */
