/* file l_day.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "l_day.h"

/* order in this array must correspond to dg87_log_day_entry_type */
/* this array must be finished by a NULL */
char *dg87_log_day_entry_type_string[] = {
  "created",
  "started",
  "terminated",
  "aborted",
  "deleted",
  "canceled",
  "ignored",
  "running",
  "wait_for_condition",
  "wait_enabled",
  "output",
  "command",
  "condition",
  "wait_tempo",
  "wait_lock",
  "wait_for_dep",
  "info",
  NULL
};

/****************************************************************************/
dg87_log_day *
dg87_log_day_new (char *name, char *filename, uid_t uid, uid_t gid)
{
  dg87_log_day *day;
  dg87_log_node root_node;
  assert (name != NULL && filename != NULL);
  if (name == NULL || filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_DAY_NAME_LENGTH) == FALSE)
    return NULL;

  /* allocate memory for day */
  day = dg87_malloc (sizeof (dg87_log_day), DG87_DAY);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  /* initialize the structure of the object */
  dg87_strnzcpy (day->name, name, DG87_DAY_NAME_LENGTH);

  /* create log file */
  if ((day->log_file = dg87_log_file_create (filename, uid, gid)) == NULL)
    {
      dg87_free (day, DG87_DAY);
      return NULL;
    }

  /* create root node */
  if ((dg87_log_file_add_node (day->log_file,
                               name, NULL, &root_node)) == FALSE)
    {
      /* TODO cleanup */
      dg87_free (day, DG87_DAY);
      return NULL;
    }

  return day;
}

/****************************************************************************/
dg87_log_day *
dg87_log_day_load (char *filename)
{
  dg87_log_day *day;
  dg87_log_file *log_file;
  dg87_log_node root_node;

  assert (filename != NULL);
  if (filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* open log file */
  if ((log_file = dg87_log_file_open (filename)) == NULL)
    return NULL;

  /* read root node */
  if ((dg87_log_file_get_first_child (log_file, NULL, &root_node)) == FALSE)
    {
      /* TODO cleanup */
      return NULL;
    }

  /* allocate memory for day */
  day = dg87_malloc (sizeof (dg87_log_day), DG87_DAY);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  /* initialize the structure of the object */
  dg87_strnzcpy (day->name, root_node.name, DG87_DAY_NAME_LENGTH);
  day->log_file = log_file;


  return day;
}

/****************************************************************************/
BOOL
dg87_log_day_delete (dg87_log_day * day, BOOL delete_file)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_log_file_close (day->log_file, delete_file) == FALSE)
    return FALSE;
  dg87_free (day, DG87_DAY);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_entry_add (dg87_log_day * day, dg87_log_day_entry_type type,
                        time_t date, char *data)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_log_file_add_entry (day->log_file,
                               date, type, data, NULL) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_entry_read_first (dg87_log_day * day, dg87_log_entry * entry)
{
  dg87_log_node root_node;
  assert (day != NULL && entry != NULL);
  if (day == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_log_file_get_first_child (day->log_file,
                                      NULL, &root_node)) == FALSE)
    {
      return FALSE;
    }
  /* read first entry (if exits) */
  if (dg87_log_file_get_first_entry (day->log_file,
                                     &root_node, entry) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_entry_read_next (dg87_log_day * day, dg87_log_entry * entry)
{
  assert (day != NULL && entry != NULL);
  if (day == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next entry (if exits) */
  if (dg87_log_file_get_next_entry (day->log_file, entry) == FALSE)
    {
      return FALSE;
    }
  return TRUE;

}

/****************************************************************************/
BOOL
dg87_log_day_entry_get_numbers (dg87_log_day * day, dg87uint * number)
{
  dg87_log_node root_node;
  assert (day != NULL && number != NULL);
  if (day == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_log_file_get_first_child (day->log_file,
                                      NULL, &root_node)) == FALSE)
    {
      return FALSE;
    }
  *number = root_node.entries_number;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_add (dg87_log_day * day, char *task_name)
{
  dg87_log_node root_node;
  dg87_log_node new_node;
  assert (day != NULL && task_name != NULL);
  if (day == NULL || task_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_log_file_get_first_child (day->log_file,
                                      NULL, &root_node)) == FALSE)
    return FALSE;
  /* create new node */
  if (dg87_log_file_add_node (day->log_file, task_name,
                              &root_node, &new_node) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_read_first (dg87_log_day * day, dg87_log_node * task_node)
{
  dg87_log_node root_node;
  assert (day != NULL && task_node != NULL);
  if (day == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_log_file_get_first_child (day->log_file,
                                      NULL, &root_node)) == FALSE)
    {
      return FALSE;
    }
  /* read first node (if exits) */
  if (dg87_log_file_get_first_child (day->log_file,
                                     &root_node, task_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_read_next (dg87_log_day * day, dg87_log_node * task_node)
{
  assert (day != NULL && task_node != NULL);
  if (day == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next task node (if exits) */
  if (dg87_log_file_get_next_node (day->log_file, task_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_get_count (dg87_log_day * day, dg87uint * number)
{
  dg87_log_node root_node;
  assert (day != NULL && number != NULL);
  if (day == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_log_file_get_first_child (day->log_file,
                                      NULL, &root_node)) == FALSE)
    {
      return FALSE;
    }
  *number = root_node.children_number;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_get_by_name (dg87_log_day * day, char *name,
                               dg87_log_node * task_node)
{
  assert (day != NULL && name != NULL && task_node != NULL);
  /* read first task */
  if (dg87_log_day_task_read_first (day, task_node) == FALSE)
    return FALSE;
  while (1)
    {
      /* compare task name */
      if (strcmp (name, task_node->name) == 0)
        return TRUE;
      /* read next task */
      if (dg87_log_day_task_read_next (day, task_node) == FALSE)
        break;
    }
  /* task not found */
  dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_entry_add (dg87_log_day * day, dg87_log_node * task_node,
                             dg87_log_day_entry_type type, time_t date,
                             char *data)
{
  assert (day != NULL && task_node != NULL);
  if (day == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_log_file_add_entry (day->log_file,
                               date, type, data, task_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_entry_read_first (dg87_log_day * day,
                                    dg87_log_node * task_node,
                                    dg87_log_entry * entry)
{
  assert (day != NULL && task_node != NULL && entry != NULL);
  if (day == NULL || task_node == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read first entry (if exits) */
  if (dg87_log_file_get_first_entry (day->log_file,
                                     task_node, entry) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_entry_read_next (dg87_log_day * day,
                                   dg87_log_node * task_node,
                                   dg87_log_entry * entry)
{
  assert (day != NULL && task_node != NULL && entry != NULL);
  if (day == NULL || task_node == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next entry (if exits) */
  if (dg87_log_file_get_next_entry (day->log_file, entry) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_task_entry_count (dg87_log_day * day, dg87_log_node * task_node,
                               dg87uint * number)
{
  assert (day != NULL && task_node != NULL && number != NULL);
  if (day == NULL || task_node == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  *number = task_node->entries_number;
  return TRUE;
}

/****************************************************************************/
dg87_log_day_entry_type
dg87_log_day_ascii_to_entry_type (char *string)
{
  int index = 0;
  assert (string != NULL);
  while (dg87_log_day_entry_type_string[index] != NULL &&
         strcmp (string, dg87_log_day_entry_type_string[index]) != 0)
    index++;
  return (dg87_log_day_entry_type_string[index] == NULL) ? -1 : index;
}

/****************************************************************************/
BOOL
dg87_log_day_job_add (dg87_log_day * day, dg87_log_node * task_node,
                      char *job_name)
{
  dg87_log_node new_node;
  assert (day != NULL && task_node != NULL && job_name != NULL);
  if (day == NULL || task_node == NULL || job_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* create new node */
  if (dg87_log_file_add_node (day->log_file, job_name,
                              task_node, &new_node) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_read_first (dg87_log_day * day, dg87_log_node * task_node,
                             dg87_log_node * job_node)
{
  assert (day != NULL && job_node != NULL);
  if (day == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read first node (if exits) */
  if (dg87_log_file_get_first_child (day->log_file,
                                     task_node, job_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_read_next (dg87_log_day * day, dg87_log_node * job_node)
{
  assert (day != NULL && job_node != NULL);
  if (day == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next job node (if exits) */
  if (dg87_log_file_get_next_node (day->log_file, job_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_get_count (dg87_log_day * day, dg87_log_node * task_node,
                            dg87uint * number)
{
  assert (day != NULL && number != NULL);
  if (day == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  *number = task_node->children_number;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_get_by_name (dg87_log_day * day, dg87_log_node * task_node,
                              char *name, dg87_log_node * job_node)
{
  assert (day != NULL && task_node != NULL && name != NULL
          && job_node != NULL);
  /* read first job */
  if (dg87_log_day_job_read_first (day, task_node, job_node) == FALSE)
    return FALSE;
  while (1)
    {
      /* compare job name */
      if (strcmp (name, job_node->name) == 0)
        return TRUE;
      /* read next job */
      if (dg87_log_day_job_read_next (day, job_node) == FALSE)
        break;
    }
  /* job not found */
  dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_entry_add (dg87_log_day * day, dg87_log_node * job_node,
                            dg87_log_day_entry_type type, time_t date,
                            char *data)
{
  assert (day != NULL && job_node != NULL);
  if (day == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_log_file_add_entry (day->log_file,
                               date, type, data, job_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_entry_read_first (dg87_log_day * day,
                                   dg87_log_node * job_node,
                                   dg87_log_entry * entry)
{
  assert (day != NULL && job_node != NULL && entry != NULL);
  if (day == NULL || job_node == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read first entry (if exits) */
  if (dg87_log_file_get_first_entry (day->log_file, job_node, entry) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_entry_read_next (dg87_log_day * day,
                                  dg87_log_node * job_node,
                                  dg87_log_entry * entry)
{
  assert (day != NULL && job_node != NULL && entry != NULL);
  if (day == NULL || job_node == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next entry (if exits) */
  if (dg87_log_file_get_next_entry (day->log_file, entry) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_day_job_entry_count (dg87_log_day * day, dg87_log_node * job_node,
                              dg87uint * number)
{
  assert (day != NULL && job_node != NULL && number != NULL);
  if (day == NULL || job_node == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  *number = job_node->entries_number;
  return TRUE;
}

/****************************************************************************/
char *
dg87_log_day_entry_type_to_ascii (dg87_log_day_entry_type entry_type)
{
  return dg87_log_day_entry_type_string[entry_type];
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_log_day_check (dg87_log_day * day, int (*print) (char *))
{
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
