/* file l_log.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>         /* uint16_t- uint32_t */

#include <libdg87/daemon.h>
#include "l_log.h"

#define DG87L_LOGFILE_SIGNATURE	"dg87l"
#define DG87L_LOGFILE_VERSION	1

/* local function prototypes */
BOOL _dg87_log_file_read_node (dg87_log_file * log_file,
                               DG87_FILEPOS position,
                               dg87_log_node * log_node);
BOOL _dg87_log_file_write_node (dg87_log_file * log_file,
                                DG87_FILEPOS position,
                                dg87_log_node * log_node);

BOOL _dg87_log_file_read_entry (dg87_log_file * log_file,
                                DG87_FILEPOS position,
                                dg87_log_entry * entry);
BOOL _dg87_log_file_write_entry (dg87_log_file * log_file,
                                 DG87_FILEPOS position,
                                 dg87_log_entry * entry);
BOOL _dg87_log_file_open_file (dg87_log_file * log_file);
BOOL _dg87_log_file_close_file (dg87_log_file * log_file);
DG87_FILEPOS _dg87_log_file_end_position (dg87_log_file * log_file);

/****************************************************************************/
dg87_log_file *
dg87_log_file_create (char *filename, uid_t uid, uid_t gid)
{
  struct stat fstat;
  uint16_t value;
  int l;
  FILE* file_stream;
  dg87_log_file *log_file;

  assert (filename != NULL);
  if (filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* verify that file doesn't exist */
  if (stat (filename, &fstat) == 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"File '%s' already exists !", filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_ALREADY_EXISTS);
      return NULL;
    }

  /* open file */
  if ((file_stream = fopen (filename, "w+")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error, 
											"Can't open file : %s", filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_DONT_EXIT);
      return NULL;
    }
  /* if daemon is running in root account, change file owner */
  if (uid != -1 && gid != -1)
    {
      if (chown (filename, uid, gid) == -1)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error, 
          	"Can't change owner of file '%s'", filename);
        }
    }
  /* write signature */
  l = strlen (DG87L_LOGFILE_SIGNATURE);
  if (fwrite(DG87L_LOGFILE_SIGNATURE, l * sizeof(char), 1, file_stream) == 0)
    {
      fclose (file_stream);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return NULL;
    }
  /* write version */
  value = htons (DG87L_LOGFILE_VERSION);
  if (fwrite(&value, sizeof(uint16_t), 1,file_stream) == 0)
    {
      fclose (file_stream);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return NULL;
    }

  /* allocate memory for object structure */
  log_file = dg87_malloc (sizeof (dg87_log_file), DG87_LOGFILE);
  if (log_file == NULL)
    {
      fclose (file_stream);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  fclose (file_stream);
  log_file->file_stream = NULL;
  log_file->root_node = DG87L_ROOT_NODE_POSITION;
  if ((log_file->filename = dg87_strdup (filename, DG87_CHAR)) == NULL)
    {
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  return log_file;
}

/****************************************************************************/
dg87_log_file *
dg87_log_file_open (char *filename)
{
  char *string;
  int version;
  uint16_t tmpvalue;
  int l;
  struct stat fstat;
  dg87_log_file *log_file;
  assert (filename != NULL);
  if (filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* verify that file exists */
  if (stat (filename, &fstat) != 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error, 
      	"File '%s' doesn't exist !", filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_DONT_EXIT);
      return NULL;
    }

  /* allocate memory for object structure */
  if ((log_file = dg87_malloc (sizeof (dg87_log_file), DG87_LOGFILE)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  if ((log_file->filename = dg87_strdup (filename, DG87_CHAR)) == NULL)
    {
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  log_file->file_stream = NULL;

  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    {
      dg87_free (log_file->filename, DG87_CHAR);
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  /* verify signature */
  l = strlen (DG87L_LOGFILE_SIGNATURE);
  string = dg87_malloc ((l + 1) * sizeof (char), DG87_CHAR);
  if (fread(string, l * sizeof(char), 1, log_file->file_stream) == 0)
    {
      dg87_free (string, DG87_CHAR);
      dg87_log_file_close (log_file, FALSE);
      dg87_free (log_file->filename, DG87_CHAR);
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return NULL;
    }
  string[l + 1] = '\0';
  if (strncmp (string, DG87L_LOGFILE_SIGNATURE, l) != 0)
    {
      dg87_free (string, DG87_CHAR);
      dg87_log_file_close (log_file, FALSE);
      dg87_free (log_file->filename, DG87_CHAR);
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_LOGFILE_WRONG_SIGN);
      return NULL;
    }
  dg87_free (string, DG87_CHAR);

  /* verify version */
  if (fread(&tmpvalue, sizeof(uint16_t), 1, log_file->file_stream) == 0)
    {
      dg87_log_file_close (log_file, FALSE);
      dg87_free (log_file->filename, DG87_CHAR);
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return NULL;
    }
  version = ntohs (tmpvalue);
  if (version != DG87L_LOGFILE_VERSION)
    {
      dg87_log_file_close (log_file, FALSE);
      dg87_free (log_file->filename, DG87_CHAR);
      dg87_free (log_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_LOGFILE_WRONT_VERSION);
      return NULL;
    }

  log_file->root_node = DG87L_ROOT_NODE_POSITION;
  _dg87_log_file_close_file (log_file);
  return log_file;
}
/****************************************************************************/
DG87_FILEPOS _dg87_log_file_end_position (dg87_log_file * log_file)
{
	assert(log_file != NULL);
	/* go to end of file */
	if (fseek (log_file->file_stream, 0, SEEK_END) == -1)
					return -1;
	/* return the position of the end of file */
	return ftell (log_file->file_stream);
}

/****************************************************************************/
BOOL
dg87_log_file_close (dg87_log_file * log_file, BOOL delete_file)
{
  assert (log_file != NULL);
  if (log_file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  _dg87_log_file_close_file (log_file);
  if (delete_file == TRUE)
    {
      if (unlink (log_file->filename) == -1)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error, 
          	"Can't delete file '%s'", log_file->filename);
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_DELETE_FILE);
          return FALSE;
        }
    }
  dg87_free (log_file->filename, DG87_CHAR);
  dg87_free (log_file, DG87_LOGFILE);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_file_add_node (dg87_log_file * log_file, char *name,
                        dg87_log_node * parent_node, dg87_log_node * log_node)
{
  dg87_log_node last_node;
  assert (log_file != NULL && log_node != NULL);
  if (log_file == NULL || log_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    return FALSE;
  /* TODO control name length */
  if (parent_node == NULL)
    {
      /* create root node */
      /* TODO control length < DG87L_LOGNODE_NAME_LENGTH */
      strcpy (log_node->name, name);
      log_node->position = DG87L_ROOT_NODE_POSITION;
      log_node->next_node = 0;
      log_node->children_number = 0;
      log_node->first_child_node = 0;
      log_node->last_child_node = 0;
      log_node->entries_number = 0;
      log_node->first_entry = 0;
      log_node->last_entry = 0;
      if (_dg87_log_file_write_node (log_file,
                                     log_node->position, log_node) == FALSE)
        {
          /* TODO clear memory */
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }
    }
  else
    {
      /* read last node (if exists) */
      if (parent_node->last_child_node != 0)
        {
          if (_dg87_log_file_read_node (log_file,
                                        parent_node->last_child_node,
                                        &last_node) == FALSE)
            {
              _dg87_log_file_close_file (log_file);
              return FALSE;
            }
        }

      /* create new node */
			log_node->position = _dg87_log_file_end_position (log_file);
			if (log_node->position == -1)
        {
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }
      /* TODO control name length < 255) */
      strcpy (log_node->name, name);
      log_node->next_node = 0;
      log_node->children_number = 0;
      log_node->first_child_node = 0;
      log_node->last_child_node = 0;
      log_node->entries_number = 0;
      log_node->first_entry = 0;
      log_node->last_entry = 0;

      /* write new node at end of file */
      if (_dg87_log_file_write_node (log_file,
                                     log_node->position, log_node) == FALSE)
        {
          /*TODO pas cool */
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }

      /* udpate previous last node (if exists) */
      if (parent_node->last_child_node != 0)
        {
          last_node.next_node = log_node->position;
          if (_dg87_log_file_write_node (log_file,
                                         parent_node->last_child_node,
                                         &last_node) == FALSE)
            {
              /*TODO pas cool */
              _dg87_log_file_close_file (log_file);
              return FALSE;
            }
        }

      /* update parent node */
      if (parent_node->first_child_node == 0)
        parent_node->first_child_node = log_node->position;
      parent_node->last_child_node = log_node->position;
      parent_node->children_number++;
      if (_dg87_log_file_write_node (log_file,
                                     parent_node->position,
                                     parent_node) == FALSE)
        {
          /*TODO pas cool */
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }

    }

  _dg87_log_file_close_file (log_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_file_get_first_child (dg87_log_file * log_file,
                               dg87_log_node * parent_node,
                               dg87_log_node * log_node)
{
  assert (log_file != NULL && log_node != NULL);
  if (log_file == NULL || log_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    return FALSE;
  if (parent_node == NULL)
    {
      /* read root node */
      if ((_dg87_log_file_read_node (log_file,
                                     DG87L_ROOT_NODE_POSITION,
                                     log_node)) == FALSE)
        {
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }
    }
  else
    {
      /* read first child node */
      if ((_dg87_log_file_read_node (log_file,
                                     parent_node->first_child_node,
                                     log_node)) == FALSE)
        {
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }
    }
  _dg87_log_file_close_file (log_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_file_get_next_node (dg87_log_file * log_file, dg87_log_node * node)
{
  assert (log_file != NULL);
  if (log_file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    return FALSE;
  if (_dg87_log_file_read_node (log_file, node->next_node, node) == FALSE)
    {
      _dg87_log_file_close_file (log_file);
      return FALSE;
    }
  _dg87_log_file_close_file (log_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_file_add_entry (dg87_log_file * log_file,
                         time_t date, int type, char *data,
                         dg87_log_node * node)
{
  DG87_FILEPOS node_position;
  dg87_log_entry last_entry;
  dg87_log_entry new_entry;
  dg87_log_node snode;
  assert (log_file != NULL);
  if (log_file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (node == NULL)
    {
      /* TODO attention node peut renvoyer sur snode qui est local ! */
      node_position = log_file->root_node;
      snode.position = log_file->root_node;
      node = &snode;
    }
  else
    {
      node_position = node->position;
    }
  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    return FALSE;

  /* read node */
  if (_dg87_log_file_read_node (log_file, node_position, node) == FALSE)
    {
      _dg87_log_file_close_file (log_file);
      return FALSE;
    }

  /* read last entry (if exists) */
  if (node->last_entry != 0)
    {
      if (_dg87_log_file_read_entry (log_file,
                                     node->last_entry, &last_entry) == FALSE)
        {
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }
    }

  /* create new entry */
	new_entry.position = _dg87_log_file_end_position (log_file);
	if (new_entry.position == -1)
    {
      return FALSE;
    }
  new_entry.next_entry = 0;
  new_entry.date = date;
  new_entry.type = type;
  if (data == NULL)
    new_entry.data[0] = '\0';
  else
    /* TODO control data length < 255 */
    strcpy (new_entry.data, data);

  /* write new entry at end of file */
  if (_dg87_log_file_write_entry (log_file,
                                  new_entry.position, &new_entry) == FALSE)
    {
      /*TODO pas cool */
      _dg87_log_file_close_file (log_file);
      return FALSE;
    }

  /* udpate previous last entry (if exists) */
  if (node->last_entry != 0)
    {
      last_entry.next_entry = new_entry.position;
      if (_dg87_log_file_write_entry (log_file,
                                      node->last_entry, &last_entry) == FALSE)
        {
          /*TODO pas cool */
          _dg87_log_file_close_file (log_file);
          return FALSE;
        }
    }

  /* update node */
  if (node->first_entry == 0)
    node->first_entry = new_entry.position;
  node->last_entry = new_entry.position;
  node->entries_number++;
  if (_dg87_log_file_write_node (log_file, node->position, node) == FALSE)
    {
      /*TODO pas cool */
      _dg87_log_file_close_file (log_file);
      return FALSE;
    }
  _dg87_log_file_close_file (log_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_file_get_first_entry (dg87_log_file * log_file,
                               dg87_log_node * parent_node,
                               dg87_log_entry * first_entry)
{
  assert (log_file != NULL && parent_node != NULL && first_entry != NULL);
  if (log_file == NULL || parent_node == NULL || first_entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    return FALSE;
  if (_dg87_log_file_read_entry (log_file,
                                 parent_node->first_entry,
                                 first_entry) == FALSE)
    {
      _dg87_log_file_close_file (log_file);
      return FALSE;
    }
  _dg87_log_file_close_file (log_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_log_file_get_next_entry (dg87_log_file * log_file,
                              dg87_log_entry * entry)
{
  assert (log_file != NULL && entry != NULL);
  if (log_file == NULL || entry == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_log_file_open_file (log_file) == FALSE)
    return FALSE;
  if (_dg87_log_file_read_entry (log_file, entry->next_entry, entry) == FALSE)
    {
      _dg87_log_file_close_file (log_file);
      return FALSE;
    }
  _dg87_log_file_close_file (log_file);
  return TRUE;
}

/***************************************************************************/
BOOL
_dg87_log_file_write_node (dg87_log_file * log_file,
                           DG87_FILEPOS position, dg87_log_node * log_node)
{
  uint32_t value32;
  uint16_t value16;
  uint8_t value8;
  assert (log_file != NULL && log_node != NULL);
  /* set file position */
  if (fseek (log_file->file_stream, position, SEEK_SET) == (off_t) - 1)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of next node */
  value32 = htonl (log_node->next_node);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write number of children nodes */
  value16 = htons (log_node->children_number);
  if (fwrite(&value16, sizeof(uint16_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of first child */
  value32 = htonl (log_node->first_child_node);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of last child */
  value32 = htonl (log_node->last_child_node);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write number of entries */
  value16 = htons (log_node->entries_number);
  if (fwrite(&value16, sizeof(uint16_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of first entry */
  value32 = htonl (log_node->first_entry);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of last entry */
  value32 = htonl (log_node->last_entry);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write name length */
  value8 = strlen (log_node->name);
  if (fwrite(&value8, sizeof(uint8_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write name */
	if (fwrite(log_node->name, value8 * sizeof(char), 1, 
													log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  return TRUE;
}

/***************************************************************************/
BOOL
_dg87_log_file_read_node (dg87_log_file * log_file,
                          DG87_FILEPOS position, dg87_log_node * log_node)
{
  uint32_t value32;
  uint16_t value16;
  uint8_t value8;
  assert (log_file != NULL && log_node != NULL);
  if (position == 0)
    return FALSE;
  /* set file position */
  if (fseek (log_file->file_stream, position, SEEK_SET) == (off_t) - 1)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->position = position;
  /* read file position of next node */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->next_node = ntohl (value32);
  /* read number of children nodes */
  if (fread(&value16, sizeof(uint16_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->children_number = ntohs (value16);
  /* read file position of first child */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->first_child_node = ntohl (value32);
  /* read file position of last child */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->last_child_node = ntohl (value32);
  /* read number of entries */
  if (fread(&value16, sizeof(uint16_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->entries_number = ntohs (value16);
  /* read file position of first entry */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->first_entry = ntohl (value32);
  /* read file position of last entry */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->last_entry = ntohl (value32);
  /* read name length */
  if (fread(&value8, sizeof(uint8_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  /* read name */
  if (fread(log_node->name, value8, 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  log_node->name[value8] = '\0';

  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_log_file_read_entry (dg87_log_file * log_file,
                           DG87_FILEPOS position, dg87_log_entry * entry)
{
  uint32_t value32;
  uint8_t value8;
  assert (log_file != NULL && entry != NULL);
  if (position == 0)
    return FALSE;
  /* set file position */
  if (fseek (log_file->file_stream, position, SEEK_SET) == (off_t) - 1)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  entry->position = position;
  /* read file position of next entry */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  entry->next_entry = ntohl (value32);
  /* read date */
  if (fread(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  entry->date = ntohl (value32);
  /* read entry type */
  if (fread(&value8, sizeof(uint8_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  entry->type = value8;
  /* read data length */
  if (fread(&value8, sizeof(uint8_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  /* read data */
  if (value8 > 0)
    {
  		if (fread(entry->data, value8, 1,log_file->file_stream) == 0)
        {
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
          return FALSE;
        }
    }
  entry->data[value8] = '\0';

  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_log_file_write_entry (dg87_log_file * log_file,
                            DG87_FILEPOS position, dg87_log_entry * entry)
{
  uint32_t value32;
  uint8_t value8;
  assert (log_file != NULL && entry != NULL);
  /* set file position */
  if (fseek (log_file->file_stream, position, SEEK_SET) == (off_t) - 1)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of next entry */
  value32 = htonl (entry->next_entry);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write date */
  value32 = htonl (entry->date);
  if (fwrite(&value32, sizeof(uint32_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write entry type */
  value8 = entry->type;
  if (fwrite(&value8, sizeof(uint8_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write data length */
  value8 = strlen (entry->data);
  if (fwrite(&value8, sizeof(uint8_t), 1,log_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write data */
  if (value8 > 0)
    {
      if (fwrite (entry->data, value8, 1,log_file->file_stream) == -1)
        {
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
          return FALSE;
        }
    }

  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_log_file_open_file (dg87_log_file * log_file)
{
  if (log_file->file_stream != NULL)
    return TRUE;
  if ((log_file->file_stream = fopen (log_file->filename, "r+")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't open file '%s' !",
				log_file->filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_CAN_NOT_OPEN);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_log_file_close_file (dg87_log_file * log_file)
{
  if (log_file->file_stream == NULL)
    return TRUE;
  fclose (log_file->file_stream);
  log_file->file_stream = NULL;
  return TRUE;
}

/****************************************************************************/
/* End of file */
