/* file main.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <assert.h>

#include <config.h>
#include <libdg87/dgerr.h>

#if DG87_USE_LONG_OPTIONS
#include <getopt.h>             /* include <config.h> must be before this line */
#endif /*DG87_USE_LONG_OPTIONS */

#include <libdg87/server.h>
#include <libdg87/daemon.h>
#include <libdg87/interpret.h>
#include <libdg87/dgerr.h>
#include "d_ref.h"
#include "engine.h"
#include "d_cmdtask.h"
#include "cmdtarget.h"
#include "cmdqueue.h"
#include "d_cmdjob.h"
#include "d_cmdref.h"
#include "x_cmdday.h"
#include "x_cmdtask.h"
#include "x_cmdjob.h"
#include "l_cmdday.h"
#include "l_cmdtask.h"
#include "l_cmdjob.h"
#include "s_cmdref.h"
#include "s_cmdtask.h"
#include "s_cmdjob.h"
#include "r_cmdjob.h"
#include "r_cmdfile.h"

#include "cmdengine.h"
#include "d_cmdrule.h"
#include "cmdvariable.h"

extern char *optarg;
extern int optind;
extern int opterr;


/* day names */
const char* day_name_array[] = { "Monday", "Tuesday", "Wednesday", 
				"Thursday", "Friday", "Saturday", "Sunday"}; 

/* month names */
const char *month_name[] = { "January", "Febuary", "March", "April", 
				"May", "June", "July", "August", "September", "October", 
				"November", "December" };

/* local prototypes */
int cmd_shutdown (dg87_connection * connection);
#ifdef DEBUG
int cmd_check (dg87_connection * connection);
#endif

/* array of Diogene87's commands */
static dg87_interpret_cmd cmd_array[] = {

#ifdef DEBUG
  {"check", cmd_check, "Check data (debug release only)", "",
   dg87_command_generic, dg87_command_access_none},
#endif

  /* Engine functions */
  {"ei", cmd_ei, "Engine Information", "B:S:E:",
   dg87_command_engine, dg87_command_access_consult},
  {"ejm", cmd_ejm, "Engine Jobs Monitor", "B:S:E:hrectwTQD",
   dg87_command_day, dg87_command_access_consult},
  {"el", cmd_el, "Engine Log", "B:S:E:",
   dg87_command_day, dg87_command_access_consult},
  {"ese", cmd_ese, "Engine Set Enabled", "",
   dg87_command_engine, dg87_command_access_manage},
  {"etm", cmd_etm, "Engine Tasks Monitor", "B:S:E:hrectwTQD",
   dg87_command_day, dg87_command_access_consult},

  {"ja", cmd_ja, "Job Add",
   "", dg87_command_job, dg87_command_access_admin},
  {"jc", cmd_jc, "Job Count", "",
   dg87_command_job, dg87_command_access_consult},
  {"ji", cmd_ji, "Job Information", "B:S:E:",
   dg87_command_job, dg87_command_access_consult},
  {"jk", cmd_jk, "Job Kill", "",
   dg87_command_job, dg87_command_access_admin},
  {"jl", cmd_jl, "Job List", "hsB:S:E:I:N:",
   dg87_command_job, dg87_command_access_consult},

  {"fa", cmd_fa, "File Add",
   "", dg87_command_job, dg87_command_access_admin},
  {"fc", cmd_fc, "File Count", "",
   dg87_command_job, dg87_command_access_consult},
  {"fk", cmd_fk, "File Kill", "",
   dg87_command_job, dg87_command_access_admin},
  {"fl", cmd_fl, "File List", "hB:S:E:I:N:",
   dg87_command_job, dg87_command_access_consult},


  /* log day functions */
  {"ldc", cmd_ldc, "Log Day Count", "",
   dg87_command_log, dg87_command_access_consult},
  {"ldk", cmd_ldk, "Log Day Kill", "",
   dg87_command_log, dg87_command_access_manage},
  {"ldl", cmd_ldl, "Log Day List", "B:S:E:I:N:",
   dg87_command_log, dg87_command_access_consult},
  {"ldec", cmd_ldec, "Log Day Entry Count", "",
   dg87_command_log, dg87_command_access_consult},
  {"ldel", cmd_ldel, "Log Day Entry List", "B:S:E:I:N:",
   dg87_command_log, dg87_command_access_consult},

  /* log job functions */
  {"ljc", cmd_ljc, "Log Job Count", "",
   dg87_command_log, dg87_command_access_consult},
  {"ljl", cmd_ljl, "Log Job List", "B:S:E:I:N:",
   dg87_command_log, dg87_command_access_consult},
  {"ljea", cmd_ljea, "Log Job Entry Add", "",
   dg87_command_log, dg87_command_access_manage},
  {"ljec", cmd_ljec, "Log Job Entry Count", "",
   dg87_command_log, dg87_command_access_consult},
  {"ljel", cmd_ljel, "Log Job Entry List", "oB:S:E:I:N:",
   dg87_command_log, dg87_command_access_consult},

  /* log task functions */
  {"ltc", cmd_ltc, "Log Task Count", "",
   dg87_command_log, dg87_command_access_consult},
  {"ltl", cmd_ltl, "Log Task List", "B:S:E:I:N:",
   dg87_command_log, dg87_command_access_consult},
  {"ltec", cmd_ltec, "Log Task Entry Count", "",
   dg87_command_log, dg87_command_access_consult},
  {"ltel", cmd_ltel, "Log Task Entry List", "B:S:E:I:N:",
   dg87_command_log, dg87_command_access_consult},


  /* Queue functions */
  {"qa", cmd_qa, "Queue add", "",
   dg87_command_target, dg87_command_access_admin},
  {"qc", cmd_qc, "Queue count", "",
   dg87_command_target, dg87_command_access_consult},
  {"qi", cmd_qi, "Queue information", "B:S:E:",
   dg87_command_target, dg87_command_access_consult},
  {"qk", cmd_qk, "Queue kill", "",
   dg87_command_target, dg87_command_access_admin},
  {"ql", cmd_ql, "Queue list", "dtB:S:E:I:N:",
   dg87_command_target, dg87_command_access_consult},
  {"qrtl", cmd_qrtl, "Queue ref Task List", "dB:S:E:I:N:",
   dg87_command_target, dg87_command_access_consult},
  {"qrtc", cmd_qrtc, "Queue ref Task Count", "",
   dg87_command_target, dg87_command_access_consult},
  {"qsd", cmd_qsd, "Queue Set Description", "",
   dg87_command_target, dg87_command_access_manage},
  {"qsm", cmd_qsm, "Queue Set Max", "",
   dg87_command_target, dg87_command_access_manage},
  {"qsn", cmd_qsn, "Queue Set Name", "",
   dg87_command_target, dg87_command_access_manage},
  {"qso", cmd_qso, "Queue Set Open", "",
   dg87_command_target, dg87_command_access_manage},

  {"qxjc", cmd_qxjc, "Queue eXp Job Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"qxjl", cmd_qxjl, "Queue eXp Job List", "sdB:S:E:I:N:",
   dg87_command_day, dg87_command_access_consult},
  {"qxtc", cmd_qxtc, "Queue eXp Task Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"qxtl", cmd_qxtl, "Queue eXp Task List", "dB:S:E:I:N:",
   dg87_command_day, dg87_command_access_consult},


  /* Ref commands */
  {"ra", cmd_ra, "Ref Add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rc", cmd_rc, "Ref Count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rd", cmd_rd, "Ref Duplicate", "",
   dg87_command_ref, dg87_command_access_admin},
  {"ri", cmd_ri, "Ref Information", "B:S:E:",
   dg87_command_ref, dg87_command_access_consult},
  {"rk", cmd_rk, "Ref Kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rl", cmd_rl, "Ref List", "B:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rp", cmd_rp, "Ref Plan", "d",
   dg87_command_day, dg87_command_access_admin},
  {"rs", cmd_rs, "Ref Simulate", "B:S:E:I:N:d",
   dg87_command_ref, dg87_command_access_consult},
  {"rsa", cmd_rsa, "Ref Set Auto", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rsn", cmd_rsn, "Ref Set Name", "",
   dg87_command_ref, dg87_command_access_manage},

  /* Ref Job functions */
  {"rja", cmd_rja, "Ref Job Add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rjc", cmd_rjc, "Ref Job Count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rjda", cmd_rjda, "Ref Job Dep Add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rjdc", cmd_rjdc, "Ref Job Dep count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rjdk", cmd_rjdk, "Ref Job Dep Kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rjdl", cmd_rjdl, "Ref Job Dep List", "tB:S:E:I:N:b:a:",
   dg87_command_ref, dg87_command_access_consult},
  {"rji", cmd_rji, "Ref Job Information", "B:S:E:",
   dg87_command_ref, dg87_command_access_consult},
  {"rjk", cmd_rjk, "Ref Job Kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rjl", cmd_rjl, "Ref Job List", "dtB:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rjsa", cmd_rjsa, "Ref Job Set Argument", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rjsd", cmd_rjsd, "Ref Job Set Description", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rjsoec", cmd_rjsoec, "Ref Job Set On Error Continue", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rjsp", cmd_rjsp, "Ref Job Set Path", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rjsn", cmd_rjsn, "Ref Job Set Name", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rjst", cmd_rjst, "Ref Job Set Tempo", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rjswad", cmd_rjswad, "Ref Job Set Wait All Dependencies", "",
   dg87_command_ref, dg87_command_access_manage},

  /* Ref Rule functions */
  {"rra", cmd_rra, "Ref Rule add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rrc", cmd_rrc, "Ref Rule count",
   "", dg87_command_ref, dg87_command_access_consult},
  {"rrk", cmd_rrk, "Ref Rule kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rrl", cmd_rrl, "Ref Rule list", "uB:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rrsn", cmd_rrsn, "Ref Rule Set Name", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rrsr", cmd_rrsr, "Ref Rule Simulate Range", "ndB:S:E:",
   dg87_command_ref, dg87_command_access_consult},
  {"rrsy", cmd_rrsy, "Ref Rule Simulate Year", "",
   dg87_command_ref, dg87_command_access_consult},

  /* Ref Rule Operation functions */
  {"rroa", cmd_rroa, "Ref Rule Operation add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rroc", cmd_rroc, "Ref Rule Operation count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rrok", cmd_rrok, "Ref Rule Operation kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rrol", cmd_rrol, "Ref Rule Operation list", "hiB:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rrom", cmd_rrom, "Ref Rule Operation Move", "",
   dg87_command_ref, dg87_command_access_manage},


  /* Ref Task functions */
  {"rta", cmd_rta, "Ref Task add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtc", cmd_rtc, "Ref Task count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rtca", cmd_rtca, "Ref Task Condition Add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtcc", cmd_rtcc, "Ref Task Condition count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rtck", cmd_rtck, "Ref Task Condition Kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtcl", cmd_rtcl, "Ref Task Condition List", "B:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rtd", cmd_rtd, "Ref Task Duplicate", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rtda", cmd_rtda, "Ref Task Dep Add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtdc", cmd_rtdc, "Ref Task Dep count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rtdk", cmd_rtdk, "Ref Task Dep Kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtdl", cmd_rtdl, "Ref Task Dep List", "tB:S:E:I:N:b:a:",
   dg87_command_ref, dg87_command_access_consult},
  {"rti", cmd_rti, "Ref Task Information", "B:S:E:",
   dg87_command_ref, dg87_command_access_consult},
  {"rtk", cmd_rtk, "Ref Task kill", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtl", cmd_rtl, "Ref Task list", "dtB:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rts", cmd_rts, "Ref Task search", "dtB:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rtsl", cmd_rtsl, "Ref Task Set Lock", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtsn", cmd_rtsn, "Ref Task Set Name", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtsr", cmd_rtsr, "Ref Task Simulate Range", "ndB:S:E:",
   dg87_command_ref, dg87_command_access_consult},
  {"rtsy", cmd_rtsy, "Ref Task Simulate Year", "",
   dg87_command_ref, dg87_command_access_consult},

  /* Ref Task Operation functions */
  {"rtoa", cmd_rtoa, "Ref Task Operation add", "",
   dg87_command_ref, dg87_command_access_admin},
  {"rtoc", cmd_rtoc, "Ref Task Operation count", "",
   dg87_command_ref, dg87_command_access_consult},
  {"rtok", cmd_rtok, "Ref Task Operation kill", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtol", cmd_rtol, "Ref Task Operation list", "hiB:S:E:I:N:",
   dg87_command_ref, dg87_command_access_consult},
  {"rtom", cmd_rtom, "Ref Task Operation Move", "",
   dg87_command_ref, dg87_command_access_manage},

  {"rtsc", cmd_rtsc, "Ref Task Set Cyclic", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtsd", cmd_rtsd, "Ref Task Set Description", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtse", cmd_rtse, "Ref Task Set Enable", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtsoec", cmd_rtsoec, "Ref Task Set On Error Continue", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtst", cmd_rtst, "Ref Task Set Target", "",
   dg87_command_ref, dg87_command_access_manage},
  {"rtswad", cmd_rtswad, "Ref Task Set Wait All Dependencies", "",
   dg87_command_ref, dg87_command_access_manage},

  /* Save function */
  {"save", cmd_save, "Save", "",
   dg87_command_engine, dg87_command_access_manage},

  /* Shutdown function */
  {"shutdown", cmd_shutdown, "Stop the execution of dg87d", "",
   dg87_command_engine, dg87_command_access_manage},

  /* Stat Ref functions */
  {"src", cmd_src, "Stat Ref Count", "",
   dg87_command_stat, dg87_command_access_consult},
  {"srk", cmd_srk, "Stat Ref Kill", "",
   dg87_command_stat, dg87_command_access_manage},
  {"srl", cmd_srl, "Stat Ref List", "B:S:E:I:N:",
   dg87_command_stat, dg87_command_access_consult},

  /* Stat job functions */
  {"sjc", cmd_sjc, "Stat Job Count", "",
   dg87_command_stat, dg87_command_access_consult},
  {"sji", cmd_sji, "Stat Job Information", "B:S:E:dh",
   dg87_command_stat, dg87_command_access_consult},
  {"sjl", cmd_sjl, "Stat Job List", "B:S:E:I:N:sh",
   dg87_command_stat, dg87_command_access_consult},

  /* stat task functions */
  {"stc", cmd_stc, "Stat Task Count", "",
   dg87_command_stat, dg87_command_access_consult},
  {"sti", cmd_sti, "Stat Task Information", "B:S:E:dh",
   dg87_command_stat, dg87_command_access_consult},
  {"stl", cmd_stl, "Stat Task List", "B:S:E:I:N:sh",
   dg87_command_stat, dg87_command_access_consult},

  /* Target functions */
  {"ta", cmd_ta, "Target Add", "",
   dg87_command_target, dg87_command_access_admin},
  {"tc", cmd_tc, "Target Count", "",
   dg87_command_target, dg87_command_access_consult},
  {"ti", cmd_ti, "Target Information", "B:S:E:",
   dg87_command_target, dg87_command_access_consult},
  {"tk", cmd_tk, "Target Kill", "",
   dg87_command_target, dg87_command_access_manage},
  {"tl", cmd_tl, "Target List", "dB:S:E:I:N:",
   dg87_command_target, dg87_command_access_consult},
  {"tsd", cmd_tsd, "Target Set Description", "",
   dg87_command_target, dg87_command_access_manage},
  {"tsh", cmd_tsh, "Target Set HostName", "",
   dg87_command_target, dg87_command_access_manage},
  {"tsn", cmd_tsn, "Target Set Name", "",
   dg87_command_target, dg87_command_access_manage},
  {"tso", cmd_tso, "Target Set Open", "",
   dg87_command_target, dg87_command_access_manage},
  {"tsp", cmd_tsp, "Target Set ip_port", "",
   dg87_command_target, dg87_command_access_manage},

  /* Variable functions */
  {"va", cmd_va, "Variable Value Add", "",
   dg87_command_variable, dg87_command_access_admin},
  {"vc", cmd_vc, "Variable Count", "",
   dg87_command_variable, dg87_command_access_consult},
  {"vi", cmd_vi, "Variable Information", "B:S:E:",
   dg87_command_variable, dg87_command_access_consult},
  {"vk", cmd_vk, "Variable Kill", "",
   dg87_command_variable, dg87_command_access_manage},
  {"vl", cmd_vl, "Variable List", "vB:S:E:I:N:",
   dg87_command_variable, dg87_command_access_consult},
  {"vsv", cmd_vsv, "Variable Set Description", "",
   dg87_command_variable, dg87_command_access_manage},

  /* Day functions */
  {"xdc", cmd_xdc, "eXploitation Day Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"xdi", cmd_xdi, "eXploitation Day Information", "B:S:E:",
   dg87_command_day, dg87_command_access_consult},
  {"xdk", cmd_xdk, "eXploitation Day Kill", "",
   dg87_command_day, dg87_command_access_admin},
  {"xdl", cmd_xdl, "eXploitation Day List", "B:S:E:I:N:",
   dg87_command_day, dg87_command_access_consult},
  {"xdsa", cmd_xdsa, "eXploitation Day Set AutoPurged", "",
   dg87_command_day, dg87_command_access_manage},
  {"xdse", cmd_xdse, "eXploitation Day Set Enabled", "",
   dg87_command_day, dg87_command_access_manage},

  /* Job functions */
  {"xjc", cmd_xjc, "eXploitation Job Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"xjdc", cmd_xjdc, "eXploitation Job Dependence Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"xjdl", cmd_xjdl, "eXploitation Job Dependence List",
   "stB:S:E:I:N:b:a:", dg87_command_day, dg87_command_access_consult},
  {"xjdu", cmd_xjdu, "eXploitation Job Dependence Update", "",
   dg87_command_day, dg87_command_access_manage},
  {"xji", cmd_xji, "eXploitation Job Information", "B:S:E:",
   dg87_command_day, dg87_command_access_consult},
  {"xjl", cmd_xjl, "eXploitation Job List", "sdtB:S:E:I:N:",
   dg87_command_day, dg87_command_access_consult},
  {"xjsa", cmd_xjsa, "eXploitation Job Set Argument", "",
   dg87_command_day, dg87_command_access_manage},
  {"xjse", cmd_xjse, "eXploitation Job Set Enabled", "",
   dg87_command_day, dg87_command_access_manage},
  {"xjsoec", cmd_xjsoec, "eXploitation Job Set On Error Continue", "",
   dg87_command_day, dg87_command_access_manage},
  {"xjsp", cmd_xjsp, "eXploitation Job Set Path", "",
   dg87_command_day, dg87_command_access_manage},
  {"xjst", cmd_xjst, "eXploitation Job Set Tempo", "",
   dg87_command_day, dg87_command_access_manage},
  {"xju", cmd_xju, "eXploitation Job Update", "",
   dg87_command_day, dg87_command_access_manage},
  {"xjswad", cmd_xjswad, "eXploitation Job Set Wait All Dependencies", "",
   dg87_command_day, dg87_command_access_manage},

  /* Task functions */
  {"xtc", cmd_xtc, "eXploitation Task Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"xtcc", cmd_xtcc, "eXploitation Task Condition Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"xtcl", cmd_xtcl, "eXploitation Task Condition List",
   "sB:S:E:I:N:", dg87_command_day, dg87_command_access_consult},
  {"xtcv", cmd_xtcv, "eXploitation Task Condition Validate", "",
   dg87_command_day, dg87_command_access_manage},
  {"xtdc", cmd_xtdc, "eXploitation Task Dependence Count", "",
   dg87_command_day, dg87_command_access_consult},
  {"xtdl", cmd_xtdl, "eXploitation Task Dependence List",
   "stB:S:E:I:N:b:a:", dg87_command_day, dg87_command_access_consult},
  {"xtdu", cmd_xtdu, "eXploitation Task Dependence Update", "",
   dg87_command_day, dg87_command_access_manage},
  {"xti", cmd_xti, "eXploitation Task Information", "B:S:E:",
   dg87_command_day, dg87_command_access_consult},
  {"xtl", cmd_xtl, "eXploitation Task List", "sdtB:S:E:I:N:",
   dg87_command_day, dg87_command_access_consult},
  {"xtse", cmd_xtse, "eXploitation Task Set Enable", "",
   dg87_command_day, dg87_command_access_admin},
  {"xtsoec", cmd_xtsoec, "eXploitation Task Set On Error Continue", "",
   dg87_command_day, dg87_command_access_manage},
  {"xtswad", cmd_xtswad, "eXploitation Task Set Wait All Dependencies", "",
   dg87_command_day, dg87_command_access_manage},
  {"xtu", cmd_xtu, "eXploitation Task Update", "",
   dg87_command_day, dg87_command_access_manage},

  {NULL, NULL, NULL, NULL, 0, 0}
};

/******************************************************************************/
#ifdef DEBUG
/* TODO find a way to remove this local variable */
static dg87_connection *p_local_connection;
/* callback function for messages printing */
int
callback (char *message)
{
  assert (message != NULL);
  dg87_connection_printf (p_local_connection, message);
  return 0;
}

int
cmd_check (dg87_connection * connection)
{
  dg87_context *context;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;

  context = connection->server->data;
  if (context == NULL)
    {
      dg87_connection_printf (connection, "ERROR : Context is NULL !\n");
      return 1;
    }
  p_local_connection = connection;
  if (dg87_context_check (context, callback) != 0)
    {
      dg87_connection_printf (connection, "ERROR !\n");
      return 1;
    }

  dg87_connection_printf (connection, "All is right ;-)\n", PACKAGE);
  return 0;
}
#endif
/******************************************************************************/
int
cmd_shutdown (dg87_connection * connection)
{
  assert (connection != NULL);
  dg87_connection_printf (connection, 
									gettext ("%s will stop...\n"), DIO87D_NAME);
  return -1;
}



/******************************************************************************/
int
signal_function (dg87_server * server, int signo)
{
  dg87_context *context;
  assert (server != NULL);
  if (server == NULL)
    return FALSE;
  context = server->data;

  switch (signo)
    {
    case SIGHUP:
      dg87_context_read_config (context);
			dg87_daemon_log_printf(dg87_daemon_log_warning,
											"Signal 'SIGHUP' received : reread configuration file");
      break;
    case SIGTERM:
      break;
    case SIGINT:
      break;
    case SIGPIPE:
      break;
    case SIGUSR1:              /* save database */
			dg87_daemon_log_printf(dg87_daemon_log_warning,
											"Signal 'SIGUSR1' received : save database.");
      if (dg87_context_save (context) == FALSE)
				dg87_daemon_log_printf(dg87_daemon_log_error,
											"Save error !");
      else
				dg87_daemon_log_printf(dg87_daemon_log_warning,
											"Save done !");
      break;
    default:
			dg87_daemon_log_printf(dg87_daemon_log_warning,
											"SIGNAL NOT HANDLED");
      break;
    }

  return FALSE;
}

/******************************************************************************/
BOOL
work_function (dg87_server * server)
{
  dg87_context *context;
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = server->data;
  return dg87_engine_run (context->engine, server);
}

/******************************************************************************/
static void
usage ()
{
	char** p;
#define gettext_noop(String) (String)
#ifdef DG87_USE_LONG_OPTIONS
	static char* msg[] = {
gettext_noop ("Usage : dio87d [OPTIONS]\nOptions :\n"),
gettext_noop (" -C file_name, --configuration file_name\n"),
gettext_noop ("\tuse the specified configuration file.\n"),
gettext_noop (" -d, --daemon\n"),
gettext_noop ("\tstart process as daemon\n"),
gettext_noop (" -I, --initialize\n"),
gettext_noop ("\tcreate database file and exit\n"),
gettext_noop (" -S, --suspend\n"),
gettext_noop ("\tsuspend engine execution at startup\n"),
gettext_noop ("\nReport bugs to admin@diogene87.org\n"),
NULL
	};
#else /*DG87_USE_LONG_OPTIONS*/
	static char* msg[] = {
gettext_noop ("Usage : dio87d [OPTIONS]\nOptions :\n"),
gettext_noop (" -C file_name\n"),
gettext_noop ("\tuse the specified configuration file.\n"),
gettext_noop (" -d\n"),
gettext_noop ("\tstart process as daemon\n"),
gettext_noop (" -I\n"),
gettext_noop ("\tcreate database file and exit\n"),
gettext_noop (" -S\n"),
gettext_noop ("\tsuspend engine execution at startup\n"),
gettext_noop ("\nReport bugs to admin@diogene87.org\n"),
NULL
};
#endif /*DG87_USE_LONG_OPTIONS*/
	p = msg;
	while (*p != NULL){
		fprintf(stderr, "%s", gettext (*p));
		p++;
	}
}

/******************************************************************************/
static void
version (void)
{
  fprintf (stderr, gettext( "%s (%s) %s\n"\
						"Copyright (C) 2003,2004,2005 Dominique Parisot.\n"\
						"Diogene87 comes with NO WARRANTY,\n"\
						"to the extent permitted by law.\n"\
						"You may redistribute copies of Diogene87\n"\
						"under the terms of the GNU General Public License.\n"\
						"For more information about these matters,\n"\
						"see the file named COPYING.\n"),
					DIO87D_NAME, PACKAGE, VERSION);

}

/******************************************************************************/
int
main (int argc, char *argv[])
{
  int optch;
  int res;
  static char stropts[] = "C:hvdIS";
#if DG87_USE_LONG_OPTIONS
  static struct option lopts[] = {
    /* name, has_arg, flag, val */
    {"configuration", 1, 0, 'C'},
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'v'},
    {"daemon", 0, 0, 'd'},
    {"initialize", 0, 0, 'I'},
    {"suspend", 0, 0, 'S'},
    {0, 0, 0, 0}
  };
  int cmdx;
#endif /*DG87_USE_LONG_OPTIONS */

  dg87_daemon *daemon = NULL;
  dg87_context *context;
  dg87_server *server;
  void *data;
  BOOL start_daemon = FALSE;
  BOOL suspend_engine = FALSE;
  BOOL new_database = FALSE;
  BOOL init_database = FALSE;
  char *conf_file = NULL;
  struct stat sb;

	/* gettext */
	if (setlocale (LC_ALL,"") == NULL)
					fprintf(stderr, "Warning : Can't set locale !\n");
	if (bindtextdomain (PACKAGE, LOCALEDIR) == NULL)
					fprintf(stderr, "Warning : Can't bind text domain !\n");
	if (textdomain(PACKAGE) == NULL)
					fprintf(stderr, "Warning : Can't set text domain !\n");

	/* initialize error to no_error */
	dg87_err_clear_error();
	dg87_update_current_time();

  /* interpret options */
#if DG87_USE_LONG_OPTIONS
  while ((optch = getopt_long (argc, argv, stropts, lopts, &cmdx)) != EOF)
#else /*DG87_USE_LONG_OPTIONS */
  while ((optch = getopt (argc, argv, stropts)) != -1)
#endif /*not DG87_USE_LONG_OPTIONS */
    {
      switch (optch)
        {
        case 'I':              /* init database */
          init_database = TRUE;
          break;
        case 'N':              /* new database */
          new_database = TRUE;
          break;
        case 'C':              /* configuration file */
          conf_file = dg87_strdup (optarg, DG87_CHAR);
          break;
        case 'S':              /* stop engine (for recovery) */
          suspend_engine = TRUE;
          break;
        case 'h':              /* help */
          usage ();
          return 1;
        case 'v':              /* version */
          version ();
          return 1;
        case 'd':              /* start as daemon */
          start_daemon = TRUE;
          break;
        default:
          fprintf (stderr, gettext ("Use -h for help.\n"));
          return 1;
        }
    }
	/* check that no arguments are supplied */
	if (optind < argc ) 
	{
		usage ();
		return 1;
	}

  /* create the context object */
  if (conf_file == NULL)
    conf_file = dg87_strdup (DIO87D_CONFIGFILE, DG87_CHAR);
  if ((context = dg87_context_new (conf_file)) == NULL)
    {
      fprintf (stderr, gettext ("Can't create the object '%s' !\n"),
											"connection");
      exit (1);
    }
  data = context;
  dg87_free (conf_file, DG87_CHAR);

  /* daemonize */
  if (start_daemon == TRUE)
    {
      daemon = dg87_daemon_new (DIO87D_NAME,
                                dg87_conf_get_field_value (context->conf,
                                                           "USER_ACCOUNT"));
      if (daemon == NULL)
        {
      		fprintf (stderr, gettext ("Can't create the object '%s' !\n"),
													"daemon");
          dg87_context_delete (context);
          return 1;
        }

      res = dg87_daemon_start (daemon);
      if (res < 0)
        {
          fprintf (stderr, gettext ("Can't daemonize !\n"));
          dg87_daemon_delete (daemon);
          dg87_context_delete (context);
          return 1;
        }

      if (res == 0)
        {
          /* the parent process terminates here */
          dg87_daemon_delete (daemon);
          dg87_context_delete (context);
          return 0;
        }
      context->daemon = daemon;
    }

	/* print some informations */
	dg87_daemon_log_printf(dg87_daemon_log_info,
									"%s-%s-%s", PACKAGE, DIO87D_NAME, VERSION);
	dg87_daemon_log_printf(dg87_daemon_log_info,
						"Copyright (C) 2003,2004,2005 Dominique Parisot.");
	dg87_daemon_log_printf(dg87_daemon_log_info,
						"GNU GENERAL PUBLIC LICENSE Version 2.");

  /* create the server object */
  server = dg87_server_new (PACKAGE, DIO87D_NAME, VERSION,
                            context->ip_port, context->ip_addr,
                            work_function, signal_function, data, cmd_array,
                            context->pid_file);
  if (server == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
				gettext ("Can't create the object '%s' !"), "server");
      dg87_context_delete (context);
      return 1;
    }
  if (dg87_context_set_server (context, server) == FALSE)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
				gettext ("Can't create the object '%s' !"), "server");
      return 1;
    }

  /* option -I : init database and exit */
  if (init_database == TRUE)
    {
			dg87_daemon_log_printf(dg87_daemon_log_info,
											"Initialize database, because '-I' option was used !");
      if (stat (context->storage->data_file, &sb) != -1 || errno != ENOENT)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	gettext ("Database already exists : %s"),
                   context->storage->data_file);
          dg87_server_delete (server);
          dg87_context_delete (context);
          return 1;
        }
			dg87_daemon_log_printf(dg87_daemon_log_info,
      	gettext ("Create database :%s"), 
											context->storage->data_file);
      if (dg87_context_save (context) == FALSE)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	gettext ("Can't save database '%s' ! %s"),
                   context->storage->data_file, dg87_err_get_error_str ());
          dg87_server_delete (server);
          dg87_context_delete (context);
          return 1;
        }
      dg87_server_delete (server);
      dg87_context_delete (context);
      return 0;
    }

  /* set the current directory */
	/* TODO move this code in a function : this code is duplicated in context.c */
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	gettext ("Configuration file : [%s]"), context->conf->config_file);
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	gettext ("Data directory : [%s]"), context->data_directory);
  if (chdir (context->data_directory) == -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	gettext ("Can't open the data directory '%s' !"),
               context->data_directory);
      dg87_context_delete (context);
      return 1;
    }


  /* load database */
  if (new_database == TRUE)
    {
			dg87_daemon_log_printf(dg87_daemon_log_warning,
      	gettext ("Don't load database, because -N option was specified."));
    }
  else
    {
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	gettext ("Load database :%s"), 
				context->storage->data_file);
      if (dg87_context_load (context) == FALSE)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	gettext ("Can't load database '%s' ! %s"),
                   context->storage->data_file, dg87_err_get_error_str ());
          dg87_context_delete (context);
          return 1;
        }

      /* load log days */
      if (dg87_context_log_load_days (context) == FALSE)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	gettext ("Can't load log days ! %s"),
                   dg87_err_get_error_str ());
          dg87_context_delete (context);
          return 1;
        }

      /* load stat referentials */
      if (dg87_context_sta_load_referentials (context) == FALSE)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	gettext ("Can't load stat refs ! %s"),
                   dg87_err_get_error_str ());
          dg87_context_delete (context);
          return 1;
        }

    }
	/* disable engine if -s option was used */
	if (suspend_engine == TRUE) {
		dg87_daemon_log_printf(dg87_daemon_log_warning,
  		gettext ("Suspend engine execution (option -s was used)."));
		dg87_engine_set_enabled(context->engine, FALSE);
	}

	/* server is ready */
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	gettext ("Server ready."));
  if (dg87_server_run (server) == TRUE)
    {
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	gettext ("Save database :%s"), 
											context->storage->data_file);
      if (dg87_context_save (context) == FALSE)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	gettext ("Can't save database '%s' ! %s"),
                   context->storage->data_file, dg87_err_get_error_str ());
          dg87_context_delete (context);
          dg87_server_delete (server);
          return 1;
        }
    }

  dg87_server_delete (server);
  dg87_context_delete (context);
  if (start_daemon == TRUE)
    {
      dg87_daemon_stop (daemon);
      dg87_daemon_delete (daemon);
    }
  return 0;
}

/******************************************************************************/
/* End of file */
