/* file r_cmdfile.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>
#include <libdg87/daemon.h>
#include <libdg87/command.h>
#include <libdg87/daemon.h>     /* log function */

#include "context.h"
#include "util.h"
#include "r_cmdfile.h"


/******************************************************************************/
/* fk
 * File Kill
 * Usage : fk day_name TaskName FileName 
 */
int
cmd_fk (dg87_connection * connection)
{
  dg87_r_file *file;
  dg87_context *context;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "fk DayName TaskName FileName\n");
      return 1;
    }

  /* find the file */
  file = dg87_r_util_find_file (connection, command->argv[0],   /* day */
                                command->argv[1],       /* task */
                                command->argv[2]);      /* path */
  if (file == NULL)
    return 1;

  /* delete the file */
  dg87_err_clear_error ();
  context = connection->server->data;
  if (dg87_r_context_file_delete (context, file) == FALSE)
    {
      dg87_connection_printf (connection, gettext ("Can't delete the file '%s' '%s' '%s' ! %s\n"), command->argv[0],      /* day */
                              command->argv[1], /* task */
                              command->argv[2], /* path */
                              dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************
 * cmd_fa
 * Ref File Add
 * Usage : fa DayName TaskName Path HostName IpPort
 */
int
cmd_fa (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_r_file *file;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 5)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "fa DayName TaskName Path HostName ip_port");
      return 1;
    }

  /* add the file */
  dg87_err_clear_error ();
  context = connection->server->data;
  file = dg87_r_context_file_add (context, command->argv[0],    /* day */
                                  command->argv[1],     /* task */
                                  command->argv[2],     /* path */
                                  command->argv[3],     /* HostName */
                                  atoi (command->argv[4]));     /* IP port */
  if (file == NULL)
    {
      dg87_connection_printf (connection, 
											gettext ("Can't add the file '%s' '%s' '%s' ! %s\n"), 
											command->argv[0],     /* day */
                      command->argv[1], /* task */
                      command->argv[2], /* file */
                      dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_fc
 * Usage : fc
 * File Count
 * return the number of files.
 */
int
cmd_fc (dg87_connection * connection)
{
  dg87_context *context;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  dg87_connection_printf (connection, "%i\n", context->file_list->count);
  return 0;
}

/******************************************************************************/
/* cmd_fl
 * Usage : fl [options]
 * File List
 * Options
 * -d : print description of file 
 * -t : print task name of file
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first file to display (first index is 0)
 * -Nnumber : number of files to display (no limit if -N < 1)
 * -s : state
 */
int
cmd_fl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  int start = 0;                /* index of the first file to display */
  int number = 0;               /* number of files to display */
  int count = 0;                /* number of files actually display */
  dg87_item *item = NULL;
  dg87_r_file *file;
  dg87_context *context;
  dg87_command *command;
  BOOL show_host = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "fl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first file to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of task to display */
          number = atoi (optarg);
          break;
        case 'h':              /* show hostname and ip_port */
          show_host = TRUE;
          break;

        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first file to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (context->file_list, start);
  else
    item = context->file_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of file to display */
      count++;
      if (number > 0 && count > number)
        break;
      file = item->object;

      /* print FileName */
      dg87_connection_printf (connection, "%s%s%s%s%s%s",
                              beg, file->day_name,
                              sep, file->task_name, sep, file->path);
      if (show_host == TRUE)
        dg87_connection_printf (connection, "%s%s%s%i",
                                sep, file->hostname, sep, file->ip_port);
      dg87_connection_printf (connection, "%s\n", end);
      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* End of file */
