/* file s_cmdjob.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <assert.h>
#include <string.h>

#include "context.h"
#include "s_cmdref.h"
#include "util.h"

/******************************************************************************/
/* cmd_sjl
 * Stat Job List 
 * Usage : sjl [options] referential_name task_name
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 * -s : print statistical informations : minimum, maximum, average duration
 * -h : print duration with `hh:mm:ss' format
 */
int
cmd_sjl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_sta_ref *ref;
  dg87_statnode task_node;
  dg87_command *command;
  dg87_statnode job_node;
  BOOL print_stat = FALSE;
  BOOL human_format = FALSE;
  dg87uint mini, maxi, average;
  char buffer[15];

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "sjl [options] RefName TaskName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_sta_referential (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_sta_task_node (connection, ref,
                                    command->argv[1], &task_node) == FALSE)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 's':              /* print statistical informations */
          print_stat = TRUE;
          break;
        case 'h':              /* use 'hh:mm:ss' format */
          human_format = TRUE;
          break;
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* read first job */
  if (dg87_sta_ref_job_read_first (ref, &task_node, &job_node) == FALSE)
    {
      /* quit if no entries or if reading error */
      free (sep);
      free (beg);
      free (end);
      return 0;
    }

  /* for each job */
  while (1)
    {
      /* print job informations */
      dg87_connection_printf (connection, "%s%s", beg, job_node.name);

      if (print_stat == TRUE)
        {
          /* get durations */
          if (dg87_sta_ref_job_stat (ref, &job_node,
                                     &average, &mini, &maxi) == FALSE)
            {
              dg87_connection_printf (connection, 
															gettext ("Can't get stat durations !\n"));
              free (sep);
              free (beg);
              free (end);
              return 1;
            }
          if (human_format == TRUE)
            {
              dg87_duration_to_ascii (average, buffer);
              dg87_connection_printf (connection, "%s%s", sep, buffer);
              dg87_duration_to_ascii (mini, buffer);
              dg87_connection_printf (connection, "%s%s", sep, buffer);
              dg87_duration_to_ascii (maxi, buffer);
              dg87_connection_printf (connection, "%s%s", sep, buffer);
            }
          else
            {
              dg87_connection_printf (connection, "%s%i", sep, average);
              dg87_connection_printf (connection, "%s%i", sep, mini);
              dg87_connection_printf (connection, "%s%i", sep, maxi);
            }
        }

      dg87_connection_printf (connection, "%s\n", end);
      /* quit if there is no next node */
      if (job_node.next_node == 0)
	      break;
      /* read next job */
      if (dg87_sta_ref_job_read_next (ref, &job_node) == FALSE)
        {
          /* quit if no entries or if reading error */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* cmd_sjc
 * Stat Job Count
 * return the number of jobs of a jobs.
 * Usage : sjc referential_name task_name
 */
int
cmd_sjc (dg87_connection * connection)
{
  dg87_sta_ref *ref;
  dg87_statnode task_node;
  dg87_command *command;
  dg87uint number;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "sjc RefName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_sta_referential (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the task */
  if (dg87_util_find_sta_task_node (connection, ref,
                                    command->argv[1], &task_node) == FALSE)
    return 1;

  if (dg87_sta_ref_job_get_count (ref, &task_node, &number) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't get the jobs number of the "\
																			"task '%s' of the referential '%s'\n"),
                              task_node.name, ref->name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", number);
  return 0;
}

/******************************************************************************/
/* cmd_sji
 * Stat Job Information 
 * returns statistical durations or last durations of a job. 
 * Usage : sji [options] referential_name task_name job_name
 * Options:
 * -d : print 30 last job durations (with start time)
 * -h : print duration with `hh:mm:ss' format
 */
int
cmd_sji (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_sta_ref *ref;
  dg87_context *context;
  dg87_command *command;
  dg87_statnode task_node;
  dg87_statnode job_node;
  BOOL print_last_durations = FALSE;
  BOOL human_format = FALSE;
  char buffer[15];
  int i;
  char *str_date;
  dg87uint mini, maxi, average;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  command = connection->command;
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "sji [options] RefName TaskName JobName");
      return 1;
    }
  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");
  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'd':              /* print last durations */
          print_last_durations = TRUE;
          break;
        case 'h':              /* print duration with hh:mm:ss format */
          human_format = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the ref */
  if ((ref =
       dg87_util_find_sta_referential (connection, command->argv[0])) == NULL)
    {
      free (sep);
      free (beg);
      free (end);
      return 1;
    }
  /* find the task */
  if (dg87_util_find_sta_task_node (connection, ref,
                                    command->argv[1], &task_node) == FALSE)
    {
      free (sep);
      free (beg);
      free (end);
      return 1;
    }
  /* find the job */
  if (dg87_util_find_sta_job_node (connection, ref, &task_node,
                                   command->argv[2], &job_node) == FALSE)
    {
      free (sep);
      free (beg);
      free (end);
      return 1;
    }

  if (print_last_durations == TRUE)
    {
      /* print last durations */
      for (i = 0; i < DG87_STATFILE_STATENTRYCOUNT; i++)
        {
          if (job_node.stat_entries[i].start_time == 0)
            continue;
          str_date = dg87_date_to_ascii (job_node.stat_entries[i].start_time);
          if (human_format == TRUE)
            {
              dg87_duration_to_ascii (job_node.stat_entries[i].duration,
                                      buffer);
              dg87_connection_printf (connection, "%s%s%s%s%s\n", beg,
                                      str_date, sep, buffer, end);
            }
          else
            {
              dg87_connection_printf (connection,
                                      "%s%s%s%i%s\n", beg, str_date, sep,
                                      job_node.stat_entries[i].duration, end);
            }
          dg87_free (str_date, DG87_CHAR);
        }
    }
  else
    {
      if (dg87_sta_ref_job_stat (ref, &job_node,
                                 &average, &mini, &maxi) == FALSE)
        {
          dg87_connection_printf (connection, 
													gettext ("Can't get stat informations !\n"));
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
      if (human_format == TRUE)
        {
          dg87_duration_to_ascii (average, buffer);
          dg87_connection_printf (connection, "%s%s%s%s%s\n",
                                  beg, "average", sep, buffer, end);
          dg87_duration_to_ascii (mini, buffer);
          dg87_connection_printf (connection, "%s%s%s%s%s\n",
                                  beg, "mini", sep, buffer, end);
          dg87_duration_to_ascii (maxi, buffer);
          dg87_connection_printf (connection, "%s%s%s%s%s\n",
                                  beg, "maxi", sep, buffer, end);
        }
      else
        {
          dg87_connection_printf (connection, "%s%s%s%i%s\n",
                                  beg, "average", sep, average, end);
          dg87_connection_printf (connection, "%s%s%s%i%s\n",
                                  beg, "mini", sep, mini, end);
          dg87_connection_printf (connection, "%s%s%s%i%s\n",
                                  beg, "maxi", sep, maxi, end);
        }
    }
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* End of file */
