/* file s_ref.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "s_ref.h"

/****************************************************************************/
dg87_sta_ref *
dg87_sta_ref_new (char *name, char *filename, uid_t uid, uid_t gid)
{
  dg87_sta_ref *ref;
  dg87_statnode root_node;
  assert (name != NULL && filename != NULL);
  if (name == NULL || filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_REF_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory for ref */
  ref = dg87_malloc (sizeof (dg87_sta_ref), DG87_REF);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  /* initialize the structure of the object */
  dg87_strnzcpy (ref->name, name, DG87_REF_NAME_LENGTH);
  /* create stat file */
  if ((ref->stat_file = dg87_statfile_create (filename, uid, gid)) == NULL)
    {
      dg87_free (ref, DG87_REF);
      return NULL;
    }
  /* create root node */
  if ((dg87_statfile_add_node (ref->stat_file,
                               name, NULL, &root_node)) == FALSE)
    {
      dg87_statfile_close (ref->stat_file, TRUE);
      dg87_free (ref, DG87_REF);
      return NULL;
    }
  return ref;
}

/****************************************************************************/
dg87_sta_ref *
dg87_sta_ref_load (char *filename)
{
  dg87_sta_ref *ref;
  dg87_statfile *stat_file;
  dg87_statnode root_node;

  assert (filename != NULL);
  if (filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* open stat file */
  if ((stat_file = dg87_statfile_open (filename)) == NULL)
    return NULL;
  /* read root node */
  if ((dg87_statfile_get_first_child (stat_file, NULL, &root_node)) == FALSE)
    {
      dg87_statfile_close (stat_file, FALSE);
      return NULL;
    }
  /* allocate memory for ref */
  ref = dg87_malloc (sizeof (dg87_sta_ref), DG87_REF);
  if (ref == NULL)
    {
      dg87_statfile_close (stat_file, FALSE);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  /* initialize the structure of the object */
  dg87_strnzcpy (ref->name, root_node.name, DG87_REF_NAME_LENGTH);
  ref->stat_file = stat_file;
  return ref;
}

/****************************************************************************/
BOOL
dg87_sta_ref_delete (dg87_sta_ref * ref, BOOL delete_file)
{
  assert (ref != NULL);
  if (ref == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_statfile_close (ref->stat_file, delete_file) == FALSE)
    return FALSE;
  dg87_free (ref, DG87_REF);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_add (dg87_sta_ref * ref, char *task_name)
{
  dg87_statnode root_node;
  dg87_statnode new_node;
  assert (ref != NULL && task_name != NULL);
  if (ref == NULL || task_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_statfile_get_first_child (ref->stat_file,
                                      NULL, &root_node)) == FALSE)
    return FALSE;
  /* create new node */
  if (dg87_statfile_add_node (ref->stat_file, task_name,
                              &root_node, &new_node) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_read_first (dg87_sta_ref * ref, dg87_statnode * task_node)
{
  dg87_statnode root_node;
  assert (ref != NULL && task_node != NULL);
  if (ref == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_statfile_get_first_child (ref->stat_file,
                                      NULL, &root_node)) == FALSE)
    {
      return FALSE;
    }
  /* read first node (if exits) */
  if (dg87_statfile_get_first_child (ref->stat_file,
                                     &root_node, task_node) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_read_next (dg87_sta_ref * ref, dg87_statnode * task_node)
{
  assert (ref != NULL && task_node != NULL);
  if (ref == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next task node (if exits) */
  if (dg87_statfile_get_next_node (ref->stat_file, task_node) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_get_count (dg87_sta_ref * ref, dg87uint * number)
{
  dg87_statnode root_node;
  assert (ref != NULL && number != NULL);
  if (ref == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read root node */
  if ((dg87_statfile_get_first_child (ref->stat_file,
                                      NULL, &root_node)) == FALSE)
    return FALSE;
  *number = root_node.children_number;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_get_by_name (dg87_sta_ref * ref, char *name,
                               dg87_statnode * task_node)
{
  assert (ref != NULL && name != NULL && task_node != NULL);
  /* read first task */
  if (dg87_sta_ref_task_read_first (ref, task_node) == FALSE)
    return FALSE;
  while (1)
    {
      /* compare task name */
      if (strcmp (name, task_node->name) == 0)
        return TRUE;
      /* read next task */
      if (dg87_sta_ref_task_read_next (ref, task_node) == FALSE)
        break;
    }
  /* task not found */
  dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_update (dg87_sta_ref * ref, dg87_statnode * task_node,
                          time_t start_time, dg87uint duration)
{
  assert (ref != NULL && task_node != NULL);
  if (ref == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_statfile_update_node (ref->stat_file, task_node,
                                 start_time, duration) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_task_stat (dg87_sta_ref * ref, dg87_statnode * task_node,
                        dg87uint * average, dg87uint * mini, dg87uint * maxi)
{
  assert (ref != NULL && task_node != NULL &&
          average != NULL && mini != NULL && maxi != NULL);
  if (dg87_statfile_stat_node (ref->stat_file, task_node,
                               average, mini, maxi) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_add (dg87_sta_ref * ref, dg87_statnode * task_node,
                      char *job_name)
{
  dg87_statnode new_node;
  assert (ref != NULL && task_node != NULL && job_name != NULL);
  if (ref == NULL || task_node == NULL || job_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* create new node */
  if (dg87_statfile_add_node (ref->stat_file, job_name,
                              task_node, &new_node) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_read_first (dg87_sta_ref * ref, dg87_statnode * task_node,
                             dg87_statnode * job_node)
{
  assert (ref != NULL && job_node != NULL);
  if (ref == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read first node (if exits) */
  if (dg87_statfile_get_first_child (ref->stat_file,
                                     task_node, job_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_read_next (dg87_sta_ref * ref, dg87_statnode * job_node)
{
  assert (ref != NULL && job_node != NULL);
  if (ref == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read next job node (if exits) */
  if (dg87_statfile_get_next_node (ref->stat_file, job_node) == FALSE)
    {
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_get_count (dg87_sta_ref * ref, dg87_statnode * task_node,
                            dg87uint * number)
{
  assert (ref != NULL && number != NULL);
  if (ref == NULL || number == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  *number = task_node->children_number;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_get_by_name (dg87_sta_ref * ref, dg87_statnode * task_node,
                              char *name, dg87_statnode * job_node)
{
  assert (ref != NULL && task_node != NULL && name != NULL
          && job_node != NULL);
  /* read first job */
  if (dg87_sta_ref_job_read_first (ref, task_node, job_node) == FALSE)
    return FALSE;
  while (1)
    {
      /* compare job name */
      if (strcmp (name, job_node->name) == 0)
        return TRUE;
      /* read next job */
      if (dg87_sta_ref_job_read_next (ref, job_node) == FALSE)
        break;
    }
  /* job not found */
  dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_update (dg87_sta_ref * ref, dg87_statnode * job_node,
                         time_t start_time, dg87uint duration)
{
  assert (ref != NULL && job_node != NULL);
  if (ref == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_statfile_update_node (ref->stat_file, job_node,
                                 start_time, duration) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_sta_ref_job_stat (dg87_sta_ref * ref, dg87_statnode * job_node,
                       dg87uint * average, dg87uint * mini, dg87uint * maxi)
{
  assert (ref != NULL && job_node != NULL &&
          average != NULL && mini != NULL && maxi != NULL);
  if (dg87_statfile_stat_node (ref->stat_file, job_node,
                               average, mini, maxi) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_sta_ref_check (dg87_sta_ref * ref, int (*print) (char *))
{
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
