/* file s_stat.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>         /* uint16_t- uint32_t */

#include <libdg87/daemon.h>
#include "s_stat.h"

#define DG87S_LOGFILE_SIGNATURE	"dg87s"
#define DG87S_LOGFILE_VERSION	1

/* local function prototypes */
BOOL _dg87_statfile_read_node (dg87_statfile * stat_file,
                               DG87_FILEPOS position,
                               dg87_statnode * stat_node);
BOOL _dg87_statfile_write_node (dg87_statfile * stat_file,
                                DG87_FILEPOS position,
                                dg87_statnode * stat_node);

BOOL _dg87_statfile_open_file (dg87_statfile * stat_file);
BOOL _dg87_statfile_close_file (dg87_statfile * stat_file);
DG87_FILEPOS _dg87_statfile_end_position (dg87_statfile * stat_file);

/****************************************************************************/
dg87_statfile *
dg87_statfile_create (char *filename, uid_t uid, uid_t gid)
{
  struct stat fstat;
  uint16_t value;
  int l;
  FILE* file_stream;
  dg87_statfile *stat_file;

  assert (filename != NULL);
  if (filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* verify that file doesn't exist */
  if (stat (filename, &fstat) == 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"File '%s' already exists !", filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_ALREADY_EXISTS);
      return NULL;
    }

  /* open file */
  if ((file_stream = fopen (filename, "a+")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't open file %s", filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_DONT_EXIT);
      return NULL;
    }
  /* if daemon is running in root account, change file owner */
  if (uid != -1 && gid != -1)
    {
      if (chown (filename , uid, gid) == -1)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	"Can't change owner of file '%s'", filename);
        }
    }
  /* write signature */
  l = strlen (DG87S_LOGFILE_SIGNATURE);
  if (fwrite(DG87S_LOGFILE_SIGNATURE, l * sizeof(char), 1, file_stream) == 0)
    {
      fclose (file_stream);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return NULL;
    }
  /* write version */
  value = htons (DG87S_LOGFILE_VERSION);
  if (fwrite(&value, sizeof(uint16_t), 1,file_stream) == 0)
    {
      fclose (file_stream);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return NULL;
    }

  /* allocate memory for object structure */
  stat_file = dg87_malloc (sizeof (dg87_statfile), DG87_LOGFILE);
  if (stat_file == NULL)
    {
      fclose (file_stream);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
	fclose (file_stream);
  stat_file->file_stream = NULL;
  stat_file->root_node = DG87S_ROOT_NODE_POSITION;
  if ((stat_file->filename = dg87_strdup (filename, DG87_CHAR)) == NULL)
    {
      dg87_free (stat_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  return stat_file;
}

/****************************************************************************/
dg87_statfile *
dg87_statfile_open (char *filename)
{
  char *string;
  int version;
  uint16_t tmpvalue;
  int l;
  struct stat fstat;
  dg87_statfile *stat_file;
  assert (filename != NULL);
  if (filename == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* verify that file exists */
  if (stat (filename, &fstat) != 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"File '%s' doesn't exist !", filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_DONT_EXIT);
      return NULL;
    }
  /* allocate memory for object structure */
  if ((stat_file = dg87_malloc (sizeof (dg87_statfile),
                                DG87_LOGFILE)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  if ((stat_file->filename = dg87_strdup (filename, DG87_CHAR)) == NULL)
    {
      dg87_free (stat_file, DG87_LOGFILE);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  stat_file->file_stream = NULL;

  /* open file */
  if (_dg87_statfile_open_file (stat_file) == FALSE)
    {
      dg87_statfile_close (stat_file, FALSE);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return NULL;
    }
  /* verify signature */
  l = strlen (DG87S_LOGFILE_SIGNATURE);
  string = dg87_malloc ((l + 1) * sizeof (char), DG87_CHAR);
  if (fread(string, l * sizeof(char), 1, stat_file->file_stream) == 0)
    {
      dg87_free (string, DG87_CHAR);
      dg87_statfile_close (stat_file, FALSE);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return NULL;
    }
  string[l + 1] = '\0';
  if (strncmp (string, DG87S_LOGFILE_SIGNATURE, l) != 0)
    {
      dg87_free (string, DG87_CHAR);
      dg87_statfile_close (stat_file, FALSE);
      dg87_err_set_error (DG87_ERR_LOGFILE_WRONG_SIGN);
      return NULL;
    }
  dg87_free (string, DG87_CHAR);

  /* verify version */
  if (fread(&tmpvalue, sizeof(uint16_t), 1, stat_file->file_stream) == 0)
    {
      dg87_statfile_close (stat_file, FALSE);
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return NULL;
    }
  version = ntohs (tmpvalue);
  if (version != DG87S_LOGFILE_VERSION)
    {
      dg87_statfile_close (stat_file, FALSE);
      dg87_err_set_error (DG87_ERR_LOGFILE_WRONT_VERSION);
      return NULL;
    }

  stat_file->root_node = DG87S_ROOT_NODE_POSITION;
  _dg87_statfile_close_file (stat_file);
  return stat_file;
}

/****************************************************************************/
BOOL
dg87_statfile_close (dg87_statfile * stat_file, BOOL delete_file)
{
  assert (stat_file != NULL);
  if (stat_file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  _dg87_statfile_close_file (stat_file);
  if (delete_file == TRUE)
    {
      if (unlink (stat_file->filename) == -1)
        {
					dg87_daemon_log_printf(dg87_daemon_log_error,
          	"Can't delete file '%s'", stat_file->filename);
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_DELETE_FILE);
          return FALSE;
        }
    }
  dg87_free (stat_file->filename, DG87_CHAR);
  dg87_free (stat_file, DG87_LOGFILE);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_statfile_add_node (dg87_statfile * stat_file, char *name,
                        dg87_statnode * parent_node,
                        dg87_statnode * stat_node)
{
  dg87_statnode last_node;
  int i;
  assert (stat_file != NULL && stat_node != NULL);
  if (stat_file == NULL || stat_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_statfile_open_file (stat_file) == FALSE)
    return FALSE;

  /* initialize new node */
  /* TODO control name length < 255) */
  strcpy (stat_node->name, name);
  stat_node->next_node = 0;
  stat_node->children_number = 0;
  stat_node->first_child_node = 0;
  stat_node->last_child_node = 0;
  for (i = 0; i < DG87_STATFILE_STATENTRYCOUNT; i++)
    {
      stat_node->stat_entries[i].start_time = 0;
      stat_node->stat_entries[i].duration = 0;
    }

  /* TODO control name length */
  if (parent_node == NULL)
    {
      /* create root node */
      stat_node->position = DG87S_ROOT_NODE_POSITION;
      if (_dg87_statfile_write_node (stat_file,
                                     stat_node->position, stat_node) == FALSE)
        {
          _dg87_statfile_close_file (stat_file);
          return FALSE;
        }
    }
  else
    {
      /* read last node (if exists) */
      if (parent_node->last_child_node != 0)
        {
          if (_dg87_statfile_read_node (stat_file,
                                        parent_node->last_child_node,
                                        &last_node) == FALSE)
            {
              _dg87_statfile_close_file (stat_file);
              return FALSE;
            }
        }

      /* create new node */
			stat_node->position = _dg87_statfile_end_position (stat_file);
			if (stat_node->position == -1)
        {
          _dg87_statfile_close_file (stat_file);
          return FALSE;
        }

      /* write new node at end of file */
      if (_dg87_statfile_write_node (stat_file,
                                     stat_node->position, stat_node) == FALSE)
        {
          /*TODO pas cool */
          _dg87_statfile_close_file (stat_file);
          return FALSE;
        }

      /* udpate previous last node (if exists) */
      if (parent_node->last_child_node != 0)
        {
          last_node.next_node = stat_node->position;
          if (_dg87_statfile_write_node (stat_file,
                                         parent_node->last_child_node,
                                         &last_node) == FALSE)
            {
              /*TODO pas cool */
              _dg87_statfile_close_file (stat_file);
              return FALSE;
            }
        }

      /* update parent node */
      if (parent_node->first_child_node == 0)
        parent_node->first_child_node = stat_node->position;
      parent_node->last_child_node = stat_node->position;
      parent_node->children_number++;
      if (_dg87_statfile_write_node (stat_file,
                                     parent_node->position,
                                     parent_node) == FALSE)
        {
          /*TODO pas cool */
          _dg87_statfile_close_file (stat_file);
          return FALSE;
        }
    }
  _dg87_statfile_close_file (stat_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_statfile_get_first_child (dg87_statfile * stat_file,
                               dg87_statnode * parent_node,
                               dg87_statnode * stat_node)
{
  assert (stat_file != NULL && stat_node != NULL);
  if (stat_file == NULL || stat_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_statfile_open_file (stat_file) == FALSE)
    return FALSE;
  if (parent_node == NULL)
    {
      /* read root node */
      if ((_dg87_statfile_read_node (stat_file,
                                     DG87S_ROOT_NODE_POSITION,
                                     stat_node)) == FALSE)
        {
          _dg87_statfile_close_file (stat_file);
          return FALSE;
        }
    }
  else
    {
      /* read first child node */
      if ((_dg87_statfile_read_node (stat_file,
                                     parent_node->first_child_node,
                                     stat_node)) == FALSE)
        {
          _dg87_statfile_close_file (stat_file);
          return FALSE;
        }
    }
  _dg87_statfile_close_file (stat_file);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_statfile_get_next_node (dg87_statfile * stat_file, dg87_statnode * node)
{
  assert (stat_file != NULL);
  if (stat_file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open file */
  if (_dg87_statfile_open_file (stat_file) == FALSE)
    return FALSE;
  if (_dg87_statfile_read_node (stat_file, node->next_node, node) == FALSE)
    {
      _dg87_statfile_close_file (stat_file);
      return FALSE;
    }
  _dg87_statfile_close_file (stat_file);
  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_statfile_write_node (dg87_statfile * stat_file,
                           DG87_FILEPOS position, dg87_statnode * stat_node)
{
  int i;
  uint32_t value32;
  uint16_t value16;
  uint8_t value8;
  assert (stat_file != NULL && stat_node != NULL);
  /* set file position */
  if (fseek (stat_file->file_stream, position, SEEK_SET) == (off_t) - 1)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of next node */
  value32 = htonl (stat_node->next_node);
  if (fwrite(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write number of children nodes */
  value16 = htons (stat_node->children_number);
  if (fwrite(&value16, sizeof(uint16_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of first child */
  value32 = htonl (stat_node->first_child_node);
  if (fwrite(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write file position of last child */
  value32 = htonl (stat_node->last_child_node);
  if (fwrite(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write name length */
  value8 = strlen (stat_node->name);
  if (fwrite(&value8, sizeof(uint8_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write name */
	if (fwrite(stat_node->name, value8 * sizeof(char), 1, 
													stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
      return FALSE;
    }
  /* write each stat entry */
  for (i = 0; i < DG87_STATFILE_STATENTRYCOUNT; i++)
    {
      /* write start date */
      value32 = htonl (stat_node->stat_entries[i].start_time);
  		if (fwrite(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
        {
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
          return FALSE;
        }
      /* write duration */
      value16 = htons (stat_node->stat_entries[i].duration);
  		if (fwrite(&value16, sizeof(uint16_t), 1,stat_file->file_stream) == 0)
        {
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_WRITE);
          return FALSE;
        }
    }
  return TRUE;
}

/***************************************************************************/
BOOL
_dg87_statfile_read_node (dg87_statfile * stat_file,
                          DG87_FILEPOS position, dg87_statnode * stat_node)
{
  uint32_t value32;
  uint16_t value16;
  uint8_t value8;
  int i;
  assert (stat_file != NULL && stat_node != NULL);
  if (position == 0)
    return FALSE;
  /* set file position */
  if (fseek (stat_file->file_stream, position, SEEK_SET) == (off_t) - 1)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  stat_node->position = position;
  /* read file position of next node */
  if (fread(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  stat_node->next_node = ntohl (value32);
  /* read number of children nodes */
  if (fread(&value16, sizeof(uint16_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  stat_node->children_number = ntohs (value16);
  /* read file position of first child */
  if (fread(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  stat_node->first_child_node = ntohl (value32);
  /* read file position of last child */
  if (fread(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  stat_node->last_child_node = ntohl (value32);
  /* read name length */
  if (fread(&value8, sizeof(uint8_t), 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  /* read name */
  if (fread(stat_node->name, value8, 1,stat_file->file_stream) == 0)
    {
      dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
      return FALSE;
    }
  stat_node->name[value8] = '\0';
  /* read each stat entry */
  for (i = 0; i < DG87_STATFILE_STATENTRYCOUNT; i++)
    {
      /* read start date */
  		if (fread(&value32, sizeof(uint32_t), 1,stat_file->file_stream) == 0)
        {
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
          return FALSE;
        }
      stat_node->stat_entries[i].start_time = ntohl (value32);
      /* read duration */
  		if (fread(&value16, sizeof(uint16_t), 1,stat_file->file_stream) == 0)
        {
          dg87_err_set_error (DG87_ERR_LOG_CAN_NOT_READ);
          return FALSE;
        }
      stat_node->stat_entries[i].duration = ntohs (value16);
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_statfile_update_node (dg87_statfile * stat_file, dg87_statnode * node,
                           time_t start_time, dg87uint duration)
{
  int i;
  assert (stat_file != NULL && node != NULL);
  /* statistical entries rotation */
  for (i = 1; i < DG87_STATFILE_STATENTRYCOUNT; i++)
    {
      node->stat_entries[i - 1].start_time = node->stat_entries[i].start_time;
      node->stat_entries[i - 1].duration = node->stat_entries[i].duration;
    }
  /* store new entry */
  i = DG87_STATFILE_STATENTRYCOUNT - 1;
  node->stat_entries[i].start_time = start_time;
  node->stat_entries[i].duration = duration;

  /* open file */
  if (_dg87_statfile_open_file (stat_file) == FALSE)
    return FALSE;
  /* rewrite entry */
  if (_dg87_statfile_write_node (stat_file, node->position, node) == FALSE)
    return FALSE;
  /* close file */
  if (_dg87_statfile_close_file (stat_file) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_statfile_stat_node (dg87_statfile * stat_file, dg87_statnode * node,
                         dg87uint * average, dg87uint * min, dg87uint * max)
{
  int i, number = 0;
  dg87uint duration, mini = 0, maxi = 0, total = 0;
  assert (stat_file != NULL && node != NULL && average != NULL &&
          min != NULL && max != NULL);

  for (i = 1; i < DG87_STATFILE_STATENTRYCOUNT; i++)
    {
      if ((duration = node->stat_entries[i].duration) > 0)
        {
          number++;
          total += duration;
          if (mini == 0 || duration < mini)
            mini = duration;
          if (maxi == 0 || duration > maxi)
            maxi = duration;
        }
    }
  *min = mini;
  *max = maxi;
  *average = (number > 0) ? total / number : 0;
  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_statfile_open_file (dg87_statfile * stat_file)
{
  if (stat_file->file_stream != NULL)
    return TRUE;
  if ((stat_file->file_stream = fopen (stat_file->filename, "r+")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't open file '%s' !",
				stat_file->filename);
      dg87_err_set_error (DG87_ERR_LOGFILE_CAN_NOT_OPEN);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
_dg87_statfile_close_file (dg87_statfile * stat_file)
{
  if (stat_file->file_stream == NULL)
    return TRUE;
  fclose (stat_file->file_stream);
  stat_file->file_stream = NULL;
  return TRUE;
}
/****************************************************************************/
DG87_FILEPOS _dg87_statfile_end_position (dg87_statfile * stat_file)
{
	assert(stat_file != NULL);
	/* go to end of file */
	if (fseek (stat_file->file_stream, 0, SEEK_END) == -1)
					return -1;
	/* return the position of the end of file */
	return ftell (stat_file->file_stream);
}
/****************************************************************************/
/* End of file */
