/* file target.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <config.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <libdg87/list.h>
#include <libdg87/vector.h>
#include "context.h"
#include "target.h"
#include "queue.h"


/****************************************************************************/
dg87_target *
dg87_target_new (char *name, BOOL system, dg87_context* context)
{
  dg87_target *target;
	dg87_queue *system_queue;
  assert (name != NULL && context != NULL);
  if (name == NULL || context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_TARGET_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((target = dg87_malloc (sizeof (dg87_target), DG87_TARGET)) == NULL)
    goto alloc_error;
  if ((target->queue_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
	/* create the system queue `batch' */
	system_queue = dg87_target_queue_add (target, "batch", TRUE);
	if (system_queue == NULL)
		goto alloc_error;

  /* initialize structure */
  dg87_strnzcpy (target->name, name, DG87_TARGET_NAME_LENGTH);
  target->description[0] = 0;
  target->ip_port = atoi (DIO87D_IPPORT);
  target->hostname = NULL;
  target->open = TRUE;
  target->system = system;
	target->context = context;
	DG87_SAVE_NEEDED;
  return target;

alloc_error:
  if (target != NULL)
    dg87_target_delete (target, TRUE);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_target_delete (dg87_target * target, BOOL force)
{
  dg87_queue *queue;
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (target->system == TRUE && force == FALSE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_TARGET);
      return FALSE;
    }
  if (target->queue_vector != NULL)
    {
      while (target->queue_vector->count > 0)
        {
          queue =
            dg87_vector_remove_object_by_index (target->queue_vector, 0);
          dg87_queue_delete (queue, TRUE);
        }
      dg87_vector_delete (target->queue_vector);
    }
  if (target->hostname != NULL)
    dg87_free (target->hostname, DG87_CHAR);

  dg87_free (target, DG87_TARGET);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_load (dg87_target * target, dg87_storage * storage)
{
  char *string;
  dg87ubyte b;
  dg87uint value;
  assert (target != NULL && storage != NULL);
  if (target == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read target description */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
	if (dg87_target_set_description (target, string) == FALSE)
		{
			dg87_free (string, DG87_CHAR);
			return FALSE;
		}
  dg87_free (string, DG87_CHAR);
  /* read long host name (can be NULL) */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
	if (target->system == FALSE)
		{
  		if (dg87_target_set_hostname (target, string) == FALSE)
    		{
      		dg87_free (string, DG87_CHAR);
      		return FALSE;
    		}
		}
  dg87_free (string, DG87_CHAR);
  /* read IP port */
  if (dg87_storage_read_uint (storage, &value) == FALSE)
    return FALSE;
	if (target->system == FALSE)
		{
  	if (dg87_target_set_ip_port (target, value, FALSE) == FALSE)
			return FALSE;
		}
  /* read opened property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_target_set_open (target, b) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_save (dg87_target * target, dg87_storage * storage)
{
  dg87int value;
  assert (target != NULL && storage != NULL);
  if (target == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write target description */
  if (dg87_storage_write_string (storage, target->description) == FALSE)
    return FALSE;
  /* write long host name (can be NULL) */
  if (dg87_storage_write_string (storage,
                                 (target->hostname == NULL) ? "" :
                                 target->hostname) == FALSE)
    return FALSE;
  /* write IP port */
  value = target->ip_port;
  if (dg87_storage_write_uint (storage, value) == FALSE)
    return FALSE;
  /* write opened property */
  if (dg87_storage_write_ubyte (storage, target->open) == FALSE)
    return FALSE;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_set_hostname (dg87_target * target, char *hostname)
{
  /* hostname can be NULL */
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (target->system == TRUE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_TARGET);
      return FALSE;
    }
  /* free previous string */
  if (target->hostname != NULL)
    {
      dg87_free (target->hostname, DG87_CHAR);
      target->hostname = NULL;
    }
  /* copy new host name */
  if (hostname != NULL && strlen (hostname) > 0)
    {
      target->hostname = dg87_strdup (hostname, DG87_CHAR);
      if (target->hostname == NULL)
        return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_set_open (dg87_target * target, BOOL value)
{
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (value != FALSE && value != TRUE)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  target->open = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_set_name (dg87_target * target, char *name)
{
  assert (target != NULL && name != NULL);
  if (target == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (target->system == TRUE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_TARGET);
      return FALSE;
    }
  if (dg87_check_name (name, DG87_TARGET_NAME_LENGTH) == FALSE)
    return FALSE;
	if (dg87_context_target_rename (target->context, target, name) == FALSE)
		return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}
/****************************************************************************/
BOOL
dg87_target_set_description (dg87_target * target, char *description)
{
  assert (target != NULL && description != NULL);
  if (target == NULL || description == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_description (description, DG87_TARGET_DESC_LENGTH) == FALSE)
    return FALSE;
  dg87_strnzcpy (target->description, description, DG87_TARGET_DESC_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_set_ip_port (dg87_target * target, dg87int value, BOOL force)
{
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (force == FALSE && target->system == TRUE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_TARGET);
      return FALSE;
    }
  if (value < 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  target->ip_port = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_queue *
dg87_target_queue_add (dg87_target * target, char *queue_name, BOOL system)
{
  dg87_queue *queue;
  assert (target != NULL && queue_name != NULL);
  if (target == NULL || queue_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  queue = dg87_queue_new (target, queue_name, system);
  if (queue == NULL)
    return NULL;

  /* vector generate error if an other queue has the same name */
  /* key must point on queue->name */
  if (dg87_vector_add_object (target->queue_vector, queue, queue->name)
      == FALSE)
    {
      dg87_queue_delete (queue, TRUE);
      return NULL;
    }
  return queue;
}
/****************************************************************************/
BOOL
dg87_target_queue_rename (dg87_target* target, dg87_queue* queue,
                               char *new_name)
{
  char old_name[DG87_QUEUE_NAME_LENGTH + 1];
  assert (target != NULL && queue != NULL && new_name != NULL);
  if (target == NULL || queue == NULL || new_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  if (strlen (new_name) > DG87_QUEUE_NAME_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  /* remove object from vector */
  if (dg87_vector_remove_object (target->queue_vector, queue) == FALSE)
    return FALSE;
  /* copy new name */
  dg87_strnzcpy (old_name, queue->name, DG87_QUEUE_NAME_LENGTH);
  dg87_strnzcpy (queue->name, new_name, DG87_QUEUE_NAME_LENGTH);
  /* add the object in the vector with the new key */
  if (dg87_vector_add_object (target->queue_vector, queue, queue->name) == FALSE)
    {
      dg87_strnzcpy (queue->name, old_name, DG87_QUEUE_NAME_LENGTH);
      dg87_vector_add_object (target->queue_vector, queue, queue->name);
      return FALSE;
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}


/****************************************************************************/
int
dg87_target_queue_count (dg87_target * target)
{
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (target->queue_vector);
}

/****************************************************************************/
dg87_queue *
dg87_target_queue_get_by_name (dg87_target * target, char *name)
{
  assert (target != NULL && name != NULL);
  if (target == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_key (target->queue_vector, name);
}

/****************************************************************************/
dg87_queue *
dg87_target_queue_get_by_index (dg87_target * target, int index)
{
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (target->queue_vector, index);
}

/****************************************************************************/
BOOL
dg87_target_queue_delete (dg87_target * target, dg87_queue * queue)
{
  assert (target != NULL && queue != NULL);
  if (target == NULL || queue == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_vector_remove_object (target->queue_vector, queue) == FALSE)
    return FALSE;
  if (dg87_queue_delete (queue, FALSE) == FALSE)
    {
      /* add queue in vector again */
      dg87_vector_add_object (target->queue_vector, queue, queue->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_target_queue_delete_by_name (dg87_target * target, char *name)
{
  dg87_queue *queue;
  assert (target != NULL && name != NULL);
  if (target == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  queue = dg87_vector_get_object_by_key (target->queue_vector, name);
  if (queue == NULL)
    return FALSE;
  return dg87_target_queue_delete (target, queue);
}

/****************************************************************************/
BOOL
dg87_target_queue_delete_by_index (dg87_target * target, int index)
{
  dg87_queue *queue;
  assert (target != NULL);
  if (target == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  queue = dg87_vector_get_object_by_index (target->queue_vector, index);
  if (queue == NULL)
    return FALSE;
  return dg87_target_queue_delete (target, queue);
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_target_check (dg87_target * target, int (*print) (char *))
{
  dg87_element *element;
  dg87_queue *queue;

  if (strlen (target->name) == 0)
    {
      print ("Target : Name is zero length !\n");
      return 1;
    }
  /* check queue_vector */
  if (target->queue_vector == NULL)
    {
      print ("Target : QueueVector is NULL !\n");
      return 1;
    }
  if (dg87_vector_check (target->queue_vector, print) != 0)
    return 1;
  element = target->queue_vector->first;
  while (element != NULL)
    {
      queue = element->object;
      if (dg87_queue_check (queue, print) != 0)
        return 1;
      if (queue->target != target)
        {
          print ("Target : Queue Target pointer incorrect !\n");
          return 1;
        }
      element = element->next;
    }
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
