/* file target.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_TARGET_H__
#define __DG87_TARGET_H__

#include "global.h"
#include <libdg87/vector.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include "d_task.h"

struct _dg87_target
{
  char name[DG87_TARGET_NAME_LENGTH + 1];
  char description[DG87_TARGET_DESC_LENGTH + 1];
  char *hostname;               /* can be null */
  dg87uint ip_port;
  BOOL open;
  BOOL system;                  /* can't be deleted if true */
  dg87_vector *queue_vector;
	dg87_context* context; 
};

/*
 * dg87_target_new 
 * create a new target and initialize the name property.
 * if the system argument is TRUE, the target can't be deleted.
 */
dg87_target *dg87_target_new (char *name, BOOL system, dg87_context* context);

/*
 * dg87_target_delete
 * if the `force' argument is TRUE, a system target can be deleted.
 */
BOOL dg87_target_delete (dg87_target * target, BOOL force);

/*
 * dg87_target_load
 */
BOOL dg87_target_load (dg87_target * target, dg87_storage * storage);

/*
 * dg87_target_save
 */
BOOL dg87_target_save (dg87_target * target, dg87_storage * storage);

/*
 * dg87_target_set_hostname
 * set the hostname of the target. The hostname can be null : the
 * target name will be used to contact remote computer.
 */
BOOL 
dg87_target_set_hostname (dg87_target * target, char *hostname);

/*
 * dg87_target_set_name
 * change the name of the target.
 */
BOOL dg87_target_set_name (dg87_target * target, char *new_name);

/*
 * dg87_target_set_description
 * set the description of the target.
 */
BOOL dg87_target_set_description (dg87_target * target, char *description);

/* dg87_target_set_ip_port 
 * Change the IP Port on which the target is listen.
 * By default the IP port of the localhost target can't be changed, unless
 * the `force' argument is true.
 * Return FALSE if it can't be changed.
 */
BOOL dg87_target_set_ip_port (dg87_target * target, dg87int value, BOOL force);

/*
 * dg87_target_set_open
 * set the value for the "open" property.
 * return FALSE if the open's value can't be change.
 */
BOOL dg87_target_set_open (dg87_target * target, BOOL value);


/*************************************************************************/
/*  functions for queue                                                  */
/*************************************************************************/

/*
 * dg87_target_queue_add
 * create a Queue object, add this object in the queue_vector
 * of the target and return a pointer on this object.
 * if the `system' argument property is true, the queue can't be deleted.
 * Return NULL if the Queue object can't be create.
 */
dg87_queue *dg87_target_queue_add (dg87_target * target, char *queue_name,
                                   BOOL system);

/*
 * dg87_target_queue_rename 
 * You don't have to use this function. Use dg87_queue_set_name instead.
 * This function is called by dg87_queue_set_name to change the name of a 
 * queue.
 * It returns FALSE if another queue has the same name.
 */
BOOL
dg87_target_queue_rename (dg87_target* target, dg87_queue* queue,
                               char *new_name);
/*
 * dg87_target_queue_count
 * return the number of queue in the target.
 * return -1 if target is NULL.
 */
int dg87_target_queue_count (dg87_target * target);


/*
 * dg87_target_queue_get_by_name
 * return a pointer on the queue which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_queue *dg87_target_queue_get_by_name (dg87_target * target, char *name);

/*
 * dg87_target_queue_get_by_index
 * return a pointer on the queue at the given index.
 */
dg87_queue *dg87_target_queue_get_by_index (dg87_target * target, int index);


/*
 * dg87_target_queue_delete_by_name
 * delete the queue which has the name given in parameter.
 */
BOOL dg87_target_queue_delete_by_name (dg87_target * target, char *name);

/*
 * dg87_target_queue_delete_by_index
 * delete the queue at the given index.
 */
BOOL dg87_target_queue_delete_by_index (dg87_target * target, int index);

/*
 * dg87_target_queue_delete
 */
BOOL dg87_target_queue_delete (dg87_target * target, dg87_queue * queue);

#ifdef DEBUG
/*
 * dg87_target_ckeck
 */
int dg87_target_check (dg87_target * target, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_TARGET_H__*/
/* End of file */
