/* file util.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <string.h>
#include <assert.h>

#include "util.h"
#include "context.h"
#include "engine.h"
#include "variable.h"
#include "x_day.h"
#include "r_job.h"
#include "r_file.h"

/******************************************************************************/
dg87_ref_model *
dg87_util_find_ref_model (dg87_connection * connection, char *name)
{
  dg87_ref_model *ref;
  dg87_context *context;
  assert (connection != NULL && name != NULL);
  if (connection == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  context = connection->server->data;
  dg87_err_clear_error ();
  ref = dg87_context_ref_model_get_by_name (context, name);
  if (ref == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't find the referential '%s' ! %s\n"),
											name, dg87_err_get_error_str ());
      return NULL;
    }
  return ref;
}

/******************************************************************************/
dg87_target *
dg87_util_find_target (dg87_connection * connection, char *name)
{
  dg87_target *target;
  dg87_context *context;
  assert (connection != NULL && name != NULL);
  if (connection == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  context = connection->server->data;
  dg87_err_clear_error ();
  target = dg87_context_target_get_by_name (context, name);
  if (target == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the target '%s' ! %s\n"),
                              name, dg87_err_get_error_str ());
      return NULL;
    }
  return target;
}

/******************************************************************************/
dg87_queue *
dg87_util_find_queue (dg87_connection * connection,
                      dg87_target * target, char *queue_name)
{
  dg87_queue *queue;
  assert (connection != NULL && target != NULL && queue_name != NULL);
  if (connection == NULL || target == NULL || queue_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  queue = dg87_target_queue_get_by_name (target, queue_name);
  if (queue == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the queue '%s' in the "\
																			"target '%s' ! %s\n"),
                              queue_name, target->name,
                              dg87_err_get_error_str ());
      return NULL;
    }
  return queue;
}

/******************************************************************************/
dg87_ref_task *
dg87_util_find_ref_model_task (dg87_connection * connection,
                               dg87_ref_model * ref, char *name)
{
  dg87_ref_task *task;
  assert (connection != NULL && ref != NULL && name != NULL);
  if (connection == NULL || ref == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  task = dg87_ref_model_task_get_by_name (ref, name);
  if (task == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the task '%s' in the "\
																			"referential '%s' ! %s\n"),
                              name, ref->name, dg87_err_get_error_str ());
      return NULL;
    }
  return task;
}

/******************************************************************************/
dg87_ref_job *
dg87_util_find_ref_job (dg87_connection * connection,
                        dg87_ref_task * task, char *job_name)
{
  dg87_ref_job *job;
  assert (connection != NULL && task != NULL && job_name != NULL);
  assert (connection != NULL && task != NULL && job_name != NULL);
  if (connection == NULL || task == NULL || job_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  job = dg87_ref_task_job_get_by_name (task, job_name);
  if (job == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the job '%s' in the task "\
																			"'%s' of the referental '%s' ! %s\n"),
                              job_name, task->name, task->ref->name,
                              dg87_err_get_error_str ());
      return NULL;
    }
  return job;
}

/******************************************************************************/
dg87_ref_rule *
dg87_util_find_ref_rule (dg87_connection * connection,
                         dg87_ref_model * ref, char *name)
{
  dg87_ref_rule *rule;
  assert (connection != NULL && ref != NULL && name != NULL);
  if (connection == NULL || ref == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  rule = dg87_ref_model_rule_get_by_name (ref, name);
  if (rule == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the rule '%s' in the "\
																			"referential '%s' ! %s\n"),
                              name, ref->name, dg87_err_get_error_str ());
      return NULL;
    }
  return rule;
}

/******************************************************************************/
dg87_variable *
dg87_util_find_variable (dg87_connection * connection, char *name)
{
  dg87_variable *variable;
  dg87_context *context;
  assert (connection != NULL && name != NULL);
  if (connection == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  context = connection->server->data;
  dg87_err_clear_error ();
  variable = dg87_engine_variable_get_by_name (context->engine, name);
  if (variable == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the variable '%s' ! %s\n"),
                              name, dg87_err_get_error_str ());
      return NULL;
    }
  return variable;

}

/******************************************************************************/
dg87_exp_day *
dg87_util_find_exp_day (dg87_connection * connection, char *name)
{
  dg87_exp_day *day;
  dg87_context *context;
  assert (connection != NULL && name != NULL);
  if (connection == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  context = connection->server->data;
  dg87_err_clear_error ();
  day = dg87_context_exp_day_get_by_name (context, name);
  if (day == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the day '%s' ! %s\n"),
                              name, dg87_err_get_error_str ());
      return NULL;
    }
  return day;
}

/******************************************************************************/
dg87_exp_task *
dg87_util_find_exp_task (dg87_connection * connection,
                         dg87_exp_day * day, char *name)
{
  dg87_exp_task *task;
  assert (connection != NULL && day != NULL && name != NULL);
  if (connection == NULL || day == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  task = dg87_exp_day_task_get_by_name (day, name);
  if (task == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the task '%s' in the "\
																			"day '%s' ! %s\n"),
                              name, day->name, dg87_err_get_error_str ());
      return NULL;
    }
  return task;

}

/******************************************************************************/
dg87_exp_job *
dg87_util_find_exp_job (dg87_connection * connection,
                        dg87_exp_task * task, char *name)
{
  dg87_exp_job *job;
  assert (connection != NULL && task != NULL && name != NULL);
  if (connection == NULL || task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  job = dg87_exp_task_job_get_by_name (task, name);
  if (job == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the job '%s' in the task "
																			"'%s' of the day '%s' !\n"),
                              name, task->name, task->day->name,
															dg87_err_get_error_str ());
      return NULL;
    }
  return job;
}


/******************************************************************************/
dg87_log_day *
dg87_util_find_log_day (dg87_connection * connection, char *name)
{
  dg87_log_day *day;
  dg87_context *context;
  assert (connection != NULL && name != NULL);
  if (connection == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  context = connection->server->data;
  dg87_err_clear_error ();
  if ((day = dg87_context_log_day_get_by_name (context, name)) == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the log day '%s' ! %s\n"),
                              name, dg87_err_get_error_str ());
      return NULL;
    }
  return day;
}

/******************************************************************************/
BOOL
dg87_util_find_log_task_node (dg87_connection * connection,
                              dg87_log_day * day, char *name,
                              dg87_log_node * task_node)
{
  dg87_context *context;
  assert (connection != NULL && name != NULL && task_node != NULL);
  if (connection == NULL || name == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = connection->server->data;
  dg87_err_clear_error ();

  if (dg87_log_day_task_get_by_name (day, name, task_node) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the task '%s' in "\
																			"log day '%s' ! %s\n"),
                              name, day->name, dg87_err_get_error_str ());
      return FALSE;
    }
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_util_find_log_job_node (dg87_connection * connection,
                             dg87_log_day * day, dg87_log_node * task_node,
                             char *name, dg87_log_node * job_node)
{
  dg87_context *context;
  assert (connection != NULL && task_node != NULL
          && name != NULL && job_node != NULL);
  if (connection == NULL || task_node == NULL
      || name == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = connection->server->data;
  dg87_err_clear_error ();

  if (dg87_log_day_job_get_by_name (day, task_node, name, job_node) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the job '%s' in the task "\
																			"'%s' of the log day '%s' ! %s\n"),
                              name, task_node->name, day->name,
                              dg87_err_get_error_str ());
      return FALSE;
    }
  return TRUE;
}

/******************************************************************************/
dg87_r_job *
dg87_r_util_find_job (dg87_connection * connection,
                      char *day_name, char *task_name, char *job_name)
{
  dg87_r_job *job = NULL;
  dg87_item *item;
  dg87_context *context;
  assert (connection != NULL && task_name != NULL &&
          job_name != NULL && day_name != NULL);
  if (connection == NULL || task_name == NULL
      || job_name == NULL || day_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  context = connection->server->data;
  item = context->job_list->first;
  while (item != NULL)
    {
      job = item->object;
      if (strcmp (job->task_name, task_name) == 0 &&
          strcmp (job->job_name, job_name) == 0 &&
          strcmp (job->day_name, day_name) == 0)
        return job;
      item = item->next;
    }

  /* we don't find the job */
  dg87_connection_printf (connection,
                          gettext ("Can't find the job '%s' of the task '%s' "\
																	"of the day '%s' !\n"),
                          day_name, task_name, job_name);
  return NULL;
}

/******************************************************************************/
dg87_r_file *
dg87_r_util_find_file (dg87_connection * connection,
                       char *day_name, char *task_name, char *path)
{
  dg87_r_file *file = NULL;
  dg87_item *item;
  dg87_context *context;
  assert (connection != NULL && task_name != NULL &&
          day_name != NULL && path != NULL);
  if (connection == NULL || day_name == NULL
      || task_name == NULL || path == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  dg87_err_clear_error ();
  context = connection->server->data;
  item = context->file_list->first;
  while (item != NULL)
    {
      file = item->object;
      if (strcmp (file->day_name, day_name) == 0 &&
          strcmp (file->task_name, task_name) == 0 &&
          strcmp (file->path, path) == 0)
        return file;
      item = item->next;
    }

  /* we don't find the file */
  dg87_connection_printf (connection,
                          gettext ("Can't find the file '%s' of the task '%s' "\
																	"of the day '%s' !\n"),
                          day_name, task_name, path);
  return NULL;
}

/******************************************************************************/
dg87_sta_ref *
dg87_util_find_sta_referential (dg87_connection * connection, char *name)
{
  dg87_sta_ref *ref;
  dg87_context *context;
  assert (connection != NULL && name != NULL);
  if (connection == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  context = connection->server->data;
  dg87_err_clear_error ();
  if ((ref =
       dg87_context_sta_referential_get_by_name (context, name)) == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the stat referential "\
																			"'%s' ! %s\n"),
                              name, dg87_err_get_error_str ());
      return NULL;
    }
  return ref;
}

/******************************************************************************/
BOOL
dg87_util_find_sta_task_node (dg87_connection * connection,
                              dg87_sta_ref * ref, char *name,
                              dg87_statnode * task_node)
{
  dg87_context *context;
  assert (connection != NULL && name != NULL && task_node != NULL);
  if (connection == NULL || name == NULL || task_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = connection->server->data;
  dg87_err_clear_error ();

  if (dg87_sta_ref_task_get_by_name (ref, name, task_node) == FALSE)
    {
      dg87_connection_printf (connection,
															gettext ("Can't find the task '%s' in the "\
																			"stat referential '%s' ! %s\n"),
                              name, ref->name, dg87_err_get_error_str ());
      return FALSE;
    }
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_util_find_sta_job_node (dg87_connection * connection,
                             dg87_sta_ref * ref, dg87_statnode * task_node,
                             char *name, dg87_statnode * job_node)
{
  dg87_context *context;
  assert (connection != NULL && task_node != NULL
          && name != NULL && job_node != NULL);
  if (connection == NULL || task_node == NULL
      || name == NULL || job_node == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = connection->server->data;
  dg87_err_clear_error ();

  if (dg87_sta_ref_job_get_by_name (ref, task_node, name, job_node) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find the job '%s' in the task "\
																			"'%s' of stat referential '%s' ! %s\n"),
                              name, task_node->name, ref->name,
                              dg87_err_get_error_str ());
      return FALSE;
    }
  return TRUE;
}

/******************************************************************************/
/* End of file */
