/* file variable.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "context.h"
#include "variable.h"

/****************************************************************************/
dg87_variable *
dg87_variable_new (char *name, char *value)
{
  dg87_variable *variable;
  assert (name != NULL && value != NULL);
  if (name == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_VARIABLE_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((variable = dg87_malloc (sizeof (dg87_variable),
                               DG87_VARIABLE)) == NULL)
    goto alloc_error;
  if ((variable->value = dg87_strdup (value, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  dg87_strnzcpy (variable->name, name, DG87_VARIABLE_NAME_LENGTH);
  variable->linked_rules = 0;
  variable->linked_ref_conditions = 0;
  variable->linked_exp_conditions = 0;
  variable->linked_ref_tasks = 0;
  variable->linked_exp_tasks = 0;
	DG87_SAVE_NEEDED;
  return variable;
alloc_error:
  if (variable != NULL)
    dg87_variable_delete (variable);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_variable_delete (dg87_variable * variable)
{
  assert (variable != NULL);
  if (variable == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (variable->linked_rules > 0)
    {
      dg87_err_set_error (DG87_ERR_VARIABLE_USED_BY_RULES);
      return FALSE;
    }
  if (variable->linked_exp_conditions > 0 ||
      variable->linked_ref_conditions > 0)
    {
      dg87_err_set_error (DG87_ERR_VARIABLE_USED_BY_CONDITIONS);
      return FALSE;
    }
  if (variable->linked_exp_tasks > 0 || variable->linked_ref_tasks > 0)
    {
      dg87_err_set_error (DG87_ERR_VARIABLE_USED_BY_TASKS);
      return FALSE;
    }
  if (variable->value != NULL)
    dg87_free (variable->value, DG87_CHAR);
  dg87_free (variable, DG87_VARIABLE);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_variable_load (dg87_variable * variable, dg87_storage * storage)
{
  char *string;
  assert (variable != NULL && storage != NULL);
  if (variable == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read variable value */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_variable_set_value (variable, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_variable_save (dg87_variable * variable, dg87_storage * storage)
{
  assert (variable != NULL && storage != NULL);
  if (variable == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write variable value */
  if (dg87_storage_write_string (storage, variable->value) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_variable_set_value (dg87_variable * variable, char *value)
{
  char *str_copy;
  assert (variable != NULL && value != NULL);
  if (variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* copy new value */
  str_copy = dg87_strdup (value, DG87_CHAR);
  if (str_copy == NULL)
    return FALSE;
  /* clear previous value */
  dg87_free (variable->value, DG87_CHAR);
  variable->value = str_copy;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_variable_check (dg87_variable * variable, int (*print) (char *))
{
  if (strlen (variable->name) == 0)
    {
      print ("Variable : name length is zero !\n");
      return 1;
    }

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
