/* file x_caller.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <config.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <assert.h>

#include <libdg87/daemon.h>
#include <libdg87/server.h>
#include <libdg87/strbuf.h>

#include "engine.h"
#include "context.h"
#include "x_caller.h"
#include "x_day.h"
#include "variable.h"
#include "l_day.h"
#include "s_ref.h"

/******************************************************************************/
BOOL
dg87_caller__submit_job (dg87_context * context, dg87_exp_job * job)
{
  dg87_target *target;
  dg87_strbuf *strbuf = NULL;
  char *arg_buffer = NULL;
  char *path_buffer = NULL;
  char *hostname = NULL;

  assert (context != NULL && job != NULL);
  if (context == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  /* replace argument variables by values */
  arg_buffer = dg87_engine_replace_variables (context->engine,
                                              (job->argument ==
                                               NULL) ? "" : job->argument);
  if (arg_buffer == NULL)
    goto error;

  /* replace path variables by values */
  path_buffer = dg87_engine_replace_variables (context->engine,
                                               (job->path ==
                                                NULL) ? "" : job->path);
  if (path_buffer == NULL)
    goto error;

	hostname = dg87_context_get_hostname (context, TRUE);
	if (hostname == NULL)
		goto error;

  if (job->task->queue->target->system == TRUE)
  {
		/* execution on the same daemon : hostname="" and ip_port=0 */
		if (dg87_r_context_job_add(context, 
														job->task->day->name,
														job->task->name, 
														job->name,
														path_buffer, arg_buffer, 
														"", 0 /* hostname="" and ip_port=0 */
														) != NULL)
          dg87_exp_job_set_state (job, dg87_exp_job_state_submit, -1);
	} else {

		/* execution on a remote daemon : send a 'ja' command message */
	  strbuf = dg87_strbuf_new ();
	  if (strbuf == NULL)
	    goto error;
	  /*ja TaskName JobName DayName Path Argument hostName ip_port */
	  dg87_strbuf_printf (strbuf,
			      "ja \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" %i",
			      job->task->day->name,
			      job->task->name, job->name,
			      path_buffer, arg_buffer, hostname, context->ip_port);

	  target = job->task->queue->target;
	  if (dg87_server_post_message (context->server,
					(target->hostname == NULL) ? target->name : target->hostname,
					target->ip_port, strbuf->buffer) == TRUE)
			dg87_exp_job_set_state(job, dg87_exp_job_state_submit, -1);

	  dg87_strbuf_delete (strbuf);
	}

	dg87_free (arg_buffer, DG87_CHAR);
	dg87_free (path_buffer, DG87_CHAR);
	dg87_free (hostname, DG87_CHAR);

  return TRUE;

error:
	/* TODO change the state of the job */
  if (arg_buffer != NULL)
    dg87_free (arg_buffer, DG87_CHAR);
  if (path_buffer != NULL)
    dg87_free (path_buffer, DG87_CHAR);
  if (hostname != NULL)
    dg87_free (hostname, DG87_CHAR);
  if (strbuf != NULL)
    dg87_strbuf_delete (strbuf);
  return FALSE;
}

/******************************************************************************/
BOOL
dg87_caller__send_task_state (dg87_exp_task * task)
{
  dg87_context *context;
  time_t tm;
  char *data;
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_log_day_entry_type entry_type;
  assert (task != NULL);

  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((data = dg87_exp_task_state_to_ascii (task->state)) == NULL)
    return FALSE;
  if ((entry_type = dg87_log_day_ascii_to_entry_type (data)) == -1)
    return FALSE;
  context = task->day->context;
  /* find the log day and the task node */
  if ((day = dg87_context_log_day_get_by_name (context,
                                               task->day->name)) == NULL)
    return FALSE;
  if (dg87_log_day_task_get_by_name (day, task->name, &task_node) == FALSE)
    return FALSE;
	/* add an entry */
  tm = dg87_current_time;
  data = "";
  if (dg87_log_day_task_entry_add (day, &task_node, entry_type,
                                   tm, data) == FALSE)
      return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__send_condition_state (dg87_exp_condition * condition)
{
  dg87_strbuf *strbuf;
  dg87_context *context;
  time_t tm;
  char *condition_string;
  dg87_log_day *day;
  dg87_log_node task_node;
  dg87_log_day_entry_type entry_type;
  assert (condition != NULL);

  if (condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = condition->task->day->context;

  /* find the log day and the task node */
  if ((day = dg87_context_log_day_get_by_name (context,
																			 condition->task->day->name)) ==
      NULL)
    return FALSE;
  if (dg87_log_day_task_get_by_name (day, condition->task->name,
                                     &task_node) == FALSE)
    return FALSE;

  /* create a buffer with the condition and the state */
  if ((condition_string = dg87_exp_condition_to_ascii (condition)) == NULL)
    return FALSE;
  if ((strbuf = dg87_strbuf_new ()) == NULL)
    {
      dg87_free (condition_string, DG87_CHAR);
      return FALSE;
    }
  dg87_strbuf_printf (strbuf, "(%s) %s",
                      condition_string,
                      dg87_exp_condition_state_to_ascii (condition->state));
  dg87_free (condition_string, DG87_CHAR);

  /* add a log entry */
  tm = dg87_current_time;
  entry_type = dg87_log_day_entry_type_condition;
  if (dg87_log_day_task_entry_add (day, &task_node, entry_type,
                                   tm, strbuf->buffer) == FALSE)
    {
  		dg87_strbuf_delete (strbuf);
      return FALSE;
    }
  dg87_strbuf_delete (strbuf);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__send_job_state (dg87_exp_job * job)
{
  dg87_context *context;
  time_t tm;
  char *data;
  dg87_log_day *day;
  dg87_log_node job_node;
  dg87_log_node task_node;
  dg87_log_day_entry_type entry_type;
  assert (job != NULL);

  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((data = dg87_exp_job_state_to_ascii (job->state)) == NULL)
    return FALSE;
  if ((entry_type = dg87_log_day_ascii_to_entry_type (data)) == -1)
    return FALSE;

  context = job->task->day->context;
  /* find the log day, the task node and the job node */
  if ((day = dg87_context_log_day_get_by_name (context,
                                               job->task->day->name)) == NULL)
    return FALSE;
  if (dg87_log_day_task_get_by_name (day, job->task->name, &task_node) ==
      FALSE)
    return FALSE;
  if (dg87_log_day_job_get_by_name (day, &task_node,
                                    job->name, &job_node) == FALSE)
    return FALSE;
  tm = dg87_current_time;
  data = "";
  if (dg87_log_day_job_entry_add (day, &job_node, entry_type, tm, data) ==
      FALSE)
    return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__send_day_command (dg87_exp_day * day, char *command)
{
  dg87_context *context;
  time_t tm;
  dg87_log_day *log_day;
  dg87_log_day_entry_type entry_type = dg87_log_day_entry_type_command;

  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  context = day->context;
  /* find the log day */
  if ((log_day =
       dg87_context_log_day_get_by_name (context, day->name)) == NULL)
    return FALSE;
	/* add an entry */
  tm = dg87_current_time;
  if (dg87_log_day_entry_add (log_day, entry_type, tm, command) == FALSE)
    return FALSE;

  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__send_task_command (dg87_exp_task * task, char *command)
{
	dg87_log_day *log_day;
	dg87_log_node task_node;
  time_t tm;

  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	tm = dg87_current_time;
  /* find the log day and the task node */
  if ((log_day = dg87_context_log_day_get_by_name (task->day->context, 
							 task->day->name)) == NULL)
					return FALSE;
  if (dg87_log_day_task_get_by_name (log_day, 
													task->name, &task_node) == FALSE)
					return FALSE;
	/* add an entry */
  if (dg87_log_day_task_entry_add (log_day, &task_node, 
													dg87_log_day_entry_type_command,
                                  tm, command) == FALSE)
					return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__send_job_command (dg87_exp_job * job, char *command)
{
	dg87_log_day *log_day;
	dg87_log_node task_node;
	dg87_log_node job_node;
  time_t tm;

  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

	tm = dg87_current_time;

  /* find the log day, the task entry and the job entry */
  if ((log_day = dg87_context_log_day_get_by_name (job->task->day->context, 
							 job->task->day->name)) == NULL)
					return FALSE;
  if (dg87_log_day_task_get_by_name (log_day, 
													job->task->name, &task_node) == FALSE)
					return FALSE;
  if (dg87_log_day_job_get_by_name (log_day, 
													&task_node, job->name, &job_node) == FALSE)
					return FALSE;
	/* add an entry */
  if (dg87_log_day_job_entry_add (log_day, &job_node, 
													dg87_log_day_entry_type_command,
                                  tm, command) == FALSE)
					return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__send_file_test (dg87_exp_condition * condition)
{
  dg87_strbuf *strbuf;
  dg87_context *context;
  dg87_target *target;
  char *hostname;

  assert (condition != NULL);
  if (condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  context = condition->task->day->context;

  strbuf = dg87_strbuf_new ();
  if (strbuf == NULL)
    return FALSE;

  hostname = dg87_context_get_hostname (context, TRUE);
  if (hostname == NULL)
    {
      dg87_strbuf_delete (strbuf);
      return FALSE;
    }

  dg87_strbuf_printf (strbuf,
                      "fa \"%s\" \"%s\" \"%s\" \"%s\" %i",
                      condition->task->day->name, condition->task->name,
                      condition->data.condition_File->path,
                      hostname, context->ip_port);
  dg87_free (hostname, DG87_CHAR);

  target = condition->task->queue->target;
  dg87_server_post_message (context->server,
                            (target->hostname ==
                             NULL) ? target->name : target->hostname,
                            target->ip_port, strbuf->buffer);
  dg87_strbuf_delete (strbuf);
  return TRUE;

}

/******************************************************************************/
BOOL
dg87_caller__create_day_entry (dg87_exp_day * day)
{
  assert (day != NULL && day->referential_name != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_context_sta_referential_add (day->context, day->referential_name)
      == NULL)
    return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__create_task_entry (dg87_exp_task * task)
{
  dg87_context *context;
  dg87_statnode task_node;
  dg87_sta_ref *ref;
  assert (task != NULL);
  assert (task->day->referential_name != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = task->day->context;
  if ((ref = dg87_context_sta_referential_get_by_name (context,
                                                       task->day->
                                                       referential_name)) ==
      NULL)
    return FALSE;
  if (dg87_sta_ref_task_get_by_name (ref, task->name, &task_node) == FALSE)
    if (dg87_sta_ref_task_add (ref, task->name) == FALSE)
      return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__update_task_entry (dg87_exp_task * task)
{
  dg87_context *context;
  dg87_statnode task_node;
  dg87_sta_ref *ref;
  time_t date;
  dg87int duration;
  assert (task != NULL && task->day->referential_name != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = task->day->context;
  if ((ref = dg87_context_sta_referential_get_by_name (context,
                                                       task->day->
                                                       referential_name)) ==
      NULL)
    return FALSE;
  if (dg87_sta_ref_task_get_by_name (ref, task->name, &task_node) == FALSE)
    return FALSE;
  date = dg87_current_time;
  duration = task->end_time - task->start_time;
	if (duration < 0) 
	{
		dg87_daemon_log_printf(dg87_daemon_log_error,
			"task duration < 0");
		duration = 0;
	}
  if (dg87_sta_ref_task_update (ref, &task_node, date, duration) == FALSE)
    return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__create_job_entry (dg87_exp_job * job)
{
  dg87_context *context;
  dg87_statnode task_node;
  dg87_statnode job_node;
  dg87_sta_ref *ref;
  assert (job != NULL && job->task->day->referential_name != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = job->task->day->context;
  if ((ref = dg87_context_sta_referential_get_by_name (context,
                                                       job->task->day->
                                                       referential_name)) ==
      NULL)
    return FALSE;
  if (dg87_sta_ref_task_get_by_name (ref, job->task->name, &task_node) ==
      FALSE)
    return FALSE;
  if (dg87_sta_ref_job_get_by_name (ref, &task_node,
                                    job->name, &job_node) == FALSE)
    if (dg87_sta_ref_job_add (ref, &task_node, job->name) == FALSE)
      return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_caller__update_job_entry (dg87_exp_job * job)
{
  dg87_context *context;
  dg87_statnode task_node;
  dg87_statnode job_node;
  dg87_sta_ref *ref;
  time_t date;
  dg87int duration;
  assert (job != NULL && job->task->day->referential_name != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  context = job->task->day->context;
  if ((ref = dg87_context_sta_referential_get_by_name (context,
                                                       job->task->day->
                                                       referential_name)) ==
      NULL)
    return FALSE;
  if (dg87_sta_ref_task_get_by_name (ref, job->task->name, &task_node) ==
      FALSE)
    return FALSE;
  if (dg87_sta_ref_job_get_by_name (ref, &task_node,
                                    job->name, &job_node) == FALSE)
    return FALSE;
  date = dg87_current_time;
  duration = job->end_time - job->start_time;
	if (duration < 0) 
	{
		dg87_daemon_log_printf(dg87_daemon_log_error,
			"job duration < 0");
		duration = 0;
	}
  if (dg87_sta_ref_job_update (ref, &job_node, date, duration) == FALSE)
    return FALSE;
  return TRUE;
}

/******************************************************************************/
/* End of file */
