/* file x_cmdjob.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include <libdg87/strbuf.h>
#include "x_caller.h"
#include "x_cmdjob.h"
#include "x_day.h"
#include "util.h"

/******************************************************************************/
/* cmd_xjc
 * Usage : xjc DayName TaskName
 * eXp Task Job Count
 * return the number of jobs in the TASK.
 */
int
cmd_xjc (dg87_connection * connection)
{
  dg87_exp_task *task;
  dg87_exp_day *day;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjc DayName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* return jobs count */
  dg87_connection_printf (connection, "%i\n", task->job_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_xjl
 * xjl [options] DayName TaskName
 * eXp Job List
 * Options
 * -d : print description of job 
 * -t : print task name of job
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 * -s : state of job
 */
int
cmd_xjl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first task to display */
  int number = 0; /* number of jobs to display */
  int count = 0;  /* number of jobs actually display */
  dg87_element *element = NULL;
  dg87_exp_task *task;
  dg87_exp_job *job;
  BOOL show_description = FALSE;
  BOOL show_task_name = FALSE;
  BOOL show_state = FALSE;
  dg87_exp_day *day;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "xjl [options] DayName TaskName");
      return 1;
    }
  /* find the day */
  if ((day = dg87_util_find_exp_day (connection, command->argv[0])) == NULL)
    return 1;
  /* find the task */
  if ((task = dg87_util_find_exp_task (connection, day,
                                       command->argv[1])) == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        case 'd': /* description */
          show_description = TRUE;
          break;
        case 't': /* Task */
          show_task_name = TRUE;
          break;
        case 's': /* show state */
          show_state = TRUE;
          break;

        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first job to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (task->job_vector, start);
  else
    element = task->job_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      job = element->object;

      /* response */
      dg87_connection_printf (connection, beg);
      if (show_task_name == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", job->task->name, sep);
        }
      dg87_connection_printf (connection, job->name);
      /* print state */
      if (show_state == TRUE)
        dg87_connection_printf (connection, "%s%s", sep,
                                dg87_exp_job_state_to_ascii (job->state));
      /* print description */
      if (show_description == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep, job->description);
        }
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xji
 * exploit job information 
 * get information for a job
 * Usage : xji [options] DayName TaskName JobName
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_xji (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_exp_task *task;
  dg87_exp_job *job;
  dg87_exp_day *day;
  dg87_command *command;
  char *str_date;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xji [options] DayName TaskName JobName");
      return 1;
    }
  /* find the day */
  if ((day = dg87_util_find_exp_day (connection, command->argv[0])) == NULL)
    return 1;

  /* find the task */
  if ((task = dg87_util_find_exp_task (connection, day,
                                       command->argv[1])) == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* print informations */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Job Name", sep, 
													job->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Name", sep, 
													job->task->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Day Name", sep, 
													job->task->day->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Description", sep, 
													job->description, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Referential", sep,
                          (job->task->day->referential_name ==
                           NULL) ? "" : job->task->day->referential_name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Target", sep,
                          (job->task->queue == NULL) ? "none" :
                          job->task->queue->target->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Target Queue", sep,
                          (job->task->queue == NULL) ? "none" :
                          job->task->queue->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Path", sep,
                          (job->path == NULL) ? "none" : job->path, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Argument", sep,
                          (job->argument ==
                           NULL) ? "none" : job->argument, end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Start Tempo", sep, 
													job->start_tempo, end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "End Tempo", sep, 
													job->end_tempo, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Wait All Dependencies", sep,
                          dg87_bool_to_ascii (job->wait_all_dependencies),
                          end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "On Error Continue", sep,
                          dg87_bool_to_ascii (job->on_error_continue), end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Enabled", sep,
                          dg87_bool_to_ascii (job->enabled), end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "State", sep,
                          dg87_exp_job_state_to_ascii (job->state), end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Exit code", sep, 
													job->exit_code, end);

  str_date = (job->start_time == 0) ? 
					dg87_strdup ("not started", DG87_CHAR) :
					dg87_date_to_ascii (job->start_time);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
									beg, "Start time", sep, 
									str_date, end);
  dg87_free (str_date, DG87_CHAR);

  str_date =
    (job->end_time == 0) ? dg87_strdup ("not terminated", DG87_CHAR) : 
		dg87_date_to_ascii (job-> end_time);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", 
								beg, "End time", sep,
								str_date, end);
  dg87_free (str_date, DG87_CHAR);

  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xjse
 * exploit job set enabled 
 * change the enabled property of the job
 * Usage : xjse DayName TaskName JobName BooleanValue 
 */
int
cmd_xjse (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *job;
  dg87_command *command;
  BOOL value;
  dg87_strbuf *log_msg;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjse DayName TaskName JobName BooleanValue\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  if ((day = dg87_util_find_exp_day (connection, command->argv[0])) == NULL)
    return 1;
  /* find the task */
  if ((task = dg87_util_find_exp_task (connection, day,
                                       command->argv[1])) == NULL)
    return 1;
  /* find the job */
  if ((job = dg87_util_find_exp_job (connection,
                                     task, command->argv[2])) == NULL)
    return 1;
  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[3])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[3]);
      return 1;
    }

  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xjse '%s' (by '%s')", command->argv[3], /* value */
                      (connection->user == NULL) ?
                      "anonymous" : connection->user->name);
  dg87_caller__send_job_command (job, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  if (dg87_exp_job_set_enabled (job, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change Enabled property for job '%s' "\
                              "of the task '%s' !\n"),
											job->name, job->task->name);
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* xjsa
 * eXploitation Job Set Argument 
 * change the argument of a job.
 * Usage : xjsa DayName TaskName JobName argument
 */
int
cmd_xjsa (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjsa DayName TaskName JobName argument");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  if (dg87_exp_job_set_argument (job, command->argv[3]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set argument for the job '%s' "\
                              "in the task '%s' ! %s\n"),
                              job->name, job->task->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* xjsp
 * eXploitation Job Set Path 
 * change the path of a job.
 * Usage : xjsp DayName TaskName JobName path 
 */
int
cmd_xjsp (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjsp DayName TaskName JobName path");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  if (dg87_exp_job_set_path (job, command->argv[3]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set the path of the job '%s' "\
                              	"in the task '%s' ! %s\n"),
                              job->name, job->task->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* xju
 * exploit job update 
 * change the state of the job
 * Usage : xju DayName TaskName JobName State [exit_code]
 */
int
cmd_xju (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *job;
  dg87_exp_job_state state;
  dg87_command *command;
  dg87_strbuf *log_msg;
  int exit_code;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xju DayName TaskName JobName State [exit_code]\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  if ((day = dg87_util_find_exp_day (connection, command->argv[0])) == NULL)
    return 1;
  /* find the task */
  if ((task = dg87_util_find_exp_task (connection, day,
                                       command->argv[1])) == NULL)
    return 1;
  /* find the job */
  if ((job = dg87_util_find_exp_job (connection,
                                     task, command->argv[2])) == NULL)
    return 1;
  state = dg87_exp_job_ascii_to_state (command->argv[3]);
  switch (state)
    {
    case dg87_exp_job_state_ignore:
    case dg87_exp_job_state_ignored:
    case dg87_exp_job_state_canceled:
    case dg87_exp_job_state_ready_to_run:
    case dg87_exp_job_state_running:
    case dg87_exp_job_state_deleted:
      exit_code = -1;
      break;
    case dg87_exp_job_state_terminated:
      exit_code = 0;
      break;
    case dg87_exp_job_state_exit:
      if (command->argc != 5)
        {
          dg87_connection_printf (connection,
                                  gettext ("exit_code must be specified.\n"));
          return 1;
        }
      exit_code = atoi (command->argv[4]);
      break;
    case dg87_exp_job_state_aborted:
      if (command->argc != 5)
        {
          dg87_connection_printf (connection,
                                  gettext ("exit_code must be specified.\n"));
          return 1;
        }
      exit_code = atoi (command->argv[4]);
      break;
    default:
      dg87_connection_printf (connection,
                              gettext ("Incorrect state value '%s'.\n"),
                              command->argv[3]);
      return 1;
    }
  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xju '%s' %i (by '%s')", 
									command->argv[3],       /*state */
                  (exit_code == -1) ? 0 : exit_code, /*exit_code */
                  (connection->user == NULL) ?
                  "anonymous" : connection->user->name);
  dg87_caller__send_job_command (job, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  if (dg87_exp_job_set_state (job, state, exit_code) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the state of job '%s' "\
															"of the task '%s' ! %s\n"),
											job->name, task->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* xjdl
 * eXploit Job Dependence List
 * Usage : xjdl [options] DayName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -a job_name : only jobdep after this job
 *  -b job_name : only jobdep before this job 
 *  -t : print dependence type
 *  -s : print dependence state
 */
int
cmd_xjdl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;   /* index of the first job to display */
  int number = 0;  /* number of jobs to display */
  int count = 0;   /* number of job actually display */
  dg87_item *item = NULL;
  int filter = 0;  /* 0 = no; 1 = before; 2 = after */
  char *filter_job_name = NULL;
  dg87_exp_job *filter_job = NULL;
  dg87_exp_task *task = NULL;
  dg87_exp_day *day = NULL;
  dg87_exp_jobdep *job_dep;
  BOOL show_dep_type = FALSE;
  BOOL show_dep_state = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjdl [options] DayName TaskName");
      return 1;
    }

  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first job to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of job to display */
          number = atoi (optarg);
          break;
        case 'b': /* only job before */
          filter_job_name = strdup (optarg);
          filter = 1;
          break;
        case 'a': /* only job after */
          filter_job_name = strdup (optarg);
          filter = 2;
          break;
        case 't': /* type of dependence */
          show_dep_type = TRUE;
          break;
        case 's': /* state of dependence */
          show_dep_state = TRUE;
          break;
        default:  /* unknown option */
          if (filter_job_name != NULL)
            free (filter_job_name);
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the filter job */
  if (filter > 0)
    {
      filter_job = dg87_util_find_exp_job (connection, task, filter_job_name);
      if (filter_job == NULL)
        {
          free (filter_job_name);
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first JobDep to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (task->job_dep_list, start);
  else
    item = task->job_dep_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      job_dep = item->object;
      if (filter == 0 ||
          (filter == 1 && job_dep->downstream_job == filter_job) ||
          (filter == 2 && job_dep->upstream_job == filter_job))
        {
          /* print job before and after */
          dg87_connection_printf (connection, "%s%s%s%s", beg,
                                  job_dep->upstream_job->name, sep,
                                  job_dep->downstream_job->name);
          /* print job dependence type */
          if (show_dep_type == TRUE)
            {
              dg87_connection_printf (connection,
                                      "%s%s", sep,
                                      dg87_exp_jobdep_type_to_ascii
                                      (job_dep->type));
            }
          /* print task dependence state */
          if (show_dep_state == TRUE)
            {
              dg87_connection_printf (connection,
                                      "%s%s", sep,
                                      dg87_exp_jobdep_state_to_ascii
                                      (job_dep->state));
            }

          dg87_connection_printf (connection, "%s\n", end);
        }
      item = item->next;
    }
  /* free memory */
  if (filter_job_name != NULL)
    free (filter_job_name);
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xjdc
 * eXp Job Dependence Count
 * Usage : xjdc TaskName
 */
int
cmd_xjdc (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjdc DayName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", task->job_dep_list->count);
  return 0;
}

/******************************************************************************/
/* xjdu
 * eXp Job Dependence Update 
 * Usage : xtdu DayName TaskName JobBefore JobAfter Type NewState
 */
int
cmd_xjdu (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *upstream_job;
  dg87_exp_job *downstream_job;
  dg87_exp_jobdep *jobdep = NULL;
  dg87_exp_jobdep_type type;
  dg87_exp_jobdep_state new_state;
  dg87_command *command;
  dg87_item *item;
  dg87_strbuf *log_msg;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 6)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjdu DayName Taskname JobBefore JobAfter Type NewState");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job before */
  upstream_job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (upstream_job == NULL)
    return 1;

  /* find the job after */
  downstream_job =
    dg87_util_find_exp_job (connection, task, command->argv[3]);
  if (downstream_job == NULL)
    return 1;

  /* find dependence type */
  type = dg87_exp_jobdep_ascii_to_type (command->argv[4]);
  if (type == dg87_exp_jobdep_type_unknown)
    {
      dg87_connection_printf (connection,
                              gettext ("Unknown dependence type '%s'\n"),
                              command->argv[4]);
      return 1;
    }

  /* get new state */
  new_state = dg87_exp_jobdep_ascii_to_state (command->argv[5]);
  if (new_state == dg87_exp_jobdep_state_unknown)
    {
      dg87_connection_printf (connection,
                              gettext ("Unknown dependence state '%s'\n"),
                              command->argv[5]);
      return 1;
    }

  /* find the dependence */
  item = task->job_dep_list->first;
  while (item != NULL)
    {
      jobdep = item->object;
      if (jobdep->upstream_job == upstream_job &&
          jobdep->downstream_job == downstream_job && jobdep->type == type)
        break;
      item = item->next;
    }
  if (item == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find a dependence between "\
																			"jobs %s and %s with type %s in "\
																			"the task %s of the day %s\n"),
                              upstream_job->name, downstream_job->name,
                              dg87_exp_jobdep_type_to_ascii (type),
                              task->name, day->name);
      return 1;
    }

  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xjdu '%s' '%s' '%s' '%s' (by '%s')", 
									command->argv[2],  /* upstream task */
                  command->argv[3], /* downstream task */
                  command->argv[4], /* type */
                  command->argv[5], /* new state */
                  (connection->user == NULL) ?
                  "anonymous" : connection->user->name);
  dg87_caller__send_job_command (downstream_job, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  /* update state */
  if (dg87_exp_jobdep_set_state (jobdep, new_state) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't change the state of dependence "\
																			"between jobs %s and %s in the "\
																			"task %s of the day %s\n"),
                              upstream_job->name, downstream_job->name,
                              task->name, day->name);
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* xjsoec
 * eXploitation Job Set on_error_continue
 * Usage : xjsoec DayName TaskName JobName true/false
 */
int
cmd_xjsoec (dg87_connection * connection)
{
  dg87_exp_task *task;
  dg87_exp_job *job;
  BOOL value;
  dg87_command *command;
  dg87_exp_day *day;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjsoec DayName TaskName JobName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;
  /* find the job */
  job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[3])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[3]);
      return 1;
    }

  /* change the property on_error_continue */
  dg87_err_clear_error ();
  if (dg87_exp_job_set_on_error_continue (job, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the job '%s' "\
															"of the task '%s' in the day "\
															"'%s' ! %s\n"),
											"on_error_continue",
											job->name, task->name, task->day->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* xjswad
 * eXploitation Job Set _wait_all_dependencies
 * Usage : xjswad DayName TaskName JobName true/false
 */
int
cmd_xjswad (dg87_connection * connection)
{
  dg87_exp_task *task;
  dg87_exp_job *job;
  BOOL value;
  dg87_command *command;
  dg87_exp_day *day;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjswad DayName TaskName JobName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;
  /* find the job */
  job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[3])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[3]);
      return 1;
    }

  /* change the property wait_all_dependencies */
  dg87_err_clear_error ();
  if (dg87_exp_job_set_wait_all_dependencies (job, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the job '%s' "\
															"of the task '%s' in the day "\
															"'%s' ! %s\n"),
															"wait_all_dependencies",
                              job->name, task->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* xjst
 * eXploitation Job Set Tempo
 * change the start_tempo and end_tempo of a job
 * Usage : xjst DayName TaskName JobName start_tempo end_tempo
 */
int
cmd_xjst (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_job *job;
  int start_tempo, end_tempo;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xjst DayName TaskName JobName start_tempo end_tempo");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* find the job */
  job = dg87_util_find_exp_job (connection, task, command->argv[2]);
  if (job == NULL)
    return 1;

  dg87_err_clear_error ();
  /* change tempo */
  start_tempo = atoi (command->argv[3]);
  end_tempo = atoi (command->argv[4]);
  dg87_err_clear_error ();
  if (dg87_exp_job_set_tempo (job, start_tempo, end_tempo) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't set the tempo of the job '%s' "\
                              "in the task '%s' of the day '%s' ! "\
															"%s\n"),
                              job->name, job->task->name, job->task->day->name,
                              dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* End of file */
