/* file x_cmdtask.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <fcntl.h>
#include <assert.h>

#include <libdg87/strbuf.h>
#include "x_caller.h"
#include "x_cmdtask.h"
#include "x_day.h"
#include "util.h"
#include "context.h"

/******************************************************************************/
/* cmd_xtc
 * eXp Task Count
 * return the number of Tasks in a day.
 * Usage : xtc DayName
 */
int
cmd_xtc (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"xtc DayName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", day->task_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_xtl
 * eXp Task List
 * Usage : xtl [options]  DayName
 * options
 * -d : print description of task
 * -t : show target and queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 * -s : state of task
 */
int
cmd_xtl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;   /* index of the first task to display */
  int number = 0;  /* number of tasks to display */
  int count = 0;   /* number of tasks actually display */
  dg87_element *element = NULL;
  dg87_exp_task *task;
  BOOL show_description = FALSE;
  BOOL show_target = FALSE;
  BOOL show_state = FALSE;
  dg87_exp_day *day;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "xtl [options] DayName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        case 'd': /* description */
          show_description = TRUE;
          break;
        case 't': /* show target */
          show_target = TRUE;
          break;
        case 's': /* show state */
          show_state = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the first Task to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (day->task_vector, start);
  else
    element = day->task_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      task = element->object;
      /* print taks name */
      dg87_connection_printf (connection, "%s%s", beg, task->name);
      /* print target and queue */
      if (show_target == TRUE)
        dg87_connection_printf (connection,
                                "%s%s%s%s", sep,
                                (task->queue == NULL) ? "none" :
                                task->queue->target->name,
                                sep,
                                (task->queue == NULL) ? "none" :
                                task->queue->name);
      /* print state */
      if (show_state == TRUE)
        dg87_connection_printf (connection, "%s%s", sep,
                                dg87_exp_task_state_to_ascii (task->state));

      /* print description */
      if (show_description == TRUE)
        dg87_connection_printf (connection, "%s%s", sep, task->description);

      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xti
 * eXp Task Information 
 * get information for a Task 
 * Usage : xti DayName TaskName
 */
int
cmd_xti (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_exp_task *task;
  dg87_exp_day *day;
  dg87_command *command;
  char *str_date;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control the argument number */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xti DayName TaskName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default: /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* print informations */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Name", sep, 
													task->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Day Name", sep, 
													task->day->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Description", sep, 
													task->description, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Referential", sep,
                          (task->day->referential_name ==
                           NULL) ? "" : task->day->referential_name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Day", sep, task->day->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Wait All Dependencies", sep,
                          dg87_bool_to_ascii (task->wait_all_dependencies),
                          end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "On Error Continue", sep,
                          dg87_bool_to_ascii (task->on_error_continue), end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Enabled", sep,
                          dg87_bool_to_ascii (task->enabled), end);
  if (task->iteration == 0)
    {
      dg87_connection_printf (connection, "%s%s%s%s%s\n",
                              beg, "Iterations", sep, 
															"unlimited", end);
    }
  else
    {
      dg87_connection_printf (connection, "%s%s%s%i%s\n",
                              beg, "Iterations", sep, 
															task->iteration, end);
    }
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Interval", sep, 
													task->interval, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "From", sep,
                          (task->from_begin == TRUE) ? "begin" : "end", end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "Actual Iteration", sep,
                          task->actual_iteration, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Lock Variable", sep,
                          (task->lock_variable == NULL) ? "none"
                          : task->lock_variable->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Lock Action", sep,
                          (task->lock_variable == NULL) ? "none"
                          : (task->lock_action == TRUE) ? "lock" : "unlock",
                          end);

  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Target", sep,
                          (task->queue ==
                           NULL) ? "none" : task->queue->target->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Queue", sep,
                          (task->queue ==
                           NULL) ? "none" : task->queue->name, end);
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "State", sep,
                          dg87_exp_task_state_to_ascii (task->state), end);
	/* start time */
  str_date = (task->start_time == 0) ? dg87_strdup ("not started", DG87_CHAR) :
	dg87_date_to_ascii (task->start_time);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", beg, 
													"Start time", sep,
                          str_date, end);
  dg87_free (str_date, DG87_CHAR);
  /* end time */
  str_date = (task->end_time == 0) ? dg87_strdup ("not terminated", DG87_CHAR) :
  dg87_date_to_ascii (task->end_time);
  dg87_connection_printf (connection, "%s%s%s%s%s\n", beg, 
													"End time", sep,
                          str_date, end);
  dg87_free (str_date, DG87_CHAR);

  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xtse
 * exploit task set enabled 
 * change the enabled property of the task 
 * Usage : xtse DayName TaskName BooleanValue 
 */
int
cmd_xtse (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_command *command;
  BOOL value;
  dg87_strbuf *log_msg;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtse DayName TaskName BooleanValue\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  if ((day = dg87_util_find_exp_day (connection, command->argv[0])) == NULL)
      return 1;
  /* find the task */
  if ((task = dg87_util_find_exp_task (connection, day,
                                       command->argv[1])) == NULL)
      return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }
  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xtse '%s' (by '%s')", 
											command->argv[2], /* value */
                      (connection->user == NULL) ?
                      	"anonymous" : connection->user->name);
  dg87_caller__send_task_command (task, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  if (dg87_exp_task_set_enabled (task, value) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't change the 'Enabled' property "\
																			"for task '%s' !\n"),
                              task->name);
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* xtu
 * exploit task update 
 * change the state of the task 
 * Usage : xtu DayName TaskName State
 */
int
cmd_xtu (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *task;
  dg87_exp_task_state state;
  dg87_command *command;
  dg87_strbuf *log_msg;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtu DayName TaskName State\n");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the day */
  if ((day = dg87_util_find_exp_day (connection, command->argv[0])) == NULL)
      return 1;

  /* find the task */
  if ((task = dg87_util_find_exp_task (connection, day,
                                       command->argv[1])) == NULL)
      return 1;

  state = dg87_exp_task_ascii_to_state (command->argv[2]);
  switch (state)
    {
    case dg87_exp_task_state_ignore:
    case dg87_exp_task_state_ignored:
    case dg87_exp_task_state_running:
    case dg87_exp_task_state_canceled:
    case dg87_exp_task_state_terminated:
    case dg87_exp_task_state_deleted:
    case dg87_exp_task_state_aborted:
      /* correct value : continue */
      break;
    default:
      dg87_connection_printf (connection,
                              gettext ("Incorrect state value '%s'.\n"),
                              command->argv[2]);
      return 1;
    }

  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xtu '%s' (by '%s')", 
											command->argv[2],  /* new state */
                      (connection->user == NULL) ?
                      "anonymous" : connection->user->name);
  dg87_caller__send_task_command (task, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  /* change state */
  if (dg87_exp_task_set_state (task, state) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the state of the task '%s' ! %s\n"),
											task->name, dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* xtcl
 * eXpl Task Condition List
 * Usage : xtcl [options] DayName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -s : condition state
 */
int
cmd_xtcl (dg87_connection * connection)
{
  char *str_hour;
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first condition to display */
  int number = 0; /* number of conditions to display */
  int count = 0;  /* number of conditions actually display */
  dg87_item *item = NULL;
  dg87_exp_condition *condition;
  dg87_exp_task *task;
  dg87_exp_day *day;
  BOOL show_state = FALSE;
  dg87_command *command;
  dg87ubyte delta;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "xtcl [options] DayName TaskName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first condition to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of condition to display */
          number = atoi (optarg);
          break;
        case 's': /* show conditition state */
          show_state = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the first condition to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (task->condition_list, start);
  else
    item = task->condition_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      condition = item->object;
      switch (condition->type)
        {
        case condition_hmin:
          delta = condition->data.condition_hmin->day_delta;
          str_hour =
            dg87_exp_condition_minute_to_hour (condition->data.
                                               condition_hmin->minutes);
          dg87_connection_printf (connection, 
																	"%s%s%s%s%s+%i", beg, "hmin",
                                  sep, str_hour, sep, delta);
          dg87_free (str_hour, DG87_CHAR);
          break;
        case condition_hmax:
          delta = condition->data.condition_hmin->day_delta;
          str_hour =
            dg87_exp_condition_minute_to_hour (condition->data.
                                               condition_hmin->minutes);
          dg87_connection_printf (connection, 
																	"%s%s%s%s%s+%i", beg, "hmax",
                                  sep, str_hour, sep, delta);
          dg87_free (str_hour, DG87_CHAR);
          break;
        case condition_hlate:
          delta = condition->data.condition_hmin->day_delta;
          str_hour =
            dg87_exp_condition_minute_to_hour (condition->data.
                                               condition_hmin->minutes);
          dg87_connection_printf (connection, 
																	"%s%s%s%s%s+%i", beg, "hlate",
                                  sep, str_hour, sep, delta);
          dg87_free (str_hour, DG87_CHAR);
          break;
        case condition_file:
          dg87_connection_printf (connection,
                                  "%s%s%s%s", beg, "file", sep,
                                  condition->data.condition_File->path);
          break;
        case condition_manual:
          dg87_connection_printf (connection,
                                  "%s%s%s%s", beg, "manual", sep,
                                  condition->data.condition_manual->message);
          break;
        case condition_ifequal:
          dg87_connection_printf (connection,
                                  "%s%s%s%s%s%s", beg,
                                  dg87_exp_condition_type_to_ascii
                                  (condition->type), sep,
                                  condition->data.condition_ifequal->
                                  variable->name, sep,
                                  condition->data.condition_ifequal->value);
          break;
        case condition_ifnequal:
          dg87_connection_printf (connection,
                                  "%s%s%s%s%s%s", beg,
                                  dg87_exp_condition_type_to_ascii
                                  (condition->type), sep,
                                  condition->data.condition_ifequal->
                                  variable->name, sep,
                                  condition->data.condition_ifequal->value);
          break;


        default:
          dg87_connection_printf (connection,
                                  gettext ("Unknown type of condition !\n"));
          break;
        }
      if (show_state == TRUE)
			{
				/* add an empty column for manual and file conditions */
				if (condition->type == condition_manual ||
												condition->type == condition_file)
        	dg87_connection_printf (connection, "%s%s", sep, "");
				/* print the condition state */
        dg87_connection_printf (connection, "%s%s", sep,
                                dg87_exp_condition_state_to_ascii (condition->
                                                                   state));
			}
      dg87_connection_printf (connection, "%s\n", end);
      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xtcc
 * eXp Task Condition Count
 * Usage : xtcc DayName TaskName
 */
int
cmd_xtcc (dg87_connection * connection)
{
  dg87_exp_task *task;
  dg87_exp_day *day;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtcc DayName TaskName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", task->condition_list->count);
  return 0;
}

/******************************************************************************/
/* xtcv 
 * eXploitation Task Condition Validate
 * Usage : xtcv DayName TaskName ConditionType Data1 [data2]
 */
int
cmd_xtcv (dg87_connection * connection)
{
  dg87_exp_task *task;
  dg87_exp_day *day;
  char *argument;
  dg87_exp_condition *condition;
  dg87_command *command;
  dg87_strbuf *log_msg;
  dg87_exp_condition_type type;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtcv DayName TaskName ConditionType Data1 [data2]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret condition type */
  type = dg87_exp_condition_ascii_to_type (command->argv[2]);
  if (type == condition_error)
    {
      dg87_connection_printf (connection,
                              gettext ("Invalid condition type '%s'\n"),
                              command->argv[2]);
      return 1;
    }
  /* find the condition */
  argument = (command->argc == 5) ? command->argv[4] : NULL;
  if ((condition = dg87_exp_task_condition_find (task, type, command->argv[3],
                                                  argument)) == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't find this condition in task '%s' of "\
                              "the day '%s'\n"), 
											task->name, day->name);
      return 1;
    }

  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xtcv '%s' (by '%s')",
                      /* TODO print condition and not condition index */
                      command->argv[2], /* condition index */
                      (connection->user == NULL) ?
                      "anonymous" : connection->user->name);
  dg87_caller__send_task_command (task, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  if (dg87_exp_condition_validate (condition) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't validate the condition '%i' "\
																			"of the task '%s' ! %s\n"),
                              index, task->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* xtdl
 * eXploit Task Dependence List
 * Usage : xtdl [options] DayName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -a task_name : only taskdep after this task 
 *  -b task_name : only taskdep before this task 
 *  -t : print dependence type
 *  -s : print dependence state
 */
int
cmd_xtdl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first taskdep to display */
  int number = 0; /* number of taskdeps to display */
  int count = 0;  /* number of taskdeps actually display */
  dg87_item *item = NULL;
  int filter = 0;               /* 0 = no; 1 = before; 2 = after */
  char *task_name_filter = NULL;
  dg87_exp_task *filter_task = NULL;
  dg87_exp_day *day = NULL;
  dg87_exp_taskdep *taskdep;
  BOOL show_dep_type = FALSE;
  BOOL show_dep_state = FALSE;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "xtdl [options] DayName");
      return 1;
    }

  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first job to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of job to display */
          number = atoi (optarg);
          break;
        case 'b': /* only job before */
          task_name_filter = strdup (optarg);
          filter = 1;
          break;
        case 'a': /* only job after */
          task_name_filter = strdup (optarg);
          filter = 2;
          break;
        case 't': /* type of dependence */
          show_dep_type = TRUE;
          break;
        case 's': /* state of dependence */
          show_dep_state = TRUE;
          break;
        default:  /* unknown option */
          if (task_name_filter != NULL)
            free (task_name_filter);
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* find the task */
  if (filter > 0)
    {
      filter_task =
        dg87_util_find_exp_task (connection, day, task_name_filter);
      if (filter_task == NULL)
        {
					/* TODO print an error message */
          free (task_name_filter);
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first taskdep to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (day->taskdep_list, start);
  else
    item = day->taskdep_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      taskdep = item->object;
      if (filter == 0 ||
          (filter == 1 && taskdep->downstream_task == filter_task) ||
          (filter == 2 && taskdep->upstream_task == filter_task))
        {
          /* print task before and after */
          dg87_connection_printf (connection, "%s%s%s%s", beg,
                                  taskdep->upstream_task->name, sep,
                                  taskdep->downstream_task->name);
          /* print task dependence type */
          if (show_dep_type == TRUE)
            {
              dg87_connection_printf (connection,
                                      "%s%s", sep,
                                      dg87_exp_taskdep_type_to_ascii
                                      (taskdep->type));
            }
          /* print task dependence state */
          if (show_dep_state == TRUE)
            {
              dg87_connection_printf (connection,
                                      "%s%s", sep,
                                      dg87_exp_taskdep_state_to_ascii
                                      (taskdep->state));
            }

          dg87_connection_printf (connection, "%s\n", end);
        }
      item = item->next;
    }
  /* free memory */
  if (task_name_filter != NULL)
    free (task_name_filter);
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xtdc
 * eXp Task Dependence Count
 * Usage : xtdc DayName
 */
int
cmd_xtdc (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "xtdc DayName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  dg87_connection_printf (connection, "%i\n", day->taskdep_list->count);
  return 0;
}

/******************************************************************************/
/* xtdu
 * eXp Task Dependence Update 
 * Usage : xtdu DayName TaskBefore TaskAfter Type NewState
 */
int
cmd_xtdu (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_exp_task *upstream_task;
  dg87_exp_task *downstream_task;
  dg87_exp_taskdep *taskdep = NULL;
  dg87_exp_taskdep_type type;
  dg87_exp_taskdep_state new_state;
  dg87_command *command;
  dg87_item *item;
  dg87_strbuf *log_msg;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 5)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtdu DayName TaskBefore TaskAfter Type NewState");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* find the task before */
  upstream_task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (upstream_task == NULL)
    return 1;

  /* find the task before */
  downstream_task =
    dg87_util_find_exp_task (connection, day, command->argv[2]);
  if (downstream_task == NULL)
    return 1;

  /* find dependence type */
  type = dg87_exp_taskdep_ascii_to_type (command->argv[3]);
  if (type == dg87_exp_taskdep_type_unknown)
    {
      dg87_connection_printf (connection,
                              gettext ("Unknown dependence type '%s'\n"),
                              command->argv[3]);
      return 1;
    }

  /* get new state */
  new_state = dg87_exp_taskdep_ascii_to_state (command->argv[4]);
  if (new_state == dg87_exp_taskdep_state_unknown)
    {
      dg87_connection_printf (connection,
                              gettext ("Unknown dependence state '%s'\n"),
                              command->argv[4]);
      return 1;
    }

  /* find the dependence */
  item = day->taskdep_list->first;
  while (item != NULL)
    {
      taskdep = item->object;
      if (taskdep->upstream_task == upstream_task &&
          taskdep->downstream_task == downstream_task
          && taskdep->type == type)
        break;
      item = item->next;
    }
  if (item == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't find a dependence between tasks "\
                              			"%s and %s with type %s in the day %s !\n"),
                              upstream_task->name, downstream_task->name,
                              dg87_exp_taskdep_type_to_ascii (type),
                              day->name);
      return 1;
    }

  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xtdu '%s' '%s' '%s' '%s' (by '%s')", 
											command->argv[1], /* upstream task */
                      command->argv[2], /* downstream task */
                      command->argv[3], /* type */
                      command->argv[4], /* new state */
                      (connection->user == NULL) ?
                      "anonymous" : connection->user->name);
  dg87_caller__send_task_command (downstream_task, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  /* update state */
  if (dg87_exp_taskdep_set_state (taskdep, new_state) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the state of the dependence "\
															"between tasks %s and %s in the day %s\n"),
											upstream_task->name, downstream_task->name,
											day->name);
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* xtsoec
 * eXploitation Task Set On Error Continue 
 * Usage : rtsoec DayName TaskName true/false
 */
int
cmd_xtsoec (dg87_connection * connection)
{
  dg87_exp_task *task;
  BOOL value;
  dg87_command *command;
  dg87_exp_day *day;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtsoec DayName TaskName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* find the task */
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }

  /* change the property on_error_continue */
  dg87_err_clear_error ();
  if (dg87_exp_task_set_on_error_continue (task, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the task '%s' "\
															"of the day '%s' ! %s\n"),
															"on_error_continue", task->name, 
															task->day->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* xtswad
 * eXploitation Task Set Wait All Dependencies
 * Usage : xtswad DayName TaskName true/false
 */
int
cmd_xtswad (dg87_connection * connection)
{
  dg87_exp_task *task;
  BOOL value;
  dg87_command *command;
  dg87_exp_day *day;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xtswad DayName TaskName true/false");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  task = dg87_util_find_exp_task (connection, day, command->argv[1]);
  if (task == NULL)
    return 1;

  /* interpret boolean value */
  if ((value = dg87_ascii_to_bool (command->argv[2])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[2]);
      return 1;
    }

  /* change the property wait_all_dependencies */
  dg87_err_clear_error ();
  if (dg87_exp_task_set_wait_all_dependencies (task, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't set the '%s' property of the task '%s' "\
															"of the day '%s' ! %s\n"),
															"wait_all_dependencies", task->name, 
															task->day->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}
/******************************************************************************/
/* End of file */
