/* file x_day.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "x_day.h"
#include <libdg87/element.h>
#include "global.h"

/****************************************************************************/
dg87_exp_day *
dg87_exp_day_new (char *name, time_t start_time, dg87_context * context)
{
  dg87_exp_day *day;

  assert (name != NULL && context != NULL);
  if (name == NULL || context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_DAY_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((day = dg87_malloc (sizeof (dg87_exp_day), DG87_DAY)) == NULL)
    goto alloc_error;
  if ((day->task_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((day->taskdep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  /* initialize the structure of the object */
  dg87_strnzcpy (day->name, name, DG87_DAY_NAME_LENGTH);
  day->enabled = TRUE;
  day->auto_purged = FALSE;
  day->start_time = start_time;
  day->context = context;
  day->referential_name = NULL;
	DG87_SAVE_NEEDED;
  return day;

alloc_error:
  if (day != NULL)
    dg87_exp_day_delete (day);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_exp_day_delete (dg87_exp_day * day)
{
  dg87_exp_task *task;
  dg87_exp_taskdep *taskdep;
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete all taskdep objects */
  if (day->taskdep_list != NULL)
    {
      while (day->taskdep_list->count > 0)
        {
          taskdep = dg87_list_remove_object_by_index (day->taskdep_list, 0);
          dg87_exp_taskdep_delete (taskdep);
        }
      dg87_list_delete (day->taskdep_list);
    }

  /* delete all task objects */
  if (day->task_vector != NULL)
    {
      while (day->task_vector->count > 0)
        {
          task = dg87_vector_remove_object_by_index (day->task_vector, 0);
          dg87_list_clear (task->upstream_taskdep_list);
          dg87_list_clear (task->downstream_taskdep_list);
          dg87_exp_task_delete (task);
        }
      dg87_vector_delete (day->task_vector);
    }
  if (day->referential_name != NULL)
    dg87_free (day->referential_name, DG87_CHAR);
  dg87_free (day, DG87_DAY);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_day_load (dg87_exp_day * day, dg87_storage * storage)
{
  BOOL value;
  char *string;
  assert (day != NULL && storage != NULL);
  if (day == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* 'enabled' property */
  if (dg87_storage_read_ubyte (storage, &value) == FALSE)
    return FALSE;
  day->enabled = value;
  if (dg87_storage_read_ubyte (storage, &value) == FALSE)
    return FALSE;
  day->auto_purged = value;
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_exp_day_set_referential_name (day, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_day_save (dg87_exp_day * day, dg87_storage * storage)
{
  assert (day != NULL && storage != NULL);
  if (day == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* 'enabled' property */
  if (dg87_storage_write_ubyte (storage, day->enabled) == FALSE)
    return FALSE;
  /* 'auto_purged' property */
  if (dg87_storage_write_ubyte (storage, day->auto_purged) == FALSE)
    return FALSE;
  /* referential name */
  if (dg87_storage_write_string (storage,
                                 (day->referential_name ==
                                  NULL) ? "" : day->referential_name) ==
      FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_day_set_enabled (dg87_exp_day * day, BOOL value)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  day->enabled = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_day_set_auto_purged (dg87_exp_day * day, BOOL auto_purged)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  day->auto_purged = auto_purged;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_day_set_referential_name (dg87_exp_day * day, char *referential_name)
{
  assert (day != NULL && referential_name != NULL);
  if (day == NULL || referential_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (day->referential_name != NULL)
    {
      /* TODO change this error */
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((day->referential_name =
       dg87_strdup (referential_name, DG87_CHAR)) == NULL)
    return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_exp_task *
dg87_exp_day_task_add (dg87_exp_day * day, char *task_name)
{
  dg87_exp_task *task = NULL;
  assert (day != NULL && task_name != NULL);
  if (day == NULL || task_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  task = dg87_exp_task_new (task_name, day);
  if (task == NULL)
    return NULL;

  /* add the task in the task list */
  /* vector generate error if an other task has not the same name */
  /* key must point on task->name */
  if (dg87_vector_add_object (day->task_vector, task, task->name) == FALSE)
    {
      dg87_exp_task_delete (task);
      return NULL;
    }
  return task;
}

/****************************************************************************/
int
dg87_exp_day_task_count (dg87_exp_day * day)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (day->task_vector);
}

/****************************************************************************/
dg87_exp_task *
dg87_exp_day_task_get_by_name (dg87_exp_day * day, char *name)
{
  dg87_element *element;
  assert (day != NULL && name != NULL);
  if (day == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }

  /* browse the list to find an item with the same name */
  element = day->task_vector->first;
  while (element != NULL)
    {
      if (strcmp (element->key, name) == 0)
        return element->object;
      element = element->next;
    }
  dg87_err_set_error (DG87_ERR_CAN_NOT_FIND_OBJECT);
  return NULL;
}

/****************************************************************************/
dg87_exp_task *
dg87_exp_day_task_get_by_index (dg87_exp_day * day, int index)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (day->task_vector, index);
}

/****************************************************************************/
BOOL
dg87_exp_day_task_delete (dg87_exp_day * day, dg87_exp_task * task)
{
  assert (day != NULL && task != NULL);
  if (day == NULL || task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_vector_remove_object (day->task_vector, task) == FALSE)
    return FALSE;
  if (dg87_exp_task_delete (task) == FALSE)
    {
      /* add task in vector again */
      dg87_vector_add_object (day->task_vector, task, task->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_day_task_delete_by_name (dg87_exp_day * day, char *name)
{
  dg87_exp_task *task;
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task = dg87_vector_get_object_by_key (day->task_vector, name);
  if (task == NULL)
    return FALSE;
  return dg87_exp_day_task_delete (day, task);
}

/****************************************************************************/
BOOL
dg87_exp_day_task_delete_by_index (dg87_exp_day * day, int index)
{
  dg87_exp_task *task;
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task = dg87_vector_get_object_by_index (day->task_vector, index);
  if (task == NULL)
    return FALSE;
  return dg87_exp_day_task_delete (day, task);
}

/****************************************************************************/
dg87_exp_taskdep *
dg87_exp_day_taskdep_add (dg87_exp_day * day, dg87_exp_task * upstream_task,
                          dg87_exp_task * downstream_task,
                          dg87_exp_taskdep_type type)
{
  dg87_exp_taskdep *taskdep;

  assert (day != NULL && upstream_task != NULL && downstream_task != NULL);
  if (day == NULL || upstream_task == NULL || downstream_task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* create and add the taskdep */
  if ((taskdep =
       dg87_exp_taskdep_new (upstream_task, downstream_task, type)) == NULL)
    return NULL;

  dg87_list_add_object (day->taskdep_list, taskdep);
  dg87_list_add_object (upstream_task->downstream_taskdep_list, taskdep);
  dg87_list_add_object (downstream_task->upstream_taskdep_list, taskdep);

  return taskdep;
}

/****************************************************************************/
int
dg87_exp_day_taskdep_count (dg87_exp_day * day)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (day->taskdep_list);
}

/****************************************************************************/
dg87_exp_taskdep *
dg87_exp_day_taskdep_get (dg87_exp_day * day, int index)
{
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (day->taskdep_list, index);
}

/****************************************************************************/
BOOL
dg87_exp_day_taskdep_delete_by_index (dg87_exp_day * day, int index)
{
  dg87_exp_taskdep *taskdep;
  assert (day != NULL);
  if (day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  taskdep = dg87_list_remove_object_by_index (day->taskdep_list, index);
  if (taskdep == NULL)
    return FALSE;
  return dg87_exp_taskdep_delete (taskdep);
}

/****************************************************************************/
BOOL
dg87_exp_day_taskdep_delete (dg87_exp_day * day, dg87_exp_taskdep * taskdep)
{
  assert (day != NULL && taskdep != NULL);
  if (day == NULL || taskdep == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_remove_object (day->taskdep_list, taskdep) == FALSE)
    return FALSE;
  return dg87_exp_taskdep_delete (taskdep);
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_exp_day_check (dg87_exp_day * day, int (*print) (char *))
{
  dg87_element *element;
  dg87_item *item;
  dg87_exp_taskdep *taskdep;

  /* check task vector */
  if (day->task_vector == NULL)
    {
      print ("Day : TaskVector is null !\n");
      return 1;
    }
  if (dg87_vector_check (day->task_vector, print) != 0)
    return 1;
  element = day->task_vector->first;
  while (element != NULL)
    {
      if (dg87_exp_task_check (element->object, print) != 0)
        return 1;
      element = element->next;
    }

  /* check Taskdep list */
  if (day->taskdep_list == NULL)
    {
      print ("Day : Taskdep list is null !\n");
      return 1;
    }
  if (dg87_list_check (day->taskdep_list, print) != 0)
    return 1;
  item = day->taskdep_list->first;
  while (item != NULL)
    {
      taskdep = item->object;
      if (dg87_exp_taskdep_check (taskdep, print) != 0)
        return 1;
      item = item->next;
    }
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
