/* file x_job.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "x_job.h"
#include "x_caller.h"
#include "x_day.h"
#include <libdg87/strbuf.h>

/* order in this array must correspond to dg87_exp_job_state */
/* this array must be finished by a NULL */
char *dg87_exp_job_state_string[] = {
  "uninitialized",
  "ignore",
  "wait_for_dep",
  "wait_enabled",
  "ready_to_run",
  "canceled",
  "ready_to_submit",
  "submit",
  "deleted",
  "running",
  "exit",
  "terminated",
  "ignored",
  "aborted",
  "abort_stopped",
  "delete_stopped",
  "wait_start_tempo",
  "wait_end_tempo",
  /*"ignored", */
  NULL
};

/****************************************************************************/
dg87_exp_job *
dg87_exp_job_new (dg87_exp_task * task, char *name)
{
  dg87_exp_job *job;
  assert (task != NULL && name != NULL);
  if (task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_JOB_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((job = dg87_malloc (sizeof (dg87_exp_job), DG87_JOB)) == NULL)
    goto alloc_error;
  /* initialize structure */
  dg87_strnzcpy (job->name, name, DG87_JOB_NAME_LENGTH);
  job->task = task;
  job->description[0] = 0;
  job->path = NULL;
  job->argument = NULL;
  job->start_tempo = 0;
  job->end_tempo = 0;
  job->state = dg87_exp_job_state_uninitialized;
  job->exit_code = 0;
  job->time = 0;
  job->wait_all_dependencies = TRUE;
  job->on_error_continue = TRUE;
  job->enabled = TRUE;
  job->start_time = 0;
  job->end_time = 0;
	DG87_SAVE_NEEDED;
  return job;

alloc_error:
  if (job != NULL)
    dg87_exp_job_delete (job);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_exp_job_delete (dg87_exp_job * job)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* ensure that job isn't in running jobs list of the queue */
  if (job->task->queue != NULL)
    dg87_queue_remove_running_job (job->task->queue, job);
  /* free memory */
  if (job->path != NULL)
    dg87_free (job->path, DG87_CHAR);
  if (job->argument != NULL)
    dg87_free (job->argument, DG87_CHAR);
  dg87_free (job, DG87_JOB);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_load (dg87_exp_job * job, dg87_storage * storage)
{
  char *string;
  dg87long long_value;
  dg87int int_value;
  dg87ubyte b, b2;
  assert (job != NULL && storage != NULL);
  if (job == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read job description */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_exp_job_set_description (job, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);
  /* read job path */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_exp_job_set_path (job, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);
  /* read job argument */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_exp_job_set_argument (job, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);

  /* read start tempo and end tempo */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_storage_read_ubyte (storage, &b2) == FALSE)
    return FALSE;
  if (dg87_exp_job_set_tempo (job, b, b2) == FALSE)
    return FALSE;

  /* read wait_all_dependencies */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_exp_job_set_wait_all_dependencies (job, b) == FALSE)
    return FALSE;

  /* read on_error_continue */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_exp_job_set_on_error_continue (job, b) == FALSE)
    return FALSE;
  /* TODO enabled state time exit_code */
  /* read 'enabled' property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  job->enabled = b;
  /* read 'state' property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  job->state = b;
  /* read 'time' property */
  if (dg87_storage_read_long (storage, &long_value) == FALSE)
    return FALSE;
  job->time = long_value;
  /* read 'exit_code' property */
  if (dg87_storage_read_int (storage, &int_value) == FALSE)
    return FALSE;
  job->exit_code = int_value;
  /* read 'start_time' property */
  if (dg87_storage_read_long (storage, &long_value) == FALSE)
    return FALSE;
  job->start_time = long_value;
  /* read 'end_time' property */
  if (dg87_storage_read_long (storage, &long_value) == FALSE)
    return FALSE;
  job->end_time = long_value;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_save (dg87_exp_job * job, dg87_storage * storage)
{
  assert (job != NULL && storage != NULL);
  if (job == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write job description */
  if (dg87_storage_write_string (storage, job->description) == FALSE)
    return FALSE;
  /* write job path */
  if (dg87_storage_write_string (storage,
                                 (job->path == NULL) ? "" : job->path) ==
      FALSE)
    return FALSE;
  /* write job argument */
  if (dg87_storage_write_string (storage,
                                 (job->argument ==
                                  NULL) ? "" : job->argument) == FALSE)
    return FALSE;
  /* write start tempo */
  if (dg87_storage_write_ubyte (storage, job->start_tempo) == FALSE)
    return FALSE;
  /* write end tempo */
  if (dg87_storage_write_ubyte (storage, job->end_tempo) == FALSE)
    return FALSE;
  /* write wait_all_dependencies */
  if (dg87_storage_write_ubyte (storage, job->wait_all_dependencies) == FALSE)
    return FALSE;
  /* write on_error_continue */
  if (dg87_storage_write_ubyte (storage, job->on_error_continue) == FALSE)
    return FALSE;
  /* 'enabled' property */
  if (dg87_storage_write_ubyte (storage, job->enabled) == FALSE)
    return FALSE;
  /* 'state' property */
  if (dg87_storage_write_ubyte (storage, job->state) == FALSE)
    return FALSE;
  /* 'time' property */
  if (dg87_storage_write_long (storage, job->time) == FALSE)
    return FALSE;
  /* 'exit_code' property */
  if (dg87_storage_write_int (storage, job->exit_code) == FALSE)
    return FALSE;
  /* 'start_time' property */
  if (dg87_storage_write_long (storage, job->start_time) == FALSE)
    return FALSE;
  /* 'end_time' property */
  if (dg87_storage_write_long (storage, job->end_time) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
dg87_exp_job_status
dg87_exp_job_get_status (dg87_exp_job * job)
{
  assert (job != NULL);
  switch (job->state)
    {
    case dg87_exp_job_state_uninitialized:
    case dg87_exp_job_state_ignore:
    case dg87_exp_job_state_wait_for_dep:
    case dg87_exp_job_state_wait_enabled:
      return dg87_exp_job_status_waiting;

    case dg87_exp_job_state_ready_to_run:
    case dg87_exp_job_state_aborted_stopped:
    case dg87_exp_job_state_deleted_stopped:
    case dg87_exp_job_state_wait_start_tempo:
    case dg87_exp_job_state_wait_end_tempo:
    case dg87_exp_job_state_ready_to_submit:
    case dg87_exp_job_state_submit:
    case dg87_exp_job_state_running:
    case dg87_exp_job_state_exit:
      return dg87_exp_job_status_running;

    case dg87_exp_job_state_terminated:
    case dg87_exp_job_state_ignored:
    case dg87_exp_job_state_canceled:
      return dg87_exp_job_status_terminated;

    case dg87_exp_job_state_deleted:
    case dg87_exp_job_state_aborted:
      return dg87_exp_job_status_aborted;
    }
  return -1;
}

/****************************************************************************/
dg87_exp_job_state
dg87_exp_job_manage (dg87_exp_job * job)
{
  dg87_item *item;
  dg87_exp_jobdep *job_dep;
  dg87_exp_job_state new_state;

  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  /* if job isn't initialized, find if it is waiting for dependencies */
  if (job->state == dg87_exp_job_state_uninitialized)
    {
      item = job->task->job_dep_list->first;
      while (item != NULL)
        {
          job_dep = item->object;
          if (job_dep->downstream_job == job)
            break;
          item = item->next;
        }
      if (item == NULL)
				{
        	dg87_exp_job_set_state (job, dg87_exp_job_state_wait_enabled, -1);
					return job->state;
				}
      else
				{
        	dg87_exp_job_set_state (job, dg87_exp_job_state_wait_for_dep, -1);
					return job->state;
				}
    }

  if (job->state == dg87_exp_job_state_wait_for_dep ||
      job->state == dg87_exp_job_state_ignore)
    {
      if (job->wait_all_dependencies == TRUE)
        {
          /* Job is waiting for all dependencies (AND) 
           * 1- Job is ready to run if all dependencies are
           * validated.
           * 2- Job is canceled if at least one dependence is
           * canceled.
           * 3- Job is waiting if at least one dependence is
           * waiting */
          new_state = dg87_exp_job_state_wait_enabled;
          item = job->task->job_dep_list->first;
          while (item != NULL)
            {
              job_dep = item->object;
              item = item->next;
              if (job_dep->downstream_job != job)
                continue;
              /* if a dep is canceled, then job is canceled */
              if (job_dep->state == dg87_exp_jobdep_state_canceled)
                {
                  new_state = dg87_exp_job_state_canceled;
                  break;
                }
              /* if a dep is waiting, then job is waiting */
              if (job_dep->state == dg87_exp_jobdep_state_waiting)
                {
                  new_state = dg87_exp_job_state_wait_for_dep;
                  break;
                }

            }

        }
      else
        {
          /* Job is waiting for one dependence (OR). 
           * 1- Job is ready to run if at least dependence is
           * validated.
           * 2- Job is canceled if all dependencies are validated.
           * 3- Job is waiting for dependence if at least one 
           * dependence is waiting.
           */
          new_state = dg87_exp_job_state_canceled;
          item = job->task->job_dep_list->first;
          while (item != NULL)
            {
              job_dep = item->object;
              item = item->next;
              if (job_dep->downstream_job != job)
                continue;
              /*if a dep is validated, then job is validated */
              if (job_dep->state == dg87_exp_jobdep_state_validated)
                {
                  new_state = dg87_exp_job_state_wait_enabled;
                  break;
                }
              /* if a dep is waiting, then job is waiting */
              if (job_dep->state == dg87_exp_jobdep_state_waiting)
                {
                  new_state = dg87_exp_job_state_wait_for_dep;
                  break;
                }

            }
        }

      switch (job->state)
        {
        case dg87_exp_job_state_wait_for_dep:
          dg87_exp_job_set_state (job, new_state, -1);
					return job->state;
        case dg87_exp_job_state_ignore:
          if (new_state != dg87_exp_job_state_wait_for_dep)
						{
            	dg87_exp_job_set_state (job, dg87_exp_job_state_ignored, -1);
							return job->state;
						}
        default:
          /* just to avoid warnings */
          break;

        }
    }
  /* job must be enabled to be ready to run */
  if (job->state == dg87_exp_job_state_wait_enabled && job->enabled == TRUE)
		{
    	dg87_exp_job_set_state (job, dg87_exp_job_state_ready_to_run, -1);
			return job->state;
		}
  /* if job is ready to run, wait start tempo */
  if (job->state == dg87_exp_job_state_ready_to_run)
    {
      job->time = time (0);
      dg87_exp_job_set_state (job, dg87_exp_job_state_wait_start_tempo, -1);
			return job->state;
    }
  /* after start tempo, launch the job */
  if (job->state == dg87_exp_job_state_wait_start_tempo)
    {
      if (time (0) >= job->start_tempo + job->time)
        {
          /* check path is not an empty string */
          if (job->path == NULL || job->path[0] == '\0')
            {
              dg87_exp_job_set_state (job, dg87_exp_job_state_deleted, -1);
							return job->state;
            }
          /* check queue is not NULL */
          if (job->task->queue == NULL)
            {
              dg87_exp_job_set_state (job, dg87_exp_job_state_deleted, -1);
							return job->state;
            }
          dg87_exp_job_set_state (job, dg87_exp_job_state_ready_to_submit, -1);
					return job->state;
        }
    }
  /* if job is exited, wait for end tempo */
  if (job->state == dg87_exp_job_state_exit)
    {
      job->time = time (0);
      dg87_exp_job_set_state (job, dg87_exp_job_state_wait_end_tempo, -1);
			return job->state;
    }
  /* after end tempo, terminate job */
  if (job->state == dg87_exp_job_state_wait_end_tempo &&
      time (0) >= job->end_tempo + job->time)

    {
      if (job->exit_code == 0)
				{
        	/* normal termination */
        	dg87_exp_job_set_state (job, dg87_exp_job_state_terminated, -1);
					return job->state;
				}
      else
        {
          /* stop or continue after an abort */
          if (job->on_error_continue == TRUE)
						{
          		dg87_exp_job_set_state (job, dg87_exp_job_state_aborted, -1);
							return job->state;
						}
          else
						{
            	dg87_exp_job_set_state (job, dg87_exp_job_state_aborted_stopped,
                                    -1);
							return job->state;
						}
        }
    }

  return job->state;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_state (dg87_exp_job * job, dg87_exp_job_state new_state,
                        int exit_code)
{
  dg87_exp_jobdep *jobdep;
  dg87_item *jobdeitem;
  dg87_queue *queue;

  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* exit if there is nothing to do */
  if (job->state == new_state)
    return TRUE;
	dg87_daemon_log_printf(dg87_daemon_log_debug,
  	"Job %s : %s -> %s", job->name,
          dg87_exp_job_state_string[job->state],
          dg87_exp_job_state_string[new_state]);

  /* stop deleted job if on_error_continue=FALSE */
  if (new_state == dg87_exp_job_state_deleted
      && job->on_error_continue == FALSE)
    new_state = dg87_exp_job_state_deleted_stopped;

  job->state = new_state;

	/* memorize start time */
  if (job->state == dg87_exp_job_state_running && job->start_time == 0)
    job->start_time = time (0);
  /* memorize end time */
  if (job->state == dg87_exp_job_state_exit && job->end_time == 0)
    {
      job->end_time = time (0);
      /* send message to dg87s */
      if (job->task->day->referential_name != NULL)
        dg87_caller__update_job_entry (job);
    }

  if (exit_code != -1)
    job->exit_code = exit_code;
  queue = job->task->queue;

  if (job->state == dg87_exp_job_state_uninitialized)
    {
      job->start_time = 0;
      job->end_time = 0;
    }

  switch (dg87_exp_job_get_status (job))
    {

    case dg87_exp_job_status_waiting:
      if (queue != NULL)
        dg87_queue_remove_running_job (queue, job);
      break;
    case dg87_exp_job_status_running:
      if (queue != NULL)
        dg87_queue_add_running_job (queue, job);
      break;
    case dg87_exp_job_status_terminated:
    case dg87_exp_job_status_aborted:
      job->end_time = time (0);
      if (queue != NULL)
        dg87_queue_remove_running_job (queue, job);
      /* update job dependencies */
      jobdeitem = job->task->job_dep_list->first;
      while (jobdeitem != NULL)
        {
          jobdep = jobdeitem->object;
          dg87_exp_jobdep_evaluate (jobdep);
          jobdeitem = jobdeitem->next;
        }
      break;
    }

  dg87_caller__send_job_state (job);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_description (dg87_exp_job * job, char *description)
{
  assert (job != NULL && description != NULL);
  if (job == NULL || description == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_description (description, DG87_JOB_DESC_LENGTH) == FALSE)
    return FALSE;
  dg87_strnzcpy (job->description, description, DG87_JOB_DESC_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_tempo (dg87_exp_job * job, int start_tempo, int end_tempo)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (start_tempo < 0 || start_tempo > JOB_MAX_TEMPO
      || end_tempo < 0 || end_tempo > JOB_MAX_TEMPO)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  job->start_tempo = start_tempo;
  job->end_tempo = end_tempo;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_path (dg87_exp_job * job, char *path)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  if (job->path != NULL)
    dg87_free (job->path, DG87_CHAR);
  job->path = NULL;
  if (path == NULL || strlen (path) == 0)
    return TRUE;

  job->path = dg87_malloc (strlen (path) + 1, DG87_CHAR);
  if (job->path == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  strcpy (job->path, path);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_argument (dg87_exp_job * job, char *argument)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  if (job->argument != NULL)
    dg87_free (job->argument, DG87_CHAR);
  job->argument = NULL;
  if (argument == NULL || strlen (argument) == 0)
    return TRUE;

  job->argument = dg87_malloc (strlen (argument) + 1, DG87_CHAR);
  if (job->argument == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  strcpy (job->argument, argument);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_wait_all_dependencies (dg87_exp_job * job, BOOL wait)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job->wait_all_dependencies = wait;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_on_error_continue (dg87_exp_job * job, BOOL value)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job->on_error_continue = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_set_enabled (dg87_exp_job * job, BOOL value)
{
  assert (job != NULL);
  if (job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job->enabled = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_exp_job_state
dg87_exp_job_ascii_to_state (char *string)
{
  int index = 0;
  assert (string != NULL);
  while (dg87_exp_job_state_string[index] != NULL &&
         strcmp (string, dg87_exp_job_state_string[index]) != 0)
    index++;
  return index;
}

/****************************************************************************/
char *
dg87_exp_job_state_to_ascii (dg87_exp_job_state state)
{
  return dg87_exp_job_state_string[state];
}

/****************************************************************************/
BOOL
dg87_exp_job_find_upstream_jobs (dg87_exp_job * job, dg87_list * job_list)
{
  dg87_exp_jobdep *job_dep;
  dg87_item *jobdep_item;
  assert (job != NULL && job_list != NULL);
  if (job == NULL || job_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (job_list, job) == FALSE)
    return FALSE;
  jobdep_item = job->task->job_dep_list->first;
  while (jobdep_item != NULL)
    {
      job_dep = jobdep_item->object;
      jobdep_item = jobdep_item->next;
      if (job_dep->downstream_job != job)
        continue;
      if (dg87_exp_job_find_upstream_jobs (job_dep->upstream_job,
                                           job_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_job_find_downstream_jobs (dg87_exp_job * job, dg87_list * job_list)
{
  dg87_exp_jobdep *job_dep;
  dg87_item *jobdep_item;
  assert (job != NULL && job_list != NULL);
  if (job == NULL || job_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (job_list, job) == FALSE)
    return FALSE;
  jobdep_item = job->task->job_dep_list->first;
  while (jobdep_item != NULL)
    {
      job_dep = jobdep_item->object;
      jobdep_item = jobdep_item->next;
      if (job_dep->upstream_job != job)
        continue;
      if (dg87_exp_job_find_downstream_jobs (job_dep->downstream_job,
                                             job_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_exp_job_check (dg87_exp_job * job, int (*print) (char *))
{
  if (strlen (job->name) == 0)
    {
      print ("Job : name length is zero !\n");
      return 1;
    }
  if (job->task == NULL)
    {
      print ("Job : Task pointer is null !\n");
      return 1;
    }
  if (job->start_tempo < 0 || job->start_tempo > JOB_MAX_TEMPO)
    {
      print ("Job : incorrect value for start_tempo !\n");
      return 1;
    }
  if (job->end_tempo < 0 || job->end_tempo > JOB_MAX_TEMPO)
    {
      print ("Job : incorrect value for end_tempo\n");
      return 1;
    }
  /* path and argument could be null -> no test */

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
