/* file x_job.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_EXP_JOB_H__
#define __DG87_EXP_JOB_H__

#include <time.h>
#include <config.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include "global.h"
#include "x_task.h"

#define JOB_MAX_TEMPO 3600

typedef enum
{
  dg87_exp_job_state_uninitialized = 0,
  dg87_exp_job_state_ignore,
  dg87_exp_job_state_wait_for_dep,
  dg87_exp_job_state_wait_enabled,
  dg87_exp_job_state_ready_to_run,
  dg87_exp_job_state_canceled,
  dg87_exp_job_state_ready_to_submit,
  dg87_exp_job_state_submit,
  dg87_exp_job_state_deleted,
  dg87_exp_job_state_running,
  dg87_exp_job_state_exit,
  dg87_exp_job_state_terminated,
  dg87_exp_job_state_ignored,
  dg87_exp_job_state_aborted,
  dg87_exp_job_state_aborted_stopped,
  dg87_exp_job_state_deleted_stopped,
  dg87_exp_job_state_wait_start_tempo,
  dg87_exp_job_state_wait_end_tempo
    /* don't forget to update dg87_exp_job_state_string and dg87_exp_job_get_status
     * in job.c when adding a new state ! */
}
dg87_exp_job_state;

typedef enum
{
  dg87_exp_job_status_waiting,
  dg87_exp_job_status_running,  /* waittempo, running ... */
  dg87_exp_job_status_terminated,       /* terminated, canceled */
  dg87_exp_job_status_aborted   /* aborted, deleted */
}
dg87_exp_job_status;

struct _dg87_exp_job
{
  char name[DG87_JOB_NAME_LENGTH + 1];
  char description[DG87_JOB_DESC_LENGTH + 1];
  dg87_exp_task *task;
  char *path;                   /* path for the script */
  char *argument;               /* argument for the script */
  int start_tempo;              /* temporization before execution in seconds */
  int end_tempo;                /* temporization after execution in seconds */
  dg87_exp_job_state state;
  BOOL wait_all_dependencies;
  BOOL on_error_continue;
  BOOL enabled;
  time_t time;
  int exit_code;
  time_t start_time, end_time;
};

/*
 * dg87_exp_job_new 
 * create a new job and initialize the name property
 */
dg87_exp_job *dg87_exp_job_new (dg87_exp_task * task, char *name);

/*
 * dg87_exp_job_delete
 */
BOOL dg87_exp_job_delete (dg87_exp_job * job);

/*
 * dg87_exp_job_load
 */
BOOL dg87_exp_job_load (dg87_exp_job * job, dg87_storage * storage);

/*
 * dg87_exp_job_save
 */
BOOL dg87_exp_job_save (dg87_exp_job * job, dg87_storage * storage);

/*
 * dg87_exp_job_set_description
 * set the description of the job.
 */
BOOL dg87_exp_job_set_description (dg87_exp_job * job, char *description);

/*
 * dg87_exp_job_get_status
 */
dg87_exp_job_status dg87_exp_job_get_status (dg87_exp_job * job);

/* 
 * dg87_exp_job_manage
 */
dg87_exp_job_state dg87_exp_job_manage (dg87_exp_job * job);

/* 
 * dg87_exp_job_set_state
 * change the state of the job and update dependencies after
 * this job
 */
BOOL dg87_exp_job_set_state (dg87_exp_job * job, dg87_exp_job_state new_state,
                             int exit_code);

/*
 * dg87_exp_job_set_tempo
 * set the start_tempo and EndTempo for the job.
 */
BOOL dg87_exp_job_set_tempo (dg87_exp_job * job, int start_tempo,
                             int end_tempo);

/*
 * dg87_exp_job_set_path
 * set the path of the job.
 * path could be NULL or zero length.
 */
BOOL dg87_exp_job_set_path (dg87_exp_job * job, char *path);

/*
 * dg87_exp_job_set_argument
 * set the argument of the job.
 * argument could be NULL or zero length.
 */
BOOL dg87_exp_job_set_argument (dg87_exp_job * job, char *argument);

/*
 * dg87_exp_job_set_wait_all_dependencies
 */
BOOL dg87_exp_job_set_wait_all_dependencies (dg87_exp_job * job, BOOL wait);

/*
 * dg87_exp_job_set_on_error_continue
 */
BOOL dg87_exp_job_set_on_error_continue (dg87_exp_job * job, BOOL value);

/*
 * dg87_exp_job_set_enabled
 */
BOOL dg87_exp_job_set_enabled (dg87_exp_job * job, BOOL value);

/*
 * dg87_exp_job_find_upstream_jobs
 * This function adds to the list given in parameter all upstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_exp_job_find_upstream_jobs (dg87_exp_job * job,
                                      dg87_list * job_list);

/*
 * dg87_exp_job_find_downstream_jobs
 * This function adds to the list given in parameter all downstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_exp_job_find_downstream_jobs (dg87_exp_job * job,
                                        dg87_list * job_list);


/*
 * dg87_exp_job_ascii_to_state
 * convert a string into a job's state
 */
dg87_exp_job_state dg87_exp_job_ascii_to_state (char *string);

/*
 * dg87_exp_job_state_to_ascii
 * convert a job's state to a string
 */
char *dg87_exp_job_state_to_ascii (dg87_exp_job_state state);

#ifdef DEBUG
/*
 * dg87_exp_job_ckeck
 */
int dg87_exp_job_check (dg87_exp_job * job, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_EXP_JOB_H__*/
/* End of file */
