/* file x_task.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "x_job.h"
#include "x_jobdep.h"
#include "x_task.h"
#include "x_condition.h"
#include "context.h"
#include "engine.h"
#include "x_caller.h"
#include "x_day.h"
#include "variable.h"
#include <libdg87/list.h>
#include <libdg87/vector.h>
#include <libdg87/strbuf.h>

/* order in this array must correspond to dg87_exp_job_state */
/* this array must be finished by a NULL */
char *dg87_exp_task_state_string[] = {
  "uninitialized",
  "ignore",
  "wait_for_dep",
  "wait_lock",
  "wait_for_condition",
  "wait_enabled",
  "canceled",
  "running",
  "ignored",
  "aborted",
  "abort_stopped",
  "terminated",
  "deleted",
  "wait_tempo",
  NULL
};

/****************************************************************************/
dg87_exp_task *
dg87_exp_task_new (char *name, dg87_exp_day * day)
{
  dg87_exp_task *task;
  assert (name != NULL && day != NULL);
  if (name == NULL || day == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_TASK_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((task = dg87_malloc (sizeof (dg87_exp_task), DG87_TASK)) == NULL)
    goto alloc_error;
  if ((task->job_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;
  if ((task->job_dep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->upstream_taskdep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->downstream_taskdep_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((task->condition_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  /* initialize structure */
  dg87_strnzcpy (task->name, name, DG87_TASK_NAME_LENGTH);
  task->description[0] = 0;
  task->queue = NULL;
  task->day = day;
  task->state = dg87_exp_task_state_uninitialized;
  task->wait_all_dependencies = TRUE;
  task->on_error_continue = TRUE;
  task->enabled = TRUE;
  task->start_time = 0;
  task->end_time = 0;
  task->iteration = 1;          /* task is not cyclical */
  task->interval = 0;           /* not used with no-cyclical task */
  task->from_begin = TRUE;      /* not used with no-cyclical task */
  task->actual_iteration = 1;   /* first iteration */
  task->lock_variable = NULL;
  task->lock_action = FALSE;
	DG87_SAVE_NEEDED;
  return task;
alloc_error:
  if (task != NULL)
    dg87_exp_task_delete (task);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_exp_task_delete (dg87_exp_task * task)
{
  dg87_exp_job *job;
  dg87_exp_jobdep *job_dep;
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* decrement lock variable counter */
  if (task->lock_variable != NULL)
    task->lock_variable->linked_exp_tasks--;
  /* can't delete task if task has dependencies whith other task */
  if (task->upstream_taskdep_list != NULL &&
      task->upstream_taskdep_list->count > 0)
    {
      dg87_err_set_error (DG87_ERR_TASK_HAS_DEPENDENCES);
      return FALSE;
    }
  if (task->downstream_taskdep_list != NULL
      && task->downstream_taskdep_list->count > 0)
    {
      dg87_err_set_error (DG87_ERR_TASK_HAS_DEPENDENCES);
      return FALSE;
    }

  if (task->upstream_taskdep_list != NULL)
    dg87_list_delete (task->upstream_taskdep_list);
  if (task->downstream_taskdep_list != NULL)
    dg87_list_delete (task->downstream_taskdep_list);

  /* remove Task from Queue */
  if (task->queue != NULL)
    dg87_list_remove_object (task->queue->exp_task_list, task);

  /* delete all JobDep objects */
  if (task->job_dep_list != NULL)
    {
      while (task->job_dep_list->count > 0)
        {
          job_dep = dg87_list_remove_object_by_index (task->job_dep_list, 0);
          dg87_exp_jobdep_delete (job_dep);
        }
      dg87_list_delete (task->job_dep_list);
    }

  /* delete all Job objects */
  if (task->job_vector != NULL)
    {
      while (task->job_vector->count > 0)
        {
          job = dg87_vector_remove_object_by_index (task->job_vector, 0);
          dg87_exp_job_delete (job);
        }
      dg87_vector_delete (task->job_vector);
    }

  /* delete all Condition objects */
  if (task->condition_list != NULL)
    {
      while (task->condition_list->count > 0)
        {
          condition = dg87_list_remove_object_by_index
            (task->condition_list, 0);
          dg87_exp_condition_delete (condition);
        }
      dg87_list_delete (task->condition_list);
    }

  dg87_free (task, DG87_TASK);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_load (dg87_exp_task * task,
                    dg87_storage * storage, dg87_context * context)
{
  dg87ubyte b;
  dg87uint ui1, ui2;
  char *string;
  dg87_target *target;
  dg87_queue *queue;
  dg87_variable *variable;
  dg87long long_value;
  assert (task != NULL && storage != NULL && context != NULL);
  if (task == NULL || storage == NULL || context == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read task description */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_exp_task_set_description (task, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);

  /* read wait_all_dependencies */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_exp_task_set_wait_all_dependencies (task, b) == FALSE)
    return FALSE;

  /* read on_error_continue */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_exp_task_set_on_error_continue (task, b) == FALSE)
    return FALSE;

  /* read cyclical properties */
  if (dg87_storage_read_uint (storage, &ui1) == FALSE)
    return FALSE;
  if (dg87_storage_read_uint (storage, &ui2) == FALSE)
    return FALSE;
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_exp_task_set_cyclical (task, ui1, ui2, b) == FALSE)
    return FALSE;
  /* actual iteration */
  if (dg87_storage_read_uint (storage, &ui1) == FALSE)
    return FALSE;
  task->actual_iteration = ui1;
  /* read lock properties */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (b == TRUE)
    {
      /* find variable */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((variable =
           dg87_engine_variable_get_by_name (context->engine,
                                             string)) == NULL)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      /* read lock_action */
      if (dg87_storage_read_ubyte (storage, &b) == FALSE)
        return FALSE;
      if (dg87_exp_task_set_lock (task, variable, b) == FALSE)
        return FALSE;
    }
  /* read target and queue */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (b == TRUE)
    {                           /* queue is not NULL */
      /* find target */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((target =
           dg87_context_target_get_by_name (context, string)) == NULL)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      /* find queue */
      if (dg87_storage_read_string (storage, &string) == FALSE)
        return FALSE;
      if ((queue = dg87_target_queue_get_by_name (target, string)) == NULL)
        {
          dg87_free (string, DG87_CHAR);
          return FALSE;
        }
      dg87_free (string, DG87_CHAR);
      /* set queue */
      if (dg87_exp_task_set_queue (task, queue) == FALSE)
        return FALSE;
    }
  /* 'enabled' property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  task->enabled = b;

  /* 'state' property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  task->state = b;

  /* read 'start_time' property */
  if (dg87_storage_read_long (storage, &long_value) == FALSE)
    return FALSE;
  task->start_time = long_value;
  /* read 'end_time' property */
  if (dg87_storage_read_long (storage, &long_value) == FALSE)
    return FALSE;
  task->end_time = long_value;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_save (dg87_exp_task * task, dg87_storage * storage)
{
  assert (task != NULL && storage != NULL);
  if (task == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write task description */
  if (dg87_storage_write_string (storage, task->description) == FALSE)
    return FALSE;
  /* write wait_all_dependencies */
  if (dg87_storage_write_ubyte (storage, task->wait_all_dependencies) == FALSE)
    return FALSE;
  /* write on_error_continue */
  if (dg87_storage_write_ubyte (storage, task->on_error_continue) == FALSE)
    return FALSE;
  /* write cyclical properties */
  if (dg87_storage_write_uint (storage, task->iteration) == FALSE)
    return FALSE;
  if (dg87_storage_write_uint (storage, task->interval) == FALSE)
    return FALSE;
  if (dg87_storage_write_ubyte (storage, task->from_begin) == FALSE)
    return FALSE;
  if (dg87_storage_write_uint (storage, task->actual_iteration) == FALSE)
    return FALSE;
  /* save lock properties */
  if (dg87_storage_write_ubyte (storage, (task->lock_variable == NULL) ?
                               FALSE : TRUE) == FALSE)
    return FALSE;
  if (task->lock_variable != NULL)
    {
      if (dg87_storage_write_string (storage,
                                     task->lock_variable->name) == FALSE)
        return FALSE;
      if (dg87_storage_write_ubyte (storage, task->lock_action) == FALSE)
        return FALSE;

    }
  /* save target and queue */
  if (dg87_storage_write_ubyte (storage,
                               (task->queue == NULL) ? FALSE : TRUE) == FALSE)
    return FALSE;
  if (task->queue != NULL)
    {
      if (dg87_storage_write_string (storage,
                                     task->queue->target->name) == FALSE)
        return FALSE;
      if (dg87_storage_write_string (storage, task->queue->name) == FALSE)
        return FALSE;
    }
  /* 'enabled' property */
  if (dg87_storage_write_ubyte (storage, task->enabled) == FALSE)
    return FALSE;
  /* 'state' property */
  if (dg87_storage_write_ubyte (storage, task->state) == FALSE)
    return FALSE;
  /* 'start_time' property */
  if (dg87_storage_write_long (storage, task->start_time) == FALSE)
    return FALSE;
  /* 'end_time' property */
  if (dg87_storage_write_long (storage, task->end_time) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
dg87_exp_task_state
dg87_exp_task_manage (dg87_exp_task * task)
{
  dg87_element *element;
  dg87_exp_job *job;
  dg87_item *item;
  dg87_exp_taskdep *taskdep;
  dg87_exp_jobdep *job_dep;
  dg87_exp_condition *condition;
  dg87_exp_task_state new_state;
  dg87_variable *lock_variable;
  BOOL ready;
  int n_terminated = 0;
  int n_aborted = 0;
  int n_running = 0;
  int n_waiting = 0;
  time_t end_time;

  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;             /* not cool */
    }

  /* initialize state if function never been call */
  if (task->state == dg87_exp_task_state_uninitialized)
		{
    	dg87_exp_task_set_state (task, dg87_exp_task_state_wait_for_dep);
			return task->state;
		}

  /* find if task is always waiting for dependencies */
  if (task->state == dg87_exp_task_state_wait_for_dep ||
      task->state == dg87_exp_task_state_ignore)
    {
      if (task->wait_all_dependencies == TRUE)
        {
          /* Task is waiting for all dependencies (AND) 
           * 1- Task is ready to run if all dependencies are
           * validated.
           * 2- Task is canceled if at least one dependence is
           * canceled.
           * 3- Task is waiting if at least one dependence is
           * waiting */
          new_state = dg87_exp_task_state_wait_lock;
          item = task->upstream_taskdep_list->first;
          while (item != NULL)
            {
              taskdep = item->object;
              item = item->next;
              /* if a dep is canceled, then task is canceled */
              if (taskdep->state == dg87_exp_taskdep_state_canceled)
                {
                  new_state = dg87_exp_task_state_canceled;
                  break;
                }
              /* if a dep is waiting, then task is waiting */
              if (taskdep->state == dg87_exp_taskdep_state_waiting)
                {
                  new_state = dg87_exp_task_state_wait_for_dep;
                  break;
                }

            }
        }
      else
        {
          /* Task is waiting for one dependence (OR). 
           * 1- Task is ready to run if at least dependence is
           * validated.
           * 2- Task is canceled if all dependencies are validated.
           * 3- Task is waiting for dependence if at least one 
           * dependence is waiting.
           */
          new_state = dg87_exp_task_state_canceled;
          item = task->upstream_taskdep_list->first;
          while (item != NULL)
            {
              taskdep = item->object;
              item = item->next;
              /*if a dep is validated,then task is validated */
              if (taskdep->state == dg87_exp_taskdep_state_validated)
                {
                  new_state = dg87_exp_task_state_wait_lock;
                  break;
                }
              /* if a dep is waiting, then task is waiting */
              if (taskdep->state == dg87_exp_taskdep_state_waiting)
                {
                  new_state = dg87_exp_task_state_wait_for_dep;
                  break;
                }

            }
        }
      switch (task->state)
        {
        case dg87_exp_task_state_wait_for_dep:
          dg87_exp_task_set_state (task, new_state);
					return task->state;
        case dg87_exp_task_state_ignore:
          if (new_state != dg87_exp_task_state_wait_for_dep)
						{
            	dg87_exp_task_set_state (task, dg87_exp_task_state_ignored);
							return task->state;
						}
        default:
          /* just to avoid warnings */
          break;
        }

    }

  /* wait lock-variable */
  if (task->state == dg87_exp_task_state_wait_lock)
    {
      ready = FALSE;
      lock_variable = task->lock_variable;
      if (lock_variable == NULL)
        {
          /* task don't use lock-variable */
          ready = TRUE;
        }
      else if (task->lock_action == FALSE)
        {
          /* unlock variable */
          dg87_variable_set_value (lock_variable, "");
          ready = TRUE;
        }
      else if (strcmp (lock_variable->value, "") == 0)
        {
          /* lock variable */
          dg87_variable_set_value (lock_variable, "locked");
          ready = TRUE;
        }
      if (ready == TRUE)
				{
        	dg87_exp_task_set_state (task,
                                 dg87_exp_task_state_wait_for_condition);
					return task->state;
				}
    }

  /* hlate : if an hlate condition is finished then
   * the task must be canceled */
  if (task->state == dg87_exp_task_state_wait_for_condition)
    {
      item = task->condition_list->first;
      while (item != NULL)
        {
          condition = item->object;
          if (condition->type == condition_hlate)
            {
              dg87_exp_condition_get_state (condition);
              if (condition->state == dg87_exp_condition_finished)
                {
                  dg87_exp_task_set_state (task,
                                           dg87_exp_task_state_canceled);
									return task->state;
                }
            }
          item = item->next;
        }
    }


  /* hmax : if an hmax condition is finished then
   * the task is ready to run */
  if (task->state == dg87_exp_task_state_wait_for_condition)
    {
      item = task->condition_list->first;
      while (item != NULL)
        {
          condition = item->object;
          if (condition->type == condition_hmax)
            {
              dg87_exp_condition_get_state (condition);
              if (condition->state == dg87_exp_condition_finished)
                {
                  dg87_exp_task_set_state (task,
                                           dg87_exp_task_state_wait_enabled);
									return task->state;
                }
            }
          item = item->next;
        }
    }


  /* hmin, manual, file, if_equal and if_not_equal conditions */
  /* all of this condition must be finished for the task to be
   * enabled to run */
  if (task->state == dg87_exp_task_state_wait_for_condition)
    {
      /* hmin */
      ready = TRUE;
      item = task->condition_list->first;
      while (item != NULL)
        {
          condition = item->object;
          if (condition->type == condition_hmin)
            {
              dg87_exp_condition_get_state (condition);
              if (condition->state == dg87_exp_condition_waiting)
                {
                  ready = FALSE;
                  break;
                }
            }
          item = item->next;
        }

      /* manual condition */
      if (ready == TRUE)
        {
          item = task->condition_list->first;
          while (item != NULL)
            {
              condition = item->object;
              if (condition->type == condition_manual)
                {
                  dg87_exp_condition_get_state (condition);
                  if (condition->state == dg87_exp_condition_waiting)
                    {
                      ready = FALSE;
                      break;
                    }
                }
              item = item->next;
            }
        }

      /* if_equal and if_not_equal conditions */
      if (ready == TRUE)
        {
          item = task->condition_list->first;
          while (item != NULL)
            {
              condition = item->object;
              if (condition->type == condition_ifequal ||
                  condition->type == condition_ifnequal)
                {
                  dg87_exp_condition_get_state (condition);
                  if (condition->state == dg87_exp_condition_waiting)
                    {
                      ready = FALSE;
                      break;
                    }
                }
              item = item->next;
            }
        }

      /* file condition */
      if (ready == TRUE)
        {

          item = task->condition_list->first;
          while (item != NULL)
            {
              condition = item->object;
              if (condition->type == condition_file)
                {
                  dg87_exp_condition_get_state (condition);
                  if (condition->state == dg87_exp_condition_waiting)
                    {
                      ready = FALSE;
                      break;
                    }
                }
              item = item->next;
            }
        }

      if (ready == TRUE)
				{
        	dg87_exp_task_set_state (task, dg87_exp_task_state_wait_enabled);
					return task->state;
				}
    }
  /* the task must be enabled to be start */
  if (task->state == dg87_exp_task_state_wait_enabled && task->enabled == TRUE)
		{
    	dg87_exp_task_set_state (task, dg87_exp_task_state_running);
			return task->state;
		}
  /* remind start time */
  if (task->state == dg87_exp_task_state_running && task->start_time == 0)
		{
    	task->start_time = time (0);
			DG87_SAVE_NEEDED;
			return task->state;
		}
  /* if task is running, update each jobs */
  if (task->state == dg87_exp_task_state_running)
    {
      /* for each job of the task */
      element = task->job_vector->first;
      while (element != NULL)
        {
          job = element->object;
          element = element->next;
          dg87_exp_job_manage (job);
          switch (dg87_exp_job_get_status (job))
            {
            case dg87_exp_job_status_running:
              n_running++;
              break;
            case dg87_exp_job_status_waiting:
              n_waiting++;
              break;
            case dg87_exp_job_status_terminated:
              n_terminated++;
              break;
            case dg87_exp_job_status_aborted:
              n_aborted++;
              break;
            }
        }
      if (n_running > 0 || n_waiting > 0)
        {
          dg87_exp_task_set_state (task, dg87_exp_task_state_running);
					return task->state;
        }
      else if (n_aborted > 0)
        {
          if (task->on_error_continue == TRUE)
						{
            	dg87_exp_task_set_state (task, dg87_exp_task_state_aborted);
							return task->state;
						}
          else
						{
            	dg87_exp_task_set_state (task,
                                     dg87_exp_task_state_aborted_stopped);
							return task->state;
						}
        }
      else
        {
          /* task is terminated -> wait tempo for
           * cyclical tasks */
          dg87_exp_task_set_state (task, dg87_exp_task_state_wait_tempo);
					return task->state;
        }
    }

  /* temporization for cyclical tasks */
  if (task->state == dg87_exp_task_state_wait_tempo)
    {

      /* remind end time */
      if (task->end_time == 0)
        {
          task->end_time = time (0);
					DG87_SAVE_NEEDED;
          /* send message to dg87s */
          if (task->day->referential_name != NULL)
            dg87_caller__update_task_entry (task);
        }

      /* calculate end time of temporization */
      if (task->from_begin == TRUE)
        end_time = task->start_time + task->interval;
      else
        end_time = task->end_time + task->interval;

      /* if temporization is terminated */
      if (end_time <= time (0))
        {
          if (task->iteration != 0 && task->actual_iteration
              >= task->iteration)
            {
              /* no more iterations : job is terminated */
              dg87_exp_task_set_state (task, dg87_exp_task_state_terminated);
							return task->state;
            }
          else
            {
              /* reinitialize each jobs of the task */
              element = task->job_vector->first;
              while (element != NULL)
                {
                  job = element->object;
                  element = element->next;
                  dg87_exp_job_set_state (job,
                                          dg87_exp_job_state_uninitialized,
                                          -1);
                }
              /* reinitialize each jobs' dependencies 
               * of the task */
              item = task->job_dep_list->first;
              while (item != NULL)
                {
                  job_dep = item->object;
                  item = item->next;
                  dg87_exp_jobdep_set_state (job_dep,
                                             dg87_exp_jobdep_state_waiting);
                }

              /* reinitialize each conditions */
              item = task->condition_list->first;
              while (item != NULL)
                {
                  condition = item->object;
                  item = item->next;
                  dg87_exp_condition_set_state (condition,
                                                dg87_exp_condition_waiting);
                }
              /* reinitialize task */
              task->start_time = 0;
              task->end_time = 0;
              task->actual_iteration++;
              dg87_exp_task_set_state (task,
                                       dg87_exp_task_state_wait_for_condition);
							return task->state;
            }
        }
    }                           /* cyclical */

  return task->state;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_lock (dg87_exp_task * task, dg87_variable * variable,
                        BOOL lock_action)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* decrement variable counter */
  if (task->lock_variable != NULL)
    task->lock_variable->linked_exp_tasks--;
  task->lock_variable = variable;
  task->lock_action = lock_action;
  /* increment variable counter */
  if (task->lock_variable != NULL)
    task->lock_variable->linked_exp_tasks++;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_state (dg87_exp_task * task, dg87_exp_task_state new_state)
{
  dg87_exp_taskdep *taskdep;
  dg87_item *taskdeitem;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* quit if there is nothing to do */
  if (task->state == new_state)
    return TRUE;

	dg87_daemon_log_printf(dg87_daemon_log_debug,
  	"Task %s : %s -> %s", task->name,
          dg87_exp_task_state_string[task->state],
          dg87_exp_task_state_string[new_state]);

  task->state = new_state;

  /* update job dependencies if necessary */
  switch (task->state)
    {
    case dg87_exp_task_state_terminated:
    case dg87_exp_task_state_aborted:
    case dg87_exp_task_state_canceled:
    case dg87_exp_task_state_ignored:
      taskdeitem = task->downstream_taskdep_list->first;
      while (taskdeitem != NULL)
        {
          taskdep = taskdeitem->object;
          dg87_exp_taskdep_evaluate (taskdep);
          taskdeitem = taskdeitem->next;
        }
    default:
      /* Nothing to do */
      break;
    }

  dg87_caller__send_task_state (task);
	DG87_SAVE_NEEDED;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_description (dg87_exp_task * task, char *description)
{
  assert (task != NULL && description != NULL);
  if (task == NULL || description == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_description (description, DG87_TASK_DESC_LENGTH) == FALSE)
    return FALSE;
  dg87_strnzcpy (task->description, description, DG87_TASK_DESC_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_cyclical (dg87_exp_task * task, dg87uint iteration,
                            dg87uint interval, BOOL from_begin)
{

  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->iteration = iteration;
  if (task->iteration == 1)
    {
      /* no cyclical task */
      interval = 0;
      from_begin = TRUE;
    }
  task->interval = interval;
  task->from_begin = from_begin;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_queue (dg87_exp_task * task, dg87_queue * queue)
{
  dg87_list *list;

  assert (task != NULL);
  if (task == NULL)
    {                           /* no test on queue, it can be null ! */
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  /* nothing to do, already done */
  if (task->queue == queue)
    return TRUE;
  /* remove task of previous target */
  if (task->queue != NULL)
    {
      list = task->queue->exp_task_list;
      dg87_list_remove_object (list, task);
    }
  /* add this task to TaskList of queue */
  task->queue = queue;
  /* add task of actual Queue */
  if (task->queue != NULL)
    {
      list = task->queue->exp_task_list;
      dg87_list_add_object (list, task);
    }
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_on_error_continue (dg87_exp_task * task, BOOL value)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->on_error_continue = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_enabled (dg87_exp_task * task, BOOL value)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->enabled = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_set_wait_all_dependencies (dg87_exp_task * task, BOOL wait)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  task->wait_all_dependencies = wait;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
dg87_exp_job *
dg87_exp_task_job_add (dg87_exp_task * task, char *job_name)
{
  dg87_exp_job *job;
  assert (task != NULL && job_name != NULL);
  if (task == NULL || job_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job = dg87_exp_job_new (task, job_name);
  if (job == NULL)
    return NULL;

  /* vector generate error if an other job has the same name */
  /* key must point on job->name */
  if (dg87_vector_add_object (task->job_vector, job, job->name) == FALSE)
    {
      dg87_exp_job_delete (job);
      return NULL;
    }
  return job;
}

/****************************************************************************/
int
dg87_exp_task_job_count (dg87_exp_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (task->job_vector);
}

/****************************************************************************/
dg87_exp_job *
dg87_exp_task_job_get_by_name (dg87_exp_task * task, char *name)
{
  assert (task != NULL && name != NULL);
  if (task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_key (task->job_vector, name);
}

/****************************************************************************/
dg87_exp_job *
dg87_exp_task_job_get_by_index (dg87_exp_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (task->job_vector, index);
}

/****************************************************************************/
BOOL
dg87_exp_task_job_delete (dg87_exp_task * task, dg87_exp_job * job)
{
  dg87_exp_jobdep *job_dep;
  dg87_item *item;
  assert (task != NULL && job != NULL);
  if (task == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* remove all dependencies link to this job */
  item = task->job_dep_list->first;
  while (item != NULL)
    {
      job_dep = item->object;
      if (job_dep->upstream_job == job || job_dep->downstream_job == job)
        {
          dg87_item *temp = item;
          item = item->next;
          dg87_list_remove_item (task->job_dep_list, temp);
          dg87_exp_jobdep_delete (job_dep);
        }
      else
        {
          item = item->next;
        }
    }
  if (dg87_vector_remove_object (task->job_vector, job) == FALSE)
    return FALSE;
  if (dg87_exp_job_delete (job) == FALSE)
    {
      /* add job in vector again */
      /* TODO les dpendances supprimes au dbut sont perdues */
      dg87_vector_add_object (task->job_vector, job, job->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_job_delete_by_name (dg87_exp_task * task, char *name)
{
  dg87_exp_job *job;
  assert (task != NULL && name != NULL);
  if (task == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job = dg87_vector_get_object_by_key (task->job_vector, name);
  if (job == NULL)
    return FALSE;
  return dg87_exp_task_job_delete (task, job);
}

/****************************************************************************/
BOOL
dg87_exp_task_job_delete_by_index (dg87_exp_task * task, int index)
{
  dg87_exp_job *job;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job = dg87_vector_get_object_by_index (task->job_vector, index);
  if (job == NULL)
    return FALSE;
  return dg87_exp_task_job_delete (task, job);
}

/****************************************************************************/
dg87_exp_jobdep *
dg87_exp_task_jobdep_add (dg87_exp_task * task, dg87_exp_job * upstream_job,
                          dg87_exp_job * downstream_job,
                          dg87_exp_jobdep_type type)
{
  dg87_exp_jobdep *job_dep;

  assert (task != NULL && upstream_job != NULL && downstream_job != NULL);
  if (task == NULL || upstream_job == NULL || downstream_job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (upstream_job->task != task || downstream_job->task != task)
    {
      dg87_err_set_error (DG87_ERR_JOBS_NOT_IN_SAME_TASK);
      return NULL;
    }

  /* create and add the job dep */
  if ((job_dep =
       dg87_exp_jobdep_new (upstream_job, downstream_job, type)) == NULL)
    return NULL;
  dg87_list_add_object (task->job_dep_list, job_dep);

  return job_dep;
}

/****************************************************************************/
int
dg87_exp_task_jobdep_count (dg87_exp_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->job_dep_list);
}

/****************************************************************************/
dg87_exp_jobdep *
dg87_exp_task_jobdep_get (dg87_exp_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->job_dep_list, index);
}

/****************************************************************************/
BOOL
dg87_exp_task_jobdep_delete_by_index (dg87_exp_task * task, int index)
{
  dg87_exp_jobdep *job_dep;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  job_dep = dg87_list_remove_object_by_index (task->job_dep_list, index);
  if (job_dep == NULL)
    return FALSE;
  dg87_exp_jobdep_delete (job_dep);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_task_jobdep_delete (dg87_exp_task * task, dg87_exp_jobdep * job_dep)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_remove_object (task->job_dep_list, job_dep) == FALSE)
    return FALSE;
  return dg87_exp_jobdep_delete (job_dep);
}

/****************************************************************************/
int
dg87_exp_task_upstream_taskdep_count (dg87_exp_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->upstream_taskdep_list);
}

/****************************************************************************/
dg87_exp_taskdep *
dg87_exp_task_upstream_taskdep_get (dg87_exp_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->upstream_taskdep_list, index);
}

/****************************************************************************/
int
dg87_exp_task_downstream_taskdep_count (dg87_exp_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->downstream_taskdep_list);
}

/****************************************************************************/
dg87_exp_taskdep *
dg87_exp_task_downstream_taskdep_get (dg87_exp_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->downstream_taskdep_list, index);
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_hmin_add (dg87_exp_task * task, int minutes,
                                  dg87ubyte day_delta)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_hmin_new (minutes, day_delta, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_hmax_add (dg87_exp_task * task, int minutes,
                                  dg87ubyte day_delta)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_hmax_new (minutes, day_delta, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_hlate_add (dg87_exp_task * task, int minutes,
                                   dg87ubyte day_delta)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_hlate_new (minutes, day_delta, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_file_add (dg87_exp_task * task, char *file_path)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_file_new (file_path, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_manual_add (dg87_exp_task * task, char *message)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_manual_new (message, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_ifequal_add (dg87_exp_task * task,
                                     dg87_variable * variable, char *value)
{
  dg87_exp_condition *condition;
  assert (task != NULL && variable != NULL && value != NULL);
  if (task == NULL || variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_ifequal_new (variable, value, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_ifnotequal_add (dg87_exp_task * task,
                                        dg87_variable * variable, char *value)
{
  dg87_exp_condition *condition;
  assert (task != NULL && variable != NULL && value != NULL);
  if (task == NULL || variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  condition = dg87_exp_condition_ifnotequal_new (variable, value, task);
  if (condition == NULL)
    return NULL;
  dg87_list_add_object (task->condition_list, condition);
  return condition;
}

/****************************************************************************/
int
dg87_exp_task_condition_count (dg87_exp_task * task)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_list_count (task->condition_list);
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_get (dg87_exp_task * task, int index)
{
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_list_get_object_by_index (task->condition_list, index);
}

/****************************************************************************/
BOOL
dg87_exp_task_condition_delete (dg87_exp_task * task, int index)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  condition = dg87_list_remove_object_by_index (task->condition_list, index);
  if (condition == NULL)
    return FALSE;
  dg87_exp_condition_delete (condition);
  return TRUE;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_task_condition_find (dg87_exp_task * task,
                              dg87_exp_condition_type type, char *data1,
                              char *data2)
{
  dg87_item *item;
  int minutes = 0;
  dg87_exp_condition *condition;
  assert (task != NULL && data1 != NULL);       /* data2 can be null */
  if (task == NULL || data1 == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* control second argument */
  switch (type)
    {
    case condition_hmin:
    case condition_hmax:
    case condition_hlate:
    case condition_ifequal:
    case condition_ifnequal:
      if (data2 == NULL)
        return NULL;
      break;
    case condition_file:
    case condition_manual:
      if (data2 != NULL)
        return NULL;
      break;
    case condition_error:
      return NULL;
    }
  /* for each condition of the task */
  item = task->condition_list->first;
  while (item != NULL)
    {
      condition = item->object;
      item = item->next;

      if (condition->type != type)
        continue;

      switch (condition->type)
        {

        case condition_hmin:
          minutes = dg87_exp_condition_hour_to_minute (data1);
          if (condition->data.condition_hmin->minutes != minutes)
            continue;
          if (condition->data.condition_hmin->day_delta != atoi (data2))
            continue;
          return condition;

        case condition_hmax:
          minutes = dg87_exp_condition_hour_to_minute (data1);
          if (condition->data.condition_hmax->minutes != minutes)
            continue;
          if (condition->data.condition_hmax->day_delta != atoi (data2))
            continue;
          return condition;

        case condition_hlate:
          minutes = dg87_exp_condition_hour_to_minute (data1);
          if (condition->data.condition_hlate->minutes != minutes)
            continue;
          if (condition->data.condition_hlate->day_delta != atoi (data2))
            continue;
          return condition;

        case condition_file:
          if (strcmp (condition->data.condition_File->path, data1) != 0)
            continue;
          return condition;

        case condition_manual:
          if (strcmp (condition->data.condition_manual->message, data1) != 0)
            continue;
          return condition;

        case condition_ifequal:
          if (strcmp
              (condition->data.condition_ifequal->variable->name, data1) != 0)
            continue;
          if (strcmp (condition->data.condition_ifequal->value, data2) != 0)
            continue;
          return condition;

        case condition_ifnequal:
          if (strcmp
              (condition->data.condition_ifnotequal->variable->name,
               data1) != 0)
            continue;
          if (strcmp (condition->data.condition_ifnotequal->value, data2)
              != 0)
            continue;
          return condition;

        case condition_error:  /* to remove compilators warning */
          return NULL;

        }                       /* end switch */
      item = item->next;
    }                           /* next condition */
  return NULL;
}

/****************************************************************************/
dg87_exp_task_state
dg87_exp_task_ascii_to_state (char *string)
{
  int index = 0;
  assert (string != NULL);
  while (dg87_exp_task_state_string[index] != NULL &&
         strcmp (string, dg87_exp_task_state_string[index]) != 0)
    index++;
  return index;
}

/****************************************************************************/
char *
dg87_exp_task_state_to_ascii (dg87_exp_task_state state)
{
  return dg87_exp_task_state_string[state];
}

/****************************************************************************/
BOOL
dg87_exp_task_find_upstream_tasks (dg87_exp_task * task,
                                   dg87_list * task_list)
{
  dg87_exp_taskdep *taskdep;
  dg87_item *taskdep_item;
  assert (task != NULL && task_list != NULL);
  if (task == NULL || task_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (task_list, task) == FALSE)
    return FALSE;
  taskdep_item = task->upstream_taskdep_list->first;
  while (taskdep_item != NULL)
    {
      taskdep = taskdep_item->object;
      taskdep_item = taskdep_item->next;
      if (dg87_exp_task_find_upstream_tasks (taskdep->upstream_task,
                                             task_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/*****************************************************************************/
BOOL
dg87_exp_task_find_downstream_tasks (dg87_exp_task * task,
                                     dg87_list * task_list)
{
  dg87_exp_taskdep *taskdep;
  dg87_item *taskdep_item;
  assert (task != NULL && task_list != NULL);
  if (task == NULL || task_list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_list_add_object (task_list, task) == FALSE)
    return FALSE;
  taskdep_item = task->downstream_taskdep_list->first;
  while (taskdep_item != NULL)
    {
      taskdep = taskdep_item->object;
      taskdep_item = taskdep_item->next;
      if (dg87_exp_task_find_downstream_tasks (taskdep->downstream_task,
                                               task_list) == FALSE)
        return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_exp_task_check (dg87_exp_task * task, int (*print) (char *))
{
  dg87_element *element;
  dg87_item *item;
  dg87_exp_job *job;

  /* check name */
  if (strlen (task->name) == 0)
    {
      print ("Task : name length is zero !\n");
      return 1;
    }
  /* check day */
  if (task->day == NULL)
    {
      print ("Task : Queue.day is NULL !\n");
      return 1;
    }
  /* check job_vector */
  if (task->job_vector == NULL)
    {
      print ("Task : JobVector is null !\n");
      return 1;
    }
  if (dg87_vector_check (task->job_vector, print) != 0)
    return 1;
  element = task->job_vector->first;
  while (element != NULL)
    {
      job = element->object;
      if (dg87_exp_job_check (job, print) != 0)
        return 1;
      if (job->task != task)
        {
          print ("Task : Job Task pointer incorrect !\n");
          return 1;
        }
      element = element->next;
    }

  /* check Jobdep list */
  if (task->job_dep_list == NULL)
    {
      print ("Task : Jobdep list is null !\n");
      return 1;
    }
  if (dg87_list_check (task->job_dep_list, print) != 0)
    return 1;
  item = task->job_dep_list->first;
  while (item != NULL)
    {
      if (dg87_exp_jobdep_check (item->object, print) != 0)
        return 1;
      item = item->next;
    }

  /* check Taskdep_before_list */
  if (task->upstream_taskdep_list == NULL)
    {
      print ("Task : Taskdep_before_list is null !\n");
      return 1;
    }
  if (dg87_list_check (task->upstream_taskdep_list, print) != 0)
    return 1;

  /* check Taskdep_after_list */
  if (task->downstream_taskdep_list == NULL)
    {
      print ("Task : Taskdep_after_list is null !\n");
      return 1;
    }
  if (dg87_list_check (task->downstream_taskdep_list, print) != 0)
    return 1;

  /* check Queue */
  if (task->queue != NULL)
    {
      if (dg87_list_find_item (task->queue->exp_task_list, task) == NULL)
        {
          print ("Task : task not in Queue.TaskList !\n");
          return 1;
        }
    }

  /* check condition_list */
  if (task->condition_list == NULL)
    {
      print ("Task : ConditionList is null !\n");
      return 1;
    }
  if (dg87_list_check (task->condition_list, print) != 0)
    return 1;
  item = task->condition_list->first;
  while (item != NULL)
    {
      if (dg87_exp_condition_check (item->object, print) != 0)
        return 1;
      item = item->next;
    }

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
