/* file x_task.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_EXP_TASK_H__
#define __DG87_EXP_TASK_H__

#include <time.h>

#include "global.h"
#include "x_job.h"
#include "x_jobdep.h"
#include "x_taskdep.h"
#include "x_condition.h"
#include "queue.h"
#include <libdg87/vector.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>


typedef enum
{
  dg87_exp_task_state_uninitialized,
  dg87_exp_task_state_ignore,
  dg87_exp_task_state_wait_for_dep,
  dg87_exp_task_state_wait_lock,
  dg87_exp_task_state_wait_for_condition,
  dg87_exp_task_state_wait_enabled,
  dg87_exp_task_state_canceled,
  dg87_exp_task_state_running,
  dg87_exp_task_state_ignored,
  dg87_exp_task_state_aborted,
  dg87_exp_task_state_aborted_stopped,
  dg87_exp_task_state_terminated,
  dg87_exp_task_state_deleted,
  dg87_exp_task_state_wait_tempo
    /* don't forget to update :
     *    - 'etm' command 
     *    - dg87_exp_task_state_string 
     *    - dg87l day's entries type
     * when adding a new state ! */
}
dg87_exp_task_state;

struct _dg87_exp_task
{
  char name[DG87_TASK_NAME_LENGTH + 1];
  char description[DG87_TASK_DESC_LENGTH + 1];
  dg87_vector *job_vector;      /* jobs of the task */
  dg87_list *job_dep_list;      /* dependencies between jobs of the task */
  dg87_list *upstream_taskdep_list;
  dg87_list *downstream_taskdep_list;
  dg87_list *condition_list;
  dg87_queue *queue;            /* queue where to execute the TASK */
  dg87_exp_day *day;
  dg87_exp_task_state state;
  BOOL wait_all_dependencies;
  BOOL on_error_continue;
  BOOL enabled;
  time_t start_time;
  time_t end_time;
  /* cyclical properties */
  dg87uint iteration;           /* number of iteration for cyclical tasks */
  dg87uint interval;            /* temporization between two executions */
  BOOL from_begin;              /* temporization starts from begin/end of task */
  dg87uint actual_iteration;    /* number of actual iteration */
  /* lock properties */
  dg87_variable *lock_variable; /* variable to [un]lock */
  BOOL lock_action;             /* TRUE : lock, FALSE unlock */
};

/*
 * dg87_exp_task_new 
 * create a new task and initialize the name property
 */
dg87_exp_task *dg87_exp_task_new (char *name, dg87_exp_day * day);

/*
 * dg87_exp_task_delete
 */
BOOL dg87_exp_task_delete (dg87_exp_task * task);

/*
 * dg87_exp_task_load
 */
BOOL dg87_exp_task_load (dg87_exp_task * task, dg87_storage * storage,
                         dg87_context * context);

/*
 * dg87_exp_task_save
 */
BOOL dg87_exp_task_save (dg87_exp_task * task, dg87_storage * storage);

/*
 * dg87_exp_task_set_description
 * set the description of the task.
 */
BOOL dg87_exp_task_set_description (dg87_exp_task * task, char *description);

/*
 * dg87_exp_task_set_cyclical
 * set cyclical properties of a task.
 * Arguments are :
 * - the number of iterations (unlimited if zero, no cyclical if one)
 * - the number of seconds for temporization between two executions
 * - a boolean value which indicates if the temporization start from begin
 * or from end of the task
 */
BOOL dg87_exp_task_set_cyclical (dg87_exp_task * task, dg87uint iteration,
                                 dg87uint interval, BOOL from_begin);

/*
 * dg87_exp_task_set_lock
 * set lock properties of the task.
 * If variable is NULL, task doesn't use a variable lock.
 * If lock_action is TRUE, variable must lock variable before starting.
 * if lock_action is FALSE, variable must unlock variable just after ending.
 *
 */
BOOL dg87_exp_task_set_lock (dg87_exp_task * task, dg87_variable * variable,
                             BOOL lock_action);


/* 
 * dg87_exp_task_set_state
 */
BOOL dg87_exp_task_set_state (dg87_exp_task * task,
                              dg87_exp_task_state state);

/* 
 * dg87_exp_task_manage
 */
dg87_exp_task_state dg87_exp_task_manage (dg87_exp_task * task);

/*
 * dg87_exp_task_set_queue
 * set the queue where the job of the TASK will be execute.
 */
BOOL dg87_exp_task_set_queue (dg87_exp_task * task, dg87_queue * queue);


/*
 * dg87_exp_task_set_on_error_continue
 */
BOOL dg87_exp_task_set_on_error_continue (dg87_exp_task * task, BOOL value);

/*
 * dg87_exp_task_set_enabled
 */
BOOL dg87_exp_task_set_enabled (dg87_exp_task * task, BOOL value);

/*
 * dg87_exp_task_set_wait_all_dependencies
 */
BOOL dg87_exp_task_set_wait_all_dependencies (dg87_exp_task * task,
                                              BOOL wait);

/*
 * dg87_exp_task_find_upstream_tasks
 * This function adds to the list given in parameter all upstream tasks
 * of a task.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_exp_task_find_upstream_tasks (dg87_exp_task * task,
                                        dg87_list * task_list);

/*
 * dg87_exp_task_find_downstream_tasks
 * This function adds to the list given in parameter all downstream tasks
 * of a task.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_exp_task_find_downstream_tasks (dg87_exp_task * task,
                                          dg87_list * task_list);


/*************************************************************************/
/*  functions for job                                                    */
/*************************************************************************/

/*
 * dg87_exp_task_job_add
 * create a job object, add this object in the JobVector
 * of the task and return a pointer on this object.
 * Return NULL if the Job object can't be create.
 */
dg87_exp_job *dg87_exp_task_job_add (dg87_exp_task * task, char *job_name);

/*
 * dg87_exp_task_job_count
 * return the number of job in the task.
 * return -1 if task is NULL.
 */
int dg87_exp_task_job_count (dg87_exp_task * task);


/*
 * dg87_exp_task_job_get_by_name
 * return a pointer on the job which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_exp_job *dg87_exp_task_job_get_by_name (dg87_exp_task * task,
                                             char *name);

/*
 * dg87_exp_task_job_get_by_index
 * return a pointer on the job at the given index.
 */
dg87_exp_job *dg87_exp_task_job_get_by_index (dg87_exp_task * task,
                                              int index);


/*
 * dg87_exp_task_job_delete_by_name
 * delete the job which has the name given in parameter.
 */
BOOL dg87_exp_task_job_delete_by_name (dg87_exp_task * task, char *name);

/*
 * dg87_exp_task_job_delete_by_index
 * delete the job at the given index.
 */
BOOL dg87_exp_task_job_delete_by_index (dg87_exp_task * task, int index);

/*
 * dg87_exp_task_job_delete
 * delete the job given in parameter 
 */
BOOL dg87_exp_task_job_delete (dg87_exp_task * task, dg87_exp_job * job);

/*************************************************************************/
/*  functions for job dependencies                                        */
/*************************************************************************/

/*
 * dg87_exp_task_jobdep_add
 * add a dependence between two jobs of the TASK. The name of the jobs must
 * be pass in parameters.
 */
dg87_exp_jobdep *dg87_exp_task_jobdep_add (dg87_exp_task * task,
                                           dg87_exp_job * upstream_job,
                                           dg87_exp_job * downstream_job,
                                           dg87_exp_jobdep_type type);

/*
 * dg87_exp_task_jobdep_count
 * return the number of jobdep the task.
 * return -1 if task is NULL.
 */
int dg87_exp_task_jobdep_count (dg87_exp_task * task);

/*
 * dg87_exp_task_jobdep_get
 * return a pointer on the jobdep at the given index.
 */
dg87_exp_jobdep *dg87_exp_task_jobdep_get (dg87_exp_task * task, int index);

/*
 * dg87_exp_task_jobdep_delete_by_index
 * delete the jobdep at the given index.
 */
BOOL dg87_exp_task_jobdep_delete_by_index (dg87_exp_task * task, int index);

/*
 * dg87_exp_task_jobdep_delete
 * delete the jobdep given in parameter 
 */
BOOL dg87_exp_task_jobdep_delete (dg87_exp_task * task,
                                  dg87_exp_jobdep * job_dep);

/*************************************************************************/
/*  functions for taskdep dependencies                                      */
/*************************************************************************/

/*
 * dg87_exp_task_upstream_taskdep_count
 * return the number of taskdep "before" the task.
 * return -1 if task is NULL.
 */
int dg87_exp_task_upstream_taskdep_count (dg87_exp_task * task);

/*
 * dg87_exp_task_upstream_taskdep_get
 * return a pointer on the taskdep "before" at the given index.
 */
dg87_exp_taskdep *dg87_exp_task_upstream_taskdep_get (dg87_exp_task * task,
                                                      int index);

/*
 * dg87_exp_task_downstream_taskdep_count
 * return the number of taskdep "after" the task.
 * return -1 if task is NULL.
 */
int dg87_exp_task_downstream_taskdep_count (dg87_exp_task * task);

/*
 * dg87_exp_task_downstream_taskdep_get
 * return a pointer on the taskdep "after" at the given index.
 */
dg87_exp_taskdep *dg87_exp_task_downstream_taskdep_get (dg87_exp_task * task,
                                                        int index);

/*************************************************************************/
/*  functions for condition list                                               */
/*************************************************************************/

/*
 * dg87_exp_task_condition_hmin_add
 */
dg87_exp_condition *dg87_exp_task_condition_hmin_add (dg87_exp_task * task,
                                                      int minutes,
                                                      dg87ubyte day_delta);

/*
 * dg87_exp_task_condition_hmax_add
 */
dg87_exp_condition *dg87_exp_task_condition_hmax_add (dg87_exp_task * task,
                                                      int minutes,
                                                      dg87ubyte day_delta);

/*
 * dg87_exp_task_condition_hlate_add
 */
dg87_exp_condition *dg87_exp_task_condition_hlate_add (dg87_exp_task * task,
                                                       int minutes,
                                                       dg87ubyte day_delta);

/*
 * dg87_exp_task_condition_file_add
 */
dg87_exp_condition *dg87_exp_task_condition_file_add (dg87_exp_task * task,
                                                      char *file_path);

/*
 * dg87_exp_task_condition_manual_add
 */
dg87_exp_condition *dg87_exp_task_condition_manual_add (dg87_exp_task * task,
                                                        char *message);

/*
 * dg87_exp_task_condition_ifequal_add
 */
dg87_exp_condition *dg87_exp_task_condition_ifequal_add (dg87_exp_task * task,
                                                         dg87_variable *
                                                         variable,
                                                         char *message);
/*
 * dg87_exp_task_condition_ifnotequal_add
 */
dg87_exp_condition *dg87_exp_task_condition_ifnotequal_add (dg87_exp_task *
                                                            task,
                                                            dg87_variable *
                                                            variable,
                                                            char *message);

/*
 * dg87_exp_task_condition_count
 * return the number of condition in the task.
 * return -1 if task is NULL.
 */
int dg87_exp_task_condition_count (dg87_exp_task * task);

/*
 * dg87_exp_task_condition_get
 * return a pointer on the condition at the given index.
 */
dg87_exp_condition *dg87_exp_task_condition_get (dg87_exp_task * task,
                                                 int index);

/*
 * dg87_exp_task_condition_delete
 * delete the condition object at the given index.
 */
BOOL dg87_exp_task_condition_delete (dg87_exp_task * task, int index);

/*
 * dg87_exp_task_condition_find
 */
dg87_exp_condition *dg87_exp_task_condition_find (dg87_exp_task * task,
                                                  dg87_exp_condition_type
                                                  type, char *data1,
                                                  char *data2);
/*
 * dg87_exp_task_state_to_ascii
 * convert a string into a task's state
 */
dg87_exp_task_state dg87_exp_task_ascii_to_state (char *string);

/*
 * dg87_exp_task_state_to_ascii
 * convert a task's state to a string
 */
char *dg87_exp_task_state_to_ascii (dg87_exp_task_state state);

#ifdef DEBUG
/*
 * dg87_exp_task_ckeck
 */
int dg87_exp_task_check (dg87_exp_task * task, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_EXP_TASK_H__*/
/* End of file */
