/* file cmdconnection.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "daemon.h"
#include "command.h"
#include "cmdconnection.h"

/******************************************************************************/
int
cmd_cc (dg87_connection * connection)
{
  dg87_list *connection_list;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  connection_list = connection->server->connection_list;
  dg87_connection_printf (connection, "%i\n", connection_list->count);
  return 0;
}

/******************************************************************************/
int
cmd_ck (dg87_connection * connection)
{
  dg87_command *command;
  dg87_item *item;
  dg87_connection *connection_item = NULL;
  int ip_port;
  char *hostname;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "ck host port");
      return 1;
    }
  /* find the connection */
  ip_port = atoi (command->argv[1]);
  hostname = command->argv[0];
  item = connection->server->connection_list->first;
  while (item != NULL)
    {
      connection_item = item->object;
      if (ip_port == connection_item->ip_port &&
	  strcmp (hostname, connection_item->hostname) == 0)
	break;
      item = item->next;
    }
  if (item == NULL)
    {
      dg87_connection_printf (connection,
			      "No connection with the host '%s' on the port '%i'\n",
			      hostname, ip_port);
      return 1;
    }
  if (connection_item != NULL)
    dg87_connection_close (connection_item);
  return 0;
}

/******************************************************************************/
/* cmd_cl
 * Connection list
 * Usage : cl [options]
 * options :
 * -u : show user name
 */
int
cmd_cl (dg87_connection * connection)
{
  dg87_command *command;
  int optch, iopt;
  char *optarg;
  char *sep, *beg, *end;	/* separator, begin, end */
  dg87_item *item;
  dg87_connection *connection_item;
  BOOL show_user = FALSE;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "cl [options]");
      return 1;
    }
  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
	{
	case 'S':		/* separator */
	  free (sep);
	  sep = strdup (optarg);
	  break;
	case 'B':		/* begin separator */
	  free (beg);
	  beg = strdup (optarg);
	  break;
	case 'E':		/* end separator */
	  free (end);
	  end = strdup (optarg);
	  break;
	case 'u':		/* user name */
	  show_user = TRUE;
	  break;
	default:		/* unknown option */
	  dg87_connection_printf (connection, "Unknown option");
	  free (sep);
	  free (beg);
	  free (end);
	  return 1;
	}
    }


  item = connection->server->connection_list->first;
  while (item != NULL)
    {
      connection_item = item->object;
      item = item->next;
      dg87_connection_printf (connection, "%s%s%s%i",
			      beg, connection_item->hostname,
			      sep, connection_item->ip_port);
      if (show_user == TRUE)
	{
	  dg87_connection_printf (connection, "%s%s", sep,
				  (connection_item->user == NULL) ?
				  "?" : connection_item->user->name);
	}
      dg87_connection_printf (connection, "%s\n", end);
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
int
cmd_user (dg87_connection * connection)
{
  dg87_command *command;
  dg87_user *user;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "user UserName Password");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
			      "Unknown option '%c' for command '%s'\n",
			      command->optv[0], command->function_name);
      return 1;
    }
  /* find the user */
  user = dg87_server_user_get_by_name (connection->server, command->argv[0]);
  if (user == NULL)
    {
      dg87_connection_printf (connection,
			      "Unknown user '%s'.\n", command->argv[0]);
      return 1;
    }
  /* verify password */
  if (dg87_user_verify_password (user, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection, "Incorrect password.\n");
      return 1;
    }

  /* link user and connection */
  if (dg87_connection_set_user (connection, user) == FALSE)
    {
      dg87_connection_printf (connection,
			      "Can't connect user '%s'. %s\n",
			      user->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
int
cmd_password (dg87_connection * connection)
{
  dg87_command *command;
  dg87_user *user;
  char *password;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc < 1 || command->argc > 2)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "password [UserName] NewPassword");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
			      "Unknown option '%c' for command '%s'\n",
			      command->optv[0], command->function_name);
      return 1;
    }
  /* find the user */
  if (command->argc == 1)
    {
      user = connection->user;
      if (user == NULL)
	{
	  dg87_connection_printf (connection, "No user connected.");
	  return 1;
	}
      password = command->argv[0];
    }
  else
    {
      /* check user privilege (to change password of another user) */
      if (connection->user->privileges[dg87_command_user]
	  < dg87_command_access_manage)
	{
	  dg87_connection_printf (connection, "Authorization denied.\n");
	  return 1;
	}

      user = dg87_server_user_get_by_name (connection->server,
					   command->argv[0]);
      if (user == NULL)
	{
	  dg87_connection_printf (connection,
				  "Unknown user '%s'\n", command->argv[0]);
	  return 1;
	}
      password = command->argv[1];
    }
  dg87_user_set_password (user, password);
  return 0;
}

/******************************************************************************/
/* End of file */
