/* file cmduser.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "daemon.h"
#include "command.h"
#include "cmduser.h"

/******************************************************************************/
/* cmd_uc
 * User Count
 * return the number of users.
 * Usage : uc 
 */
/******************************************************************************/
int
cmd_uc (dg87_connection * connection)
{
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  dg87_connection_printf (connection, "%i\n",
			  connection->server->user_vector->count);
  return 0;
}

/******************************************************************************/
/* ua
 * User Add
 * Usage : ua UserName
 */
/******************************************************************************/
int
cmd_ua (dg87_connection * connection)
{
  dg87_user *user;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n", "ua UserName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
			      "Unknown option '%c' for command '%s'\n",
			      command->optv[0], command->function_name);
      return 1;
    }
  /* add the user */
  dg87_err_clear_error ();
  user = dg87_server_user_add (connection->server, command->argv[0]);
  if (user == NULL)
    {
      dg87_connection_printf (connection,
			      "Can't add a user '%s' ! %s\n",
			      command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* usp
 * User Set Privilege
 * Usage : usp UserName Privilege CommandFamily1 CommandFamily2 ...
 */
int
cmd_usp (dg87_connection * connection)
{
  dg87_user *user;
  dg87_command *command;
  dg87_command_privilege privilege;
  dg87_command_family family;
  int i_arg;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "usp UserName Privilege CommandFamily1 CommandFamily2 ...");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
			      "Unknown option '%c' for command '%s'\n",
			      command->optv[0], command->function_name);
      return 1;
    }
  /* find the user */
  dg87_err_clear_error ();
  user = dg87_server_user_get_by_name (connection->server, command->argv[0]);
  if (user == NULL)
    {
      dg87_connection_printf (connection,
			      "Can't find the user '%s' ! %s\n",
			      command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  /* find privilege */
  dg87_err_clear_error ();
  privilege = dg87_command_ascii_to_privilege (command->argv[1]);
  if (privilege == -1)
    {
      dg87_connection_printf (connection,
			      "Incorrect privilege argument '%s' ! %s\n",
			      command->argv[1], dg87_err_get_error_str ());

      return 1;
    }
  /* for each family */
  for (i_arg = 2; i_arg < command->argc; i_arg++)
    {

      /* find family */
      dg87_err_clear_error ();
      family = dg87_command_ascii_to_family (command->argv[i_arg]);
      if (family == -1)
	{
	  dg87_connection_printf (connection,
				  "Incorrect command family argument '%s' ! %s\n",
				  command->argv[i_arg],
				  dg87_err_get_error_str ());
	  return 1;
	}
      /* set privilege */
      if (dg87_user_set_privilege (user, family, privilege) == FALSE)
	{
	  dg87_connection_printf (connection,
				  "Can't set privilege '%s' "
				  "for command family '%s' for user '%s' ! %s\n",
				  command->argv[1],
				  command->argv[i_arg],
				  user->name, dg87_err_get_error_str ());
	  return 1;
	}

    }
  return 0;

}

/******************************************************************************/
/* ui
 * User Information
 * Usage : ui [options] UserName
 */
/******************************************************************************/
int
cmd_ui (dg87_connection * connection)
{
  char *sep, *beg, *end;	/* separator, begin, end */
  /*static char optstring[] = "B:S:E:"; */
  int optch, iopt;
  char *optarg;
  dg87_user *user;
  dg87_command *command;
  int i;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n", "ui UserName");
      return 1;
    }
  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
	{
	case 'S':		/* separator */
	  free (sep);
	  sep = strdup (optarg);
	  break;
	case 'B':		/* begin separator */
	  free (beg);
	  beg = strdup (optarg);
	  break;
	case 'E':		/* end separator */
	  free (end);
	  end = strdup (optarg);
	  break;
	default:		/* unknown option */
	  dg87_connection_printf (connection, "Unknown option");
	  free (sep);
	  free (beg);
	  free (end);
	  return 1;
	}
    }

  /* find user */
  user = dg87_server_user_get_by_name (connection->server, command->argv[0]);
  if (user == NULL)
    {
      dg87_connection_printf (connection,
			      "Can't find the user '%s' ! %s\n",
			      command->argv[0], dg87_err_get_error_str ());
      free (sep);
      free (beg);
      free (end);
      return 1;
    }


  /* User name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
			  beg, "User Name", sep, user->name, end);

  /* privileges */
  for (i = 0; i < DG87_COMMAND_FAM_MAX; i++)
    dg87_connection_printf (connection, "%s%s%s%s%s\n",
			    beg, dg87_command_family_to_ascii (i), sep,
			    dg87_command_privilege_to_ascii (user->
							     privileges[i]),
			    end);
  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* uk
 * User Kill
 * Usage : uk UserName
 */
/******************************************************************************/
int
cmd_uk (dg87_connection * connection)
{
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n", "uk UserName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
			      "Unknown option '%c' for command '%s'\n",
			      command->optv[0], command->function_name);
      return 1;
    }
  /* delete the user */
  dg87_err_clear_error ();
  if (dg87_server_user_delete_by_name (connection->server,
				       command->argv[0]) == FALSE)
    {
      dg87_connection_printf (connection,
			      "Can't delete the user '%s' ! %s\n",
			      command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_ul
 * User List
 * Usage : ul [options]
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first user to display (first index is 0)
 * -Nnumber : number of user to display (no limit if -N < 1)
 * -c : show number of connections
 */
/******************************************************************************/
int
cmd_ul (dg87_connection * connection)
{
  char *sep, *beg, *end;	/* separator, begin, end */
  /* "dtB:S:E:I:N:" */
  int optch, iopt;
  char *optarg;
  int start = 0;		/* index of the first user to display */
  int number = 0;		/* number of users to display */
  int count = 0;		/* number of user actually display */
  dg87_element *element = NULL;
  dg87_user *user;
  dg87_command *command;
  BOOL show_cnx_count = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "ul [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
	{
	case 'S':		/* separator */
	  free (sep);
	  sep = strdup (optarg);
	  break;
	case 'B':		/* begin separator */
	  free (beg);
	  beg = strdup (optarg);
	  break;
	case 'E':		/* end separator */
	  free (end);
	  end = strdup (optarg);
	  break;
	case 'I':		/* index of the first user to display */
	  start = atoi (optarg);
	  break;
	case 'N':		/* number of users to display */
	  number = atoi (optarg);
	  break;
	case 'c':		/* show active connections count */
	  show_cnx_count = TRUE;
	  break;
	default:		/* unknown option */
	  dg87_connection_printf (connection, "Unknown option");
	  free (sep);
	  free (beg);
	  free (end);
	  return 1;
	}
    }

  /* find the first user to display */
  element = NULL;
  if (start > 0)
    element =
      dg87_vector_get_element_by_index (connection->server->user_vector,
					start);
  else
    element = connection->server->user_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of user to display */
      count++;
      if (number > 0 && count > number)
	break;
      user = element->object;
      /* print user name */
      dg87_connection_printf (connection, "%s%s", beg, user->name);
      /* print connection count */
      if (show_cnx_count == TRUE)
	{
	  dg87_connection_printf (connection, "%s%i",
				  sep, user->connection_count);
	}
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* End of file */
