/* file command.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "command.h"

#define WITHOUTARG 0
#define WITHARG 1

/* TODO replace global variable by function arguments */
static char *c;			/* pointer on the current character */
static char *p;			/* pointer on the begining of argument */
static char current_option;	/* last option which was found */

/* local function to skip to end of argument */
static BOOL
skiargument (dg87_command * command)
{
  assert (command != NULL);
  if (*c == '\"')
    {
      /* skip to end of quoted string */
      p = ++c;
      while (*c != '\0')
	{
	  if (*(c - 1) != '\\' && *c == '\"')
	    break;
	  c++;
	}
      if (*c == '\0')
	{
	  command->error = DG87_COMMAND_ERR_STRNOTCLOSED;
	  return FALSE;
	}
    }
  else
    {
      /* go to end of word */
      p = c;
      while (*c != ' ' && *c != '\0')
	c++;
    }
  *c = 0;
  return TRUE;
}

/******************************************************************************/

/* local function to add an argument to the function */
static BOOL
add_argument (dg87_command * command)
{
  char **new_buffer;
  int new_size;
  assert (command != NULL);
  if (skiargument (command) == FALSE)
    return FALSE;
  if (command->argv == NULL)
    {
      new_size = 1;
      new_buffer = dg87_malloc (sizeof (char *), DG87_CHAR);
    }
  else
    {
      new_size = command->argc + 1;
      new_buffer = realloc (command->argv, sizeof (char *) * new_size);
    }
  if (new_buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  command->argc = new_size;
  command->argv = new_buffer;
  command->argv[command->argc - 1] = p;

  return TRUE;
}

/******************************************************************************/
/* local function to add an option to the function */
static BOOL
add_option (dg87_command * command, int need_arg)
{
  char *new_buffer;
  char **new_arg_buffer;
  int new_size;
  assert (command != NULL);
  if (command->optc == 0)
    {
      new_size = 1;
      new_buffer = dg87_malloc (sizeof (char), DG87_CHAR);
      new_arg_buffer = dg87_malloc (sizeof (char *), DG87_CHAR);
    }
  else
    {
      new_size = command->optc + 1;
      new_buffer = realloc (command->optv, sizeof (char) * new_size);
      new_arg_buffer = realloc (command->optargv, sizeof (char *) * new_size);
    }
  if (new_buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  if (new_arg_buffer == NULL)
    {
      dg87_free (new_buffer, DG87_CHAR);
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  command->optc = new_size;
  command->optv = new_buffer;
  command->optargv = new_arg_buffer;
  command->optv[command->optc - 1] = current_option;
  if (need_arg == WITHARG)
    {
      if (skiargument (command) == FALSE)
	return FALSE;
      command->optargv[command->optc - 1] = p;
    }
  else
    {
      command->optargv[command->optc - 1] = NULL;
    }
  return TRUE;
}


/******************************************************************************/
static void
dg87_command_split_string (dg87_command * command)
{
  char *co;			/* pointer on optstr */
  int option = 1;		/* zero if option disabled by -- */
  char *pend;			/* pointer on the end of the string '\0' */

  assert (command != NULL);

  /* replace separators by space */
  for (c = command->buffer; *c != '\0'; c++)
    if (*c == '\t')
      *c = ' ';
  pend = c;

  /* for each characters */
  c = command->buffer;
  while (c < pend)
    {

      /* skip separators */
      while (*c == ' ')
	c++;
      if (*c == '\0')
	break;

      /* quoted string as function argument */
      if (*c == '\"')
	{
	  if (add_argument (command) == FALSE)
	    return;
	  c++;
	  continue;
	}

      /* dash dot */
      if (*c == '-' && option == 1)
	{
	  /* dash dot a normal argument */
	  c++;
	  if (*c == ' ' || *c == ':' || *c == '\0')
	    {
	      c--;
	      if (add_argument (command) == FALSE)
		return;
	      c++;
	      continue;
	    }
	  /* detect -- as end of option */
	  if (*c == '-')
	    {
	      option = 0;
	      c++;
	      continue;
	    }
	  /* interpret one or more options */
	  while (*c != ' ' && *c != '\0')
	    {

	      current_option = *c;
	      /* find the option */
	      co = command->optstr;
	      while (*co != '\0' && *co != *c)
		co++;
	      if (*co == '\0')
		{
		  command->error = DG87_COMMAND_ERR_UNKNOWNOPTION;
		  command->error_option = current_option;
		  return;
		}

	      /* if the option need an argument */
	      if (*(co + 1) == ':')
		{
		  c++;
		  /* skip separators before argument */
		  while (*c == ' ' && *c != '\0')
		    c++;
		  if (*c == '\0')
		    {
		      command->error = DG87_COMMAND_ERR_NOARGUMENT;
		      command->error_option = current_option;
		      return;
		    }
		  if (add_option (command, WITHARG) == FALSE)
		    return;
		  /* stop to search other options */
		  break;
		}
	      else
		{
		  c++;
		  if (add_option (command, WITHOUTARG) == FALSE)
		    return;
		}
	    }
	  c++;
	  continue;
	}

      /* skip to end of word argument */
      if (add_argument (command) == FALSE)
	return;
      c++;
    }
}

/******************************************************************************/
dg87_command *
dg87_command_new (char *command_string)
{
  dg87_command *command;
  char *arg;
  assert (command_string != NULL);
  if (command_string == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((command = dg87_malloc (sizeof (dg87_command), DG87_COMMAND)) == NULL)
    goto alloc_error;

  /* initialize structure */
  command->argc = 0;
  command->argv = NULL;
  command->optc = 0;
  command->optv = NULL;
  command->optargv = NULL;
  command->error = 0;
  command->error_option = '\0';
  command->optstr = NULL;

  /* extract function name */
  arg = command_string;
  while (*arg != '\0' && *arg != '\t' && *arg != ' ')
    arg++;
  if (*arg == '\0')
    {
      /* function whithout arguments */
      command->buffer = dg87_strdup ("", DG87_CHAR);
    }
  else
    {
      /* function with arguments */
      *arg = '\0';
      command->buffer = dg87_strdup (arg + 1, DG87_CHAR);
    }
  if (command->buffer == NULL)
    goto alloc_error;
  if ((command->function_name =
       dg87_strdup (command_string, DG87_CHAR)) == NULL)
    goto alloc_error;
  return command;

alloc_error:
  if (command != NULL)
    dg87_command_delete (command);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_command_interpret_options (dg87_command * command,
				char *option_stringing)
{
  assert (command != NULL && option_stringing != NULL);
  if (command == NULL || option_stringing == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  command->optstr = dg87_strdup (option_stringing, DG87_CHAR);
  if (command->optstr == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }

  dg87_command_split_string (command);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_command_delete (dg87_command * command)
{
  assert (command != NULL);
  if (command == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (command->buffer != NULL)
    dg87_free (command->buffer, DG87_CHAR);
  if (command->function_name != NULL)
    dg87_free (command->function_name, DG87_CHAR);
  if (command->optstr != NULL)
    dg87_free (command->optstr, DG87_CHAR);
  if (command->argv != NULL)
    dg87_free (command->argv, DG87_CHAR);
  if (command->optv != NULL)
    dg87_free (command->optv, DG87_CHAR);
  if (command->optargv != NULL)
    dg87_free (command->optargv, DG87_CHAR);
  dg87_free (command, DG87_COMMAND);
  return TRUE;
}

/******************************************************************************/
dg87_command_family
dg87_command_ascii_to_family (char *string)
{
  assert (string != NULL);
  if (string == NULL)
    return -1;

  if (strcasecmp (string, "generic") == 0)
    return dg87_command_generic;
  if (strcasecmp (string, "communication") == 0)
    return dg87_command_communication;
  if (strcasecmp (string, "user") == 0)
    return dg87_command_user;
  if (strcasecmp (string, "engine") == 0)
    return dg87_command_engine;
  if (strcasecmp (string, "target") == 0)
    return dg87_command_target;
  if (strcasecmp (string, "referential") == 0)
    return dg87_command_ref;
  if (strcasecmp (string, "job") == 0)
    return dg87_command_job;
  if (strcasecmp (string, "day") == 0)
    return dg87_command_day;
  if (strcasecmp (string, "log") == 0)
    return dg87_command_log;
  if (strcasecmp (string, "variable") == 0)
    return dg87_command_variable;
  if (strcasecmp (string, "stat") == 0)
    return dg87_command_stat;
  return -1;
}

/******************************************************************************/
char *
dg87_command_family_to_ascii (dg87_command_family family)
{
  switch (family)
    {
    case dg87_command_generic:
      return "generic";
    case dg87_command_communication:
      return "communication";
    case dg87_command_user:
      return "user";
    case dg87_command_engine:
      return "engine";
    case dg87_command_target:
      return "target";
    case dg87_command_ref:
      return "referential";
    case dg87_command_day:
      return "day";
    case dg87_command_job:
      return "job";
    case dg87_command_variable:
      return "variable";
    case dg87_command_log:
      return "log";
    case dg87_command_stat:
      return "stat";
    }
  return "?";
}

/******************************************************************************/
dg87_command_privilege
dg87_command_ascii_to_privilege (char *string)
{
  assert (string != NULL);
  if (string == NULL)
    return -1;

  if (strcasecmp (string, "none") == 0)
    return dg87_command_access_none;
  if (strcasecmp (string, "consult") == 0)
    return dg87_command_access_consult;
  if (strcasecmp (string, "manage") == 0)
    return dg87_command_access_manage;
  if (strcasecmp (string, "admin") == 0)
    return dg87_command_access_admin;
  return -1;
}

/******************************************************************************/
char *
dg87_command_privilege_to_ascii (dg87_command_privilege privilege)
{

  switch (privilege)
    {
    case dg87_command_access_none:
      return "none";
    case dg87_command_access_consult:
      return "consult";
    case dg87_command_access_manage:
      return "manage";
    case dg87_command_access_admin:
      return "admin";
    }
  return "?";
}

/******************************************************************************/
/* End of file */
