/* file conf.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "conf.h"
#include "daemon.h"

#define DG87_CONF_MAXLINELENGTH 256

/******************************************************************************/
dg87_conf *
dg87_conf_new (char *config_file, dg87_conf_field * field_array)
{
  dg87_conf *conf;
  assert (config_file != NULL && field_array != NULL);
  if (config_file == NULL || field_array == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (strlen (config_file) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((conf = dg87_malloc (sizeof (dg87_conf), DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((conf->config_file = dg87_strdup (config_file, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  conf->field_array = field_array;
  return conf;

alloc_error:
  if (conf != NULL)
    dg87_conf_delete (conf);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_conf_delete (dg87_conf * conf)
{
  assert (conf != NULL);
  if (conf == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (conf->config_file != NULL)
    dg87_free (conf->config_file, DG87_OTHER);
  dg87_free (conf, DG87_OTHER);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_conf_read (dg87_conf * conf)
{
  FILE *fp;
  char buffer[DG87_CONF_MAXLINELENGTH + 1];
  char *value;
  int i;
  dg87_conf_field *field;

  assert (conf != NULL);
  if (conf == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* initialize each field value */
  field = conf->field_array;
  while (field->name != NULL)
    {
      if (field->value != NULL)
	dg87_free (field->value, DG87_CHAR);
      field->value = NULL;
      /* copy initial value */
      if (field->default_value != NULL)
	{
	  if ((field->value = dg87_strdup (field->default_value,
					   DG87_CHAR)) == NULL)
	    {
	      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	      return FALSE;
	    }
	}
      field++;			/* next field */
    }

  /* open configuration file */
  fp = fopen (conf->config_file, "r");
  if (fp == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	gettext ("Can't open configuration file '%s' !"), 
											conf->config_file);
      dg87_err_set_error (DG87_ERR_CAN_NOT_OPEN_CONFIG_FILE);
      return FALSE;
    }

  /* read each lines */
  while (fgets (buffer, DG87_CONF_MAXLINELENGTH, fp) != NULL)
    {


      /* SECURITE !!! */
      buffer[DG87_CONF_MAXLINELENGTH] = '\0';

      /* skip comment */
      if (buffer[0] == '#')
	continue;

      /* remove \n at end of line */
      if ((value = strrchr (buffer, '\n')) != NULL)
	*value = '\0';

      /* remove trailling spaces */
      for (i = strlen (buffer) - 1; i > 0; i--)
	{
	  if (buffer[i] != '\t' && buffer[i] != ' ')
	    break;
	  buffer[i] = '\0';
	}

      /* ignore empty lines */
      if (strlen (buffer) == 0)
	continue;

      /* search separator */
      if ((value = strchr (buffer, '=')) == NULL)
	{
	  fclose (fp);
		dg87_daemon_log_printf(dg87_daemon_log_error,
	  	"Can't find separator '=' in line '%s' !", buffer);
	  dg87_err_set_error (DG87_ERR_CONF_SYNTAX_ERROR);
	  return FALSE;
	}

      /* replace separator by '\0' */
      *(value++) = '\0';

      /* find field in array */
      field = conf->field_array;
      while (field->name != NULL)
	{
	  /* compare field name */
	  if (strcmp (buffer, field->name) != 0)
	    {
	      field++;		/* next field */
	      continue;
	    }
	  /* copy value */
	  if (field->value != NULL)
	    dg87_free (field->value, DG87_CHAR);
	  if ((field->value = dg87_strdup (value, DG87_CHAR)) == NULL)
	    {
	      fclose (fp);
	      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	      return FALSE;
	    }
	  break;
	}
      /* unknown field name */
      if (field->name == NULL)
	{
	  fclose (fp);
		dg87_daemon_log_printf(dg87_daemon_log_error,
	  	"Unknown field name '%s' !", buffer);
	  dg87_err_set_error (DG87_ERR_CONF_UNKNOWN_FIELD_NAME);
	  return FALSE;
	}
    }
  fclose (fp);
  return TRUE;
}

/******************************************************************************/
char *
dg87_conf_get_field_value (dg87_conf * conf, char *field_name)
{
  dg87_conf_field *field;
  assert (conf != NULL && field_name != NULL);
  if (conf == NULL || field_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* find field in array */
  field = conf->field_array;
  while (field->name != NULL)
    {
      /* compare field name */
      if (strcmp (field_name, field->name) == 0)
	return field->value;
      field++;
    }
  dg87_err_set_error (DG87_ERR_CONF_UNKNOWN_FIELD_NAME);
  return NULL;
}

/******************************************************************************/
/* End of file */
