/* file connection.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <sys/time.h>		/* prevent error for arm architecture */
#include <assert.h>

#include "daemon.h"
#include "connection.h"

/* local functions prototypes */
static void _dg87_connection_shutdown_socket (dg87_connection * connection);

/******************************************************************************/
dg87_connection *
dg87_connection_in_new (int fd, char *hostname, int ip_port,
			dg87_server * server)
{
  dg87_connection *connection;
  assert (hostname != NULL && server != NULL);
  if (hostname == NULL || server == NULL || fd == -1)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((connection = dg87_malloc (sizeof (dg87_connection),
				 DG87_CONNECTION)) == NULL)
    goto alloc_error;
  if ((connection->hostname = dg87_strdup (hostname, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  connection->ip_port = ip_port;
  connection->fd = fd;
  connection->buffer = NULL;
  connection->buffer_size = 0;
  connection->message_size = 0;
  connection->type = dg87_connection_type_in;
  connection->state = dg87_connection_state_wait_message;
  connection->server = server;
  connection->command = NULL;
  connection->close = FALSE;
  connection->user = NULL;
  return connection;

alloc_error:
  if (connection != NULL)
    dg87_connection_delete (connection);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
dg87_connection *
dg87_connection_out_new (char *hostname, int ip_port)
{
  dg87_connection *connection;
  assert (hostname != NULL);
  if (hostname == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((connection = dg87_malloc (sizeof (dg87_connection),
				 DG87_CONNECTION)) == NULL)
    goto alloc_error;
  if ((connection->hostname = dg87_strdup (hostname, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  connection->ip_port = ip_port;
  connection->fd = -1;
  connection->buffer = NULL;
  connection->buffer_size = 0;
  connection->message_size = 0;
  connection->type = dg87_connection_type_out;
  connection->state = dg87_connection_state_reading_message;
  connection->server = NULL;
  connection->command = NULL;
  connection->close = FALSE;
  connection->user = NULL;
  return connection;

alloc_error:
  if (connection != NULL)
    dg87_connection_delete (connection);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_connection_delete (dg87_connection * connection)
{
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (connection->user != NULL)
    {
      dg87_user_update_connection_count (connection->user, -1);
      connection->user = NULL;
    }
  if (connection->command != NULL)
    dg87_command_delete (connection->command);
  if (connection->buffer != NULL)
    dg87_free (connection->buffer, DG87_CHAR);
  if (connection->hostname != NULL)
    dg87_free (connection->hostname, DG87_CHAR);
  _dg87_connection_shutdown_socket (connection);
  dg87_free (connection, DG87_CONNECTION);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_connection_buffer_concat (dg87_connection * connection, char *string)
{
  char *new_buffer;
  int new_size;
  int string_size;

  assert (connection != NULL && string != NULL);
  if (connection == NULL || string == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  string_size = strlen (string);
  /* initialize the buffer */
  if (connection->buffer == NULL)
    {
      new_size = DG87_CONNECTION_BUFFERSIZE;
      new_buffer = dg87_malloc (new_size, DG87_CHAR);
      if (new_buffer == NULL)
	{
	  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	  return FALSE;
	}
      connection->buffer = new_buffer;
      connection->buffer_size = new_size;
      connection->message_size = 0;
      connection->buffer[0] = 0;
    }
  /* if the buffer is to small to concat the string */
  while (connection->message_size + string_size >= connection->buffer_size)
    {
      /* then expand the buffer */
      new_size = connection->buffer_size + DG87_CONNECTION_BUFFERSIZE;
      new_buffer = realloc (connection->buffer, new_size);
      if (new_buffer == NULL)
	{
	  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	  return FALSE;
	}
      connection->buffer = new_buffer;
      connection->buffer_size = new_size;
    }
  /* append the string to the buffer */
  memcpy (connection->buffer + connection->message_size,
	  string, string_size + 1);
  connection->message_size += string_size;

  return TRUE;
}

/******************************************************************************/
static void
_dg87_connection_shutdown_socket (dg87_connection * connection)
{
  assert (connection != NULL);
  if (connection->fd != -1)
    {
      /* stop receiving and sending data for this socket */
      if (shutdown (connection->fd, 2) == -1)
				dg87_daemon_log_printf(dg87_daemon_log_error,
					"Error while socket shutdown !");
      close (connection->fd);
      connection->fd = -1;
      connection->state = dg87_connection_state_connection_close;
    }
}

/******************************************************************************/
char *
dg87_connection_read (dg87_connection * connection)
{
  char *buffer = NULL;
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (connection->state != dg87_connection_state_message_available)
    {
      dg87_err_set_error (DG87_ERR_CNX_MESSAGE_NOT_AVAILABLE);
      return NULL;
    }
  buffer = connection->buffer;
  connection->buffer = NULL;
  connection->buffer_size = 0;
  connection->message_size = 0;
  connection->state = dg87_connection_state_wait_message;
  return buffer;
}

/******************************************************************************/
BOOL
dg87_connection_close (dg87_connection * connection)
{
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  connection->close = TRUE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_connection_read_command (dg87_connection * connection)
{
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (connection->state != dg87_connection_state_message_available)
    {
      dg87_err_set_error (DG87_ERR_CNX_MESSAGE_NOT_AVAILABLE);
      return FALSE;
    }

  if (connection->command != NULL)
    {
      dg87_command_delete (connection->command);
      connection->command = NULL;
    }
  connection->command = dg87_command_new (connection->buffer);
  if (connection->command == NULL)
    return FALSE;

  if (connection->buffer != NULL)
    {
      dg87_free (connection->buffer, DG87_CHAR);
      connection->buffer = NULL;
    }
  connection->buffer_size = 0;
  connection->message_size = 0;
  connection->state = dg87_connection_state_wait_message;
  return TRUE;
}

/******************************************************************************/
BOOL dg87_connection_printf
  (dg87_connection * connection, char *strformat, ...)
{
  char buffer[60];
  char c[2];
  va_list ap;
  int ival;
  long lval;
  char *string;

  assert (connection != NULL && strformat != NULL);

  if (connection == NULL || strformat == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (connection->state != dg87_connection_state_wait_message)
    {
      dg87_err_set_error (DG87_ERR_CNX_NOT_WAITING_MESSAGE);
      return FALSE;
    }

  va_start (ap, strformat);	/* ap points on the first optional argument */
  while (*strformat)
    {
      if (*strformat == '\n')
	{
	  /* print CR LF */
	  dg87_connection_buffer_concat (connection, 
			  DG87_CONNECTION_IN_TERMINATOR);
	}
      else if (*strformat != '%')
	{
	  /* not a format character (%), print it */
	  c[0] = *strformat;
	  c[1] = 0;
	  dg87_connection_buffer_concat (connection, c);
	}
      /* interpret character after % */
      else
	switch (*++strformat)
	  {
	  case 's':		/* replace '%s' by a string */
	    string = va_arg (ap, char *);
	    if (string != NULL)
	      dg87_connection_buffer_concat (connection, string);
	    break;
	  case 'c':		/* replace '%c' by a character */
	    ival = va_arg (ap, int);
	    sprintf (buffer, "%c", ival);
	    dg87_connection_buffer_concat (connection, buffer);
	    break;
	  case 'i':		/* replace '%i' by a integer */
	    ival = va_arg (ap, int);
	    sprintf (buffer, "%i", ival);
	    dg87_connection_buffer_concat (connection, buffer);
	    break;
	  case 'l':		/* replace '%l' by a long */
	    lval = va_arg (ap, long);
	    sprintf (buffer, "%li", lval);
	    dg87_connection_buffer_concat (connection, buffer);
	    break;
	  default:
	    c[0] = *strformat;
	    c[1] = 0;
	    dg87_connection_buffer_concat (connection, c);
	  }			/* end case */
      strformat++;		/* next character */

    }				/* end while */
  va_end (ap);

  return TRUE;
}

/******************************************************************************/
BOOL
dg87_connection_exit (dg87_connection * connection, int exit_code)
{
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (connection->state != dg87_connection_state_wait_message)
    {
      dg87_err_set_error (DG87_ERR_CNX_NOT_WAITING_MESSAGE);
      return FALSE;
    }
  if (exit_code == 0)
    dg87_connection_printf (connection, "OK");
  else
    dg87_connection_printf (connection, "ERROR:%i", exit_code);
  return dg87_connection_send (connection);
}

/******************************************************************************/
BOOL
dg87_connection_send (dg87_connection * connection)
{
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (connection->state != dg87_connection_state_wait_message)
    {
      dg87_err_set_error (DG87_ERR_CNX_NOT_WAITING_MESSAGE);
      return FALSE;
    }
  /* add the terminator to the message */
  if (connection->type == dg87_connection_type_in)
    dg87_connection_buffer_concat (connection,
				   DG87_CONNECTION_OUT_TERMINATOR);
  else
    dg87_connection_buffer_concat (connection, DG87_CONNECTION_IN_TERMINATOR);
  connection->state = dg87_connection_state_sending_message;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_connection_get_message (dg87_connection * connection)
{
  /* private function : don't test parameters */
  struct timeval tv;
  fd_set readfds;
  int nread, ntoread;
  int delta;
  char *new_buffer;
  int new_size;
  char *terminator;
  char *string;

  assert (connection != NULL);
  /* return if there is nothing to read */
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  FD_ZERO (&readfds);
  FD_SET (connection->fd, &readfds);
  if (select (connection->fd + 1, &readfds, NULL, NULL, &tv) == 0)
    {
      /* close connection if necessary */
      if (connection->close == TRUE)
	{
	  _dg87_connection_shutdown_socket (connection);
	  return TRUE;
	}
      return FALSE;
    }

  /* get the number of bytes to read */
  ioctl (connection->fd, FIONREAD, &ntoread);
  if (ntoread == 0)
    {
      /* connection close by remote */
      _dg87_connection_shutdown_socket (connection);
      if (connection->buffer != NULL)
	{
	  dg87_free (connection->buffer, DG87_CHAR);
	  connection->buffer = NULL;
	}
      return FALSE;
    }

  /* allocate enough memory for the buffer */
  if (connection->buffer == NULL)
    {
      delta = 0;
      connection->buffer_size = ntoread;
      connection->buffer = dg87_malloc (connection->buffer_size + 1,
					DG87_CHAR);
      if (connection->buffer == NULL)
	return FALSE;
      memset (connection->buffer, 0, connection->buffer_size + 1);
    }
  else
    {
      delta = connection->buffer_size;
      new_size = connection->buffer_size + ntoread;
      new_buffer = realloc (connection->buffer, new_size + 1);
      if (new_buffer == NULL)
	return FALSE;
      /* if realloc fails, connection->buffer point on the 
       * previous buffer */
      connection->buffer = new_buffer;
      connection->buffer_size = new_size;
    }

  /* the start reading position is buffer + delta */
  nread = read (connection->fd, connection->buffer + delta, ntoread);
  if (nread != ntoread)
    {
      /* connection is closed */
      _dg87_connection_shutdown_socket (connection);
      if (connection->buffer != NULL)
	{
	  dg87_free (connection->buffer, DG87_CHAR);
	  connection->buffer = NULL;
	}
      return FALSE;
    }
  *(connection->buffer + connection->buffer_size) = '\0';

  /* detect terminator */
  if (connection->buffer_size < 1)
    return TRUE;

  /* the type of connection determine the terminator */
  if (connection->type == dg87_connection_type_in)
    terminator = DG87_CONNECTION_IN_TERMINATOR;
  else
    terminator = DG87_CONNECTION_OUT_TERMINATOR;

  /* try to find the terminator at the end of the buffer */
  string = connection->buffer + connection->buffer_size - strlen (terminator);
  if (strcmp (string, terminator) == 0)
    {
      connection->buffer[connection->buffer_size
			 - strlen (terminator)] = '\0';
      connection->state = dg87_connection_state_message_available;
    }
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_connection_send_message (dg87_connection * connection)
{
  /* private function : don't test parameters */
  struct timeval tv;
  fd_set writefds;
  ssize_t nwrite;

  assert (connection != NULL);
  /* return if fd is not ready to be writing */
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  FD_ZERO (&writefds);
  FD_SET (connection->fd, &writefds);
  if (select (connection->fd + 1, NULL, &writefds, NULL, &tv) == 0)
    return FALSE;

  /* get the size of the output buffer */
  /*
     size = sizeof(size_t);
     getsockopt(connection->fd, SOL_SOCKET, SO_SNDBUF, &snd_buf_size, &size);
     printf("output buffer size : %i\n", snd_buf_size);
     size = sizeof(size_t);
     getsockopt(connection->fd, SOL_SOCKET, SO_RCVBUF, &snd_buf_size, &size);
     printf("input buffer size : %i\n", snd_buf_size);
   */

  /* send buffer */
  /* TODO timeout if remote can't receive all the buffer ? 
   * we can just fill the output buffer and send the remaining
   * in next updates */
  nwrite = write (connection->fd, connection->buffer,
		  connection->message_size);
  if (nwrite != connection->message_size)
    {
      connection->state = dg87_connection_state_error;
      return FALSE;
    }
  dg87_free (connection->buffer, DG87_CHAR);
  connection->buffer = NULL;
  if (connection->close == TRUE)
    {
      _dg87_connection_shutdown_socket (connection);
      return TRUE;
    }
  connection->state = dg87_connection_state_reading_message;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_connection_update (dg87_connection * connection)
{
  struct sockaddr_in address;
  struct hostent *hostnameent;
  int sockfd, length, result;

  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* open the connection if necessary */
  if (connection->fd == -1)
    {
      /* convert hostname to IP address */
      hostnameent = gethostbyname (connection->hostname);
      if (!hostnameent)
	{
	  connection->state = dg87_connection_state_error;
	  dg87_err_set_error (DG87_ERR_CNX_CAN_NOT_GET_HOST_BY_NAME);
	  return FALSE;
	}
      /* create the socket */
      if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
	{
	  dg87_err_set_error (DG87_ERR_CNX_SOCKET);
	  return FALSE;
	}
      /* connect to the remote */
      address.sin_family = AF_INET;
      address.sin_addr = *(struct in_addr *) *hostnameent->h_addr_list;
      address.sin_port = htons (connection->ip_port);
      length = sizeof (address);
      result = connect (sockfd, (struct sockaddr *) &address, length);
      if (result == -1)
	{
	  _dg87_connection_shutdown_socket (connection);
	  connection->state = dg87_connection_state_error;
	  dg87_err_set_error (DG87_ERR_CNX_CAN_NOT_CONNECT);
	  return FALSE;
	}
      connection->fd = sockfd;
    }
  /* read or write message */
  switch (connection->state)
    {
    case dg87_connection_state_reading_message:
      return dg87_connection_get_message (connection);
    case dg87_connection_state_sending_message:
      return dg87_connection_send_message (connection);
    default:
      /* nothing to do */
      break;
    }
  return FALSE;
}

/******************************************************************************/
BOOL
dg87_connection_set_user (dg87_connection * connection, dg87_user * user)
{
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  /* remove previous user */
  if (connection->user != NULL)
    dg87_user_update_connection_count (connection->user, -1);

  /* add current user */
  connection->user = user;
  if (connection->user != NULL)
    dg87_user_update_connection_count (connection->user, 1);

  return TRUE;
}

/******************************************************************************/
/* End of file */
