/* file connection.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_CONNECTION_H__
#define __DG87_CONNECTION_H__

#include "global.h"
#include "server.h"
#include "command.h"

#define DG87_CONNECTION_BUFFERSIZE 1024
#define DG87_CONNECTION_IN_TERMINATOR "\r\n"
#define DG87_CONNECTION_OUT_TERMINATOR "\r\nDiogene87>"

/*
 * dg87_connection_type
 * The type of a connection can be "in" or "out".
 * An "in" connection is established by a remote client.
 * 	Initially the program read a request message from the remote,
 *	and then the program send a response message to the remote.
 * An "out" connection is established by the program.
 * 	Initially the program send a message to the remote,
 * 	and after the program read a response message from the remote. 
 */
typedef enum dg87_connection_type_enum
{
  dg87_connection_type_in,
  dg87_connection_type_out
}
dg87_connection_type;

/*
 * dg87_connection_state
 */
typedef enum dg87_connection_state_enum
{
  /* reading_message : connection is reading a message from the remote */
  dg87_connection_state_reading_message,

  /* message_available : a message from the remote is available and can 
   * be read with a call to dg87_connection_get_message */
  dg87_connection_state_message_available,

  /* wait_message : connection is waiting from a message to send to the
   * remote. The message can be create by multiple call to
   * dg87_connection_write and dg87_connection_printf.
   * The message will be send by a call to dg87_connection_post_message */
  dg87_connection_state_wait_message,

  /* sending_message : connection is sending the message to the remote */
  dg87_connection_state_sending_message,

  /* connection_close : the connection has been close by the remote */
  dg87_connection_state_connection_close,

  /* error : the connection can't be opened */
  dg87_connection_state_error
}
dg87_connection_state;

/*
 * dg87_connection object
 */
struct _dg87_connection
{
  dg87_server *server;
  dg87_connection_type type;
  dg87_connection_state state;
  int fd;
  char *hostname;
  int ip_port;
  char *buffer;
  int buffer_size;
  int message_size;
  dg87_command *command;
  BOOL close;
  dg87_user *user;		/* user connected (can be null) */
};

/*
 * dg87_connection_in_new
 * create a connection object which type is dg87_connection_type_in
 */
dg87_connection *dg87_connection_in_new (int fd, char *hostname, int ip_port,
					 dg87_server * server);

/*
 * dg87_connection_out_new
 * create a connection object which type is dg87_connection_type_out
 */
dg87_connection *dg87_connection_out_new (char *hostname, int ip_port);

/*
 * dg87_connection_delete
 * close the connection and delete the memory for this object
 */
BOOL dg87_connection_delete (dg87_connection * connection);

/*
 * dg87_connection_read
 * read a message from the remote.
 * the function return NULL if the message is not available
 */
char *dg87_connection_read (dg87_connection * connection);

/*
 * dg87_connection_close
 */
BOOL dg87_connection_close (dg87_connection * connection);

/*
 * dg87_connection_read_command
 */
BOOL dg87_connection_read_command (dg87_connection * connection);

/*
 * dg87_connection_printf
 * append a string to the message which will be send to the remote.
 */
BOOL dg87_connection_printf
  (dg87_connection * connection, char *strformat, ...);

/* 
 * dg87_connection_send
 * transmit the message to the remote.
 */
BOOL dg87_connection_send (dg87_connection * connection);

/*
 * dg87_connection_exit
 * send "OK" to the connection if exit_code = 0 or "ERROR:exit_code" 
 */
BOOL dg87_connection_exit (dg87_connection * connection, int exit_code);

/*
 * dg87_connection_update
 * transmit a message to the remote or receive a message from the remote.
 */
BOOL dg87_connection_update (dg87_connection * connection);

/* 
 * dg87_connection_set_user
 * associate a user to identify who is connected.
 * user can be NULL if user isn't known.
 */
BOOL dg87_connection_set_user (dg87_connection * connection,
			       dg87_user * user);

#endif /*__DG87_CONNECTION_H__*/
