/* file daemon.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "global.h"
#include "daemon.h"
#include <stdio.h>
#include <stdarg.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <sys/stat.h>		/*umask */
#include <sys/types.h>		/*getpwnam */
#include <pwd.h>
#include <config.h>
#include <assert.h>
#include "strbuf.h"

/* global variable */

/*
 * dg87_daemon_log_level
 * This variable defines the priority of log messages which are sended 
 * to syslog. The default level is dg87_daemon_log_info : debug messages
 * are not sended to syslog.
 * The value of this variable is defined by the configuration file.
 */
static dg87_daemon_log_priority dg87_daemon_log_level = dg87_daemon_log_info;

static char* dg87_daemon_log_priority_name[] = 
{
	"ALERT",
	"CRITICAL",
	"ERROR",
	"WARNING",
	"NOTICE",
	"INFO",
	"DEBUG",
	NULL
};

/*****************************************************************************/
dg87_daemon *
dg87_daemon_new (char *name, char *user_name)
{
  dg87_daemon *daemon;
  assert (name != NULL);
  if (name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((daemon = dg87_malloc (sizeof (dg87_daemon), DG87_DAEMON)) == NULL)
    goto alloc_error;
  daemon->user_name = NULL;
  if (user_name != NULL)
    if ((daemon->user_name = dg87_strdup (user_name, DG87_CHAR)) == NULL)
      goto alloc_error;
  daemon->name = name;
  return daemon;

alloc_error:
  if (daemon != NULL)
    dg87_daemon_delete (daemon);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/*****************************************************************************/
int
dg87_daemon_start (dg87_daemon * daemon)
{
  int pid;
  int fd;
  struct passwd *pwp;
  assert (daemon != NULL);
  if (daemon == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }

  /* change user id */
  if (daemon->user_name != NULL)
    {
      /* search user informations */
      if ((pwp = getpwnam (daemon->user_name)) == NULL)
	{
		dg87_daemon_log_printf(dg87_daemon_log_error,
		   "Can't get user information '%s' !", daemon->user_name);
	  return -1;
	}
      /* change gid */
      if (setgid (pwp->pw_gid) == -1)
	{
		dg87_daemon_log_printf(dg87_daemon_log_error,
	  	"Can't change group (GID) !");
	  return -1;
	}
      /* change uid */
      if (setuid (pwp->pw_uid) == -1)
	{
		dg87_daemon_log_printf(dg87_daemon_log_error,
	  	"Can't change user (UID) !");
	  return -1;
	}
    }

  pid = fork ();
  if (pid < 0)
    {
      /* can't fork */
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't fork !");
      return -1;
    }
  if (pid > 0)
    {
      /* return 0 to the parent process */
      return 0;
    }
  /* create a new session to be process group leader */
  setsid ();
  pid = fork ();
  if (pid < 0)
    {
      /* can't fork */
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't fork !");
      return -1;
    }
  if (pid > 0)
    {
      /* the parent end normaly */
      return 0;
    }
  /* close all stream */
  for (fd = 3 /*TODO gettablesize() */ ; fd >= 0; --fd)
    close (fd);

  /* redirect stdin, stdout and stderr to /dev/null */
  fd = open ("/dev/null", O_RDWR);
  dup (fd);
  dup (fd);
  umask (022);
  chdir ("/tmp");

  /* open the log */
  openlog (daemon->name, LOG_PERROR | LOG_PID, LOG_DAEMON);
	dg87_daemon_log_printf(dg87_daemon_log_critical,
									"Daemon started");

  return 1;
}

/*****************************************************************************/
void
dg87_daemon_stop (dg87_daemon * daemon)
{
  /* the deamon ends here */
	dg87_daemon_log_printf(dg87_daemon_log_critical,
									"Daemon stopped");
  closelog ();
}

/*****************************************************************************/
void
dg87_daemon_delete (dg87_daemon * daemon)
{
  assert (daemon != NULL);
  if (daemon == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return;
    }
  if (daemon->user_name != NULL)
    dg87_free (daemon->user_name, DG87_CHAR);
  dg87_free (daemon, DG87_DAEMON);
}

/****************************************************************************/
void dg87_daemon_log_printf(dg87_daemon_log_priority priority, 
								char* format_string, ...)
{

	int syslog_priority;
	char buffer[60];
  char c[2];
  va_list ap;
  int ival;
  long lval;
  char *string;
	char *format;
	dg87_strbuf* strbuf;

	strbuf = dg87_strbuf_new();
	if (strbuf == NULL)
		return;

	format = format_string;
  va_start (ap, format_string); /* ap points on the first optional argument */
  while (*format)
    {
      if (*format == '\n')
			{
	  		/* print CR LF */
	  		dg87_strbuf_cat (strbuf, "\n\r");
			}
      else if (*format != '%')
			{
	  		/* not a format character (%), print it */
	  		c[0] = *format;
	  		c[1] = 0;
	  		dg87_strbuf_cat (strbuf, c);
			}
      /* interpret character after % */
      else
				switch (*++format)
	  		{
	  			case 's':		/* replace '%s' by a string */
	    			string = va_arg (ap, char *);
	    			if (string != NULL)
	      			dg87_strbuf_cat (strbuf, string);
	    			break;
	  			case 'c':		/* replace '%c' by a character */
	    			ival = va_arg (ap, int);
	    			sprintf (buffer, "%c", ival);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			case 'i':		/* replace '%i' by a integer */
	    			ival = va_arg (ap, int);
	    			sprintf (buffer, "%i", ival);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			case 'l':		/* replace '%l' by a long */
	    			lval = va_arg (ap, long);
	    			sprintf (buffer, "%li", lval);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			default:
	    			c[0] = *format;
	    			c[1] = 0;
	    			dg87_strbuf_cat (strbuf, c);
	  			}			/* end case */
      format++;			/* next character */
    }				/* end while */
  va_end (ap);

	switch (priority)
	{
		case dg87_daemon_log_alert:    syslog_priority = LOG_ALERT; break;
		case dg87_daemon_log_critical: syslog_priority = LOG_CRIT;  break;
		case dg87_daemon_log_error:    syslog_priority = LOG_ERR;   break;
		case dg87_daemon_log_warning:  syslog_priority = LOG_WARNING;break;
		case dg87_daemon_log_notice:   syslog_priority = LOG_NOTICE; break;
		case dg87_daemon_log_info:     syslog_priority = LOG_INFO;   break;
		case dg87_daemon_log_debug:    syslog_priority = LOG_DEBUG;  break;
		default: syslog_priority = LOG_DEBUG; break;

	}

	if (syslog_priority <= dg87_daemon_log_level)
	  syslog (syslog_priority, strbuf->buffer);

	fprintf(stderr, "%s - %s\n", 
			dg87_daemon_log_priority_name[priority],
			strbuf->buffer);
	dg87_strbuf_delete(strbuf);

}
/*****************************************************************************/
BOOL dg87_daemon_set_log_priority (char* log_level)
{
  int index = 0;
	if (log_level == NULL)
		dg87_daemon_log_level = dg87_daemon_log_info;
	else
	{
		while (dg87_daemon_log_priority_name[index] != NULL &&
					 strcmp (log_level, dg87_daemon_log_priority_name[index]) != 0)
			index++;
		if (dg87_daemon_log_priority_name[index] == NULL)
						return FALSE;
		dg87_daemon_log_level = index; 
	}
	return TRUE;
}
/*****************************************************************************/
char* dg87_daemon_get_log_priority (void)
{
	return dg87_daemon_log_priority_name[dg87_daemon_log_level];
}
/*****************************************************************************/
/* End of file */
