/* file interpret.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include "global.h"
#include "daemon.h"
#include "server.h"
#include "interpret.h"
#include "cmdconnection.h"
#include "cmdmessage.h"
#include "cmduser.h"

/* local prototypes */
static int dg87_interpret_cmd_exit (dg87_connection * connection);
static int dg87_interpret_cmd_version (dg87_connection * connection);
static int dg87_interpret_cmd_help (dg87_connection * connection);
#ifdef DEBUG
static int dg87_interpret_cmd_memory (dg87_connection * connection);
#endif

/* array for dg87_interpret's commands */
static dg87_interpret_cmd cmd_local_array[] = {
  {"cc", cmd_cc, "Connection Count", "",
   dg87_command_communication, dg87_command_access_consult},
  {"ck", cmd_ck, "Connection Kill", "",
   dg87_command_communication, dg87_command_access_admin},
  {"cl", cmd_cl, "Connection List", "B:S:E:u",
   dg87_command_communication, dg87_command_access_consult},
  {"exit", dg87_interpret_cmd_exit, "Exit Diogene", "",
   dg87_command_generic, dg87_command_access_none},
  {"help", dg87_interpret_cmd_help, "Give help on commands", "",
   dg87_command_generic, dg87_command_access_none},
  {"mc", cmd_mc, "Message Count", "",
   dg87_command_communication, dg87_command_access_consult},
#ifdef DEBUG
  {"memory", dg87_interpret_cmd_memory,
   "Memory usage (debug release only)", "v",
   dg87_command_generic, dg87_command_access_none},
#endif
  {"mk", cmd_mk, "Message Kill", "",
   dg87_command_communication, dg87_command_access_admin},
  {"ml", cmd_ml, "Message List", "B:S:E:sm",
   dg87_command_communication, dg87_command_access_consult},

  {"password", cmd_password, "Change user password", "",
   dg87_command_generic, dg87_command_access_consult},

  {"ua", cmd_ua, "User Add", "",
   dg87_command_user, dg87_command_access_admin},
  {"uc", cmd_uc, "User Count", "",
   dg87_command_user, dg87_command_access_consult},
  {"ui", cmd_ui, "User Information", "B:S:E:",
   dg87_command_user, dg87_command_access_consult},
  {"uk", cmd_uk, "User Kill", "",
   dg87_command_user, dg87_command_access_admin},
  {"ul", cmd_ul, "User List", "B:S:E:I:N:c",
   dg87_command_user, dg87_command_access_consult},
  {"usp", cmd_usp, "User Set Privilege", "",
   dg87_command_user, dg87_command_access_manage},

  {"user", cmd_user, "Identify user", "",
   dg87_command_generic, dg87_command_access_none},

  {"version", dg87_interpret_cmd_version,
   "Return the name and the version of the program",
   "", dg87_command_generic, dg87_command_access_none},
  {NULL, NULL, NULL, NULL, 0}
};

/******************************************************************************/
static int
dg87_interpret_cmd_exit (dg87_connection * connection)
{
  assert (connection != NULL);
  dg87_connection_printf (connection, "Bye\n");
  dg87_connection_close (connection);
  return 0;
}

/******************************************************************************/
static int
dg87_interpret_cmd_version (dg87_connection * connection)
{
  assert (connection != NULL);
  dg87_connection_printf (connection, "%s-%s-%s\n",
			  connection->server->package_name,
			  connection->server->program_name,
			  connection->server->program_version);
  return 0;
}

/******************************************************************************/
static int
dg87_interpret_cmd_help (dg87_connection * connection)
{
  int i_cmd;
  int brk = 0;
  dg87_interpret_cmd *command;
  dg87_interpret *interpret;
  assert (connection != NULL);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  interpret = connection->server->interpret;
  switch (connection->command->argc)
    {

    case 0:			/* print the command list */
      dg87_connection_printf (connection, "Available functions are :\n");
      /* internal functions */
      for (i_cmd = 0; cmd_local_array[i_cmd].name != NULL; i_cmd++)
	{
	  if (dg87_user_verify_privilege (connection->user,
					  cmd_local_array[i_cmd].family,
					  cmd_local_array[i_cmd].privilege)
	      == TRUE)
	    {
	      dg87_connection_printf (connection,
				      "%s \t", cmd_local_array[i_cmd].name);
	      if (brk++ == 8)
		{
		  dg87_connection_printf (connection, "\n");
		  brk = 0;
		}
	    }
	}
      /* client functions */
      for (i_cmd = 0; interpret->cmd_array[i_cmd].name != NULL; i_cmd++)
	{
	  if (dg87_user_verify_privilege (connection->user,
					  interpret->cmd_array[i_cmd].family,
					  interpret->cmd_array[i_cmd].
					  privilege) == TRUE)
	    {
	      dg87_connection_printf (connection, "%s \t",
				      interpret->cmd_array[i_cmd].name);
	      if (brk++ == 8)
		{
		  dg87_connection_printf (connection, "\n");
		  brk = 0;
		}
	    }
	}
      dg87_connection_printf (connection, "\n");
      break;

    case 1:			/* help for one command */

      /* search in local array */
      command = cmd_local_array;
      while (command->name != NULL && strcmp (command->name,
					      connection->command->
					      argv[0]) != 0)
	command++;
      /* search in client array */
      if (command->name == NULL)
	{
	  command = interpret->cmd_array;
	  while (command->name != NULL
		 && strcmp (command->name, connection->command->argv[0]) != 0)
	    command++;
	}

      if (command->name == NULL)
	{
	  dg87_connection_printf (connection,
				  "Help : Unknown command '%s'\n",
				  connection->command->argv[0]);
	  break;
	}

      if (dg87_user_verify_privilege (connection->user,
				      command->family,
				      command->privilege) == FALSE)
	{
	  dg87_connection_printf (connection,
				  "Help : Unauthorized command '%s'\n",
				  connection->command->argv[0]);
	  break;

	}

      /* print help command */
      dg87_connection_printf (connection,
			      "%s %s : %s\n", "Function ",
			      command->name, command->help_message);
      break;

    default:
      dg87_connection_printf (connection, "Incorrect use\n");
    }
  return 0;
}

/******************************************************************************/
#ifdef DEBUG
int
dg87_interpret_cmd_memory (dg87_connection * connection)
{
  dg87_command *command;
  char *buffer;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->optc == 0)
    dg87_connection_printf (connection, "%l\n", dg87_usage ());
  else
    {
      if (command->optv[0] != 'v')
	{
	  dg87_connection_printf (connection,
				  "Unknown option '%c' for command '%s'\n",
				  command->optv[0], command->function_name);
	  return 1;
	}
      buffer = dg87_stat ();
      if (buffer == NULL)
	return 1;
      dg87_connection_printf (connection, buffer);
      free (buffer);
    }
  return 0;
}
#endif
/******************************************************************************/
dg87_interpret *
dg87_interpret_new (dg87_server * server, dg87_interpret_cmd * cmd_array)
{
  dg87_interpret *interpret;
  assert (server != NULL && cmd_array != NULL);
  if (server == NULL || cmd_array == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if ((interpret = dg87_malloc (sizeof (dg87_interpret),
				DG87_INTERPRET)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  interpret->server = server;
  interpret->cmd_array = cmd_array;
  return interpret;
}

/******************************************************************************/
BOOL
dg87_interpret_delete (dg87_interpret * interpret)
{
  assert (interpret != NULL);
  if (interpret == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_free (interpret, DG87_INTERPRET);
  return TRUE;
}

/******************************************************************************/
int
dg87_interpret_execute (dg87_interpret * interpret,
			dg87_connection * connection)
{
  int retval = 0;
  dg87_interpret_cmd *cmd = NULL;
  assert (interpret != NULL && connection != NULL);
  if (interpret == NULL || connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }

  /* read the command and initialize the Command object */
  if (dg87_connection_read_command (connection) == FALSE)
    return -2;

  /* find the function in the local functions array */
  cmd = cmd_local_array;
  while (cmd->name != NULL &&
	 strcmp (connection->command->function_name, cmd->name) != 0)
    cmd++;

  /* find the function in the server functions array */
  if (cmd->name == NULL)
    {
      cmd = interpret->cmd_array;
      while (cmd->name != NULL &&
	     strcmp (connection->command->function_name, cmd->name) != 0)
	cmd++;
    }

  if (cmd->name == NULL)
    {
      dg87_connection_printf (connection, "Unknown command '%s'\n",
			      connection->command->function_name);
      dg87_connection_exit (connection, DG87_ERR_UNKNOWN_COMMAND);
      return -2;
    }

  /* control access privilege */
  if (connection->user != NULL &&
      dg87_user_verify_privilege (connection->user,
				  cmd->family, cmd->privilege) == FALSE)
    {
      dg87_connection_printf (connection, "Permission denied\n");
      dg87_connection_exit (connection, DG87_ERR_PERM_DENIED);
      return -2;
    }

  /* interpret options */
  if (dg87_command_interpret_options (connection->command, cmd->option_string)
      == FALSE)
    {
      dg87_connection_printf (connection, "Interpretation Error\n",
			      connection->command->function_name);
      dg87_connection_exit (connection, DG87_ERR_INTERPRET_ERROR);
      return -2;
    }
  switch (connection->command->error)
    {
    case DG87_COMMAND_ERR_STRNOTCLOSED:
      dg87_connection_printf (connection, "Quoted string not closed.\n");
      dg87_connection_send (connection);
      return -2;
    case DG87_COMMAND_ERR_UNKNOWNOPTION:
      dg87_connection_printf (connection,
			      "Unknown option '-%c' for "
			      "command '%s'.\n",
			      connection->command->error_option,
			      connection->command->function_name);

      dg87_connection_send (connection);
      return -2;
    case DG87_COMMAND_ERR_NOARGUMENT:
      dg87_connection_printf (connection,
			      "No argument for option '-%c' in "
			      "command '%s'.\n",
			      connection->command->error_option,
			      connection->command->function_name);
      dg87_connection_send (connection);
      return -2;
    }

  if (connection->command->error != 0)
    dg87_connection_exit (connection, connection->command->error);

  /* call the function */
  retval = cmd->function (connection);
  switch (retval)
    {
    case RETVAL_OK:
    case RETVAL_SHUTDOWN:
      dg87_connection_exit (connection, RETVAL_OK);
      break;
    case RETVAL_NOT_TERMINATED:
      break;
    default:
      dg87_connection_exit (connection, retval);
    }

  return retval;
}

/******************************************************************************/
/* End of file */
