/* file list.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "list.h"

/**************************************************************************/
dg87_list *
dg87_list_new (void)
{
  dg87_list *list;
  if ((list = dg87_malloc (sizeof (dg87_list), DG87_LIST)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  list->first = NULL;
  list->last = NULL;
  list->count = 0;
  return list;
}

/**************************************************************************/
BOOL
dg87_list_delete (dg87_list * list)
{
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_list_clear (list);
  dg87_free (list, DG87_LIST);
  return TRUE;
}

/**************************************************************************/
BOOL
dg87_list_clear (dg87_list * list)
{
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (list->count > 0)
    {
      dg87_item *current = list->first;
      dg87_item *next = current->next;
      while (current != NULL)
	{
	  dg87_item_delete (current);
	  current = next;
	  if (current != NULL)
	    next = current->next;
	}
    }
  list->first = NULL;
  list->last = NULL;
  list->count = 0;
  return TRUE;
}


/**************************************************************************/
BOOL
dg87_list_add_object (dg87_list * list, void *object)
{
  dg87_item *item;
  dg87_item *temp;
  assert (list != NULL && object != NULL);
  if (list == NULL || object == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  item = dg87_item_new (object);
  if (item == NULL)
    return FALSE;
  if (list->last == NULL)
    {
      /* the list is empty, add the first element */
      list->first = item;
      list->last = item;
      list->count = 1;
    }
  else
    {
      /* add the element at the end */
      temp = list->last;
      temp->next = item;
      item->prev = temp;
      list->last = item;
      list->count++;
    }
  return TRUE;
}

/**************************************************************************/
BOOL
dg87_list_insert_object (dg87_list * list, void *object, dg87uint index)
{
  dg87_item *item, *temp;
  dg87uint i = 0;

  assert (list != NULL && object != NULL);
  if (list == NULL || object == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (index > list->count)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }

  item = dg87_item_new (object);
  if (item == NULL)
    return FALSE;

  /* if the list is empty, we add at the begin of the list */
  if (list->first == NULL)
    {
      /* the list is empty */
      list->first = item;
      list->last = item;
      list->count = 1;
    }
  else if (index == list->count)
    {
      /* add at the end of the list */
      temp = list->last;
      temp->next = item;
      item->prev = temp;
      list->last = item;
      list->count++;
    }
  else
    {
      /* the list is not empty */
      i = 0;
      temp = list->first;
      /* find the item at the given index */
      while (temp->next != NULL && i < index)
	{
	  temp = temp->next;
	  i++;
	}
      /* taskdate the links */
      item->next = item;
      if (temp->prev == NULL)
	list->first = item;
      else
	temp->prev->next = item;
      item->prev = temp->prev;
      temp->prev = item;
      list->count++;
    }

  return TRUE;
}

/**************************************************************************/
dg87uint
dg87_list_count (dg87_list * list)
{
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return list->count;
}

/**************************************************************************/
dg87_item *
dg87_list_get_item (dg87_list * list, dg87uint index)
{
  dg87uint i;
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (index >= list->count)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }

  if (index == 0)
    return list->first;
  else if (index == list->count - 1)
    return list->last;
  else
    {
      dg87_item *item = list->first;
      for (i = 0; i < index; i++)
	item = item->next;
      return item;
    }
  return NULL;
}

/**************************************************************************/
BOOL
dg87_list_move_item (dg87_list * list, dg87uint index, int delta)
{
  dg87_item *first_item;
  dg87_item *next_item;
  void *object;
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (index >= list->count)
    {
      dg87_err_set_error (DG87_ERR_INDEX_OUT_OF_BOUND);
      return FALSE;
    }
  switch (delta)
    {
    case 1:
      if (index == list->count - 1)
	{
	  /* can't move next element */
	  dg87_err_set_error (DG87_ERR_INDEX_OUT_OF_BOUND);
	  return FALSE;
	}
      first_item = dg87_list_get_item (list, index);
      next_item = first_item->next;
      break;
    case -1:
      if (index == 0)
	{
	  /* can't move first element */
	  dg87_err_set_error (DG87_ERR_INDEX_OUT_OF_BOUND);
	  return FALSE;
	}
      first_item = dg87_list_get_item (list, index - 1);
      next_item = first_item->next;
      break;
    default:
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  object = first_item->object;
  first_item->object = next_item->object;
  next_item->object = object;
  return TRUE;
}

/**************************************************************************/
void *
dg87_list_get_object_by_index (dg87_list * list, dg87uint index)
{
  dg87_item *item;
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  item = dg87_list_get_item (list, index);
  if (item == NULL)
    return NULL;
  return item->object;
}

/**************************************************************************/
BOOL
dg87_list_remove_object (dg87_list * list, void *object)
{
  dg87_item *item;
  assert (list != NULL && object != NULL);
  if (list == NULL || object == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  item = dg87_list_find_item (list, object);
  if (item == NULL)
    return FALSE;

  dg87_list_remove_item (list, item);
  return TRUE;
}

/**************************************************************************/
void *
dg87_list_remove_object_by_index (dg87_list * list, dg87uint index)
{
  dg87_item *item;
  assert (list != NULL);
  if (list == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  item = dg87_list_get_item (list, index);
  if (item == NULL)
    return NULL;

  return dg87_list_remove_item (list, item);
}

/**************************************************************************/
void *
dg87_list_remove_item (dg87_list * list, dg87_item * item)
{
  void *object;
  assert (list != NULL && item != NULL);
  if (list == NULL || item == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  object = item->object;

  /* change the link of the previous item in the list */
  if (item->prev == NULL)
    list->first = item->next;
  else
    item->prev->next = item->next;

  /* change the link of the next item in the list */
  if (item->next == NULL)
    list->last = item->prev;
  else
    item->next->prev = item->prev;

  /* delete the item */
  list->count--;
  dg87_item_delete (item);
  return object;
}

/***************************************************************************/
dg87_item *
dg87_list_find_item (dg87_list * list, void *object)
{
  dg87_item *item;
  assert (list != NULL && object != NULL);
  if (list == NULL || object == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* browse the list to find the item associate to the object */
  item = list->first;
  while (item != NULL && item->object != object)
    item = item->next;
  return item;
}

/***************************************************************************/
BOOL
dg87_list_intersection (dg87_list * list1, dg87_list * list2)
{
  dg87_item *item1, *item2;
  assert (list1 != NULL && list2 != NULL);
  if (list1 == NULL || list2 == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* for each object of the first list */
  item1 = list1->first;
  while (item1 != NULL)
    {
      /* for each object of the first list */
      item2 = list2->first;
      while (item2 != NULL)
	{
	  /* if an object is in the two lists */
	  if (item1->object == item2->object)
	    return TRUE;
	  /* next object */
	  item2 = item2->next;
	}
      /* next object */
      item1 = item1->next;
    }
  return FALSE;
}

/***************************************************************************/
#ifdef DEBUG
int
dg87_list_check (dg87_list * list, int (*print) (char *))
{
  dg87_item *item;
  dg87uint count = 0;
  item = list->first;
  while (item != NULL)
    {

      /* check object */
      if (item->object == NULL)
	{
	  print ("List : Object pointer is NULL !\n");
	  return 1;
	}

      /* check next element */
      if (item->next == NULL)
	{
	  /* last element */
	  if (list->last != item)
	    {
	      print ("List : Last element incorrect !\n");
	      return 1;
	    }
	}
      else if (item->next->prev != item)
	{
	  print ("List : _previous pointer incorrect !\n");
	  return 1;
	}

      /* check previous element */
      if (item->prev == NULL)
	{
	  /* first element */
	  if (list->first != item)
	    {
	      print ("List : First element incorrect !\n");
	      return 1;
	    }
	}
      else if (item->prev->next != item)
	{
	  print ("List : _previous pointer incorrect !\n");
	  return 1;
	}

      /* next element */
      count++;
      item = item->next;

    }
  /* check object's count */
  if (count != list->count)
    {
      print ("List : incorrect count !\n");
      return 1;
    }
  /* check empty list */
  if (list->count == 0 || list->first == NULL || list->last == NULL)
    {
      if (list->count != 0 || list->first != NULL || list->last != NULL)
	{
	  print ("List : incorrect empty list\n");
	  return 1;
	}
    }
  return 0;
}
#endif /*DEBUG*/
/***************************************************************************/
/* End of file */
