/* file message.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <config.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <assert.h>

#include "global.h"
#include "message.h"
#include "daemon.h"

/******************************************************************************/
BOOL
dg87_message_verify_banner (char *banner)
{
  char *char1, *char2;
  int size;
  assert (banner != NULL);
  if (banner == NULL)
    return FALSE;

  /* banner sample : diogene87-dg87d-0.4.0-(PXX) 
   * this function must verify that PXX between parentheses
   * correspond to DG87_PROTOCOL_VERSION */

  /* find parentheses */
  char1 = strchr (banner, '(');
  char2 = strchr (banner, ')');
  if (char1 == NULL || char2 == NULL)
    return FALSE;

  /* at least on character between parentheses */
  if (char1 + 1 >= char2)
    return FALSE;

  char1++;
  size = char2 - char1 - 1;
  if (strncmp (DG87_PROTOCOL_VERSION, char1, size) != 0)
    return FALSE;

  return TRUE;
}

/******************************************************************************/
BOOL
dg87_message_send (dg87_message * message, char *user_name, char *password)
{
  dg87_connection *connection;
  char *response;
  char *banner;
  assert (message != NULL && user_name != NULL && password != NULL);
  if (message == NULL || user_name == NULL || password == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (message->state == dg87_message_state_cancel)
    return FALSE;

  /* create a connection object */
  if (message->connection == NULL)
    {
      if ((connection = dg87_connection_out_new (message->hostname,
						 message->ip_port)) == NULL)
				return FALSE;
      message->connection = connection;
    }

  /* restart aborted message after retry temporization */
  if (dg87_message_is_aborted (message) == TRUE)
    {
      /* verify if retry temporization is terminated */
      if (message->last_connection_retry != 0 &&
	  message->last_connection_retry +
	  DG87_CONNECTION_RETRY_TEMPO > dg87_current_time)
	{
	  return FALSE;
	}
      /* try again */
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"Temporization terminated, retry...");
      message->state = dg87_message_state_not_connected;
      message->last_connection_retry = 0;
      message->connection->state = dg87_connection_state_reading_message;
    }

  /* update connection */
  connection = message->connection;
  dg87_connection_update (connection);

  /* if connection is aborted */
  if (connection->state == dg87_connection_state_error)
    {
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"Connection error : start retry temporization.");
      message->state = dg87_message_state_connection_error;
      message->last_connection_retry = dg87_current_time;
      return FALSE;
    }

  switch (message->state)
    {
    case dg87_message_state_not_connected:
      message->state = dg87_message_state_read_banner;
      break;
    case dg87_message_state_read_banner:
      /* read the banner */
      if (connection->state == dg87_connection_state_message_available)
	{
	  banner = dg87_connection_read (connection);
	  /*  verify protocol */
	  if (banner == NULL)
	    {
	      message->state = dg87_message_state_banner_error;
	      message->last_connection_retry = dg87_current_time;
	      return FALSE;
	    }

	  if (dg87_message_verify_banner (banner) == FALSE)
	    {
				dg87_daemon_log_printf(dg87_daemon_log_debug,
	      	"wrong protocol !");
	      message->state = dg87_message_state_banner_error;
	      message->last_connection_retry = dg87_current_time;
	      dg87_free (banner, DG87_CHAR);
	      return FALSE;
	    }
	  dg87_free (banner, DG87_CHAR);
	  message->state = dg87_message_state_send_user;
	}
      return FALSE;

    case dg87_message_state_send_user:
      /* send user name and password */
      dg87_connection_printf (message->connection,
			      "user \"%s\" \"%s\"", user_name, password);
      dg87_connection_send (message->connection);
      message->state = dg87_message_state_read_user_response;
      return FALSE;

    case dg87_message_state_read_user_response:
      if (connection->state == dg87_connection_state_message_available)
	{
	  response = dg87_connection_read (connection);
	  if (response == NULL)
	    {
	      message->state = dg87_message_state_authentification_error;
	      message->last_connection_retry = dg87_current_time;
	      dg87_free (response, DG87_CHAR);
	      return FALSE;
	    }
	  if (strcmp (response, "OK") != 0)
	    {
				dg87_daemon_log_printf(dg87_daemon_log_debug,
	      	"identification error : %s", response);
	      message->state = dg87_message_state_authentification_error;
	      message->last_connection_retry = dg87_current_time;
	      dg87_free (response, DG87_CHAR);
	      return FALSE;
	    }
	  dg87_free (response, DG87_CHAR);
	  message->state = dg87_message_state_send_message;
	}
      return FALSE;

    case dg87_message_state_send_message:
      /* send the message */
      dg87_connection_printf (message->connection, message->message_string);
      dg87_connection_send (message->connection);
      message->state = dg87_message_state_read_reponse;
      return FALSE;

    case dg87_message_state_read_reponse:
      /* try to read the response */
      response = dg87_connection_read (message->connection);
      /* TODO interpret response */
      if (response == NULL)
	return FALSE;
      message->response = response;
      return TRUE;

    case dg87_message_state_cancel:
    case dg87_message_state_banner_error:
    case dg87_message_state_authentification_error:
    case dg87_message_state_connection_error:
      /* Nothing to do */
      break;
    }

  return FALSE;
}

/******************************************************************************/
dg87_message *
dg87_message_new (char *hostname, int ip_port, char *message_string)
{
  dg87_message *message;
  assert (hostname != NULL && message_string != NULL);
  if (hostname == NULL || message_string == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((message = dg87_malloc (sizeof (dg87_message), DG87_MESSAGE)) == NULL)
    goto alloc_error;
  if ((message->hostname = dg87_strdup (hostname, DG87_CHAR)) == NULL)
    goto alloc_error;
  if ((message->message_string =
       dg87_strdup (message_string, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  message->ip_port = ip_port;
  message->connection = NULL;
  message->state = dg87_message_state_not_connected;
  message->last_connection_retry = 0;
  message->response = NULL;
	DG87_SAVE_NEEDED;
  return message;

alloc_error:
  if (message != NULL)
    dg87_message_delete (message);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_message_delete (dg87_message * message)
{
  assert (message != NULL);
  if (message == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (message->connection != NULL)
    dg87_connection_delete (message->connection);
  if (message->hostname != NULL)
    dg87_free (message->hostname, DG87_CHAR);
  if (message->message_string != NULL)
    dg87_free (message->message_string, DG87_CHAR);
  if (message->response != NULL)
    dg87_free (message->response, DG87_CHAR);
  dg87_free (message, DG87_MESSAGE);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_message_is_aborted (dg87_message * message)
{
  switch (message->state)
    {
    case dg87_message_state_not_connected:
    case dg87_message_state_read_banner:
    case dg87_message_state_send_user:
    case dg87_message_state_read_user_response:
    case dg87_message_state_send_message:
    case dg87_message_state_read_reponse:
    case dg87_message_state_cancel:
      return FALSE;
    case dg87_message_state_banner_error:
    case dg87_message_state_authentification_error:
    case dg87_message_state_connection_error:
      return TRUE;
    }
  return FALSE;
}

/******************************************************************************/
BOOL
dg87_message_cancel (dg87_message * message)
{
  assert (message != NULL);
  if (message == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (message->state == dg87_message_state_cancel)
    return TRUE;

  if (message->connection != NULL)
    dg87_connection_close (message->connection);

	dg87_daemon_log_printf(dg87_daemon_log_info, 
									"Cancel message '%s'", message->message_string);

  message->state = dg87_message_state_cancel;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/******************************************************************************/
char *
dg87_message_state_to_ascii (dg87_message_state state)
{
  switch (state)
    {
    case dg87_message_state_not_connected:
      return "not connected";
    case dg87_message_state_read_banner:
      return "read banner";
    case dg87_message_state_send_user:
      return "send user name";
    case dg87_message_state_read_user_response:
      return "get user response";
    case dg87_message_state_send_message:
      return "send message";
    case dg87_message_state_read_reponse:
      return "get message response";
    case dg87_message_state_cancel:
      return "canceld";
    case dg87_message_state_banner_error:
      return "error";
    case dg87_message_state_authentification_error:
      return "authentification error";
    case dg87_message_state_connection_error:
      return "connection error";
    }
  return "Unknown state";

}

/******************************************************************************/
/* End of file */
