/* file server.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <assert.h>

#include <config.h>
#include "server.h"
#include "daemon.h"
#include "message.h"
#include "user.h"

#define CNX_MAX 10		/* max number of sockets connection */

/*****************************************************************************/
/* dg87_server_close_pid_file
 * Close and delete actual PID file.
 */
static BOOL
dg87_server_close_pid_file (dg87_server * server)
{
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* return if no PID file or if PID file is already closed */
  if (server->pid_file_str == NULL || server->fd_pid_file == 0)
    return TRUE;
  close (server->fd_pid_file);
  unlink (server->pid_file_str);
  server->fd_pid_file = 0;
  return TRUE;
}

/*****************************************************************************/
/* dg87_server_open_pid_file
 * Open PID file and write process ID in it, and lock it.
 * server->pid_file_str can be NULL if no PID file is used.
 */
static BOOL
dg87_server_open_pid_file (dg87_server * server)
{
  char str[12];
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* close previous lock file */
  dg87_server_close_pid_file (server);

  /* return if no pid file */
  if (server->pid_file_str == NULL)
    return TRUE;
  /* try to open the lock file */
  server->fd_pid_file = open (server->pid_file_str, O_RDWR | O_CREAT, 0640);
  if (server->fd_pid_file < 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
											"Can't open PID file '%s' !", server->pid_file_str);
      return FALSE;
    }
  /* try to lock the file */
  /* TODO lockf and F_LOCK do not exist in cygwin */
  if (lockf (server->fd_pid_file, F_TLOCK, 0) < 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't lock PID file '%s' !", server->pid_file_str);
      return FALSE;
    }
  /* write the pid of the deamon in the file */
  snprintf (str, 12, "%d\n", getpid ());
  if (write (server->fd_pid_file, str, strlen (str)) == -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't write in PID file '%s' !", server->pid_file_str);
      return FALSE;
    }
  /* don't close the file to keep the file locked */
  return TRUE;
}

/******************************************************************************/
static BOOL
dg87_server_create_users (dg87_server * server)
{
  dg87_user *user;
  assert (server != NULL);
  if (server == NULL)
    return FALSE;
  /* create diogene */
  user = dg87_server_user_add (server, DG87_USER_DIOGENE_NAME);
  if (user == FALSE)
    return FALSE;
  dg87_user_set_password (user, DG87_USER_DIOGENE_PASSWORD);
  dg87_user_set_privilege (user, dg87_command_day, dg87_command_access_admin);
  dg87_user_set_privilege (user, dg87_command_job, dg87_command_access_admin);
  dg87_user_set_privilege (user, dg87_command_log, dg87_command_access_admin);
  dg87_user_set_privilege (user,
			   dg87_command_stat, dg87_command_access_admin);
  user->system_account = TRUE;
  /* create administrator */
  user = dg87_server_user_add (server, DG87_USER_ADMIN_NAME);
  if (user == FALSE)
    return FALSE;
  dg87_user_set_administrator (user);
  return TRUE;
}

/******************************************************************************/
dg87_server *
dg87_server_new (char *package_name,
		 char *program_name,
		 char *program_version,
		 int ip_port,
		 char *ip_addr,
		 BOOL (*work_function) (dg87_server *),
		 int (*signal_function) (dg87_server *, int signo),
		 void *data, dg87_interpret_cmd * cmd_array,
		 char *pid_file_str)
{
  dg87_server *server;

  assert (program_name != NULL && program_version != NULL &&
	  work_function != NULL && signal_function != NULL &&
	  data != NULL && cmd_array != NULL && ip_addr != NULL);
  if (package_name == NULL || program_name == NULL ||
      program_version == NULL || work_function == NULL ||
      signal_function == NULL || data == NULL ||
      cmd_array == NULL || ip_addr == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  /* allocate memory */
  if ((server = dg87_malloc (sizeof (dg87_server), DG87_SERVER)) == NULL)
    goto alloc_error;
  if ((server->connection_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((server->message_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((server->user_vector = dg87_vector_new ()) == NULL)
    goto alloc_error;

  if (dg87_server_create_users (server) == FALSE)
    goto alloc_error;

  if ((server->interpret = dg87_interpret_new (server, cmd_array)) == NULL)
    goto alloc_error;

  server->pid_file_str = NULL;
  if (pid_file_str != NULL)
    if ((server->pid_file_str =
	 dg87_strdup (pid_file_str, DG87_CHAR)) == NULL)
      goto alloc_error;
  /* initialize structure */
  server->fd_pid_file = 0;
  server->package_name = package_name;
  server->program_name = program_name;
  server->program_version = program_version;
  server->ip_port = ip_port;
  server->ip_addr = ip_addr;
  server->work_function = work_function;
  server->signal_function = signal_function;
  server->data = data;
  server->quit = FALSE;
  server->shutdown = FALSE;

  return server;

alloc_error:
  if (server != NULL)
    dg87_server_delete (server);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/******************************************************************************/
BOOL
dg87_server_delete (dg87_server * server)
{
  dg87_item *item;
  dg87_connection *connection;
  dg87_element *element;
  dg87_user *user;
  dg87_message *message;
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete all remaining connection */
  if (server->connection_list != NULL)
    {
      if (server->connection_list->count > 0)
	{
	  item = server->connection_list->first;
	  while (item != NULL)
	    {
	      connection = item->object;
	      item = item->next;
	      dg87_connection_close (connection);
	    }
	}
      dg87_list_delete (server->connection_list);
    }

  /* cancel all message to send */
  if (server->message_list != NULL)
    {
      if (server->message_list->count > 0)
	{
	  item = server->message_list->first;
	  while (item != NULL)
	    {
	      message = item->object;
	      item = item->next;
	      dg87_message_cancel (message);
	    }
	}
      dg87_list_delete (server->message_list);
    }

  /* delete users */
  if (server->user_vector != NULL)
    {
      if (server->user_vector->count > 0)
	{
	  element = server->user_vector->first;
	  while (element != NULL)
	    {
	      user = element->object;
	      element = element->next;
	      dg87_user_delete (user);
	    }
	}
      dg87_vector_delete (server->user_vector);
    }
  if (server->interpret != NULL)
    dg87_interpret_delete (server->interpret);
  if (server->pid_file_str != NULL)
    dg87_free (server->pid_file_str, DG87_CHAR);
  dg87_free (server, DG87_SERVER);
  return TRUE;
}

/******************************************************************************/
dg87_connection *
dg87_server_add_connection (dg87_server * server, int fd,
			    char *hostname, int ip_port)
{
  dg87_connection *connection;
  assert (server != NULL && hostname != NULL);
  if (server == NULL || hostname == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  connection = dg87_connection_in_new (fd, hostname, ip_port, server);
  if (connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  if (dg87_list_add_object (server->connection_list, connection) == FALSE)
    {
      dg87_connection_delete (connection);
      return NULL;
    }
  /* send the banner */
  dg87_connection_printf (connection, "%s-%s-%s-(%s)\n",
			  server->package_name,
			  server->program_name,
			  server->program_version, DG87_PROTOCOL_VERSION);
  dg87_connection_send (connection);
  return connection;
}

/******************************************************************************/
BOOL
dg87_server_remove_connection (dg87_server * server,
			       dg87_connection * connection)
{
  assert (server != NULL && connection != NULL);
  if (server == NULL || connection == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_list_remove_object (server->connection_list, connection);
  dg87_connection_delete (connection);
  return TRUE;
}

/******************************************************************************/
/* dg87_server_prepare
 * This functions creates a new socket (server->ip_port)
 */
static int
dg87_server_prepare (dg87_server * server)
{
  struct sockaddr_in address_server;
  int optval;
  int sockfd_new;
  assert (server != NULL);
  /* open a new socket */
  if ((sockfd_new = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
											"Can't initialize socket !");
      return -1;
    }
  optval = 1;			/*1=yes, bind should reuse local addresses for this socket */
  if (setsockopt (sockfd_new, SOL_SOCKET, SO_REUSEADDR,
		  &optval, sizeof (int)) == -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't set socket option !");
      return -1;
    }
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"IP_ADDR : %s", server->ip_addr);
	dg87_daemon_log_printf(dg87_daemon_log_info,
  	"IP_PORT : %i", server->ip_port);
  memset ((char *) &address_server, 0, sizeof (struct sockaddr_in));
  address_server.sin_family = AF_INET;
  address_server.sin_port = htons (server->ip_port);
  if (inet_aton (server->ip_addr, &(address_server.sin_addr)) == 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Invalid IP_ADDR");
      address_server.sin_addr.s_addr = htonl (INADDR_ANY);
    }

  if (bind (sockfd_new, (struct sockaddr *) &address_server,
	    sizeof (address_server)) == -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't bind socket !");
      return -1;
    }

  if (listen (sockfd_new, CNX_MAX) == -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
      	"Can't listen socket !");
      return -1;
    }

  return sockfd_new;
}

/******************************************************************************/
/* TODO find a way to remove this static variable */
static dg87_server *local_server;
static void
signal_function (int signo)
{
  local_server->signal_function (local_server, signo);
  if (signo == SIGINT || signo == SIGTERM)
    {
			dg87_daemon_log_printf(dg87_daemon_log_info,
      	"Signal received : shutdown in progress.");
      local_server->shutdown = TRUE;
    }
}

/******************************************************************************/
BOOL
_dg87_server_manage_messages (dg87_server * server)
{
  dg87_item *item;
  dg87_message *message;
  /* delete messages which are canceled */
  item = server->message_list->first;
  while (item != NULL)
    {
      message = item->object;
      item = item->next;
      if (message->state == dg87_message_state_cancel)
	{
	  dg87_list_remove_object (server->message_list, message);
	  dg87_message_delete (message);
	}
    }
  /* try to send a message */
  item = server->message_list->first;
  while (item != NULL)
    {
      message = item->object;
      item = item->next;
      /* send message */
      if (dg87_message_send (message, DG87_USER_DIOGENE_NAME,
			     dg87_server_get_diogene_password (server)) ==
	  TRUE)
	{
	  dg87_list_remove_object (server->message_list, message);
	  dg87_message_delete (message);
	  return TRUE;
	}
      /* if message is not in error, exit */
      if (dg87_message_is_aborted (message) == FALSE)
	return TRUE;
    }
  return FALSE;
}

/******************************************************************************/
BOOL
dg87_server_run (dg87_server * server)
{
  int sockfd_server, sockfd_client, sockfd_new;
  socklen_t lng_client;
  struct sockaddr_in address_client;
  struct timeval tv;
  int result;
  int ip_port;
  fd_set testreadfds;
  fd_set testwritefds;
  struct sigaction sa_old;
  struct sigaction sa_new;
  dg87_item *item;
  dg87_connection *connection;

  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  local_server = server;

  if (dg87_server_open_pid_file (server) == FALSE)
    return FALSE;

  /* catch signals */
  sa_new.sa_handler = signal_function;
  sigemptyset (&sa_new.sa_mask);
  sa_new.sa_flags = 0;
  sigaction (SIGTERM, &sa_new, &sa_old);
  sigaction (SIGHUP, &sa_new, &sa_old);
  sigaction (SIGINT, &sa_new, &sa_old);
  sigaction (SIGPIPE, &sa_new, &sa_old);
  sigaction (SIGUSR1, &sa_new, &sa_old);

  ip_port = -1;
  sockfd_server = -1;
  server->quit = FALSE;
  server->shutdown = FALSE;
  while (server->quit == FALSE)
    {
			/* update dg87_current_time */
			dg87_current_time = time(NULL);

      /* if daemon starts of SIGHUP signal changes IP port */
      if (ip_port != server->ip_port)
	{

	  /* open a new socket */
	  sockfd_new = dg87_server_prepare (server);
	  if (sockfd_new < 0)
	    {
				if (sockfd_server == -1){
								/* TODO cleanup ?*/
								return FALSE;
				}
	      /* restore previous IP port value */
	      server->ip_port = ip_port;
	    }
	  else
	    {
	      /* close previous socket */
	      if (sockfd_server != -1)
						{
							dg87_daemon_log_printf(dg87_daemon_log_info,
								"Change IP Port !");
							close (sockfd_server);
						}
	      /* select new socket */
	      sockfd_server = sockfd_new;
	    }

	  /* memorize actual IP port value */
	  ip_port = server->ip_port;
	}

      /* quit if socket can't be opened */
      if (sockfd_server == -1)
	return FALSE;

      /* call work function */
      if (server->work_function (server) == TRUE)
	{
	  /* short timeout : workfunction has work to do */
	  tv.tv_sec = 0;
	  tv.tv_usec = 100;
	}
      else
	{
	  /* long timeout : workfunction has nothing to do */
	  tv.tv_sec = 0;
	  tv.tv_usec = 5000;
	}

      _dg87_server_manage_messages (server);

      /* if shutdown in progress */
      if (server->shutdown == TRUE)
	{
	  /* close all remaining connection */
	  if (server->connection_list->count > 0)
	    {
	      item = server->connection_list->first;
	      while (item != NULL)
		{
		  connection = item->object;
		  item = item->next;
		  dg87_connection_close (connection);
		}
	    }
	  server->quit = TRUE;
	}

      /* construct fds with connection */
      FD_ZERO (&testreadfds);
      FD_ZERO (&testwritefds);
      FD_SET (sockfd_server, &testreadfds);
      item = server->connection_list->first;
      while (item != NULL)
	{
	  connection = item->object;
	  item = item->next;
	  switch (connection->state)
	    {
	    case dg87_connection_state_reading_message:
	      FD_SET (connection->fd, &testreadfds);
	      break;
	    case dg87_connection_state_sending_message:
	      FD_SET (connection->fd, &testwritefds);
	      break;
	    default:
	      /* nothing to do */
	      break;
	    }
	}

      /* FD_SETSIZE as first parameter for select */
      result = select (FD_SETSIZE, &testreadfds, &testwritefds, NULL, &tv);

      /* continue if there is no request */
      if (result == 0)
	continue;

      if (result < 1)
	{
	  /* come here if signal is sended */
	  continue;
	}

      /* request for new connection */
      if (FD_ISSET (sockfd_server, &testreadfds))
	{
	  lng_client = sizeof (address_client);
	  errno = 0;
	  if ((sockfd_client = accept (sockfd_server,
				       (struct sockaddr *) &address_client,
				       &lng_client)) == -1)
	    {
				dg87_daemon_log_printf(dg87_daemon_log_info,
	      	"accept error");
	    }
	  else if (server->shutdown == TRUE)
	    {
	      /* server can't accept new connections */
	      close (sockfd_client);
	    }
	  else
	    {
	      dg87_server_add_connection (server,
					  sockfd_client,
					  inet_ntoa (address_client.sin_addr),
					  ntohs (address_client.sin_port));
	    }
	}

      /* for each Connection */
      item = server->connection_list->first;
      while (item != NULL)
	{
	  connection = item->object;
	  item = item->next;

	  /* if there is nothing to read or to write, continue */
	  if (!FD_ISSET (connection->fd, &testreadfds)
	      && !FD_ISSET (connection->fd, &testwritefds))
	    continue;

	  dg87_connection_update (connection);

	  if (connection->state == dg87_connection_state_message_available)
	    {
	      if (dg87_interpret_execute (server->interpret,
					  connection) == RETVAL_SHUTDOWN)
		{
			dg87_daemon_log_printf(dg87_daemon_log_info,
		  	"Shutdown in progress...");
		  server->shutdown = TRUE;
		}
	    }

	  if (connection->state == dg87_connection_state_connection_close)
	    {
	      dg87_server_remove_connection (server, connection);
	      continue;
	    }
	}			/* next connection */

    }
  if (sockfd_server != -1)
    close (sockfd_server);
  dg87_server_close_pid_file (server);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_server_post_message (dg87_server * server,
			  char *hostname, int ip_port, char *message_string)
{
  dg87_message *message;
  assert (server != NULL && hostname != NULL && message_string != NULL);
  if (server == NULL || hostname == NULL || message_string == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  message = dg87_message_new (hostname, ip_port, message_string);
  if (message == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return FALSE;
    }
  if (dg87_list_add_object (server->message_list, message) == FALSE)
    {
      dg87_message_delete (message);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
dg87_user *
dg87_server_user_add (dg87_server * server, char *user_name)
{
  dg87_user *user = NULL;
  assert (server != NULL && user_name != NULL);
  if (server == NULL || user_name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  user = dg87_user_new (user_name);
  if (user == NULL)
    return NULL;

  /* add the user in the UserList */
  /* vector generate error if an other user has not the same name */
  /* key must point on user->name */
  if (dg87_vector_add_object (server->user_vector, user, user->name) == FALSE)
    {
      dg87_user_delete (user);
      return NULL;
    }

  return user;
}

/****************************************************************************/
int
dg87_server_user_count (dg87_server * server)
{
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return dg87_vector_count (server->user_vector);
}

/****************************************************************************/
dg87_user *
dg87_server_user_get_by_name (dg87_server * server, char *name)
{
  dg87_element *element;
  assert (server != NULL && name != NULL);
  if (server == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (name) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }

  /* browse the list to find an item with the same name */
  element = server->user_vector->first;
  while (element != NULL)
    {
      if (strcmp (element->key, name) == 0)
	return element->object;
      element = element->next;
    }
  dg87_err_set_error (DG87_ERR_CAN_NOT_FIND_OBJECT);
  return NULL;
}

/****************************************************************************/
dg87_user *
dg87_server_user_get_by_index (dg87_server * server, int index)
{
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  return dg87_vector_get_object_by_index (server->user_vector, index);
}

/****************************************************************************/
BOOL
dg87_server_user_delete_by_name (dg87_server * server, char *name)
{
  dg87_user *user;
  assert (server != NULL && name != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  user = dg87_vector_remove_object_by_key (server->user_vector, name);
  if (user == NULL)
    return FALSE;
  if (dg87_user_delete (user) == FALSE)
    {
      /* add the user in the vector again */
      dg87_vector_add_object (server->user_vector, user, user->name);
      return FALSE;
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_server_user_delete_by_index (dg87_server * server, int index)
{
  dg87_user *user;
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  user = dg87_vector_remove_object_by_index (server->user_vector, index);
  if (user == NULL)
    return FALSE;
  dg87_user_delete (user);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_server_reload (dg87_server * server, char *new_pid_file, char *ip_addr,
		    int ip_port)
{
  char *previous_pid_file;
  int fd_previous_pid_file;
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  server->ip_port = ip_port;
  server->ip_addr = ip_addr;
  /* quit if pid files are the same */
  if (server->pid_file_str == NULL && new_pid_file == NULL)
    return TRUE;
  if (server->pid_file_str != NULL && new_pid_file != NULL &&
      strcmp (server->pid_file_str, new_pid_file) == 0)
    return TRUE;
  /* save previous PID file */
  previous_pid_file = server->pid_file_str;
  fd_previous_pid_file = server->fd_pid_file;
  server->fd_pid_file = 0;
  server->pid_file_str = NULL;
  /* initialize new PID file */
  if (new_pid_file != NULL)
    {
      server->pid_file_str = dg87_strdup (new_pid_file, DG87_CHAR);
      if (server->pid_file_str == NULL)
	{
	  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	  return FALSE;
	}
      /* open new PID file */
      if (dg87_server_open_pid_file (server) == FALSE)
	{
	  /* restore previous PID file */
	  server->pid_file_str = previous_pid_file;
	  server->fd_pid_file = fd_previous_pid_file;
	  return FALSE;
	}
    }
  /* free previous PID file */
  if (previous_pid_file != NULL)
    {
      close (fd_previous_pid_file);
      unlink (previous_pid_file);
      dg87_free (previous_pid_file, DG87_CHAR);
    }

  return TRUE;
}

/******************************************************************************/
char *
dg87_server_get_diogene_password (dg87_server * server)
{
  dg87_user *user;
  assert (server != NULL);
  if (server == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if ((user = dg87_vector_get_object_by_key (server->user_vector,
					     DG87_USER_DIOGENE_NAME)) == NULL)
    return NULL;
  return user->password;
}

/******************************************************************************/
/* End of file */
