/* file storage.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "storage.h"
#include "daemon.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h>		/* uint16_t- uint32_t */
#include <assert.h>

/***************************************************************************/
dg87_storage *
dg87_storage_new (char *data_file, char *backup_file,
		  char *signature, int min_version, int max_version,
		  char *user_account)
{
  dg87_storage *storage;
  struct passwd *pswd;
  assert (data_file != NULL && backup_file != NULL && signature != NULL);
  if (data_file == NULL || backup_file == NULL || signature == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((storage = dg87_malloc (sizeof (dg87_storage), DG87_STORAGE)) == NULL)
    goto alloc_error;
  /* initialize structure */
  storage->data_file = data_file;
  storage->backup_file = backup_file;
  storage->mode = dg87_storage_mode_closed;
  storage->signature = signature;
  storage->min_version = min_version;
  storage->max_version = max_version;
  storage->actual_version = -1;
  storage->file_stream = NULL;
  /* if we are running in root account, change file owner of data file */
  storage->uid = -1;
  storage->gid = -1;
  if (geteuid () == 0 && user_account != NULL)
    {
      if ((pswd = getpwnam (user_account)) != NULL)
	{
	  storage->uid = pswd->pw_uid;
	  storage->gid = pswd->pw_gid;
	}
    }
  return storage;

alloc_error:
  if (storage != NULL)
    dg87_storage_delete (storage);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/***************************************************************************/
BOOL
dg87_storage_delete (dg87_storage * storage)
{
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_free (storage, DG87_STORAGE);
  return TRUE;
}

/***************************************************************************/
static BOOL
dg87_storage_check_header (dg87_storage * storage)
{
  char *string;
  uint16_t tmpvalue;
  int l;
  assert (storage != NULL);
  /* verify signature */
  l = strlen (storage->signature);
  string = dg87_malloc ((l + 1) * sizeof (char), DG87_CHAR);
  if (fread(string, l * sizeof(char), 1, storage->file_stream) == 0)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  string[l + 1] = '\0';
  if (strncmp (string, storage->signature, l) != 0)
    {
      dg87_free (string, DG87_CHAR);
      dg87_err_set_error (DG87_ERR_STORAGE_WRONG_SIGN);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);

  /* verify version */
  if (fread(&tmpvalue, sizeof(uint16_t), 1, storage->file_stream) == 0)
    return FALSE;
  storage->actual_version = ntohs (tmpvalue);
  if (storage->actual_version < storage->min_version ||
      storage->actual_version > storage->max_version)
    {
      dg87_err_set_error (DG87_ERR_STORAGE_WRONG_VERSION);
      return FALSE;
    }
  return TRUE;
}

/***************************************************************************/
static BOOL
dg87_storage_write_header (dg87_storage * storage)
{
  uint16_t value;
  int l;
  assert (storage != NULL);
  /* write signature */
  l = strlen (storage->signature);
  if (fwrite(storage->signature, l * sizeof(char), 1,storage->file_stream) == 0)
    return FALSE;
  /* write version */
  value = htons (storage->max_version);
  if (fwrite(&value, sizeof(uint16_t), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
static BOOL
dg87_storage_open_write (dg87_storage * storage)
{
  /* writing on backup file */
  assert (storage != NULL);
	dg87_daemon_log_printf(dg87_daemon_log_debug,
  	"Open '%s' for writing", storage->backup_file);
  if ((storage->file_stream = fopen (storage->backup_file, "w")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
  			"Can't open '%s' for writing", storage->backup_file);
      dg87_err_set_error (DG87_ERR_STORAGE_CAN_NOT_OPEN);
      return FALSE;
    }

  /* if daemon is running in root account, change file owner */
  if (storage->uid != -1 && storage->gid != -1)
    {
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"Change file owner");
      if (chown (storage->backup_file, storage->uid, storage->gid) == -1)
			{
				dg87_daemon_log_printf(dg87_daemon_log_error,
					"Can't change file owner of '%s'",
					storage->backup_file);
			}
    }
  storage->mode = dg87_storage_mode_write;
  if (dg87_storage_write_header (storage) == FALSE)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
static BOOL
dg87_storage_open_read (dg87_storage * storage)
{
  assert (storage != NULL);
  /* reading data file */
	dg87_daemon_log_printf(dg87_daemon_log_debug,
  	"Open '%s' for reading", storage->data_file);
  if ((storage->file_stream = fopen (storage->data_file, "r")) == NULL)
    {
			dg87_daemon_log_printf(dg87_daemon_log_error,
  			"Can't open '%s' for reading !", storage->data_file);
      dg87_err_set_error (DG87_ERR_STORAGE_CAN_NOT_OPEN);
      return FALSE;
    }
  storage->mode = dg87_storage_mode_read;
  if (dg87_storage_check_header (storage) == FALSE)
    return FALSE;

  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_open (dg87_storage * storage, dg87_storage_mode mode)
{
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  switch (mode)
    {
    case dg87_storage_mode_write:
      return dg87_storage_open_write (storage);
    case dg87_storage_mode_read:
      return dg87_storage_open_read (storage);
    default:
      dg87_err_set_error (DG87_ERR_STORAGE_WRONG_MODE);
      return FALSE;
    }
  return FALSE;
}

/***************************************************************************/
BOOL
dg87_storage_close (dg87_storage * storage)
{
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (storage->mode == dg87_storage_mode_closed)
    return TRUE;
  /* close file */
  if (fclose (storage->file_stream) != 0)
    {
      dg87_err_set_error (DG87_ERR_STORAGE_CAN_NOT_CLOSE);
      return FALSE;
    }
  /* if write mode, rename backup file to data file */
  if (storage->mode == dg87_storage_mode_write)
  {
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"Rename data file '%s' to '%s'", 
				storage->backup_file,
				storage->data_file);
      if (rename (storage->backup_file, storage->data_file) == -1)
			{
				dg87_daemon_log_printf(dg87_daemon_log_error,
	  			"Can't rename data file '%s' !", storage->data_file);
	  		return FALSE;
			}
  }
  storage->mode = dg87_storage_mode_closed;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_write_string (dg87_storage * storage, char *string)
{
  uint16_t tmpvalue;
  size_t size;
  assert (storage != NULL && string != NULL);
  if (storage == NULL || string == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* TODO conversion */
  size = strlen (string);
  tmpvalue = htons (size);
  if (fwrite(&tmpvalue, sizeof(uint16_t), 1, storage->file_stream) == 0)
    return FALSE;

	if (size != 0)
	{
  	if (fwrite(string, size * sizeof(char), 1, storage->file_stream) == 0)
  	  return FALSE;
	}
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_read_string (dg87_storage * storage, char **pstring)
{
  size_t size;
  uint16_t tmpvalue;
  char *string;
  assert (storage != NULL && pstring != NULL);
  if (storage == NULL || pstring == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  *pstring = NULL;
  /* read size */
  if (fread(&tmpvalue, sizeof(uint16_t), 1, storage->file_stream) == 0)
    return FALSE;
  size = ntohs (tmpvalue);
  /* allocate memory for string */
  string = dg87_malloc ((size + 1) * sizeof (char), DG87_CHAR);
  if (string == NULL)
    return FALSE;
  /* read string */
  if (size > 0)
    {
      if (fread(string, size, 1, storage->file_stream) == 0)
	{
	  dg87_free (string, DG87_CHAR);
	  *pstring = NULL;
	  return FALSE;
	}
    }
  string[size] = '\0';
  *pstring = string;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_write_long (dg87_storage * storage, dg87long value)
{
  uint32_t tmpvalue;
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  tmpvalue = htonl (value);
  if (fwrite(&tmpvalue, sizeof(uint32_t), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_read_long (dg87_storage * storage, dg87long * value)
{
  uint32_t tmpvalue;
  assert (storage != NULL && value != NULL);
  if (storage == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fread(&tmpvalue, sizeof(uint32_t), 1,storage->file_stream) == 0)
    return FALSE;
  *value = ntohl (tmpvalue);
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_write_uint (dg87_storage * storage, dg87uint value)
{
  uint16_t tmpvalue;
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  tmpvalue = htons (value);
  if (fwrite(&tmpvalue, sizeof(uint16_t), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_read_uint (dg87_storage * storage, dg87uint * value)
{
  uint16_t tmpvalue;
  assert (storage != NULL && value != NULL);
  if (storage == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fread(&tmpvalue, sizeof(uint16_t), 1,storage->file_stream) == 0)
    return FALSE;
  *value = ntohs (tmpvalue);
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_write_int (dg87_storage * storage, dg87int value)
{
  int16_t tmpvalue;
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  tmpvalue = htons (value);
  if (fwrite(&tmpvalue, sizeof(uint16_t), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_read_int (dg87_storage * storage, dg87int * value)
{
  int16_t tmpvalue;
  assert (storage != NULL && value != NULL);
  if (storage == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fread(&tmpvalue, sizeof(uint16_t), 1,storage->file_stream) == 0)
    return FALSE;
  *value = ntohs (tmpvalue);
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_write_ubyte (dg87_storage * storage, dg87ubyte value)
{
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fwrite(&value, sizeof(dg87ubyte), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_read_ubyte (dg87_storage * storage, dg87ubyte * value)
{
  assert (storage != NULL && value != NULL);
  if (storage == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fread(value, sizeof(dg87ubyte), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}
/***************************************************************************/
BOOL
dg87_storage_write_byte (dg87_storage * storage, dg87byte value)
{
  assert (storage != NULL);
  if (storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fwrite(&value, sizeof(dg87byte), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_storage_read_byte (dg87_storage * storage, dg87byte * value)
{
  assert (storage != NULL && value != NULL);
  if (storage == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (fread(value, sizeof(dg87byte), 1,storage->file_stream) == 0)
    return FALSE;
  return TRUE;
}
/***************************************************************************/
/* End of file */
