/* file strbuf.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>
#include "strbuf.h"

/******************************************************************************/
dg87_strbuf *
dg87_strbuf_new (void)
{
  dg87_strbuf *strbuf;
  strbuf = dg87_malloc (sizeof (dg87_strbuf), DG87_STRBUF);
  if (strbuf == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  strbuf->buffer = NULL;
  strbuf->buffer_size = 0;
  strbuf->size = 0;
  if (dg87_strbuf_cat (strbuf, "") == FALSE)
    {
      dg87_strbuf_delete (strbuf);
      return NULL;
    }
  return strbuf;
}

/******************************************************************************/
BOOL
dg87_strbuf_delete (dg87_strbuf * strbuf)
{
  assert (strbuf != NULL);
  if (strbuf == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (strbuf->buffer != NULL)
    dg87_free (strbuf->buffer, DG87_CHAR);
  dg87_free (strbuf, DG87_STRBUF);

  return TRUE;
}

/******************************************************************************/
char *
dg87_strbuf_get_buffer (dg87_strbuf * strbuf)
{
  char *buffer;
  assert (strbuf != NULL);
  if (strbuf == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  buffer = strbuf->buffer;
  strbuf->buffer = NULL;
  if (dg87_strbuf_cat (strbuf, "") == FALSE)
    {
      return NULL;
    }
  return buffer;
}

/******************************************************************************/
BOOL
dg87_strbuf_cat (dg87_strbuf * strbuf, char *buffer)
{
  assert (strbuf != NULL && buffer != NULL);
  if (strbuf == NULL || buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_strbuf_ncat (strbuf, buffer, strlen (buffer));
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_strbuf_ncat (dg87_strbuf * strbuf, char *buffer, int string_size)
{
  int new_size;
  char *new_buffer;
  assert (strbuf != NULL && buffer != NULL);
  if (strbuf == NULL || buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* initialize the buffer */
  if (strbuf->buffer == NULL)
    {
      new_size = DG87_STRBUF_BUFFERSIZE;
      new_buffer = dg87_malloc (new_size, DG87_CHAR);
      if (new_buffer == NULL)
	{
	  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	  return FALSE;
	}
      strbuf->buffer = new_buffer;
      strbuf->buffer_size = new_size;
      strbuf->size = 0;
      strbuf->buffer[0] = 0;
    }
  /* if the buffer is to small to concat the string */
  while (strbuf->size + string_size >= strbuf->buffer_size)
    {
      /* then expand the buffer */
      new_size = strbuf->buffer_size + DG87_STRBUF_BUFFERSIZE;
      new_buffer = realloc (strbuf->buffer, new_size);
      if (new_buffer == NULL)
	{
	  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
	  return FALSE;
	}
      strbuf->buffer = new_buffer;
      strbuf->buffer_size = new_size;
    }
  /* append the string to the buffer */
  memcpy (strbuf->buffer + strbuf->size, buffer, string_size + 1);
  strbuf->size += string_size;

  return TRUE;

}

/******************************************************************************/
BOOL
dg87_strbuf_cpy (dg87_strbuf * strbuf, char *buffer)
{
  assert (strbuf != NULL && buffer != NULL);
  if (strbuf == NULL || buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* reinitialize string buffer */
  dg87_free (strbuf->buffer, DG87_CHAR);
  strbuf->buffer = NULL;
  strbuf->size = 0;
  strbuf->buffer_size = 0;
  /* concat new string to an empty string */
  dg87_strbuf_ncat (strbuf, buffer, strlen (buffer));
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_strbuf_ncpy (dg87_strbuf * strbuf, char *buffer, int size)
{
  assert (strbuf != NULL && buffer != NULL);
  if (strbuf == NULL || buffer == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* reinitialize string buffer */
  dg87_free (strbuf->buffer, DG87_CHAR);
  strbuf->buffer = NULL;
  strbuf->size = 0;
  strbuf->buffer_size = 0;
  /* concat new string to an empty string */
  dg87_strbuf_ncat (strbuf, buffer, size);
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_strbuf_printf (dg87_strbuf * strbuf, char *format, ...)
{
  char buffer[60];
  char c[2];
  va_list ap;
  int ival;
  long lval;
  char *string;

  assert (strbuf != NULL && format != NULL);
  if (strbuf == NULL || format == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }

  va_start (ap, format);	/* ap points on the first optional argument */
  while (*format)
    {
      /*if (*format == '\n')
			{
	  		dg87_strbuf_cat (strbuf, "\n\r");
			}
      else */if (*format != '%')
			{
	  		/* not a format character (%), print it */
	  		c[0] = *format;
	  		c[1] = 0;
	  		dg87_strbuf_cat (strbuf, c);
			}
      /* interpret character after % */
      else
				switch (*++format)
	  		{
	  			case 's':		/* replace '%s' by a string */
	    			string = va_arg (ap, char *);
	    			if (string != NULL)
	      			dg87_strbuf_cat (strbuf, string);
	    			break;
	  			case 'c':		/* replace '%c' by a character */
	    			ival = va_arg (ap, int);
	    			sprintf (buffer, "%c", ival);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			case 'i':		/* replace '%i' by a integer */
	    			ival = va_arg (ap, int);
	    			sprintf (buffer, "%i", ival);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			case 'l':		/* replace '%l' by a long */
	    			lval = va_arg (ap, long);
	    			sprintf (buffer, "%li", lval);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			default:
	    			c[0] = *format;
	    			c[1] = 0;
	    			dg87_strbuf_cat (strbuf, c);
	  			}			/* end case */
      format++;			/* next character */
    }				/* end while */
  va_end (ap);
  return TRUE;
}

/******************************************************************************/

/* End of file */
