/* file user.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>
#include "user.h"

/******************************************************************************/
dg87_user *
dg87_user_new (char *name)
{
  int i;
  dg87_user *user;
  assert (name != NULL);
  if (name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_USER_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  user = dg87_malloc (sizeof (dg87_user), DG87_USER);
  if (user == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  /* initialize structure */
  dg87_strnzcpy (user->name, name, DG87_USER_NAME_LENGTH);
  user->password[0] = '\0';
  user->connection_count = 0;
  user->system_account = FALSE;
  for (i = 1; i < DG87_COMMAND_FAM_MAX; i++)
    user->privileges[i] = dg87_command_access_none;
  user->privileges[dg87_command_generic] = dg87_command_access_consult;
	DG87_SAVE_NEEDED;
  return user;
}

/******************************************************************************/
BOOL
dg87_user_delete (dg87_user * user)
{
  assert (user != NULL);
  if (user == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (user->system_account == TRUE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_DELETE_SYSTEM_ACCOUNT);
      return FALSE;
    }
  if (user->connection_count > 0)
    {
      dg87_err_set_error (DG87_ERR_USER_CONNECTED);
      return FALSE;
    }
  dg87_free (user, DG87_USER);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_load (dg87_user * user, dg87_storage * storage)
{
  char *string;
  int i;
  dg87ubyte byte;
  assert (user != NULL && storage != NULL);
  if (user == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read user password */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_user_set_password (user, string) == FALSE)
    return FALSE;
  /* read privileges */
  for (i = 0; i < DG87_COMMAND_FAM_MAX; i++)
    {
      if (dg87_storage_read_ubyte (storage, &byte) == FALSE)
	return FALSE;
      user->privileges[i] = byte;
    }
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_save (dg87_user * user, dg87_storage * storage)
{
  int i;
  assert (user != NULL && storage != NULL);
  if (user == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write user password */
  if (dg87_storage_write_string (storage, user->password) == FALSE)
    return FALSE;
  /* write privileges */
  for (i = 0; i < DG87_COMMAND_FAM_MAX; i++)
    {
      if (dg87_storage_write_ubyte (storage, user->privileges[i]) == FALSE)
	return FALSE;
    }

  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_update_connection_count (dg87_user * user, int delta)
{
  assert (user != NULL);
  if (user == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  user->connection_count += delta;
  if (user->connection_count < 0)
    user->connection_count = 0;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_set_password (dg87_user * user, char *password)
{
  assert (user != NULL && password != NULL);
  if (user == NULL || password == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (strlen (password) > DG87_USER_PASSWORD_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_PASSWORD_TOO_LONG);
      return FALSE;
    }
  dg87_strnzcpy (user->password, password, DG87_USER_PASSWORD_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_verify_password (dg87_user * user, char *password)
{
  assert (user != NULL && password != NULL);
  if (user == NULL || password == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (strlen (password) >= DG87_USER_PASSWORD_LENGTH)
    {
      dg87_err_set_error (DG87_ERR_PASSWORD_TOO_LONG);
      return FALSE;
    }
  if (strncmp (user->password, password, DG87_USER_PASSWORD_LENGTH) != 0)
    return FALSE;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_set_privilege (dg87_user * user,
			 dg87_command_family family,
			 dg87_command_privilege privilege)
{
  assert (user != NULL);
  if (user == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (user->system_account == TRUE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_ACCOUNT);
      return FALSE;
    }
  user->privileges[family] = privilege;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_user_verify_privilege (dg87_user * user,
			    dg87_command_family family,
			    dg87_command_privilege privilege)
{
  /* user is NULL if no user is connected : this is normal */
  if (user == NULL && privilege == dg87_command_access_none)
    return TRUE;
  if (user != NULL && user->privileges[family] >= privilege)
    return TRUE;
  return FALSE;
}

/******************************************************************************/
BOOL
dg87_user_set_administrator (dg87_user * user)
{
  int i;
  assert (user != NULL);
  if (user == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  user->system_account = TRUE;
  for (i = 0; i < DG87_COMMAND_FAM_MAX; i++)
    user->privileges[i] = dg87_command_access_admin;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/******************************************************************************/
/* End of file */
