/* file vector.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "vector.h"

/***************************************************************************/
dg87_vector *
dg87_vector_new (void)
{
  dg87_vector *vector = dg87_malloc (sizeof (dg87_vector), DG87_VECTOR);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  vector->first = NULL;
  vector->last = NULL;
  vector->count = 0;
  return vector;
}

/***************************************************************************/
BOOL
dg87_vector_delete (dg87_vector * vector)
{
  dg87_element *element, *temp;
  assert (vector != NULL);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete each element in the vector */
  element = vector->first;
  while (element != NULL)
    {
      temp = element;
      element = element->next;
      dg87_element_delete (temp);
    }
  dg87_free (vector, DG87_VECTOR);
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_vector_clear (dg87_vector * vector)
{
  dg87_element *element, *temp;
  assert (vector != NULL);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete each element in the vector */
  element = vector->first;
  while (element != NULL)
    {
      temp = element;
      element = element->next;
      dg87_element_delete (temp);
    }
  vector->first = NULL;
  vector->last = NULL;
  vector->count = 0;
  return TRUE;
}

/***************************************************************************/
BOOL
dg87_vector_add_object (dg87_vector * vector, void *object, char *key)
{
  dg87_element *element;
  dg87_element *temp;
  dg87int compar = 0;
  assert (vector != NULL && key != NULL);
  if (vector == NULL || object == NULL || key == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (strlen (key) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_KEY);
      return FALSE;
    }

  /* create the associate element */
  element = dg87_element_new (object, key);
  if (element == NULL)
    return FALSE;

  /* add the element in the list */
  if (vector->count == 0)
    {
      /* list is empty */
      vector->first = element;
      vector->last = element;
      vector->count = 1;
      return TRUE;
    }
  /* optimisation if key is greater than last element */
  if (strcmp (key, vector->last->key) > 0)
    {
      temp = vector->last;
      temp->next = element;
      element->next = NULL;
      element->prev = temp;
      vector->last = element;
      vector->count++;
      return TRUE;
    }

  /* find the position for the object in the vector */
  temp = vector->first;
  while (temp != NULL)
    {
      compar = strcmp (key, temp->key);
      if (compar <= 0)
	break;
      temp = temp->next;
    }

  /* add at the end of the list */
  if (temp == NULL)
    {
      element->prev = vector->last;
      vector->last->next = element;
      vector->last = element;
      element->next = NULL;
      vector->count++;
      return TRUE;
    }

  /* add just before temp */
  if (compar != 0)
    {
      element->next = temp;
      if (temp->prev == NULL)
	vector->first = element;
      else
	temp->prev->next = element;
      element->prev = temp->prev;
      temp->prev = element;
      vector->count++;
      return TRUE;
    }
  /* an object as the same key in the vector */
  dg87_err_set_error (DG87_ERR_KEY_NOT_UNIQUE);
  dg87_element_delete (element);
  return FALSE;
}

/***************************************************************************/
void *
_dg87_vector_delete_element (dg87_vector * vector, dg87_element * element)
{
  void *object;
  assert (vector != NULL && element != NULL);
  if (vector == NULL || element == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }

  object = element->object;

  /* change the link of the previous element in the vector */
  if (element->prev == NULL)
    vector->first = element->next;
  else
    element->prev->next = element->next;

  /* change the link of the next element in the vector */
  if (element->next == NULL)
    vector->last = element->prev;
  else
    element->next->prev = element->prev;

  /* delete the element object */
  vector->count--;
  dg87_element_delete (element);
  return object;
}

/***************************************************************************/
BOOL
dg87_vector_remove_object (dg87_vector * vector, void *object)
{
  dg87_element *element;
  assert (vector != NULL && object != NULL);
  if (vector == NULL || object == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  element = vector->first;
  while (element != NULL)
    {
      if (element->object == object)
	{
	  _dg87_vector_delete_element (vector, element);
	  return TRUE;
	}
      element = element->next;
    }
  dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
  return FALSE;
}

/***************************************************************************/
void *
dg87_vector_remove_object_by_key (dg87_vector * vector, char *key)
{
  dg87_element *element;
  assert (vector != NULL && key != NULL);
  if (vector == NULL || key == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  element = dg87_vector_get_element_by_key (vector, key);
  if (element == NULL)
    {
      dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
      return NULL;
    }
  return _dg87_vector_delete_element (vector, element);
}

/***************************************************************************/
void *
dg87_vector_remove_object_by_index (dg87_vector * vector, dg87uint index)
{
  dg87_element *element;
  assert (vector != NULL);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  element = dg87_vector_get_element_by_index (vector, index);
  return _dg87_vector_delete_element (vector, element);
}

/***************************************************************************/
dg87uint
dg87_vector_count (dg87_vector * vector)
{
  assert (vector != NULL);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return -1;
    }
  return vector->count;
}

/***************************************************************************/
void *
dg87_vector_get_object_by_index (dg87_vector * vector, dg87uint index)
{
  dg87_element *element;
  assert (vector != NULL);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  element = dg87_vector_get_element_by_index (vector, index);
  return (element == NULL) ? NULL : element->object;
}

/***************************************************************************/
void *
dg87_vector_get_object_by_key (dg87_vector * vector, char *key)
{
  dg87_element *element;
  assert (vector != NULL && key != NULL);
  if (vector == NULL || key == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  element = dg87_vector_get_element_by_key (vector, key);
  if (element == NULL)
    {
      dg87_err_set_error (DG87_ERR_CANNOT_FIND_OBJECT);
      return NULL;
    }
  return element->object;
}

/***************************************************************************/
dg87_element *
dg87_vector_get_element_by_index (dg87_vector * vector, dg87uint index)
{
  dg87_element *element;
  dg87uint i = 0;
  assert (vector != NULL);
  if (vector == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (index >= vector->count)
    {
      dg87_err_set_error (DG87_ERR_INDEX_OUT_OF_BOUND);
      return NULL;
    }
  if (vector->count == 0)
    return NULL;
  if (index == 0)
    return vector->first;
  if (index == vector->count - 1)
    return vector->last;
  element = vector->first;
  while (element != NULL && i++ < index)
    element = element->next;
  return element;
}

/***************************************************************************/
dg87_element *
dg87_vector_get_element_by_key (dg87_vector * vector, char *key)
{
  dg87_element *element;
  int length;

  assert (vector != NULL && key != NULL);
  if (vector == NULL || key == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (key) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_KEY);
      return NULL;
    }

  element = vector->first;
  while (element != NULL)
    {
      length = strcmp (key, element->key);
      if (length == 0)
	/* we found the exact key */
	return element;
      else if (length < 0)
	/* actual key is greater than search key */
	return NULL;
      element = element->next;
    }
  dg87_err_set_error (DG87_ERR_CAN_NOT_FIND_OBJECT);
  return NULL;
}

/***************************************************************************/
dg87_element *
dg87_vector_find_element (dg87_vector * vector, void *object)
{
  dg87_element *element;
  assert (vector != NULL && object != NULL);
  if (vector == NULL || object == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* browse the vector to find the item associate to the object */
  element = vector->first;
  while (element != NULL && element->object != object)
    element = element->next;
  return element;
}

/***************************************************************************/
#ifdef DEBUG
int
dg87_vector_check (dg87_vector * vector, int (*print) (char *))
{
  dg87_element *element;
  dg87uint count = 0;
  element = vector->first;
  while (element != NULL)
    {

      /* check object */
      if (element->object == NULL)
	{
	  print ("Vector : Object pointer is NULL !\n");
	  return 1;
	}
      /* check key's pointer */
      if (element->key == NULL)
	{
	  print ("Vector : Key pointer is NULL !\n");
	  return 1;
	}
      /* check key length */
      if (strlen (element->key) == 0)
	{
	  print ("Vector : Key is zero length !\n");
	  return 1;
	}

      /* check next element */
      if (element->next == NULL)
	{
	  /* last element */
	  if (vector->last != element)
	    {
	      print ("Vector : Last element incorrect !\n");
	      return 1;
	    }
	}
      else if (element->next->prev != element)
	{
	  print ("Vector : _previous pointer incorrect !\n");
	  return 1;
	}

      /* check previous element */
      if (element->prev == NULL)
	{
	  /* first element */
	  if (vector->first != element)
	    {
	      print ("Vector : First element incorrect !\n");
	      return 1;
	    }
	}
      else if (element->prev->next != element)
	{
	  print ("Vector : _previous pointer incorrect !\n");
	  return 1;
	}

      /* check key order */
      if (element->prev != NULL)
	{
	  if (strcmp (element->prev->key, element->key) >= 0)
	    {
	      print ("Vector : sort problem !\n");
	      return 1;
	    }
	}

      /* next element */
      count++;
      element = element->next;

    }
  /* check object's count */
  if (count != vector->count)
    {
      print ("Vector : incorrect count !\n");
      return 1;
    }

  /* TODO check empty vector */
  if (count == 0 || vector->first == NULL || vector->last == NULL)
    {

    }
  return 0;
}
#endif /*DEBUG*/
/***************************************************************************/
/* End of file */
