/* file context.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_CONTEXT_H__
#define __DG87_CONTEXT_H__

#include <time.h>
#include <libdg87/daemon.h>
#include <libdg87/vector.h>
#include <libdg87/conf.h>
#include <libdg87/storage.h>
#include "global.h"
#include "d_ref.h"
#include "context.h"
#include "target.h"

/*
 * dg87_context object
 * The dg87_context object is the main object of the application.
 * It has a pointer on every collections : targets, referentials,
 * exploitation days, log days, stat referentials... 
 */
struct _dg87_context
{
  /* TODO ip_port is stored in dg87_context.ip_port and dg87_server.ip_port
   * when a SIGHUP is sended, configuration file is re-read, and 
   * dg87_context.ip_port value can change.
   * In this case, dg87_server tries new IP Port and if an error occurres
   * (port already used), dg87_server.ip_port keeps old value.
   */
  /* collection of targets */
  dg87_vector *target_vector;
  /* collection of referentials */
  dg87_vector *ref_vector;
  /* collection of stat referentials */
  dg87_vector *stat_ref_vector;
  /* collection of exploitation days */
  dg87_vector *exp_day_vector;
  /* collection of log days */
  dg87_vector *log_day_vector;
  /* collection of jobs which are running on this server */
  dg87_list *job_list;
  /* collection of files which are checked on this server */
  dg87_list *file_list;

  dg87_engine *engine;
  dg87_server *server;
  dg87_daemon *daemon;

	/* the system queue (localhost/batch) */
	dg87_queue *system_queue;

  /* path of the file which is used to store the PID of the daemon.
   * Its value is set by the PID_FILE field of the configuratio file.
   * Can be NULL if PID_FILE field is not specified.*/
  char *pid_file;
  /* path of the directory where the database files are stored */
  char *data_directory;
  /* fully qualified hostname which is stored in the field
   * HOSTNAME of the configuration file.
   * Can be NULL if HOSTNAME field is not specified */
  char *hostname;
  /* IP address the daemon is listening on */
  char *ip_addr;
  /* IP port the daemon is listening on */
  int ip_port;
  /* configuration file object */
  dg87_conf *conf;
  /* storage to read/write the database file */
  dg87_storage *storage;
  /* time of the last database save */
  time_t save_time;
  /* number of seconds between to saves of the database */
  dg87uint save_tempo;
  /* number of days before automatically deleting an exploitation day */
  dg87uint day_purge_retention;
};

/*
 * dg87_context_new
 * Create a dg87_context object and return a pointer on it.
 * conf_file is the path of the configuration file (/etc/diogene87/dio87d.conf)
 */
dg87_context *dg87_context_new (char *conf_file);

/*
 * dg87_context_delete
 */
BOOL dg87_context_delete (dg87_context * context);

/*
 * dg87_context_set_server
 */
BOOL dg87_context_set_server (dg87_context * context, dg87_server * server);

/*
 * dg87_context_make_periodic_save
 */
BOOL dg87_context_make_periodic_save (dg87_context * context);

/*
 * dg87_context_read_config
 * This function reads (or re-reads) the configuration file variables
 */
BOOL dg87_context_read_config (dg87_context * context);

/*
 * dg87_context_log
 * write the message in the system log.
 */
void dg87_context_log (char *message);

/* 
 * dg87_context_get_hostname
 * This functions returns the full qualified name of the server.
 * If the value of use_conf_file is true, the name of the server is searched
 * in the configuration file (field HOSTNAME).
 * You must call dg87_free to free the memory.
 */
char *dg87_context_get_hostname (dg87_context * context, BOOL use_conf_file);

/*
 * dg87_context_save
 */
BOOL dg87_context_save (dg87_context * context);

/*
 * dg87_context_load
 */
BOOL dg87_context_load (dg87_context * context);

#ifdef DEBUG
/*
 * dg87_context_ckeck
 */
int dg87_context_check (dg87_context * context, int (*print) (char *));
#endif /*DEBUG*/
/*************************************************************************/
/*  functions for targets                                                */
/*************************************************************************/
/*
 * dg87_context_target_add
 * creates a dg87_target object, adds this object in the target_vector 
 * of the context and returns a pointer on this object.
 * If the `system' argument is TRUE, the target can't be deleted.
 * Returns NULL if the target object can't be created.
 */
  dg87_target * dg87_context_target_add (dg87_context * context,
                                         char *target_name, BOOL system);

/*
 * dg87_context_target_count
 * returns the number of targets.
 */
int dg87_context_target_count (dg87_context * context);

/*
 * dg87_context_target_get_by_name
 * returns a pointer on the target which has the name
 * given in parameter. Returns NULL if it can't be found.
 */
dg87_target *dg87_context_target_get_by_name (dg87_context * context,
                                              char *name);

/*
 * dg87_context_target_get_by_index
 * returns a pointer on the target at the given position.
 * returns NULL if the index is out of bound.
 */
dg87_target *dg87_context_target_get_by_index (dg87_context * context,
                                               int index);

/*
 * dg87_context_target_delete_by_name
 * deletes the target which has the name given in parameter.
 * returns NULL if it can't be found.
 */
BOOL dg87_context_target_delete_by_name (dg87_context * context, char *name);

/*
 * dg87_context_target_delete_by_index
 * delete the target at the given index.
 * returns NULL if it can't be found.
 */
BOOL dg87_context_target_delete_by_index (dg87_context * context, int index);

/*
 * dg87_context_target_delete
 * removes the target from the target vector, and deletes the object.
 * return FALSE, if the target can't be deleted.
 */
BOOL dg87_context_target_delete (dg87_context * context,
                                 dg87_target * target);

/*
 * dg87_context_target_rename
 * You don't have to use this function. Use dg87_target_set_name instead.
 * This function is called by dg87_target_set_name to change the name of a 
 * target.
 * It returns FALSE if another target has the same name.
 */
BOOL
dg87_context_target_rename (dg87_context * context,
                               dg87_target * target, char *new_name);



/*************************************************************************/
/*  functions for referential models                                     */
/*************************************************************************/

/*
 * dg87_context_ref_model_add
 * creates a dg87_ref object, adds this object in the referential vector 
 * of the context and returns a pointer on this object.
 * Returns NULL if the referential object can't be created.
 */
dg87_ref_model *dg87_context_ref_model_add (dg87_context * context,
                                            char *referential_name);

/*
 * dg87_context_ref_model_duplicate
 * creates a new referential object by copying another, adds this object
 * to the referential vector and returns a pointer on this object. 
 * returns NULL if the referential can't be created.
 */
dg87_ref_model *dg87_context_ref_model_duplicate (dg87_context * context,
                                                  dg87_ref_model *
                                                  original_referential,
                                                  char *new_referential_name);

/*
 * dg87_context_ref_model_delete_by_name
 * removes from the referential vector, the referential which has the name 
 * given in parameter, and delete this object.
 * returns FALSE if the referential can't be deleted.
 */
BOOL
dg87_context_ref_model_delete_by_name (dg87_context * context, char *name);

/*
 * dg87_context_ref_model_get_by_name
 * returns a pointer on the referential which has the name
 * given in parameter. Returns NULL if it can't be found.
 */
dg87_ref_model *dg87_context_ref_model_get_by_name (dg87_context * context,
                                                    char *name);

/*
 * dg87_context_ref_model_rename
 * You don't have to use this function. Use dg87_ref_model_set_name instead.
 * This function is called by dg87_ref_model_set_name to change the name of a 
 * referential.
 * It returns FALSE if another referential has the same name.
 */
BOOL
dg87_context_ref_model_rename (dg87_context * context,
                               dg87_ref_model * ref, char *new_name);

/*************************************************************************/
/*  functions for exploitation days                                      */
/*************************************************************************/

/*
 * dg87_context_exp_day_add
 * creates an exploitation day object, adds this object in the exploitation
 * day vector of the context and returns a pointer on this object.
 * Returns NULL if the day object can't be created.
 */
dg87_exp_day *dg87_context_exp_day_add (dg87_context * context,
                                        char *day_name, time_t start_time);

/*
 * dg87_context_exp_day_count
 * returns the number of exploitation days.
 * returns -1 if context is NULL.
 */
int dg87_context_exp_day_count (dg87_context * context);

/*
 * dg87_context_exp_day_get_by_name
 * returns a pointer on the exploitation day which has the name
 * given in parameter. Returns NULL if it can't be found.
 */
dg87_exp_day *dg87_context_exp_day_get_by_name (dg87_context * context,
                                                char *name);

/*
 * dg87_context_exp_day_get_by_index
 * returns a pointer on the day at the given index.
 * returns NULL if the value of the index is out of bound.
 */
dg87_exp_day *dg87_context_exp_day_get_by_index (dg87_context * context,
                                                 int index);

/*
 * dg87_context_exp_day_delete_by_name
 * removes the day which has the name given in parameter, from the
 * exploitation day vector and deletes this object.
 * returns NULL if the day object can't be deleted.
 */
BOOL dg87_context_exp_day_delete_by_name (dg87_context * context, char *name);

/*
 * dg87_context_exp_day_delete_by_index
 * removes the exploitation day at the given index, from the exploitation
 * day vector and delete this object.
 * returns NULL if the index value is out of bound.
 */
BOOL dg87_context_exp_day_delete_by_index (dg87_context * context, int index);

/*************************************************************************/
/*  functions for log days                                               */
/*************************************************************************/

/*
 * dg87_context_log_load_days
 * loads the log days which are available in the data directory.
 * adds each log days in the log days vector.
 * returns NULL, if days can't be loaded.
 */
BOOL dg87_context_log_load_days (dg87_context * context);

/*
 * dg87_context_log_day_add
 * creates a log day object, adds this object in the log day vector 
 * of the context and returns a pointer on this object.
 * Returns NULL if the day object can't be created.
 */
dg87_log_day *dg87_context_log_day_add (dg87_context * context,
                                        char *day_name);

/*
 * dg87_context_log_load_day
 * loads a day from a data file.
 * returns NULL if an error occures while loading.
 */
dg87_log_day *dg87_context_log_load_day (dg87_context * context,
                                         char *filename);

/*
 * dg87_context_log_day_count
 * returns the number of log days.
 * returns -1 if context is NULL.
 */
int dg87_context_log_day_count (dg87_context * context);

/*
 * dg87_context_log_day_get_by_name
 * returns a pointer on the log day which has the name
 * given in parameter. Returns NULL if it can't be found.
 */
dg87_log_day *dg87_context_log_day_get_by_name (dg87_context * context,
                                                char *name);

/*
 * dg87_context_log_day_get_by_index
 * returns a pointer on the log day at the given position.
 * returns NULL if the value of the index if out of bound.
 */
dg87_log_day *dg87_context_log_day_get_by_index (dg87_context * context,
                                                 int index);

/*
 * dg87_context_log_day_delete_by_name
 * remove the day which has the name given in parameter, from the log day
 * vector and delete this object.
 * return FALSE if the log day can't be deleted.
 */
BOOL dg87_context_log_day_delete_by_name (dg87_context * context, char *name,
                                          BOOL delete_file);

/*
 * dg87_context_log_day_delete_by_index
 * remove the day at the given index, from the log day vector and delete
 * this object.
 * return FALSE if the log day can't be deleted.
 */
BOOL dg87_context_log_day_delete_by_index (dg87_context * context, int index,
                                           BOOL delete_file);


/*************************************************************************/
/*  functions for stat refs */
/*************************************************************************/

/*
 * dg87_context_sta_referential_add
 * creates a stat referential object, adds this object in the stat 
 * referential vector of the context and returns a pointer on this object.
 * returns NULL if the referential object can't be created.
 */
dg87_sta_ref *dg87_context_sta_referential_add (dg87_context * context,
                                                char *referential_name);

/*
 * dg87_context_sta_referential_load
 * load a stat referential from a data file.
 * return NULL if an error occurres while loading.
 */
dg87_sta_ref *dg87_context_sta_referential_load (dg87_context * context,
                                                 char *filename);

/*
 * dg87_context_sta_referential_count
 * returns the number of stat referentials.
 * returns -1 if context is NULL.
 */
int dg87_context_sta_referential_count (dg87_context * context);

/*
 * dg87_context_sta_referential_get_by_name
 * returns a pointer on the stat referential which has the name
 * given in parameter. Returns NULL if it can't be found.
 */
dg87_sta_ref *dg87_context_sta_referential_get_by_name (dg87_context *
                                                        context, char *name);

/*
 * dg87_context_sta_referential_get_by_index
 * returns a pointer on the referential at the given position.
 * returns NULL if the index value is out of bound.
 */
dg87_sta_ref *dg87_context_sta_referential_get_by_index (dg87_context *
                                                         context, int index);

/*
 * dg87_context_sta_referential_delete_by_name
 * removes the stat referential which has the name given in parameter, from
 * the stat referential vector and deletes it.
 * Returns FALSE, if the stat referential can't be deleted.
 * If the delete_file parameter is TRUE, the data file which was used to store
 * the stat is deleted.
 */
BOOL dg87_context_sta_referential_delete_by_name (dg87_context * context,
                                                  char *name,
                                                  BOOL delete_file);

/*
 * dg87_context_sta_referential_delete_by_index
 * removes the stat referential at the given index, from the stat referential
 * vector and delete it.
 * Returns FALSE, if the stat referential can't be deleted.
 * If the delete_file parameter is TRUE, the data file which was used to store
 * the stat is deleted.
 */
BOOL dg87_context_sta_referential_delete_by_index (dg87_context * context,
                                                   int index,
                                                   BOOL delete_file);

/*
 * dg87_sta_ref_load_referentials
 * reads the data directory and loads each data files which is used to store
 * a stat referential.
 * returns FALSE if an error occurres while loading.
 */
BOOL dg87_context_sta_load_referentials (dg87_context * context);

/*************************************************************************/
/*  functions for running jobs                                           */
/*************************************************************************/

/*
 * dg87_r_context_job_add
 * create a job object, adds it to the job vector and return a pointer on it.
 * The hostname and the ip_port parameters are used to know where to send
 * the outputs of the job.
 * returns NULL if the job can't be created.
 */
dg87_r_job *dg87_r_context_job_add (dg87_context * context, char *task_name,
                                    char *job_name, char *day_name,
                                    char *path, char *argument,
                                    char *hostname, int ip_port);

/*
 * dg87_r_context_job_delete
 * remove the job from the job vector and delete it.
 * returns FALSE if the job can't be deleted.
 */
BOOL dg87_r_context_job_delete (dg87_context * context, dg87_r_job * job);

/*************************************************************************/
/*  functions for files                                                  */
/*************************************************************************/

/*
 * dg87_r_context_file_add
 *
 */
dg87_r_file *dg87_r_context_file_add (dg87_context * context, char *day_name,
                                      char *task_name, char *path,
                                      char *hostname, int ip_port);

/*
 * dg87_r_context_file_delete
 */
BOOL dg87_r_context_file_delete (dg87_context * context, dg87_r_file * file);

#endif /*__DG87_CONTEXT_H__*/
/* End of file */
