/* file queue.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

#include "context.h"
#include "x_caller.h"
#include "queue.h"
#include "x_task.h"

/****************************************************************************/
dg87_queue *
dg87_queue_new (dg87_target * target, char *name, BOOL system)
{
  dg87_queue *queue;
  assert (target != NULL && name != NULL);
  if (target == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (dg87_check_name (name, DG87_QUEUE_NAME_LENGTH) == FALSE)
    return NULL;
  /* allocate memory */
  if ((queue = dg87_malloc (sizeof (dg87_queue), DG87_QUEUE)) == NULL)
    goto alloc_error;
  if ((queue->ref_task_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((queue->exp_task_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  if ((queue->running_job_list = dg87_list_new ()) == NULL)
    goto alloc_error;
  /* initialize structure */
  dg87_strnzcpy (queue->name, name, DG87_QUEUE_NAME_LENGTH);
  queue->target = target;
  queue->description[0] = 0;
  queue->open = TRUE;
  queue->system = system;
  queue->max = 100;
	DG87_SAVE_NEEDED;
  return queue;

alloc_error:
  if (queue != NULL)
    dg87_queue_delete (queue, TRUE);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_queue_delete (dg87_queue * queue, BOOL force)
{
  dg87_ref_task *ref_task;
  dg87_exp_task *exp_task;
  assert (queue != NULL);
  if (queue == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (queue->system == TRUE && force == FALSE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_QUEUE);
      return FALSE;
    }
  /* set the Queue to NULL for each referential task */
  if (queue->ref_task_list != NULL)
    {
      while (queue->ref_task_list->count > 0)
        {
          ref_task =
            dg87_list_remove_object_by_index (queue->ref_task_list, 0);
          ref_task->queue = NULL;
        }
      dg87_list_delete (queue->ref_task_list);
    }
  /* set the Queue to NULL for each exploitation task */
  if (queue->exp_task_list != NULL)
    {
      while (queue->exp_task_list->count > 0)
        {
          exp_task =
            dg87_list_remove_object_by_index (queue->exp_task_list, 0);
          exp_task->queue = NULL;
        }
      dg87_list_delete (queue->exp_task_list);
    }
  if (queue->running_job_list != NULL)
    dg87_list_delete (queue->running_job_list);

  dg87_free (queue, DG87_QUEUE);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_load (dg87_queue * queue, dg87_storage * storage)
{
  dg87uint value;
  dg87ubyte b;
  char *string;
  assert (queue != NULL && storage != NULL);
  if (queue == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read target description */
  if (dg87_storage_read_string (storage, &string) == FALSE)
    return FALSE;
  if (dg87_queue_set_description (queue, string) == FALSE)
    {
      dg87_free (string, DG87_CHAR);
      return FALSE;
    }
  dg87_free (string, DG87_CHAR);
  /* ignore max property in version 1 of data file */
  if (dg87_storage_read_uint (storage, &value) == FALSE)
    return FALSE;
  queue->max = value;
  /* read opened property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  if (dg87_queue_set_open (queue, b) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_save (dg87_queue * queue, dg87_storage * storage)
{
  assert (queue != NULL && storage != NULL);
  if (queue == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* write target description */
  if (dg87_storage_write_string (storage, queue->description) == FALSE)
    return FALSE;
  /* write max */
  if (dg87_storage_write_uint (storage, queue->max) == FALSE)
    return FALSE;
  /* write opened property */
  if (dg87_storage_write_ubyte (storage, queue->open) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_set_name (dg87_queue * queue, char *name)
{
  assert (queue != NULL && name != NULL);
  if (queue == NULL || name == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (queue->system == TRUE)
    {
      dg87_err_set_error (DG87_ERR_CAN_NOT_CHANGE_SYSTEM_QUEUE);
      return FALSE;
    }
  if (dg87_check_name (name, DG87_QUEUE_NAME_LENGTH) == FALSE)
    return FALSE;
	if (dg87_target_queue_rename (queue->target, queue, name) == FALSE)
		return FALSE;
	DG87_SAVE_NEEDED;
  return TRUE;
}
/****************************************************************************/
BOOL
dg87_queue_set_description (dg87_queue * queue, char *description)
{
  assert (queue != NULL && description != NULL);
  if (queue == NULL || description == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (dg87_check_description (description, DG87_QUEUE_DESC_LENGTH) == FALSE)
    return FALSE;
  dg87_strnzcpy (queue->description, description, DG87_QUEUE_DESC_LENGTH);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_set_open (dg87_queue * queue, BOOL value)
{
  assert (queue != NULL);
  if (queue == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (value != FALSE && value != TRUE)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  queue->open = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_set_max (dg87_queue * queue, int value)
{
  assert (queue != NULL);
  if (queue == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (value < 1)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  queue->max = value;
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_add_running_job (dg87_queue * queue, dg87_exp_job * job)
{
  assert (queue != NULL && job != NULL);
  if (queue == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (job->task->queue != queue)
    return FALSE;
  if (dg87_list_find_item (queue->running_job_list, job) == NULL)
    {
      dg87_list_add_object (queue->running_job_list, job);
    }
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_remove_running_job (dg87_queue * queue, dg87_exp_job * job)
{
  dg87_item *job_item;
  assert (queue != NULL && job != NULL);
  if (queue == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (job->task->queue != queue)
    return FALSE;
  job_item = dg87_list_find_item (queue->running_job_list, job);
  if (job_item != NULL)
    dg87_list_remove_item (queue->running_job_list, job_item);
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_queue_manage (dg87_context * context, dg87_queue * queue)
{
  dg87_item *job_item;
  dg87_exp_job *job;
  int n_running_jobs = 0;

  assert (queue != NULL);
  if (queue == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* nothing to do if queue or target is closed */
  if (queue->open == FALSE || queue->target->open == FALSE)
    return TRUE;

  /*TODO : verify that number of running jobs < max */
  job_item = queue->running_job_list->first;
  while (job_item != NULL)
    {
      job = job_item->object;
      if (job->state != dg87_exp_job_state_ready_to_submit)
        n_running_jobs++;
      job_item = job_item->next;
    }
  if (n_running_jobs >= queue->max)
    return TRUE;

  /* submit jobs which are waiting */
  job_item = queue->running_job_list->first;
  while (job_item != NULL)
    {
      job = job_item->object;
      if (job->state == dg87_exp_job_state_ready_to_submit)
        {
          if (dg87_caller__submit_job (context, job) == TRUE)
            {
							dg87_daemon_log_printf(dg87_daemon_log_debug,
              	"queue : submit job %s", job->name);
              dg87_exp_job_set_state (job, dg87_exp_job_state_submit, -1);
              n_running_jobs++;
							DG87_SAVE_NEEDED;
							dg87_daemon_log_printf(dg87_daemon_log_debug,
              	"queue : number of running jobs %i", n_running_jobs);
							dg87_daemon_log_printf(dg87_daemon_log_debug,
              	"queue : maximum running jobs %i", queue->max);
              if (n_running_jobs >= queue->max)
                {
									dg87_daemon_log_printf(dg87_daemon_log_debug,
                  	"Can't start job now !");
                  break;
                }
            }
        }
      job_item = job_item->next;
    }
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_queue_check (dg87_queue * queue, int (*print) (char *))
{
  dg87_item *item;
  dg87_ref_task *task;
  if (strlen (queue->name) == 0)
    {
      print ("Queue : name length is zero !\n");
      return 1;
    }
  if (queue->target == NULL)
    {
      print ("Queue : target pointer is NULL !\n");
      return 1;
    }

  /* check ref_task_list */
  if (queue->ref_task_list == NULL)
    {
      print ("Queue : RefTaskList is NULL !\n");
      return 1;
    }
  if (dg87_list_check (queue->ref_task_list, print) != 0)
    return 1;
  item = queue->ref_task_list->first;
  while (item != NULL)
    {
      task = item->object;
      if (task->queue != queue)
        {
          print ("Queue : Task queue incorrect !\n");
          return 1;
        }
      item = item->next;
    }

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
