/* file x_cmdday.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include <libdg87/strbuf.h>
#include "x_cmdday.h"
#include "util.h"
#include "context.h"
#include "x_caller.h"
#include "engine.h"

/******************************************************************************/
/* xdk
 * eXploitation Day Kill
 * Usage : xdk [DayName]
 */
int
cmd_xdk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_strbuf *log_msg;
  dg87_exp_day *day;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc > 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xdk [DayName]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  if (command->argc == 0)
    {
      dg87_engine_auto_purge (context->engine, TRUE);
      return 0;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;
  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xdk (by '%s')",
                      (connection->user == NULL) ?
                      "anonymous" : connection->user->name);
  dg87_caller__send_day_command (day, log_msg->buffer);
  dg87_strbuf_delete (log_msg);
  /* delete the day */
  dg87_err_clear_error ();
  if (dg87_context_exp_day_delete_by_name (context, command->argv[0]) ==
      FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the day '%s' ! %s\n"),
                              command->argv[0], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_xdc
 * eXploitation Day Count
 * return the number of Days.
 * Usage : xdc 
 */
int
cmd_xdc (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"xdc");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  dg87_connection_printf (connection, "%i\n", context->exp_day_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_xdl
 * eXploitation Day List
 * Usage : xdl [options] 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int
cmd_xdl (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;  /* index of the first task to display */
  int number = 0; /* number of days to display */
  int count = 0;  /* number of days actually display */
  dg87_element *element = NULL;
  dg87_exp_day *day;
  dg87_context *context;
  dg87_command *command;

  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "xdl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first task to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of task to display */
          number = atoi (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first day to display */
  element = NULL;
  if (start > 0)
    element =
      dg87_vector_get_element_by_index (context->exp_day_vector, start);
  else
    element = context->exp_day_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of task to display */
      count++;
      if (number > 0 && count > number)
        break;
      day = element->object;
      /* print day name */
      dg87_connection_printf (connection, "%s%s", beg, day->name);
      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xdi
 * eXploitation Day Information
 * get information for a day
 * Usage : xdi DayName
 */
int
cmd_xdi (dg87_connection * connection)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_exp_day *day;
  dg87_command *command;
  char *string;

  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xdi [options] DayName");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* Task name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Day Name", sep, 
													day->name, end);
  /* start time */
  if ((string = dg87_date_to_ascii (day->start_time)) != NULL)
    {
      dg87_connection_printf (connection, "%s%s%s%s%s\n",
                              beg, "Start Time", sep, 
															string, end);
      dg87_free (string, DG87_CHAR);
    }
  /* enabled property */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Enabled", sep,
                          dg87_bool_to_ascii (day->enabled), end);
  /* ref name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Referential", sep,
                          (day->referential_name ==
                           NULL) ? "" : day->referential_name, end);
  /* auto_purged property */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Auto Purged", sep,
                          dg87_bool_to_ascii (day->auto_purged), end);

  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* xdsa
 * eXploitation Day Set AutoPurged
 * Usage : xdsa DayName AutoPurged 
 */
int
cmd_xdsa (dg87_connection * connection)
{
  dg87_command *command;
  dg87_exp_day *day;
  BOOL auto_purged;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xdsa DayName AutoPurge");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* interpret auto_purged parameter */
  if ((auto_purged = dg87_ascii_to_bool (command->argv[1])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[1]);
      return 1;
    }
  /* change the auto_purged property */
  dg87_err_clear_error ();
  if (dg87_exp_day_set_auto_purged (day, auto_purged) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the 'auto_purge' property of "\
													"the day '%s' ! %s\n"),
											day->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* xdse
 * eXploitation Day Set Enabled 
 * Usage : xdse DayName booleanValue
 */
int
cmd_xdse (dg87_connection * connection)
{
  dg87_exp_day *day;
  dg87_command *command;
  dg87_strbuf *log_msg;
  BOOL value;

  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"xdse DayName BooleanValue");
      return 1;
    }
  /* find the day */
  day = dg87_util_find_exp_day (connection, command->argv[0]);
  if (day == NULL)
    return 1;

  /* interpet enable argument */
  if ((value = dg87_ascii_to_bool (command->argv[1])) == BOOLERROR)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s', use true or false !\n"),
											command->argv[1]);
      return 1;
    }
  if (dg87_exp_day_set_enabled (day, value) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the 'Enabledthe ' property "\
															" of the day '%s' !\n"),
											day->name);
      return 1;
    }
  /* send log entry */
  log_msg = dg87_strbuf_new ();
  dg87_strbuf_printf (log_msg, "xdse %s (by '%s')", 
									command->argv[1],/* value */
                  (connection->user == NULL) ?
                  "anonymous" : connection->user->name);
  dg87_caller__send_day_command (day, log_msg->buffer);
  dg87_strbuf_delete (log_msg);

  return 0;
}

/******************************************************************************/
/* End of file */
